/*
 * Mocha 
 * Copyright (c) 2006 IT Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file AppSndEffect.h
 *
 * This file implements the APIs of Application Sound Manager.
 *
 * @author	mobum
 */
/*:Associate with "Application Sound Manager" */

#ifndef _APP_SND_BASE_H_
#define _APP_SND_BASE_H_

#define APP_SND_INVALID_TYPE		(-1)

#define	APP_SND_MUTE_VOL			(-1)		///< Muted volume level

#define	APP_SND_VIB_INTENSITY_INVALID	(-1)		///< Invalid intensity
#define	APP_SND_VIB_DEFAULT_PLAY_CNT	1		///< default vib count

#define	APP_SND_VOL_LEVEL_0			( 0 * AM_SND_VOLUME_PER_RANGE )	///< 0 volume level 
#define	APP_SND_VOL_LEVEL_1			( 1 * AM_SND_VOLUME_PER_RANGE )	///< 1 volume level 
#define	APP_SND_VOL_LEVEL_MIN		APP_SND_VOL_LEVEL_0		///< min volume level 
#define	APP_SND_VOL_LEVEL_MAX		( AM_SND_VOLUME_CNT_MAX * AM_SND_VOLUME_PER_RANGE )	///< max volume level 

#define APP_SND_CHANNEL_CNT			1	///< Channel count

#define APP_SND_DID_CNT_MAX			2	///< Did max

#define APP_SND_STEP_RANGE_MAX		10	///< Step sound range max

#define APP_SND_INTERVAL_RANGE_MAX	10	///< Interval sound range max

#define	APP_SND_DTMF_FREQUENCY		770 ///< 770 hz
#define	APP_SND_DTMF_DURATION		100 ///< 100 ms

#define APP_SND_DTMF_STR_LEN_MAX	2	///< DTMF string lenth max

#define APP_SND_FILE_MAIN_EXT		""	///< Extention (.mmf)
#define APP_SND_FILE_ALTER_EXT		""	///< Extention (.mmf)

/**
 @brief Enum for AppSndKeyType
 */
typedef enum
{
	APP_SND_KEY_HP_INVALID,
	APP_SND_KEY_HP_MAX			///< max value of key type
} AppSndKeyType;

/**
 @brief Enum for AppSndEffectType
 */
typedef enum
{
	APP_SND_EFFECT_TOUCH_START
} AppSndEffectType;

/**
 @brief Enum for AppSndVibType
 */
typedef enum
{
	APP_SND_VIB_INVALID,	///< Invalid value
	APP_SND_VIB_MAX
} AppSndVibType;

/**
 @brief Enum for AppSndMuteType
 */
typedef enum
{
	APP_SND_MUTE,		///< Mute
	APP_SND_UNMUTE	///< Unmute
} AppSndMuteType;

/**
 @brief Enum for AppSndEffectDtmfType
 */
typedef enum
{
	APP_SND_DTMF_INVALID,	///< Invalid value
} AppSndDtmfType;

typedef enum
{
	APP_SND_REPEAT_ALL		= 0x0003,	///< Audio | Dtmf
} AppSndRepeatType;


/**
 @brief Struct for AppSndMuteInfo
 */
typedef struct
{
	DID				did;		///< did
} AppSndMuteInfo;

// Effect sound callback
typedef	BOOL (*AppSndCallback)(void* pParam);

#ifdef __cplusplus

// type -> type
template <typename T>
struct Type2Type
{
	typedef T OriginalType;
};

template<typename T>
class CAppSndSingleton
{
private:
	CAppSndSingleton(const CAppSndSingleton&);
	CAppSndSingleton& operator=(const CAppSndSingleton&);

protected:
	CAppSndSingleton() {}
	virtual ~CAppSndSingleton() {}

public:
	static T& Instance(void)
	{
		static T obj;
		return obj;
	}
};

#endif // __cplusplus

#endif // _APP_SND_BASE_H_
