/*
 * Samsung Handset Platform
 * Copyright (c) 2000 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */

/*:Associate with "ANSI C" */

#ifndef _AC_ECOM_H_
#define _AC_ECOM_H_

#include "SysEcom.h"
#include <stdarg.h>		// for va_list
#include <time.h>		// for struct tm

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * ECOM Interface
 */
ECOM_INTERFACE(EAnsiC)
{
	DECLARE_EBASE()

	// string.h 
	void*		(*Memchr)(const void* buf, int c, size_t count);
	int			(*Memcmp)(const void* pMem1, const void* pMem2, size_t length);
	int			(*WMemcmp)(const wchar_t* s1,const wchar_t*  s2, size_t n);

	void*		(*MemcpyEx)(void* pDest, const void* pSrc, size_t length, unsigned int uBufferSize, const char* szFileName, int nLine);
	void*		(*Memmove)(void* pDest, const void* pSrc, size_t length);
	void*		(*MemmoveEx)(void* pDest, const void* pSrc, size_t length);
	void*		(*MemsetEx)(void* pMem, int value, size_t length, unsigned int uBufferSize ,const char* szFileName, int nLine);
	wchar_t *	(*WMemset)(wchar_t *wcs, wchar_t wc, size_t n);
	void*		(*Memcpy16)(void* pDest, const void* pSrc, size_t length);
	void*		(*Memset16)(void* pMem, int value, size_t length);

	char*		(*StrcatEx)(char *szDest, const char *szSrc, unsigned int uDestSize, const char* szFileName, int nLine);
	char*		(*Strchr)(const char *s, int c);
	int			(*Strcmp)(const char* s1, const char* s2);
	char*		(*StrcpyEx)(char *szDest, const char *szSrc, unsigned int uDestSize, const char* szFileName, int nLine);
	int			(*Stricmp)(const char* dst, const char* src);
	size_t		(*Strlen)(const char* s);
	char*		(*StrncatEx)(char *szDest, const char *szSrc, unsigned int count, unsigned int uDestSize, const char* szFileName, int nLine);
	int			(*Strncmp)(const char* s1, const char* s2, size_t count);
	char*		(*StrncpyEx)(char *szDest, const char *szSrc, unsigned int count, unsigned int uDestSize, const char* szFileName, int nLine);
	int			(*Strnicmp)(const char* dst, const char* src, size_t n);
	char*		(*Strpbrk)(const char *s, const char *accept);
	char*		(*Strrchr)(const char* s, int c);
	char*		(*Strstr)(const char *s, const char *needle);
	char*		(*Stristr)(const char *s, const char *needle);
	char *    		(*Strlwr)(char * s);
	char*    		(*Strupr)(char* s);
		
	char*		(*Strtok)(char* token, const char* delimit);
	char *            (*StrtokS) (char * string, const char * control, char ** context);
	
	//for wide string

	size_t 		(*Wcslen)(const wchar_t *s);
	wchar_t *	(*Wcscpy)(wchar_t *dest, const wchar_t *src);
	wchar_t *	(*Wcsncpy) (wchar_t *dest, const wchar_t *src, size_t n);
	int 			(*Wcsicmp)(const wchar_t *s1, const wchar_t *s2);
	int 			(*Wcsncmp)(const wchar_t *s1, const wchar_t *s2, size_t n);
	int 			(*Wcscmp)(const wchar_t *s1, const wchar_t *s2);
	wchar_t *	(*Wcschr)(const wchar_t *s, wchar_t c);
	wchar_t *	(*Wcsstr)(const wchar_t * s1,const wchar_t * s2);
	//for wide string collation
	int 			(*WStrCollate)(const wchar_t *ws1, const wchar_t *ws2);
	int 			(*WStrnCollate)(const wchar_t *ws1, const wchar_t *ws2, size_t count);
	int 			(*WCharCollate)(wchar_t wch1,wchar_t wch2);
	
	int 			(*SprintfSEx)(char *buffer, unsigned int bufferSize, const char* szFileName, int nLine , ... );
	AcErrnoT 	(*StrcatSEx)(char *szDest, unsigned int uDestSize, const char *szSrc,const char* szFileName, int nLine);
	AcErrnoT 	(*StrncatSEx)(char *szDest, unsigned int uDestSize, const char *szSrc, unsigned int count,const char* szFileName, int nLine);
	AcErrnoT 	(*StrcpySEx)(char *szDest, unsigned int uDestSize, const char *szSrc,const char* szFileName, int nLine);
	AcErrnoT 	(*StrncpySEx)(char *szDest, unsigned int uDestSize, const char *szSrc,unsigned int count ,const char* szFileName, int nLine);
	AcErrnoT 	(*StrcatCchEx)(char *szDest, unsigned int uDestSize, const char *szSrc,const char* szFileName, int nLine);
	AcErrnoT 	(*StrncatCchEx)(char *szDest, unsigned int uDestSize, const char *szSrc, unsigned int count,const char* szFileName, int nLine);
	AcErrnoT		(*StrcpyCchEx)(char *szDest, unsigned int uDestSize, const char *szSrc,const char* szFileName, int nLine);
	AcErrnoT 	(*StrncpyCchEx)(char *szDest, unsigned int uDestSize, const char *szSrc, unsigned int count,const char* szFileName, int nLine);
	int			(*SscanfSEx) (const char* buffer,const  char* format,const char* szFileName, int nLine , ... );	

	
	
	int 			(*Strchop) (char *str);
	char * 		(*Strrev)(char *str);  
	char * 		(*Strsoundex) (char *soundexbuf, char *str);
	char * 		(*Strlcpy) (char *dest, char *src, int length);
	char * 		(*Strmcpy) (char *dest, char *src, int pos, int length);
	char * 		(*Strrcpy) (char *dest, char *src, int length);
	char * 		(*Strinsert) (char *dest, char *src, char *to_insert, int place);
	char * 		(*Strtrim) (char *str);
	char * 		(*Strrtrim) (char *str);
	char * 		(*Strltrim) (char *str);
	char * 		(*Strsqz) (char *str);
	char * 		(*Strrmchar) (char *string, char * chars_to_rm);
	char * 		(*Strrmrange) (char *str, char clo, char chi);
	char * 		(*Strrmstr) (char *string, char *substr_to_rm);
	int 			(*Strsplit) (char * strv[], int strv_size, char * str, char * toks, int sqz_consec);
	int 			(*Strsplit_csv) (char * strv[], int strv_size, char * str);
	char * 		(*Strtolower) (char *str);
	char * 		(*Strunquote) (char * str);

	
	// stdlib.h
	int			(*Atoi)(const char *nptr);
	char*		(*Itoa)(int cible, char* chaine, int radix);
	long		(*Atol)(const char* s);
	double		(*Atof)(const char* s);
	double		(*Strtod)(const char* nptr, char** endptr);
	long		(*Strtol)(const char* nptr, char** endptr, int base);
	unsigned long (*Strtoul)(const char *nptr, char **endptr, int base);
	void		(*Qsort)( void *base, size_t num, size_t width, int (*compare )(const void *elem1, const void *elem2 ) );
	void*		(*Bsearch)(const void *key, const void *base, size_t num, size_t width, int (*compare ) ( const void *elem1, const void *elem2 ) );

	int 			(*Atoin)(const char* szStr, int size); // new creation

	// ctype.h
	int			(*Isalnum)(int ch);
	int			(*Isalpha)(int ch);
	int			(*Isdigit)(int  ch);
	int			(*Isspace)(int	ch);
	int			(*Iscntrl)(int ch);
	int			(*Islower)(int ch);
	int			(*Isupper)(int ch);
	int			(*Isxdigit)(int ch);
	int			(*Tolower)(int ch);
	int			(*Toupper)(int ch);

	//for wide string 
	wchar_t 		(*Towlower)(wchar_t c);
	wchar_t 		(*Towupper)(wchar_t c);
	int 			(*Iswctype)(wchar_t c,wchar_t desc);
	int 			(*Iswupper)(wchar_t c);   
	int 			(*Iswlower)(wchar_t c);    
	int 			(*Iswalnum)(wchar_t c);
	int 			(*Iswalpha)(wchar_t c);
	int 			(*Iswprint)(wchar_t c);
	int 			(*Iswpunct)(wchar_t c);    
	int 			(*Iswcntrl)(wchar_t c);
	int 			(*Iswspace)(wchar_t c); 
	int 			(*Iswdigit)(wchar_t c);
	int 			(*Iswxdigit)(wchar_t c);
	int 			(*Isgraph)(wchar_t c);
	int 			(*Iswascii)(wchar_t c);

	// stdio.h
	int			(*Vsprintf)(char* buf,  const char* format,  va_list va);
	int                  (*Vsnprintf)(char *buf,  size_t bufsize, const  char *format,  va_list va);
	int			(*Sprintf)(char* buf,   const char* format, ...);
	int                 (*Snprintf)(char *buf, size_t bufsize, const char *format,...);
	int			(*Sscanf)(const char* buf, const char* fmt, ...);

	// math.h
	double		(*Pow)(double x, double y);
	double		(*Sqrt)(double x);
	int			(*Rand)(void);
	void		(*Srand)(unsigned int seed);
	int			(*MultiSin)(int multiplier, int sin);
	int			(*MultiCos)(int multiplier, int cos);

	// time.h
	char*		(*Asctime)(const struct tm* timeptr);
	char*		(*Ctime)(const time_t* timer);
	struct tm*	(*Gmtime)(const time_t* timer);
	struct tm*	(*Localtime)(const time_t* timer);
	time_t		(*Mktime)(struct tm* timeptr);
	time_t		(*Time)(time_t* timer);
	clock_t		(*Clock)(void);
	void			(*Tzset)(void);
	size_t		(*Strftime)(char* dst, size_t maxsize, const char* fmt, const struct tm* timeptr);
	
	// locale.h
	char *		(*SetLocale)(int category, const char *locale); 
};


/*
 * Interface Accessor
 */
#if (defined(_SHP_SIMUL) && defined(_SHP_CALL_DLL_ANSIC)) || (defined(_SHP_DLL) && ! defined(_SHP_DLL_ANSIC) && ! defined(_SHP_CALL_DLL_ANSIC))
// string.h
#	define AcDispatch			(__AcGetVtbl()->Dispatch)
#	define AcMemchr			(__AcGetVtbl()->Memchr)
#	define AcMemcmp			(__AcGetVtbl()->Memcmp)
#	define AcWMemcmp		(__AcGetVtbl()->WMemcmp)

#	define AcMemcpy(pDest, pSrc, length)			(__AcGetVtbl()->MemcpyEx(pDest, pSrc, length, (unsigned int)sizeof(pDest), __FILE__, __LINE__))
#	define AcMemmove		(__AcGetVtbl()->Memmove)
#	define AcMemmoveEx		(__AcGetVtbl()->MemmoveEx)
#	define AcMemset(pMem, value, length)			(__AcGetVtbl()->MemsetEx(pMem, value, length, (unsigned int)sizeof(pMem), __FILE__, __LINE__))
#	define AcMemcpy16		(__AcGetVtbl()->Memcpy16)
#	define AcMemset16		(__AcGetVtbl()->Memset16)
#	define AcWMemset		(__AcGetVtbl()->WMemset)


#	define AcStrcat(szDest, szSrc)			(__AcGetVtbl()->StrcatEx(szDest, szSrc, (unsigned int)sizeof(szDest), __FILE__, __LINE__))
#	define AcStrchr			(__AcGetVtbl()->Strchr)
#	define AcStrcmp			(__AcGetVtbl()->Strcmp)
#	define AcStrcpy(szDest, szSrc)			(__AcGetVtbl()->StrcpyEx(szDest, szSrc, (unsigned int)sizeof(szDest), __FILE__, __LINE__))
#	define AcStricmp		(__AcGetVtbl()->Stricmp)
#	define AcStrlen			(__AcGetVtbl()->Strlen)
#	define AcStrncat(szDest, szSrc, count)		(__AcGetVtbl()->StrncatEx(szDest, szSrc, count, (unsigned int)sizeof(szDest), __FILE__, __LINE__))
#	define AcStrncmp		(__AcGetVtbl()->Strncmp)
#	define AcStrncpy(szDest, szSrc, count)		(__AcGetVtbl()->StrncpyEx(szDest, szSrc, count, (unsigned int)sizeof(szDest), __FILE__, __LINE__))
#	define AcStrnicmp		(__AcGetVtbl()->Strnicmp)
#	define AcStrpbrk		(__AcGetVtbl()->Strpbrk)
#	define AcStrrchr		(__AcGetVtbl()->Strrchr)
#	define AcStrstr			(__AcGetVtbl()->Strstr)
#	define AcStristr		(__AcGetVtbl()->Stristr)
#	define AcStrlwr		(__AcGetVtbl()->Strlwr)
#	define AcStrupr		(__AcGetVtbl()->Strupr)
#	define AcStrtok		(__AcGetVtbl()->Strtok)
#	define AcStrtokS		(__AcGetVtbl()->StrtokS)
//for wide string
#	define AcWcslen		(__AcGetVtbl()->Wcslen)	
#	define AcWcscpy		(__AcGetVtbl()->Wcscpy)
#	define AcWcsncpy	(__AcGetVtbl()->Wcsncpy)
#	define AcWcsicmp	(__AcGetVtbl()->Wcsicmp)
#	define AcWcsncmp	(__AcGetVtbl()->Wcsncmp)
#	define AcWcscmp		(__AcGetVtbl()->Wcscmp)	
#	define AcWcschr		(__AcGetVtbl()->Wcschr)
#	define AcWcsstr		(__AcGetVtbl()->Wcsstr)

#	define AcStrchop	(__AcGetVtbl()->Strchop)
#	define AcStrrev	(__AcGetVtbl()->Strrev)
#	define AcStrsoundex	(__AcGetVtbl()->Strsoundex)
#	define AcStrlcpy	(__AcGetVtbl()->Strlcpy)
#	define AcStrmcpy	(__AcGetVtbl()->Strmcpy)
#	define AcStrrcpy	(__AcGetVtbl()->Strrcpy)
#	define AcStrinsert	(__AcGetVtbl()->Strinsert)
#	define AcStrtrim	(__AcGetVtbl()->Strtrim)
#	define AcStrrtrim	(__AcGetVtbl()->Strrtrim)
#	define AcStrltrim	(__AcGetVtbl()->Strltrim)
#	define AcStrsqz	(__AcGetVtbl()->Strsqz)
#	define AcStrrmchar	(__AcGetVtbl()->Strrmchar)
#	define AcStrrmrange	(__AcGetVtbl()->Strrmrange)
#	define AcStrrmstr	(__AcGetVtbl()->Strrmstr)
#	define AcStrsplit	(__AcGetVtbl()->Strsplit)
#	define AcStrsplit_csv	(__AcGetVtbl()->Strsplit_csv)
#	define AcStrtolower	(__AcGetVtbl()->Strtolower)
#	define AcStrunquote	(__AcGetVtbl()->Strunquote)


// stdlib.h
#	define AcAtoi			(__AcGetVtbl()->Atoi)
#	define AcItoa			(__AcGetVtbl()->Itoa)
#	define AcAtol			(__AcGetVtbl()->Atol)
#	define AcAtof			(__AcGetVtbl()->Atof)
#	define AcStrtod			(__AcGetVtbl()->Strtod)
#	define AcStrtol			(__AcGetVtbl()->Strtol)
#	define AcStrtoul		(__AcGetVtbl()->Strtoul)
#	define AcQsort			(__AcGetVtbl()->Qsort)
#	define AcBsearch		(__AcGetVtbl()->Bsearch)
#	define AcAtoin			(__AcGetVtbl()->Atoin)
// ctype.h
#	define AcIsalnum		(__AcGetVtbl()->Isalnum)
#	define AcIsalpha		(__AcGetVtbl()->Isalpha)
#	define AcIsdigit		(__AcGetVtbl()->Isdigit)
#	define AcIsspace		(__AcGetVtbl()->Isspace)	
#	define AcIscntrl		(__AcGetVtbl()->Iscntrl)
#	define AcIslower		(__AcGetVtbl()->Islower)
#	define AcIsupper		(__AcGetVtbl()->Isupper)
#	define AcIsxdight		(__AcGetVtbl()->Isxdigit)
#	define AcTolower		(__AcGetVtbl()->Tolower)
#	define AcToupper		(__AcGetVtbl()->Toupper)
//for wide string
#	define AcTowlower	(__AcGetVtbl()->Towlower)
#	define AcTowupper	(__AcGetVtbl()->Towupper)
#	define AcIswctype	(__AcGetVtbl()->Iswctype)
#	define AcIswupper	(__AcGetVtbl()->Iswupper)
#	define AcIswlower	(__AcGetVtbl()->Iswlower)
#	define AcIswalnum	(__AcGetVtbl()->Iswalnum)
#	define AcIswalpha	(__AcGetVtbl()->Iswalpha)
#	define AcIswprint		(__AcGetVtbl()->Iswprint)
#	define AcIswpunct	(__AcGetVtbl()->Iswpunct)
#	define AcIswcntrl		(__AcGetVtbl()->Iswcntrl)
#	define AcIswspace	(__AcGetVtbl()->Iswspace)
#	define AcIswdigit		(__AcGetVtbl()->Iswdigit)
#	define AcIswxdigit	(__AcGetVtbl()->Iswxdigit)
#	define AcIsgraph		(__AcGetVtbl()->Isgraph)	
#	define AcIswascii		(__AcGetVtbl()->Iswascii)

//for wide string collation
#	define AcWStrCollate		(__AcGetVtbl()->WStrCollate)
#	define AcWStrnCollate		(__AcGetVtbl()->WStrnCollate)
#	define AcWCharCollate	(__AcGetVtbl()->WCharCollate)

#	define AcSprintfS(buffer,bufferSize,...)			(__AcGetVtbl()->SprintfSEx(buffer, bufferSize, __FILE__, __LINE__, __VA_ARGS__))
#	define AcStrcatS(szDest,uDestSize,szSrc)		(__AcGetVtbl()->StrcatSEx(szDest,uDestSize,szSrc,__FILE__, __LINE__))
#	define AcStrncatS(szDest,uDestSize,szSrc,count)		(__AcGetVtbl()->StrncatSEx(szDest,uDestSize,szSrc,count,__FILE__, __LINE__))
#	define AcStrcpyS(szDest,uDestSize,szSrc)		(__AcGetVtbl()->StrcpySEx(szDest,uDestSize,szSrc,__FILE__, __LINE__))
#	define AcStrncpyS(szDest,uDestSize,szSrc,count)		(__AcGetVtbl()->StrncpySEx(szDest,uDestSize,szSrc,count,__FILE__, __LINE__))
#	define AcStrcatCch(szDest,uDestSize,szSrc)		(__AcGetVtbl()->StrcatCchEx(szDest,uDestSize,szSrc,__FILE__, __LINE__))
#	define AcStrncatCch(szDest,uDestSize,szSrc,count)		(__AcGetVtbl()->StrncatCchEx(szDest,uDestSize,szSrc,count,__FILE__, __LINE__))
#	define AcStrcpyCch(szDest,uDestSize,szSrc)		(__AcGetVtbl()->StrcpyCchEx(szDest,uDestSize,szSrc,__FILE__, __LINE__))
#	define AcStrncpyCch(szDest,uDestSize,szSrc,count)		(__AcGetVtbl()->StrncpyCchEx(szDest,uDestSize,szSrc,count,__FILE__, __LINE__))
#	define AcSscanfS(buffer,format,...)						(__AcGetVtbl()->SscanfSEx(buffer,format,__FILE__, __LINE__, __VA_ARGS__))		

// stdio.h
#	define AcVsprintf	       (__AcGetVtbl()->Vsprintf)
#	define AcVsnprintf        (__AcGetVtbl()->Vsnprintf)
#	define AcSprintf		(__AcGetVtbl()->Sprintf)
#	define AcSnprintf		(__AcGetVtbl()->Snprintf)
#	define AcSscanf			(__AcGetVtbl()->Sscanf)


// math.h
#	define AcPow			(__AcGetVtbl()->Pow)
#	define AcSqrt			(__AcGetVtbl()->Sqrt)
#	define AcRand			(__AcGetVtbl()->Rand)
#	define AcSrand			(__AcGetVtbl()->Srand)
#	define AcMultiSin		(__AcGetVtbl()->MultiSin)
#	define AcMultiCos		(__AcGetVtbl()->MultiCos)

// time.h
#	define AcAsctime		(__AcGetVtbl()->Asctime)
#	define AcCtime			(__AcGetVtbl()->Ctime)
#	define AcGmtime			(__AcGetVtbl()->Gmtime)
#	define AcLocaltime		(__AcGetVtbl()->Localtime)
#	define AcMktime			(__AcGetVtbl()->Mktime)
#	define AcTime			(__AcGetVtbl()->Time)
#	define AcClock			(__AcGetVtbl()->Clock)
#	define AcTzset			(__AcGetVtbl()->Tzset)
#	define AcStrftime		(__AcGetVtbl()->Strftime)

// locale.h
#	define AcSetLocale		(__AcGetVtbl()->SetLocale)
#endif // _SHP_DLL


/*
 * ECOM Type
 */
typedef struct
{
	DECLARE_ECOM_VTBL(EAnsiC)
	UINT32			refCount;
} TAnsiC;

extern TAnsiC ansiC;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif // _AC_ECOM_H_
