/*
 * Mocha
 * Copyright (c) 2000-2003 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */

/*:Associate with "Registry Manager" */

#ifndef	_REG_ECOM_H_
#define _REG_ECOM_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * ECOM Interface
 */
ECOM_INTERFACE(ERegMgr)
{
	DECLARE_EBASE()

	HReg	(*Load)(const char* szFileName, BOOL bCreatMode);
	BOOL	(*Unload)(HReg hReg);

	BOOL	(*Clean)(const char* szFileName);

	int		(*GetInt)(HReg hReg, const char* szSection, const char* szKey, int defaultValue);
	double	(*GetReal)(HReg hReg, const char* szSection, const char* szKey, double defaultValue);
	BOOL	(*GetString)(HReg hReg, const char* szSection, const char* szKey, const char* szDefaultValue, char* pDestBuffer, int nSize);
	BOOL	(*GetSection)(HReg hReg, const char* szSection, char* pBuffer, int nSize);
	BOOL	(*GetFirstEntry)(HReg hReg, const char* szSection, char* szKey, char* szValue);
	BOOL	(*GetNextEntry)(HReg hReg, const char* szSection, char* szKey, char* szValue);

	BOOL	(*SetInt)(HReg hReg, const char* szSection, const char* szKey, int value);
	BOOL	(*SetReal)(HReg hReg, const char* szSection, const char* szKey, double value);
	BOOL	(*SetString)(HReg hReg, const char* szSection, const char* szKey, char* pValue);
	BOOL	(*SetSection)(HReg hReg, const char* szSection, char* pDataBuffer);

	BOOL	(*RemoveKey)(HReg hReg, const char* szSection, const char* szKey);
	BOOL	(*RemoveSection)(HReg hReg, const char* szSection);

	BOOL	(*Flush)(HReg hReg);
};

/*
 * Interface Accessor
 */
// Native/DLL Call
#if (defined(_SHP_SIMUL) && defined(_SHP_CALL_DLL_REGMGR)) || (defined(_SHP_DLL) && ! defined(_SHP_DLL_REGMGR) && ! defined(_SHP_CALL_DLL_REGMGR))
#	define RegDispatch			(__RegGetVtbl()->Dispatch)
#	define RegLoad				(__RegGetVtbl()->Load)
#	define RegUnload				(__RegGetVtbl()->Unload)

#	define RegClean				(__RegGetVtbl()->Clean)

#	define RegGetInt				(__RegGetVtbl()->GetInt)
#	define RegGetReal			(__RegGetVtbl()->GetReal)
#	define RegGetString			(__RegGetVtbl()->GetString)
#	define RegGetSection			(__RegGetVtbl()->GetSection)
#	define RegGetFirstEntry			(__RegGetVtbl()->GetFirstEntry)
#	define RegGetNextEntry			(__RegGetVtbl()->GetNextEntry)

#	define RegSetInt				(__RegGetVtbl()->SetInt)
#	define RegSetReal			(__RegGetVtbl()->SetReal)
#	define RegSetString			(__RegGetVtbl()->SetString)
#	define RegSetSection			(__RegGetVtbl()->SetSection)

#	define RegRemoveKey			(__RegGetVtbl()->RemoveKey)
#	define RegRemoveSection		(__RegGetVtbl()->RemoveSection)

#	define RegFlush				(__RegGetVtbl()->Flush)
#endif // _SHP_DLL


/*
 * ECOM Type
 */
typedef struct
{
	DECLARE_ECOM_VTBL(ERegMgr)
	UINT32		refCount;
} TRegMgr;

extern TRegMgr	regMgr;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif	// _REG_ECOM_H_
