/*
 * Samsung Handset Platform
 * Copyright (c) 2000 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */

/*:Associate with "SysBase" */

#ifndef _SYS_BASE_ECOM_H_
#define _SYS_BASE_ECOM_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "SysECOM.h"

typedef enum
{
	ASSERTION_ASSERT,
	ASSERTION_REQUIRE,
	ASSERTION_ENSURE,
	ASSERTION_CHECK,
	ASSERTION_NEVER_GET_HERE,
	ASSERTION_MAX
} Assertion;


/*
 * ECOM Interface
 */
ECOM_INTERFACE(ESysBase)
{
	DECLARE_EBASE()

	BOOL		(*__RegisterClass)(EComp* pEComp, ECOMCLSID clsID, void* pClass, DID did);
	BOOL		(*UnregisterClass)(EComp* pEComp, ECOMCLSID clsID);
	void*		(*GetInterface)(ECOMCLSID clsID);
	void*		(*GetInterfaceEx)(ECOMCLSID clsID);

	void		(*SetLastError)(UINT32 err);
	UINT32		(*GetLastError)(void);

	void		(*DebugPrintf)(UINT32 code, const char* format, ...);
	void		(*_AssertBreakpoint)(Assertion assertType);
	void		(*_AssertReport)(int type, const char *reason, const char *expr, const char *szFileName, int lineNum);
	
	// DLL Context
	const char*	(*_GetDllNameByDID)(DID did);
	const char*	(*_GetAppNameByDID)(DID did);
	DID			(*_GetDidByDllName)(const char* szDLL);
	DID			(*_GetDidByAppName)(const char* szApp);
	BOOL		(*_IsDefaultPseudoDLL)(DID did);
};

/*
 * Interface Accessor
 */
// Native Call
#if (defined(_SHP_SIMUL) && defined(_SHP_CALL_DLL_SYSBASE)) || (defined(_SHP_DLL) && ! defined(_SHP_DLL_SYSBASE) && ! defined(_SHP_CALL_DLL_SYSBASE))
#	define SysDispatch		(__SysGetVtbl()->Dispatch)

#if defined(_SHP_MY_DID)
#	define SysRegisterClass(pEComp, clsID, pClass)	(__SysGetVtbl()->__RegisterClass)(pEComp, clsID, pClass, _SHP_MY_DID)
#else
#	define SysRegisterClass(pEComp, clsID, pClass)	(__SysGetVtbl()->__RegisterClass)(pEComp, clsID, pClass, DID_MOCHA_BUILT_IN)
#endif	// _SHP_MY_DID

#	define SysUnregisterClass		(__SysGetVtbl()->UnregisterClass)
#	define SysGetInterface			(__SysGetVtbl()->GetInterface)
#	define SysGetInterfaceEx		(__SysGetVtbl()->GetInterfaceEx)
#	define SysDebugPrintf			(__SysGetVtbl()->DebugPrintf)
#	define _SysAssertBreakpoint		(__SysGetVtbl()->_AssertBreakpoint)
#	define _SysAssertReport			(__SysGetVtbl()->_AssertReport)
#	define SysSetLastError			(__SysGetVtbl()->SetLastError)
#	define SysGetLastError			(__SysGetVtbl()->GetLastError)

#	define _SysGetDllNameByDID		(__SysGetVtbl()->_GetDllNameByDID)
#	define _SysGetAppNameByDID		(__SysGetVtbl()->_GetAppNameByDID)
#	define _SysGetDidByDllName		(__SysGetVtbl()->_GetDidByDllName)
#	define _SysGetDidByAppName		(__SysGetVtbl()->_GetDidByAppName)
#	define _SysIsDefaultPseudoDLL	(__SysGetVtbl()->_IsDefaultPseudoDLL)
#endif

/*
 * ECOM Type
 */
typedef struct
{
	DECLARE_ECOM_VTBL(ESysBase)
	UINT32			refCount;
} TSysBase;

extern TSysBase sysBase;


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif // _SYS_BASE_ECOM_H_
