/*
 * Mocha
 * Copyright (c) 2001, 2002 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/**
 * @file		ProtoAccount.h
 * @brief		This file implementats the Protocol Manager.
 * @version	1.0
 */
 
#ifndef _PROTO_ACCOUNT_H_
#define _PROTO_ACCOUNT_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "Shp.h"


/* Account reference type */
typedef enum
{
	PROTO_ACCOUNT_REF_IN_USE,
	PROTO_ACCOUNT_REF_IN_SERVICE
} ProtoAccountRefType;

/* Basic account information */
typedef struct
{
	HProtoAccount			hProtoAccount;		/**< Account handle */
	BOOL				bDefault;			/**< To be changeable by users */
	BOOL				bChargeFree;
	ProtoAppType			protoAppType;		/**< Application protocol type(WAP, HTTP, OBEX etc) */
	char					szAccountName[PROTO_NETINFO_NAME_LEN_MAX+1];	/**< Account name */
} ProtoAccountInfo;

/* Account name */
typedef struct
{
	char					szAccountName[PROTO_NETINFO_NAME_LEN_MAX+1];	/**< Account name */
}ProtoAccountName;

/* Account information including the network information; Each account can share the same network information. */
typedef struct
{
	BOOL				bActive;				/**< All valid accounts are TRUE. If FALSE, it is an empty account. */
	ProtoAccountInfo		accountInfo;
	ProtoNetInfo			netInfo;
} ProtoAccountInfoEx;

/* Account information to manage all account entities. */
typedef struct
{
	int					version;
	int					count;
	char					szPreferredName[PROTO_NETINFO_NAME_LEN_MAX+1];
	ProtoAccountInfoEx	accountInfoEx[PROTO_ACCOUNT_COUNT_MAX];
} ProtoAccountInfoTable;

typedef enum
{
	PROTO_WAP_NORMAL					= 0x00,
	PROTO_WAP_SECURE					= 0x01
} ProtoWapType;

typedef enum
{
	PROTO_EDIT_ENABLE					= 0x00,
	PROTO_EDIT_UNABLE					= 0x01
} ProtoEditType;


#define PROTO_CONVERT_EDITABLE_LENGTH_MAX              3
#define PROTO_CONVERT_AUTH_LENGTH_MAX    		   6
#define PROTO_CONVERT_PROTOCOL_LENGTH_MAX             5
#define PROTO_CONVERT_ADDR_LENGTH_MAX                          64
#define PROTO_CONVERT_PORT_LENGTH_MAX                            5
#define PROTO_CONVERT_SECURE_CONN_LENGTH_MAX             3
#define PROTO_CONVERT_LINGER_TIME_LENGTH_MAX             5
#define PROTO_CONVERT_IP_ADDR_LENGTH_MAX             15
#define PROTO_CONVERT_TRAFFIC_CLASS_LENGTH_MAX             12


typedef struct _CscXmlCustomerInfo
{
	char		accountName[PROTO_NETINFO_NAME_LEN_MAX+1];
	char 	editable[PROTO_CONVERT_EDITABLE_LENGTH_MAX+1];
	char		authType[PROTO_CONVERT_AUTH_LENGTH_MAX+1];
	char		apn[PROTO_APN_ADDR_LEN_MAX+1];								/* JJ */
	char		authID[PROTO_AUTH_ID_LEN_MAX+1];
	char 	authPwd[PROTO_AUTH_PASSWD_LEN_MAX+1];
	char   	protocol[PROTO_CONVERT_PROTOCOL_LENGTH_MAX+1];
	char	       homeUrl[PROTO_HOME_URL_LEN_MAX + 1];
	char        proxyAddr[PROTO_IPV4_STR_LEN_MAX+1];              // may be Proxy is host
	char        proxyPort[PROTO_CONVERT_PORT_LENGTH_MAX+1];
	char        gatewayAddr[PROTO_CONVERT_IP_ADDR_LENGTH_MAX+1];
	char		secureConn[PROTO_CONVERT_SECURE_CONN_LENGTH_MAX+1];
	char 	lingerTime[PROTO_CONVERT_LINGER_TIME_LENGTH_MAX+1];
	char		staticIPAddr[PROTO_CONVERT_IP_ADDR_LENGTH_MAX+1];
	char		dnsAddr[PROTO_CONVERT_IP_ADDR_LENGTH_MAX+1];
	char 	trafficClass[PROTO_CONVERT_TRAFFIC_CLASS_LENGTH_MAX+1];
} CscXmlCustomerInfo;


/*
 * Public Function Prototypes
 */
HProtoAccount		ProtoCreateAccount(const ProtoNetInfo* pInfo);
BOOL			ProtoDeleteAccount(HProtoAccount hProtoAccount);
BOOL			ProtoSetAccount(HProtoAccount hProtoAccount, const ProtoNetInfo* pInfo);
BOOL			ProtoGetAccount(HProtoAccount hProtoAccount, ProtoNetInfo* pInfo);
BOOL			ProtoGetAccountHandleByName(HProtoAccount* pHProtoAccount, const char* szAccountName);
BOOL			ProtoEnumerateAccountInfo(ProtoAccountInfo* pAccountInfo, int bufCount, int* pCount);
BOOL			ProtoEnumerateAccountHandles(HProtoAccount* pHBuf, int bufCount, int* pCount);
BOOL			ProtoEnumerateAccountNames(ProtoAccountName* pNameBuf, int bufCount, int* pCount);
BOOL			ProtoGetPreferredAccount(HProtoAccount* pHProtoAccount);
BOOL			ProtoSetPreferredAccount(HProtoAccount hProtoAccount);
BOOL			ProtoGetAccountStatus(HProtoAccount  hProtoAccount, BOOL* pbInService, BOOL* pbDefault);
BOOL			ProtoIsFreeAccount(HProtoAccount hProtoAccount);

#ifdef __cplusplus
}
#endif  /* __cplusplus */


#endif // _PROTO_ACCOUNT_H_

