/*
 * Mocha
 * Copyright (c) 2001, 2002 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/**
 * @file		ProtoECOM.h
 * @brief		This file implementats the Protocol Manager.
 * @version	1.0
 */
 
#ifndef _PROTO_ECOM_H_
#define _PROTO_ECOM_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * ECOM Interface
 */
ECOM_INTERFACE(EProtoMgr)
{
	DECLARE_EBASE()

	/* ProtoMgr Apis */
	HProtocol				(*RegisterClientByEventHandler)(ProtoEventClass eventClass, char* szEventHandler, char* szAppName);
	HProtocol				(*RegisterClient)(ProtoEventClass eventClass, ProtoNetworkCallback callback, char* szAppName);
	BOOL				(*UnregisterClient)(HProtocol hProtocol);
	HProtocol				(*RegisterClientByEventHandlerEx)(ProtoEventClass eventClass, char* szEventHandlerName, char* szAppName);
	HProtocol				(*RegisterClientEx)(ProtoEventClass eventClass, ProtoNetworkCallback callback, char* szAppName);
	BOOL				(*SetClientRegEx)(HProtocol	hProtocol, BOOL	bEx);
	BOOL				(*SetClientEventClassEx)(HProtocol hProtocol, ProtoEventClass eventClass, BOOL bGetEvent);

	BOOL				(*StartNetwork)(HProtocol hProtocol);
	BOOL				(*StartNetworkByAccount)(HProtoAccount hProtoAccount);
	BOOL				(*StopNetwork)(HProtocol hProtocol);
	BOOL				(*StopNetworkByAccount)(HProtoAccount hProtoAccount);

	BOOL				(*SetNetworkInfo)(HProtocol hProtocol, HProtoAccount hProtoAccount);
	const ProtoNetInfo*	(*GetNetworkInfo)(HProtocol hProtocol);
	const ProtoNetInfo*	(*GetNetworkInfoEx)(HProtocol hProtocol, HProtoAccount* pHProtoAccount);
	BOOL				(*GetNetworkStateByOpMode)(ProtoOpMode opMode, ProtoState* pProtoState);
	BOOL				(*SetInactiveTime)(HProtoAccount hProtoAccount, ProtoInactiveType type, ULONG* pTimeout);
	
	BOOL				(*EnumerateActiveAccountHandles)(HProtoAccount* pHBuf, int bufLen, int* pHCount);
	BOOL				(*EnumerateHandlesByAccount)(HProtoAccount hProtoAccount, HProtocol* pHBuf, int bufLen, int* pHCount);
	BOOL				(*BindAccount)(HProtocol hProtocol, HProtoAccount hProtoAccount);
	BOOL				(*UnbindAccount)(HProtocol hProtocol, HProtoAccount hProtoAccount);

	BOOL				(*IsDataCallPossible)(ProtoOpMode);

	/* ProtoStat Apis */
	BOOL				(*GetStatInfo)(ProtoOpMode opMode, ProtoStatInfo* pStatInfo);
	BOOL				(*ResetStatInfo)(ProtoOpMode opMode, ProtoStatType statType);

	/* ProtoAccount Apis */
	HProtoAccount			(*CreateAccount)(const ProtoNetInfo* pInfo);
	BOOL				(*DeleteAccount)(HProtoAccount hProtoAccount);
	BOOL				(*SetAccount)(HProtoAccount hProtoAccount, const ProtoNetInfo* pInfo);
	BOOL				(*GetAccount)(HProtoAccount hProtoAccount, ProtoNetInfo* pInfo);
	BOOL				(*GetAccountHandleByName)(HProtoAccount* pHProtoAccount, const char* szAccountName);
	BOOL				(*EnumerateAccountInfo)(ProtoAccountInfo* pAccountInfo, int bufCount, int* pCount);
	BOOL				(*EnumerateAccountHandles)(HProtoAccount* pHBuf, int bufCount, int* pCount);
	BOOL				(*EnumerateAccountNames)(ProtoAccountName* pNameBuf, int bufCount, int* pCount);
	BOOL				(*RestartAccountInfo)(void);
	BOOL				(*SetPreferredAccount)(HProtoAccount hProtoAccount);
	BOOL				(*GetPreferredAccount)(HProtoAccount* pHProtoAccount);
	BOOL				(*GetAccountStatus)(HProtoAccount  hProtoAccount, BOOL* pbInService, BOOL* pbDefault);	
	BOOL				(*IsFreeAccount)(HProtoAccount hProtoAccount);
};

/*
 * ECOM Type
 */
typedef struct
{
	DECLARE_ECOM_VTBL(EProtoMgr)
	UINT32				refCount;

} TProtoMgr;

extern TProtoMgr protoMgr;

#ifdef __cplusplus
}
#endif  /* __cplusplus */

#endif // _PROTO_ECOM_H_
