/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		NPGluePluginDLL..h
 * @brief		Define NPGluePluginDLL
 * @author		Sangheon Kim (sangheon77.kim@samsung.com)
 * @version		1.0
 * @date		2008.03.10	v1.0		Created
 */
#ifndef _NPGLUE_PLUGINDLL_H_
#define _NPGLUE_PLUGINDLL_H_

#if defined (SUPPORT_NP_PLUGIN) 
#include "npfunctions.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


typedef NPError (*NP_GetEntryPointsProcPtr)(NPPluginFuncs* pFuncs);

typedef NPError (*NP_InitializeProcPtr)(NPNetscapeFuncs* pFuncs);

typedef void (*NP_ShutdownProcPtr)();

typedef struct
{

	/** Name of the plugin */
	char*	szName; 
	
	/** Description of the plugin */
	char*	szDescription; 
	
	/**  
	 * Mimetypes of the plugin 
	 * Multiple types separated by the "|" character
	 * ex. "video/quicktime|audio/aiff|image/jpeg" 
	 */
	char*	szMimeTypes; 

	/**
	 * Extentions of the plugin
	 * Multiple extentions separated by the "|" character
	 * ex. "text|txt"
	 */
	 char*	szExtentions;
	
	/**
	 * Browser will call this funtion to get NPP_XXX entry points.
	 */
	NP_GetEntryPointsProcPtr fpNP_GetEntryPoints;

	/**
	 * The browser calls this function only once: when a plug-in is loaded, before the first instance is created.
	 * Browser will pass NPN_XXX entry points to plugin when calling this funtions.
	 * @see http://developer.mozilla.org/en/docs/NP_Initialize
	 */
	NP_InitializeProcPtr fpNP_Initialize;

	/**
	 * The browser calls this function once after the last instance of your plug-in is destroyed, before unloading the plug-in library itself
	 * @see http://developer.mozilla.org/en/docs/NP_Shutdown
	 */
	NP_ShutdownProcPtr fpNP_Shutdown;

}NPGluePluginDLLInfo;

#ifdef __cplusplus
}
#endif /* __cplusplus */

/**
 * This class define NPGluePluginDLL
 * Plugin must inherit NPGluePluginDLL and implement GetPluginDLLInfo.
 *
 * Plugin must define one global varibale of NPGluePluginDLL inherited type,
 * and then the Plugin will be automatically loaded into PluginDLL directory of NPGlue.
 *
 * When Browser engine load/initialize plugins in the platform,
 * Browser engine can get information about plugins through NPGlue_GetPluginDLLDirectory API.
 */
class NPGluePluginDLL
{
/*PG : this structure is added so that the browser Engine has the
 * knowledge of the sizes of strings and so it can allocate memory for
 * them before calling GetPluginDLLInfo*/ 

public:
	int		iNameStringSize; 
	int		iDescriptionStringSize; 
	int		iMimeTypesStringSize; 
	int		iExtentionsStringSize;
public:
	NPGluePluginDLL();

	/**
	 * Each inherited plugin must implement this funtions.
	 * This funtion provide entry points of this plugin.
	 */
	virtual NPError GetPluginDLLInfo( NPGluePluginDLLInfo* pInfo ) = 0;
	virtual NPError SetPluginStringInfo( ) = 0;
};


class FlashPluginDLL : public NPGluePluginDLL
{
public:
	virtual NPError GetPluginDLLInfo( NPGluePluginDLLInfo* pInfo );
	virtual NPError SetPluginStringInfo( );
};

class AudioPluginDLL : public NPGluePluginDLL
{
public:
	virtual NPError GetPluginDLLInfo( NPGluePluginDLLInfo* pInfo );
	virtual NPError SetPluginStringInfo( );
};
class SVGPluginDLL : public NPGluePluginDLL
{
public:
	virtual NPError GetPluginDLLInfo( NPGluePluginDLLInfo* pInfo );
	virtual NPError SetPluginStringInfo( );
};

#define NPGLUE_MAX_PLUGINDLL		10

typedef struct
{
	NPGluePluginDLL* DLLs[NPGLUE_MAX_PLUGINDLL];
	int numOfDLLs;
}NPGluePluginDLLDirectory;

/**
 * This funtion is used to implement glue layer of each browser engine.
 * Browser engine can get PluginDLLs information through this funtions.
 */
NPGluePluginDLLDirectory* NPGlue_GetPluginDLLDirectory();

//<TBD> this has to be under the Browser selection FLAG

NPGluePluginDLLDirectory* SFB_GetPluginDLLDirectory();




#endif //SUPPORT_NP_PLUGIN
#endif //_NPGLUE_PLUGINDLL_H_




