/**
* Samsung Handset Platform
* Copyright (c) 2006 Samsung Electronics, Inc.
* All rights reserved.
*
* This software is the confidential and proprietary information
* of Samsung Electronics, Inc. ("Confidential Information").  You
* shall not disclose such Confidential Information and shall use
* it only in accordance with the terms of the license agreement
* you entered into with Samsung Electronics.
*/
/*
*  This file defines the event management for the browser modules.
*  @author Naveen Kumar SG(naveensg@samsung.com)
*  @author Sneha Bhat(sneha.bhat@samsung.com)
*/

#ifndef _SFBAL_EVENT_H
#define _SFBAL_EVENT_H

#include "SFBalMain.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define BAL_TIMER_CALLBACK 0
#define BAL_EVENT_QUEUE_SIZE        100           /*Max no of events in the queue*/ // Basav

typedef enum
{
    BAL_EVENT_NOT_SENT,
    BAL_EVENT_SENT,
    BAL_EVENT_FAIED,
    BAL_EVENT_COMPLETE
}SFBalEventStatus;

typedef enum{
    BAL_APP_RM,
    BAL_APP_CD,
    BAL_APP_WEBCORE,
    BAL_APP_BROWSER
}SFBalAppId;

typedef enum{

    /*RM related events*/
    BAL_EVENT_RM_SEND_NEXT_REQUEST,
    BAL_EVENT_RM_PROCESS_RESULT,
	BAL_EVENT_RM_GET_FROM_CACHE,// SFB00000031
    BAL_EVENT_RM_PROCESS_FILE,
	BAL_EVENT_RM_CANCEL_REQUEST,// Anshul 19/12/07
	BAL_EVENT_RM_DELAYED_ABORT,
	BAL_EVENT_RM_GET_NEXT_CHUNK,//Anshul Aug 13
    /*CD related events*/
    /*Bridge related events*/
    /*Browser related events*/
    BAL_EVENT_BROWSER_EXPOSE,
    BAL_EVENT_BROWSER_SCROLL_EXPOSE,
	BAL_EVENT_OUT_OF_MEMORY, 
	BAL_EVENT_RENDERING_COMPLETE,
    BAL_EVENT_RELOAD,
    BAL_EVENT_JS_COLLECT_GARBAGE,
    BAL_EVENT_BROWSER_TOKENIZER_CONTINUE,
	BAL_EVENT_ASYNC_FETCH_REQUEST,
	BAL_EVENT_OPEN_NEW_WINDOW,
	BAL_EVENT_DELETE_JS_WINDOW,
    BAL_EVENT_BROWSER_EXIT_ENGINE,
	BAL_EVENT_BROWSER_SMARTLINK,
}SFBalEventId;

typedef enum
{
	BAL_EVENT_LOW_PRIORITY,
	BAL_EVENT_NORMAL_PRIORITY,  /* All Default Event */
	BAL_EVENT_HIGH_PRIORITY		/* BAL_EVENT_OUT_OF_MEMORY & BAL_EVENT_JS_COLLECT_GARBAGE*/
} SFBalEventPriority;


//struct SFBalEventDataList_st; // forward declaration
//typedef void (*SFBalEventCallback)( struct SFBalEventDataList_st *pEvent ); 
typedef HObj SFBalHSenderContext;

typedef struct SFBalEventDataList_st{
    struct SFBalEventDataList_st *next;
    SFBalEventId    eventId;
    void*       eventBuffer;
	SFBalEventPriority eSFBalEventPriority;
    SFBalAppId appId;
    SFBalEventId brEvntId;
    SFBalEventStatus eBrEventStatus;
    int param1;
    int param2;

#ifdef MULTIPLE_INSTANCE_SUPPORT //Pramod - Multiple instance support
	void* pEventWebControlInstance;
#endif

	//// to remove dependencies on upper layer. client should pass callback
	//SFBalEventCallback fpEventCallback;

	// Event must be excuted under sender's context.
	// So this context is used to restore sender's context when excuting the event.
	SFBalHSenderContext hSenderEventContext;
}SFBalEventDataList;





//void SFBalSetCurrentEventContext( SFBalHEventContext hContext );
SFBalHSenderContext SFBalGetCurrentSenderContext();
void SFBalStartSenderContext( SFBalHSenderContext hContext );
void SFBalStopSenderContext();


#ifdef MULTIPLE_INSTANCE_SUPPORT //Pramod - Multiple instance support
void
SFBalSendEvent(SFBalAppId appId,SFBalEventId eventId, void* eventBuffer , int param1, UINT32 param2, void* pWebControlInstance ); //, SFBalEventCallback fpCallback );
#else
void
SFBalSendEvent(SFBalAppId appId,SFBalEventId eventId, void* eventBuffer , int param1, UINT32 param2 ); //, SFBalEventCallback fpCallback );
#endif

BOOL
SFBalIsExposeEventPresent();

void 
SFBalRemoveEventByID(SFBalAppId appId,SFBalEventId eventId, void* pWebControlInstance );

BOOL
SFBalProcessEvents();

void
SFBalEventInit();

void
SFBalExitEvent();

#ifdef MULTIPLE_INSTANCE_SUPPORT //Pramod - Multiple instance support
	/**
	 * this is temporary patch to remove dependencies to module in upper layer
	 */
	void SFBalInstanceExitEvent( void* pWebControlInstance, void *pRmClient);
	typedef void	(*SFBalHandleUpdateContents)( void* wmRect, void* pWebControl );
	typedef void (*SFBalGoToURL)(void* eventBuffer, void* pWebControl);
	typedef void (*SFBalFetchRequest)(void* pWebControl);
	typedef void (*SFBalEngineTokenizerHandler)(void* pWebControl);
	typedef void (*SFBalRmEventHandler)( SFBalEventDataList * evt );
	typedef void (*SFBalOOMHandler)(); 
	typedef void (*SFBalJSGCHandler)();
	typedef void (*SFBalReload) (void* pWebControl);
	typedef void (*SFBalExitEngine) ( void* pWebControl );
	typedef void (*SFBalSmartLink)(void* pWebControl, int pBuffer);
	typedef void (*SFBalDeleteJsWindow)(void *pWebControl);
#else //MULTIPLE_INSTANCE_SUPPORT
	typedef void (*SFBalRmEventHandler)( SFBalEventDataList * evt );
	typedef void	(*SFBalHandleUpdateContents)( void* wmRect );
	typedef void (*SFBalOOMHandler)(); 
	typedef void (*SFBalReload) (void* pWebControl);
	typedef void (*SFBalGoToURL)();
	typedef void (*SFBalFetchRequest)();
	typedef void (*SFBalJSGCHandler)();
	typedef void (*SFBalSmartLink)(int pBuffer);
	typedef void (*SFBalEngineTokenizerHandler)();
	typedef void (*SFBalDeleteJsWindow)(void *pWebControl);
#endif

typedef struct
{
	SFBalRmEventHandler	fpRmEventHandler;
	SFBalHandleUpdateContents	fpHandleUpdateContents;
	SFBalOOMHandler	fpOOMHandler; 
	SFBalGoToURL		fpGoToURL;
	SFBalFetchRequest	fpFetchRequest;
    SFBalJSGCHandler    fpJSGCHandler;
    SFBalEngineTokenizerHandler fpEngineTokenizerHandler;
    SFBalReload fpReload;
	SFBalSmartLink	fpSmartLink;
	SFBalDeleteJsWindow fpDeleteJsWindow;
#ifdef MULTIPLE_INSTANCE_SUPPORT 
	SFBalExitEngine		fpExitEngine;
#endif
}SFBalEventCallbacks;

void SFBalSetEventCallbacks( SFBalEventCallbacks *pCallbacks );


#ifdef __cplusplus
} //extern "C"
#endif /* __cplusplus */

#endif

