/*
* ==============================================================================
*  Name        : BrowserControl_Iterator_Interface.h
*  Part of     : Browser Control.
*  Description : Defines a default iterator interface.
*  Version     :
*
*    Copyright (c) 2007,
*    All rights reserved.
* ==============================================================================
*/

#include "BrowserControl_DataTypes.h" 

#ifdef __cplusplus
extern "C"
{
#endif
/**
 * @defgroup Iterator Iterator API functions
 */

/**
* BrowserControl_Iterator_First
* @brief Purpose : Moves the iterator to the beginning of a list and gets the first item.
* @param[in] HANDLE aIteratorHandle Handle to the Iterator.
* @param[out] void* aItem Pointer to the buffer that will recieve the item.
* @param[in,out] unsigned long* aSize Size of the buffer. If the buffer is not enough, the required size is sent back in aSize.
* @return ESTATUS : ESUCCESS if the iterator could move to the beginning,
* else EFAIL. Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Iterator
*/
WEBKIT_SHP_API ESTATUS BrowserControl_Iterator_First(HANDLE aIteratorHandle, gpointer aItem, gulong* aSize);

/**
* BrowserControl_Iterator_Last
* @brief Purpose : Moves the iterator to the end of a list and gets the last item.
* @param[in] HANDLE aIteratorHandle Handle to the Iterator.
* @param[out] void* aItem Pointer to the buffer that will recieve the item.
* @param[in,out] unsigned long* aSize Size of the buffer. If the buffer is not enough, the required size is sent back in aSize.
* @return ESTATUS : ESUCCESS if the iterator could move to the end,
* else EFAIL. Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Iterator
*/
WEBKIT_SHP_API ESTATUS BrowserControl_Iterator_End(HANDLE aIteratorHandle, gpointer aItem, gulong* aSize);

/**
* BrowserControl_Iterator_Next
* @brief Purpose : Gets the next item in the list and moves the iterator forward.
* @param[in] HANDLE aIteratorHandle Handle to the Iterator.
* @param[out] void* aItem Pointer to the buffer that will recieve the item.
* @param[in,out] unsigned long* aSize Size of the buffer. If the buffer is not enough, the required size is sent back in aSize.
* @return ESTATUS : ESUCCESS if the iterator could move to the next item,
* else EFAIL. Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Iterator
*/
WEBKIT_SHP_API ESTATUS BrowserControl_Iterator_Next(HANDLE aIteratorHandle, gpointer aItem, gulong* aSize);

/**
*  BrowserControl_Iterator_Previous
* @brief Purpose : Gets the previous item in the list and moves the iterator backward.
* @param[in] HANDLE aIteratorHandle Handle to the Iterator.
* @param[out] void* aItem Pointer to the buffer that will recieve the item.
* @param[in,out] unsigned long* aSize Size of the buffer. If the buffer is not enough, the required size is sent back in aSize.
* @return ESTATUS : ESUCCESS if the iterator could move to the previous item,
* else EFAIL. Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Iterator
*/
WEBKIT_SHP_API ESTATUS BrowserControl_Iterator_Previous(HANDLE aIteratorHandle, gpointer aItem, gulong* aSize);

/**
*  BrowserControl_Iterator_SkipNext
* @brief Purpose : Skips the next item in the list and moves the iterator forward.
* @param[in] HANDLE aIteratorHandle Handle to the Iterator.
* @return ESTATUS : ESUCCESS if the iterator could skip the next item,
* else EFAIL. Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Iterator
*/
WEBKIT_SHP_API ESTATUS BrowserControl_Iterator_SkipNext(HANDLE aIteratorHandle);

/**
* BrowserControl_Iterator_SkipPrevious
* @brief Purpose : Skips the previous item in the list and moves the iterator backward.
* @param[in] HANDLE aIteratorHandle Handle to the Iterator.
* @return ESTATUS : ESUCCESS if the iterator could skip the previous item,
* else EFAIL. Exact reason of the failure can be queried using
* BrowserControl_GetLastError().
* @ingroup Iterator
*/
WEBKIT_SHP_API ESTATUS BrowserControl_Iterator_SkipPrevious(HANDLE aIteratorHandle);

#ifdef __cplusplus
} //extern "C"
#endif
