#############################################################
#
#  NetCC compile system for engine
#
#  Copyright (R) Samsung Electronics
#  Author: Pushparajan V 
#############################################################

include nc_cfg.min

AR = @$(NC_MAKE_LIB_LIST) $(ARMHOME)/bin/armar#

# dummy target to prevent netcc build error
NBUILDER:

##############################################################
#
# Step 1: Do dependency check - NC_DepChecker
# Uses: nc_deps.ncc
#
##############################################################
depcheck:
	@echo Checking $(NC_FILE_CMD_LIST) $(NC_FILE_LIB_LIST) $(NC_FILE_BAT_LIST)
	@rm -rf VesselLog.txt
	@if test -f $(NC_FILE_CMD_LIST); then rm.exe -f $(NC_FILE_CMD_LIST); fi
	@if test -f $(NC_FILE_LIB_LIST); then rm.exe -f $(NC_FILE_LIB_LIST); fi
	@if test -f $(NC_FILE_BAT_LIST); then rm.exe -f $(NC_FILE_BAT_LIST); fi

	@echo [Network C/C++ Compiler] Depend Updater started.......
	@if test ! -d $(NC_DIR_OBJECT); then mkdir.exe $(NC_DIR_OBJECT); else $(NC_DEP_CHECK) $(NC_UPDATE_OPTION); fi
	@echo [Network C/C++ Compiler] Depend Updater finished......


##############################################################
#
# Step 2: Generate command list - NC_CLGen
# Generates: nc_cmds.ncc
#
##############################################################

$(TARGET_BUILD_PATH)/%.o: %.c
	@$(NC_MAKE_CMD_LIST) $(CC) $(CFLAGS) $(DEFINES) $(INCLUDES) $(OBJ_CMD) $@ $<

$(TARGET_BUILD_PATH)/%.o: %.cpp
	@$(NC_MAKE_CMD_LIST) $(CXX) $(CXXFLAGS) $(DEFINES) $(INCLUDES) $(OBJ_CMD) $@ $<

%.c: %.y

# Archiving rule - Done in local machine (not using NetCC)
###############################################################################

$(APPSBROWSER)/$(TARGET): $(OBJECTS)
	@echo Archiving...
	@echo $? > _tmp_objs
	@$(NC_MAKE_BAT_LIST) updatearchive.bat $@ _tmp_objs
	@if test -f $(NC_FILE_BAT_LIST).bat; then rm -f $(NC_FILE_BAT_LIST).bat; fi
	@if ! test -f $(NC_FILE_LIB_LIST); then cp -f $(NC_FILE_BAT_LIST) $(NC_FILE_BAT_LIST).bat; fi
	@if test -f $(NC_FILE_BAT_LIST); then $(NC_EXE_BAT); fi
	@if test -f $(NC_FILE_LIB_LIST); then $(NC_MAKE_LIB); fi

##############################################################
#
# Step 3: Generate dependency - NC_DepMake
# Generates: nc_deps.ncc
# uses: nc_cmds.ncc
#
##############################################################

gendeps: $(OBJECTS)
	@echo "Generate dependencies.. [ nccompile.inc ]"
	@echo $(NC_ROOT_PATH) $(NC_WORK_PATH)
	@if test -f $(NC_FILE_CMD_LIST); then $(NC_MAKE_DEF); fi

##############################################################
#
# Step 4: Compute NetCC metrics
# Uses: nc_cmds.ncc
#
##############################################################

nccomp: gendeps
	@echo "Compute NetCC metrics.."
	@if test -f $(NC_FILE_CMD_LIST); then $(NC_COMPILE); fi

##############################################################
#
# Step 5: Start Network compilation
#
##############################################################

startvessel: nccomp
	@echo "Start vessel [ nccompile.inc ]"
	@if test -f $(NC_FILE_CMD_LIST); then $(NC_VESSEL_WAIT); fi
	@if test -f $(NC_FILE_CMD_LIST); then $(NC_VESSEL) ; fi

# Build rules
###############################################################################

prebuild: 
	-@if [ -f prebuild.bat ]; \
	then \
		`pwd`/prebuild.bat; \
	fi
	
createdirs: mainoutputdir
	@mkdir -p $(TARGET_BUILD_PATH)

postbuild:
	-@if [ -f postbuild.bat ]; \
	then \
		`pwd`/postbuild.bat; \
	fi

all: prebuild createdirs depcheck startvessel postbuild $(APPSBROWSER)/$(TARGET)

clean:
	$(RM) $(TARGET_BUILD_PATH);

