/*
* ==============================================================================
*  Name        : ObjectBase.h
*  Part of     : WebEngine / WEBKIT / MemMan
*  Interface   : 
*  Description : 
*  Version     : %Version%
* ==============================================================================
*/
#ifndef __OBJECTBASE_H__
#define __OBJECTBASE_H__

//  INCLUDES

// CONSTANTS

// MACROS

// DATA TYPES

// CLASS DECLARATION

/**
*  The base class for classes wishing to use memory manager to allocate and free memory.
*  This class overloads new and delete operators, and drived classes shouldn't implement
*  their own new/delete. Use of array new/delete operators (new [] and delete [] )
*  for CBaseObject-derived classes is forbidden and should be replaced by MemoryManager::Alloc().
*
*  @lib memman.lib
*  @since 1.0
*/
class CObjectBase
{
public:
	/**
	* allocate a memory buffer
	* @since 1.0
	* @param aSize size of memory buffer to be allocated
	* @return pointer to a valid memory buffer
	*/		
	inline void* operator new(guint aSize);

	/**
	* free a memory buffer
	* @since 1.0
	* @param aPtr pointer to the buffer to be freed
	* @return
	*/		
	inline void operator delete(void* aPtr);

    /**
	* placement new operator
	* @since 1.0
	* @param - not used
    * @param aPtr memory buffer to be returned
	* @return pointer to a valid memory buffer
	*/		
    inline void* operator new(guint, void* aPtr);

    /**
	* placement delete operator
	* @since 1.0
	* @param - not used 
    * @param - not used 
	* @return
	*/		
    inline void operator delete(void*, void*);

    inline void* operator new [] (guint aSize);

	inline void operator delete [] (void* aPtr);

protected:
	/**
	* default constructor, only accessible by derived classes
	* @since 1.0
	* @param 
	* @return 
	*/
	CObjectBase() {}

	/**
	* The destructor is never called directly and we shouldn't
	* make it virtual. Because otherwise we will introduce a virtual
	* table for each derived class.
	* @since 1.0
	* @param 
	* @return 
	*/
	~CObjectBase() {}
};

#include "ObjectBase.inl"

#endif