#include "WebCoreBridge.h"
#include "WebCorePageData.h"
#include "Document.h"
#include "DocLoader.h"
#include "Frame.h"
#include "NodeList.h"
#include "RenderImage.h"
#include "Image.h"
#include "CachedCSSStyleSheet.h"
#include "CachedScript.h"
#include "Page.h"

using namespace WebCore;

WebCoreBridge::WebCoreBridge( XMLFrame* _frame,
                                IWebClientBridge* _clientBridge ) 
    : m_frame( _frame ),
      m_clientBridge( _clientBridge )
    {
    m_clientBridge->SetWebCoreBridge( this );
    }

WebCoreBridge::~WebCoreBridge()
    {
    delete m_clientBridge;
    }


/*
    Mask :
    =======

    CWebCorePageDataItem::EMarkup
    CWebCorePageDataItem::EStylesheet
    CWebCorePageDataItem::EScript
*/

Vector<WebCorePageDataItem *> WebCoreBridge::GetPageData(int _typeMask,Frame *child, const char *path)
{
	Vector<WebCorePageDataItem *> itemArray;
	WebCore::Document* document = child->document();
	WebCore::DocLoader* loader = child->document()->docLoader();
	WebCorePageDataItem* dataItem = NULL;
	
	// get page markup document if requested
	if(_typeMask & WebCorePageDataItem::EMarkup)
	{
		HTMLSrcString htmlSrc ;
        document->page()->initSavePageList () ;
		document->setFilePath (WebCore::String(path)) ;
		document->getFileName () ;
		document->toString(htmlSrc);
        document->page()->initSavePageList () ;

		dataItem = new WebCorePageDataItem(WebCorePageDataItem::EMarkup);
		dataItem->SetUrl(document->url());
		dataItem->SetContentType("text/html");
		dataItem->SetCharset("ucs-2");
		itemArray.append(dataItem);
	}
	return itemArray;
}

Vector<Image *> WebCoreBridge::GetImageData()
{
    Document* document = m_frame->document();
    DocLoader* loader = m_frame->document()->docLoader();

    Vector<Image *, 10> imageList;

    PassRefPtr<HTMLCollection> imageCollection = document->images();
    printf("CELUNITE Image Save - \nImage Count = %d\n", imageCollection->length());

    for (int i=0; i< imageCollection->length() ;i++)
    {
    PassRefPtr<Node> tempNode = imageCollection->item(i);
    if(tempNode)
    {
        RenderImage *r = static_cast<RenderImage *>(tempNode->renderer());
        if (r)
        {
            CachedImage* cachedImage = r->cachedImage();
            Image* image = cachedImage->image();

            imageList.append(image);

                printf("CELUNITE Image Save - \nImage Raw Data Size = %d\n", image->data()->size());
        }
    /*
        // Test Code

        FILE * pFile;
        pFile = fopen ("myimg.gif","w");
        if (pFile!=NULL)
        {
            fwrite(image->data()->data(), image->data()->size(), 1, pFile);
        fclose (pFile);
        }
    */


        }
    }


    return imageList;
}

// End of File

