#if defined(SHP_SUPPORT_WML)
#ifndef __WMLBROWSERCONTEXT_H__
#define __WMLBROWSERCONTEXT_H__

#include "config.h"
#include "PlatformString.h"
#include "StringHash.h"
#include <wtf/HashMap.h>
#include "KURL.h"

#if PLATFORM(SHP)
#include "CMMDefs.h"
#endif // PLATFORM(SHP)

namespace WebCore	{
class WMLVariable ;
class Page ;
class Frame ;

typedef HashMap<String, WMLVariable *> AttributeMap;

class WMLBrowserContext : public Noncopyable
    CMM_MODIFIED_ADD
{
public :
		enum EVariableConversionType
		{
			EDefault
			,EEscape
			,EUnEscape
			,ENoEscape
		};

		WMLBrowserContext(Frame *frame, Page *page = 0);
		~WMLBrowserContext();

		void InitBrowserContext();
		void ResetBrowserContext();

		void ResetHistoryStack();
		void GoBackInHistory(int aNoOfSteps = 1);
		void GoForwardInHistory(int aNoOfSteps = 1);
		void addHistoryItem (const String &url) ;

		void ResetVariableInstances();
		void AddVariable(const String& aVariableName, const String& aVariableValue, EVariableConversionType type) ;
		void SetVariableValue(const String& aVariableName, const String& aNewVariableValue);
		String GetVariableValue(String& aVariableName);
		bool isVariableNameValid(const String & aVariableName);

		AttributeMap& GetVariableAttributeMap();

		void SetAccessDomain(String& aDomain);
		void SetAccessPath(String& aPath);

		String AccessDomain(){ return m_accessUrl.host();}
		String AccessPath(){ return m_accessUrl.path();}

		String* RandomVarName();
		//bool AllowDeckAccess(const String& aCurrentUrl);
		bool contains (String &) ;

		bool AllowDeckAccess(){return m_allowDeskAccess;};
		void SetDeckAccess(bool  aAllowDeckAccess) { m_allowDeskAccess = aAllowDeckAccess;};


	private:
		Frame *m_frame ;
		Page *m_page ;

		AttributeMap m_attrMaps ;
		KURL m_accessUrl;
		bool m_allowDeskAccess;
};

class WMLVariable : public Noncopyable
CMM_MODIFIED_ADD
{
public:
	WMLVariable (const String &value, 
		WMLBrowserContext::EVariableConversionType type=WMLBrowserContext::ENoEscape)
		:m_type(type)
		,m_value("")
	{
		if (!m_value.isEmpty())
			m_value.truncate(0) ;
		m_value.append(value) ;
	}

	void setVariableValue (const String &value) {
		if (!m_value.isEmpty())
			m_value.truncate(0) ;
		m_value.append(value) ;
	}

	String& getVariable (){return m_value ;}

	~WMLVariable () {}
private:
	String m_value ;

	WMLBrowserContext::EVariableConversionType m_type;
};

} //WebCore


#endif //__WMLBROWSERCONTEXT_H__
#endif //SHP_SUPPORT_WML