/******************************************************************************************************
 * PROJECT :  Samsung Safari Browser
 * Copyright (c) 2007 Software Center, Samsung Electronics, Inc.
 * All right reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. (Confidential Information). You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics.
 */

/* FILE :
 * This file describes the Plugin specific structure/variable definitions 
 * which will be used by Webkit
 *
 * @name		HTMLBgSoundElement.cpp 
 * @author		SameerPatil (sameer.p@siso.com)  [M]
 * @version		$1.0$  [O]
 * @see		    related information [O]
 ******************************************************************************************************/

#ifdef SUPPORT_NP_PLUGIN


#include "config.h"
#include "HTMLBgSoundElement.h"

#include "CSSHelper.h"
#include "CSSPropertyNames.h"
#include "Frame.h"
#include "FrameView.h"
#include "FrameLoader.h"
#include "FrameLoaderClient.h"
#include "HTMLDocument.h"
#include "HTMLObjectElement.h"
#include "HTMLNames.h"
#include "PluginView.h"
#include "Settings.h"


#define MAX_UINT32 0xFFFFFFFF

namespace WebCore {

using namespace HTMLNames;

HTMLBgSoundElement::HTMLBgSoundElement(Document* doc)
: m_doc(doc)
,HTMLPlugInElement(bgsoundTag, doc)
,m_pluginView(NULL)
{
}

HTMLBgSoundElement::~HTMLBgSoundElement()
{
	delete m_pluginView;
}

void HTMLBgSoundElement::parseMappedAttribute(MappedAttribute* attr)
{
	String val = attr->value();

	if (attr->name() == idAttr) {
		m_serviceType = val.lower();
		int pos = m_serviceType.find(";");
		if (pos != -1)
			m_serviceType = m_serviceType.left(pos);
	} else if (attr->name() == codeAttr || attr->name() == srcAttr)
		url = parseURL(val).deprecatedString();
	else if (attr->name() == loopAttr ){
		if(val.isEmpty())
			m_loopCount = -1;
		else if(equalIgnoringCase(val, "infinite"))
			m_loopCount = MAX_UINT32;
		else
			m_loopCount = val.toInt();
		}
	else if (attr->name() == volumeAttr) {
		if(val.isEmpty())
			m_loopCount = -1;
		else
			m_volume = val.toFloat();	
	}
	else
		HTMLPlugInElement::parseMappedAttribute(attr);
}


bool HTMLBgSoundElement::mapToEntry(const QualifiedName& attrName, MappedAttributeEntry& result) const
{
	if (attrName == hiddenAttr) {
		result = eUniversal;
		return false;
	}

	return HTMLPlugInElement::mapToEntry(attrName, result);
}


void HTMLBgSoundElement::insertedIntoDocument()
{
	HTMLPlugInElement::insertedIntoDocument();
}

void HTMLBgSoundElement::removedFromDocument()
{
	HTMLPlugInElement::removedFromDocument();
}


void HTMLBgSoundElement::attributeChanged(Attribute* attr, bool preserveDecls)
{
	HTMLPlugInElement::attributeChanged(attr, preserveDecls);
}


void HTMLBgSoundElement::attach()
{
	HTMLPlugInElement::attach();

#if USE(LOW_BANDWIDTH_DISPLAY)
    // don't create plugin during low bandwidth display
    if (m_doc->frame()->document()->inLowBandwidthDisplay()) {
		m_doc->frame()->loader()->needToSwitchOutLowBandwidthDisplay();
        return;
    }
#endif

#if PLATFORM(SHP)
	Frame* frame = m_doc->frame();
	Settings* settings = frame->settings();
	if(settings->isBgSoundEnabled())
#endif // PLATFORM(SHP)
		createPlugin();		
}

void HTMLBgSoundElement::detach()
{
	HTMLPlugInElement::detach();	
}

bool HTMLBgSoundElement::isURLAttribute(Attribute *attr) const
{
	return attr->name() == srcAttr;
}

String HTMLBgSoundElement::src() const
{
	return getAttribute(srcAttr);
}
void HTMLBgSoundElement::setSrc(const String& value)
{
	setAttribute(srcAttr, value);
}

String HTMLBgSoundElement::type() const
{
	return getAttribute(typeAttr);
}

void HTMLBgSoundElement::setType(const String& value)
{
	setAttribute(typeAttr, value);
}

float HTMLBgSoundElement::volume()
{
	String volValue = getAttribute(volumeAttr);
	return volValue.toFloat();
}

void HTMLBgSoundElement::setVolume(const String& value)
{
	setAttribute(volumeAttr, value);
}

int HTMLBgSoundElement::loop()
{
	String loopVal = getAttribute(loopAttr);
	return loopVal.toInt();
}

void HTMLBgSoundElement::setLoop(const String& value)
{
	setAttribute(loopAttr, value);
}

void HTMLBgSoundElement::createPlugin()
{
	 
	Vector<String> paramNames;
	Vector<String> paramValues;


	if (url.isEmpty() && m_serviceType.isEmpty())
		return;

	// add all attributes set on the bgsound object
	NamedAttrMap* a = this->attributes();
	if (a) {
		for (unsigned i = 0; i < a->length(); ++i) {
			Attribute* it = a->attributeItem(i);
			paramNames.append(it->name().localName().domString());
			paramValues.append(it->value().domString());
		}
	}

	KURL completedURL;

	if (!url.isEmpty())
		completedURL = m_doc->frame()->loader()->completeURL(url);

	if (m_doc->frame()->loader()->client()->objectContentType(completedURL, m_serviceType) == ObjectContentNetscapePlugin)
		return;

	// Create the plugin widget and init which will give the network request
	m_pluginView = PluginView::create(m_doc->frame(), IntSize(), this, completedURL, paramNames, paramValues, m_serviceType, false);
	if(m_pluginView)
	{
		m_pluginView->init();
	}
	else
	{
		// throw error
	}

}

} //namespace WebCore

#endif //SUPPORT_NP_PLUGIN
