/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Simon Hausmann <hausmann@kde.org>
 * Copyright (C) 2003, 2006, 2007 Apple Inc. All rights reserved.
 *           (C) 2006 Graham Dennis (graham.dennis@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#if defined(SHP_SUPPORT_WML)
#include "config.h"
#include "WMLAElement.h"

#include "CSSHelper.h"
#include "Document.h"
#include "Event.h"
#include "EventHandler.h"
#include "EventNames.h"
#include "Frame.h"
#include "FrameLoader.h"
#include "HTMLImageElement.h"
#include "HTMLNames.h"
#include "KeyboardEvent.h"
#include "MouseEvent.h"
#include "MutationEvent.h"
#include "RenderFlow.h"
#include "RenderImage.h"
#include "ResourceRequest.h"
#include "SelectionController.h"
#include "Settings.h"
#include "UIEvent.h"

#include "EventTargetNode.h"
#include "Event.h"
#include "EventTarget.h"
#include "EventListener.h"
#include "BackForwardList.h"
#include "Page.h"
#include "HistoryItem.h"
#include "FrameView.h"
#include "RenderButton.h"

#include "WMLCardElement.h"
#include "WMLAnchorElement.h"
#include "WMLDocument.h"
#if defined ( SFB_SUPPORT_WMLSCRIPT )
#include "HTMLScriptElement.h"
#include "Node.h"
#include "DocLoader.h"
#include "WMLScriptInterface.h"
#endif//SFB_SUPPORT_WMLSCRIPT
namespace WebCore {

using namespace HTMLNames;
using namespace EventNames;

enum EventType;

//WML <a> tag attributed definitions
//i think i can inherit this from HTMLAnchorElement, meant both href tags

WMLaElement::WMLaElement(Document *doc)
:HTMLAnchorElement( doc )
{
	/* 
		No Need to check in ParseMappedAttribute
		As DTD Says, if A is present then it has to have attribute
	*/
	m_isLink = true;
}

WMLaElement::~WMLaElement()
{

}
void WMLaElement::parseMappedAttribute(MappedAttribute *attr)
{
	//Nataraj - 20th feb
#if defined ( SFB_SUPPORT_WMLSCRIPT )
		String charset = document()->charset();
		String DocumentUrl = document()->url();	
		String ScriptUrl = attr->value() ;
		String targetUrl = document()->completeURL(ScriptUrl);
		int pos = targetUrl.reverseFind('#') ;
		String FinalTargetScriptUrl = targetUrl.substring(0,pos); 

		// if the Url is anything other than wmls, we dont need to add it to the cache
		if ( FinalTargetScriptUrl.endsWith(".wmls") )
		{
		String functionName = targetUrl.substring(pos+1); 
		document()->docLoader()->WmlrequestScript(FinalTargetScriptUrl, getAttribute(charsetAttr));
		}
		return ;
#endif//SFB_SUPPORT_WMLSCRIPT
}

void WMLaElement::defaultEventHandler(Event* ev)
{
	if(ev->isMouseEvent() && ev->type() == clickEvent)
	{
		//Get href attribute value of anchor
 		//String url = parseURL(getAttribute(hrefAttr));
#if defined ( SFB_SUPPORT_WMLSCRIPT )
		String Fragment ;
#endif//SFB_SUPPORT_WMLSCRIPT
		String url = href();
		//Get target url to which we want to navigate to
		String targetUrl = document()->completeURL(url);

		//It may be link like #card3, remove first character, get card id, card3 in this case
		
		int pos = url.find('#');
		if(pos == -1) // # not found in url, means it can be normal url like google.com
		{
			HTMLAnchorElement::defaultEventHandler(ev);
			return;
		}
		else if(pos == 0) // first char is '#', so it is local card
		{
			url.remove(0,1);
		}
		else   //For case like samepage.wml#card2, we should not call base class, should just set active id.
		{
			//Get current url from document
			String docUrl = document()->url();
			String docUrlPrefix = "";  //holds document url excluding card id
			int revpos;
			if((revpos = docUrl.reverseFind('#')) == -1)
			{
				docUrlPrefix = docUrl;
			}
			else
			{
				docUrlPrefix = docUrl.substring(0,revpos);
			}

			String targetUrlPrefix = "";  //holds target url excluding card id
			if((revpos = targetUrl.reverseFind('#')) != -1)
			{
#if defined ( SFB_SUPPORT_WMLSCRIPT )
				Fragment = targetUrl.substring(revpos+1) ;
#endif//SFB_SUPPORT_WMLSCRIPT
				targetUrlPrefix = targetUrl.substring(0,revpos);
			}
			else
			{
				targetUrlPrefix = targetUrl;
			}

			if(docUrlPrefix != targetUrlPrefix) //Check whether request is with in the page or not
			{
#if defined ( SFB_SUPPORT_WMLSCRIPT )
				if ( targetUrlPrefix.endsWith(".wmls"))
				{
					WMLDocument *doc = static_cast<WMLDocument *>(document()) ;
					Node *parent = parentNode() ;
					ASSERT (parent) ;
					if (parent->hasTagName(oneventTag)) // may be we need to add several tags here
					{
						doc->scriptInterface()->setWmlsecOnEventParams(targetUrlPrefix, Fragment) ;
					}
					else
					{
						
						doc->scriptInterface()->exeScript(targetUrlPrefix, Fragment);
					}
				}
				else
#endif//SFB_SUPPORT_WMLSCRIPT
				{
				HTMLAnchorElement::defaultEventHandler(ev); //not same page, issue fetch request
			    return;
			}
			}
			else //card in same page, set active id
			{
#if defined (SFB_SUPPORT_WMLSCRIPT)
				if ( targetUrlPrefix.endsWith(".wmls"  ))
				{
					WMLDocument *doc = static_cast<WMLDocument *>(document()) ;
					Node *parent = parentNode() ;
					ASSERT (parent) ;
					if (parent->hasTagName(oneventTag)) // may be we need to add several tags here
					{
						doc->scriptInterface()->setWmlsecOnEventParams(targetUrlPrefix, Fragment) ;
			}
					else
					{

						doc->scriptInterface()->exeScript(targetUrlPrefix, Fragment);
					}
				}
				else
#endif//SFB_SUPPORT_WMLSCRIPT
				{
					url = url.substring(pos+1);
				}
			}
		}

		//set active id first and start from event handling from that card
		document()->setActiveCardId(url);
		document()->frame()->loader()->processEventsAndFindDesiredCard(NavigationEvent,true);

		ev->setDefaultHandled(); //needed to prevent the call to top base class defaultEventHandler
	}
	//HTMLAnchorElement::defaultEventHandler(ev);
}

} //Webcore
#endif//SHP_SUPPORT_WML