#if defined(SHP_SUPPORT_WML)
#include "config.h"
#include "Element.h"
#include "WMLAccessElement.h"
#include "AtomicString.h"
#include "CSSHelper.h"
#include "CSSPropertyNames.h"
#include "CSSValueKeywords.h"
#include "EventNames.h"
#include "HTMLDocument.h"
#include "HTMLNames.h"
#include "QualifiedName.h"
#include "WMLDocument.h"
#include "WMLBrowserContext.h"
#include "Frame.h"
#include "FrameLoader.h"
#include "FrameLoaderClient.h"
#include "allocs.h" // For Calloc and Malloc 

//For SHP memory apis
# if 0 
#include "SFBal.h"
#include "AnsiCLib.h"
# endif

namespace WebCore {

using namespace HTMLNames;
//WML <access> tag att

String WMLAccessElement::domain() const
{
    return getAttribute(domainAttr);
}

void WMLAccessElement::setDomain(const String &value)
{
    setAttribute(domainAttr, value);
}

String WMLAccessElement::path() const
{
    return getAttribute(pathAttr);
}

void WMLAccessElement::setPath(const String &value)
{
    setAttribute(pathAttr, value);
}

String WMLAccessElement::id() const
{
    return getAttribute(idAttr);
}

void WMLAccessElement::setid(const String &value)
{
    setAttribute(idAttr, value);
}

String WMLAccessElement::Class() const
{
    return getAttribute(classAttr);
}

void WMLAccessElement::setClass(const String &value)
{
    setAttribute(classAttr, value);
}


WMLAccessElement::WMLAccessElement(Document *doc)
    :HTMLElement( accessTag,doc) 
{
	// set the default values for the domain and path attributes
	WMLDocument *wmldoc = static_cast<WMLDocument *>(document());
	String domain = doc->domain();
	String path("/");
	wmldoc->BrowserContext()->SetAccessDomain(domain);
	wmldoc->BrowserContext()->SetAccessPath(path);
}


#if 0                              
WMLAccessElement::WMLAccessElement(const QualifiedName& tagName,Document *doc)
    :HTMLElement( tagName,doc) 
{
}
#endif
WMLAccessElement::~WMLAccessElement()
{
}

void WMLAccessElement::parseMappedAttribute (MappedAttribute *attr)
{

	String referer = document()->frame()->loader()->referrer();
	WMLDocument *doc = static_cast<WMLDocument *>(document()) ;

	String attrValue = attr->value().domString();
	// Need to handle default values as well
	// domain default value is same as present deck's domain
	// path default value is /
	if (attr->name() == domainAttr)
	{
		doc->BrowserContext()->SetAccessDomain(attrValue) ;
	}
	else if (attr->name() == pathAttr)
	{
		doc->BrowserContext()->SetAccessPath(attrValue) ;
	}

}

void WMLAccessElement::attach()
{
	//Performs the check and updates the browser context with the access status.
	WMLDocument *doc = static_cast<WMLDocument *>(document()) ;
	bool bIsDomainAccessible = DomainAccessVerified();
	doc->BrowserContext()->SetDeckAccess(bIsDomainAccessible);
	if(!bIsDomainAccessible)
	{
		doc->frame()->loader()->client()->showErrorMessageBox("Access denied");
	}

//	if()
}

bool WMLAccessElement::DomainAccessVerified()
{
	bool host_check = false;
	int idx = 0,org_len = 0,ref_len = 0;
	WMLDocument *doc = static_cast<WMLDocument *>(document()) ;
	String referer = document()->frame()->loader()->referrer();
	bool result = false;
	KURL refererUrl(referer.deprecatedString());
	String ref_host = refererUrl.host();
	String ref_path = refererUrl.path();
	String ref_lastComp = refererUrl.lastPathComponent();
	// If the last component is a document just remove it.
	int pos = ref_path.find(ref_lastComp);
	if(pos != -1 && ((ref_lastComp.find((UChar)'.')!=-1)) )
	{
		ref_path.remove(pos-1,ref_lastComp.length()+1);
	}

	String host = doc->BrowserContext()->AccessDomain();
	String path = doc->BrowserContext()->AccessPath();
	
	UChar* p_ref_host = (UChar *)ref_host.characters();
	int p_ref_host_len = ref_host.length();
	UChar* p_host = (UChar *)host.characters();
	int p_host_len = host.length();

	UChar* ref_temp_buffer = (UChar*)malloc((p_ref_host_len + 1) * sizeof(UChar));
	UChar* temp_buffer = (UChar*)malloc((p_host_len + 1) * sizeof(UChar));

	// Check if the host matches the complete domain as specified in the access tag.
	// Comparisons of the domain labels are done in the reverse order(literally),the last domain labels
	// are compared first moving on to the first domain label.If the refering URL has all or more 
	// domain labels, then a match is confirmed.Anything less is not a match.
	// Example : Access Domain : wml.wap.com
	// Refering URL #1 : wml.wap.com - MATCH - ACCESS GRANTED
	// Refering URL #2 : test.wml.wap.com - MATCH - ACCESS GRANTED
	// Refering URL #1 : wml.waptest.com - MISMATCH - ACCESS DENIED
	for(int i = p_host_len - 1,j = p_ref_host_len - 1; i >= 0 ;i--,j--)
	{
		idx = 0,org_len = 0,ref_len = 0;

		//memset(temp_buffer,'\0',(p_host_len + 1)*sizeof(UChar));
		//memset(ref_temp_buffer,'\0',(p_ref_host_len + 1)*sizeof(UChar));
		AcMemset(temp_buffer,'\0',(p_host_len + 1)*sizeof(UChar));
		AcMemset(ref_temp_buffer,'\0',(p_ref_host_len + 1)*sizeof(UChar));


		while(host[i--]!=(UChar)'.')
		{
			temp_buffer[idx++] = host[i+1];
			org_len++;
			if(i < 0)
				break;
		}

		idx = 0;
		while(ref_host[j--]!=(UChar)'.')
		{
			ref_temp_buffer[idx++] = ref_host[j+1];
			ref_len++;
			if(j < 0)
				break;
		}
		if(org_len && ref_len)
		{
			String a(temp_buffer,org_len);
			String b(ref_temp_buffer,ref_len);

			// Compare the domain labels one at a time
			host_check = (a == b) ? true : false;
			if(!host_check)
				break;
		}

		// If the referring URL gets exhausted before the access domain,its also an error
		if(j < 0)
		{
			host_check = (i < 0) ? true : false;
			break;
		}
	}

	// Host check has been successful and so now check the path also matches.
	// Here the comparision is done in the forward direction.
	if(host_check)
	{
		result = true;
		UChar* p_ref_path = (UChar *)ref_path.characters();
		int p_ref_path_len = ref_path.length();
		UChar* p_path = (UChar *)path.characters();
		int p_path_len = path.length();

		free(ref_temp_buffer);
		free(temp_buffer);


		ref_temp_buffer = (UChar*)malloc((p_ref_path_len + 1) * sizeof(UChar));
		temp_buffer = (UChar*)malloc((p_path_len + 1) * sizeof(UChar));

		//now check that the path is compatible
		for(int i = 0,j = 0; i < p_path_len ;)
		{
			idx = 0,org_len = 0,ref_len = 0;

			//memset(temp_buffer,'\0',(p_path_len + 1)*sizeof(UChar));
			//memset(ref_temp_buffer,'\0',(p_ref_path_len + 1)*sizeof(UChar));
			AcMemset(temp_buffer,'\0',(p_path_len + 1)*sizeof(UChar));
			AcMemset(ref_temp_buffer,'\0',(p_ref_path_len + 1)*sizeof(UChar));

			while(path[i++]!=(UChar)'/')
			{
				temp_buffer[idx++] = path[i-1];
				org_len++;
				if(i > p_path_len - 1)
					break;
			}

			idx = 0;

			while(ref_path[j++]!=(UChar)'/')
			{
				ref_temp_buffer[idx++] = ref_path[j-1];
				ref_len++;
				if(j > p_ref_path_len - 1)
					break;
			}

			// The path must be same
			if(org_len && ref_len)
			{
				String a(temp_buffer,org_len);
				String b(ref_temp_buffer,ref_len);

				result = (a == b) ? true : false;
				if(!result)
					break;
			}

			// If the referring path gets exhausted before the access path,its also an error
			if(j > p_ref_path_len - 1)
			{
				result = (i > p_path_len - 1) ? true : false;
				break;
			}
		}
	}

	free(temp_buffer);	
	temp_buffer = NULL;
	free(ref_temp_buffer);
	ref_temp_buffer = NULL;

	return result;
}

} //namespace
#endif//SHP_SUPPORT_WML