/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Simon Hausmann <hausmann@kde.org>
 * Copyright (C) 2003, 2006, 2007 Apple Inc. All rights reserved.
 *           (C) 2006 Graham Dennis (graham.dennis@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#if defined(SHP_SUPPORT_WML)

#include "config.h"
#include "WMLGoElement.h"

#include "CSSHelper.h"
#include "Document.h"
#include "Event.h"
#include "EventHandler.h"
#include "EventNames.h"
#include "HTMLNames.h"

//Nataraj
#if defined ( SFB_SUPPORT_WMLSCRIPT )
#include "HTMLScriptElement.h"
#include "Node.h"
#include "DocLoader.h"
#include "WMLScriptInterface.h"
#endif//SFB_SUPPORT_WMLSCRIPT
#include "WMLAnchorElement.h"

#include "Frame.h"
#include "FrameLoader.h"
#include "HistoryItem.h"
#include "Page.h"

#include "WmlCardElement.h"
#include "WMLPostfieldElement.h"
#include "WMLDocument.h"
#include "CString.h"

#ifdef SUPPORT_AUTO_FORM_FILL
#include "AutoFormFill.h"
#include "WMLInputElement.h"
#endif //SUPPORT_AUTO_FORM_FILL

namespace WebCore {

using namespace HTMLNames;
using namespace EventNames;


//WML <go> tag attributes
String WMLGoElement::method () const
{
	String type = getAttribute(methodAttr);
	if (type.isEmpty())
		type = "GET" ;
	return type ;
}

String WMLGoElement::sendreferer() const
{
    return getAttribute(sendrefererAttr);
}

String WMLGoElement::cache_control() const
{
	return getAttribute(cache_controlAttr);
}

String WMLGoElement::accept_charset() const
{
	return getAttribute(accept_charsetAttr);
}

String WMLGoElement::enctype() const
{
	String type = getAttribute(enctypeAttr);
	if (type.isEmpty())
		type = "application/x-www-form-urlencoded" ;
	return type ;
}


void WMLGoElement::parseMappedAttribute(MappedAttribute *attr)
{
	//Nataraj - 20th feb
    if (attr->name() == hrefAttr) {
	#if defined  ( SFB_SUPPORT_WMLSCRIPT )
		String charset = document()->charset();
		String DocumentUrl = document()->url() ;	
		String ScriptUrl = attr->value() ;
		String targetUrl = document()->completeURL(ScriptUrl);
		int pos = targetUrl.reverseFind('#') ;
		String FinalTargetScriptUrl = targetUrl.substring(0,pos);
		if (FinalTargetScriptUrl.contains(".wmls"))
		{
			String functionName = targetUrl.substring(pos+1); 
			document()->docLoader()->WmlrequestScript(FinalTargetScriptUrl, getAttribute(charsetAttr));
		}
	#endif//SFB_SUPPORT_WMLSCRIPT
        bool wasLink = m_isLink;
        m_isLink = !attr->isNull();
        if (wasLink != m_isLink)
            setChanged();
		//funny part is here i am not able to access the parent, as the thing is not yet associated with the
		// dom tree, so am forced to this only after the rendering tree is created
		/*Node *parent = parentNode() ;
		ASSERT (parent) ;
		if (parent->hasTagName(anchorTag)) // may be we need to add several tags here
		{
			HTMLElement *element = static_cast<HTMLElement *>(parent) ;
			element->parseMappedAttribute (attr) ;
		}*/
    } else
        HTMLAnchorElement::parseMappedAttribute(attr);
}

RenderObject *WMLGoElement::createRenderer(RenderArena* arena, RenderStyle* style)
{
	RenderObject *obj = HTMLAnchorElement::createRenderer (arena, style) ;
	//for some miguided code here... :) for attributes

	Node *parent = parentNode() ;
	ASSERT (parent) ;
	while (parent)
	{
		if (parent->hasTagName(anchorTag)) // may be we need to add several tags here
		{
			WMLAnchorElement *anchor = static_cast<WMLAnchorElement *>(parent) ;
			anchor->setChildAttributes (this) ;
			break ;
		}
		else
			parent = parent->nextSibling () ;
	}
	return obj ;
	
}

//raghu
WMLGoElement::WMLGoElement(Document *doc)
:HTMLAnchorElement(goTag, doc )
,m_tempEvent(0)
{
}

WMLGoElement::~WMLGoElement()
{
}



void WMLGoElement::defaultEventHandler(Event* evt)
{
	if (m_isLink && evt->type() == clickEvent) 
	{
		m_tempEvent = evt;   //Store evt to make use of it in processHref
		bool bIsEventHandled = false;
		handleEvent(NavigationEvent, true, bIsEventHandled); //As it is link click, it is forward navigation.
		m_tempEvent = 0;     //Resetting
	}
	else
	{
		HTMLAnchorElement::defaultEventHandler(evt)	;
	}
}

void WMLGoElement::handleEvent(EventType eventType, bool bForwardNavigation, bool & bIsEventHandled)
{
	//Call children's handleEvent. setvar,postfield etc.
	for(Node *pChild = firstChild();pChild;pChild = pChild->nextSibling())
	{
		pChild->handleEvent(eventType,bForwardNavigation, bIsEventHandled);
	}

	//For go task, eventType doesn't matter, it has to process href attribute value
	processHref(eventType, bForwardNavigation); //Passing zero as there is no need to pass event data here.
	/*if(eventType == NavigationEvent)
	{
		String url = href();
		if(!url.isNull())
		{
			//Check local card
			String docUrl = document()->URL();
			int pos = url.find('#');
			if(pos != -1)
			{
				String cardId = url.substring(pos+1);
			
				RefPtr<HistoryItem> item = document()->frame()->page()->mainFrame()->loader()->createHistoryItemTree(document()->frame(), true);
				item->setURLString(url);
				document()->frame()->page()->backForwardList()->addItem(item);

				document()->setActiveCardId(cardId);
			}
			else
			{
				document()->frame()->loader()->urlSelected(url,getAttribute(targetAttr), NULL, false, true);
			}
		}
		processHref(0); //Passing zero as there is no need to pass event data here.
	}*/
}

String WMLGoElement::href() const
{
	WMLDocument *doc = static_cast<WMLDocument *>(document()) ;
	const String& val = getAttribute(hrefAttr).domString();
	String href ;
	String *value = document()->frame()->getVariableReplacedString (val, WMLBrowserContext::EEscape) ;
	if (value)
		href = value->copy();
	return href ;
}

static DeprecatedCString encodeCString(const CString& cstr)
{
    DeprecatedCString e = cstr.deprecatedCString();

    // http://www.w3.org/TR/html4/interact/forms.html#h-17.13.4.1
    // same safe characters as Netscape for compatibility
    static const char *safe = "-._*";
    int elen = e.length();
    DeprecatedCString encoded((elen + e.contains('\n')) * 3 + 1);
    int enclen = 0;

    for (int pos = 0; pos < elen; pos++) {
        unsigned char c = e[pos];

        if ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || (c >= '0' && c <= '9') || strchr(safe, c))
            encoded[enclen++] = c;
        else if (c == ' ')
            encoded[enclen++] = '+';
        else if (c == '\n' || (c == '\r' && e[pos + 1] != '\n')) {
            encoded[enclen++] = '%';
            encoded[enclen++] = '0';
            encoded[enclen++] = 'D';
            encoded[enclen++] = '%';
            encoded[enclen++] = '0';
            encoded[enclen++] = 'A';
        } else if (c != '\r') {
            encoded[enclen++] = '%';
            unsigned int h = c / 16;
            h += (h > 9) ? ('A' - 10) : '0';
            encoded[enclen++] = h;

            unsigned int l = c % 16;
            l += (l > 9) ? ('A' - 10) : '0';
            encoded[enclen++] = l;
        }
    }
    encoded[enclen++] = '\0';
    encoded.truncate(enclen);

    return encoded;
}


void WMLGoElement::issueFetchRequest(String &url)
{
	KURL urlstring(document()->completeURL(url).deprecatedString());
	DeprecatedString query;
	ResourceRequest request;
	RefPtr<FormData> pFormData = NULL ;
	unsigned int fieldCnt = 0;
	//Call all postfield children here and get the data to be sent to server
	for(Node *pChild=firstChild(); pChild; pChild = pChild->nextSibling())
	{
		if(pChild->hasTagName(postfieldTag))
		{
			WMLPostfieldElement *pPostfieldChild = static_cast<WMLPostfieldElement *>(pChild);
			DeprecatedString name = pPostfieldChild->getNameAttr().deprecatedString();
			DeprecatedString value = pPostfieldChild->getValueAttr().deprecatedString();
			//test code
			if(equalIgnoringCase(method(),"POST"))
			{
				if (!pFormData)
					pFormData = new FormData() ;
				if(fieldCnt != 0)
					pFormData->appendData("&",1);

				pFormData->appendData(name.ascii(),name.length());
				pFormData->appendData("=",1); 
				//encode post data ,so that spaces are converted to '+'
				CString cvalue = encodeCString(CString(value.ascii(),value.length()));
				pFormData->appendData(cvalue.data(),cvalue.length());
				fieldCnt++;
			}
			else
			{
				if(!query.isEmpty())
					query.append('&');
				query.append(name);
				query.append('=');
				query.append(value);
			}
		}
	}

	//Below functions can be used for encoding the string into utf-8,
	//Based on accept-charset, we need to do the encoding, not always to utf-8
	//urlstring.encode_string(urlstring.url());

	if(cache_control() == "no-cache")
	{
		request.setCachePolicy(ReloadIgnoringCacheData);
		request.setHTTPHeaderField("Cache-Control","no-cache");
	}
	//Note: code needs to be added for processing accept-charset attribute
	//If go has any children like postfield, it has to send that data also to server.

	if(equalIgnoringCase(method(),"GET"))
	{
		if (!query.isEmpty())
		{
			if (url.contains("?"))
			{
				urlstring.setQueryWithExistingQuery(query);
			}
			else
				urlstring.setQuery(query);
		}
	}
	else if(equalIgnoringCase(method(),"POST"))
	{
		request.setHTTPBody(pFormData);
	}
	
	if(sendreferer() == "true")
	{
		((WMLDocument *)(document()))->setSendReferer(true);
	}
	else
	{
		((WMLDocument *)(document()))->setSendReferer(false);
	}
	request.setHTTPMethod(method().upper());
	request.setHTTPContentType(enctype());
	request.setURL(urlstring);
	document()->frame()->loader()->setAsyncRequest(request) ;
	//Prepare query string for GET method.
# if 0 
	if((method().isEmpty()) || (method().upper() == "GET"))  //default/ Get specified
	{
		urlstring.setQuery(query);
		request.setURL(urlstring);
		request.setHTTPReferrer(sendreferer());
		document()->frame()->loader()->urlSelected(request, getAttribute(targetAttr), evt, false, true);
	}
	else if(method().upper() == "POST")
	{
		request.setURL(urlstring);
		request.setHTTPReferrer(sendreferer());

	//	PassRefPtr<FormData> pFormData = new FormData;
	//	pFormData->appendData(data.ascii(),data.length());
		//request.setHTTPBody(pFormData);
		//frame->loader()->submitForm("GET", m_url, formData(0), m_target, String(), String(), event);
		document()->frame()->loader()->submitForm("POST", url, pFormData,target(), enctype(), String(), evt);
		return;
	}
# endif

#ifdef SUPPORT_AUTO_FORM_FILL

	String URL (document()->baseURI());

	String strFormName = "";	//Need to get form name

	AutoFormFill::StartTransaction(URL,strFormName);

	HTMLCollection *pinputCollection = new HTMLCollection (document(), HTMLCollection::InputTag);

	for(Node *pNode = pinputCollection->firstItem(); pNode; pNode = pinputCollection->nextItem())
	{
		WMLInputElement *input = static_cast<WMLInputElement *>(pNode);
		String typeValue = input->getAttribute(typeAttr);
		String nameValue = input->getAttribute(nameAttr);

		WMLDocument* wmldoc = (WMLDocument*)document();
		String inutValue = wmldoc->BrowserContext()->GetVariableValue(nameValue);
		
		if(NULL != URL && AutoFormFill::IsElementTypeSupported(typeValue))
		{
			if(!nameValue.isEmpty() && !inutValue.isEmpty())
				AutoFormFill::AddElement(nameValue,inutValue);
		}
	}

#if 0
		//ToDo after Settings is Added m_StorePasswords
		Frame *frame = document()->frame();
		Settings* settings = frame->settings();
		if(settings->isPasswordStorageEnabled())
#endif
		AutoFormFill::CommitTransaction();
		AutoFormFill::ClearTransaction();

#endif //SUPPORT_AUTO_FORM_FILL
}

void WMLGoElement::processHref(EventType eventType, bool bForwardNavigation)
{
#if defined ( SFB_SUPPORT_WMLSCRIPT )
	String Fragment ;
#endif//SFB_SUPPORT_WMLSCRIPT
	//Get href attribute value of anchor
	String url = href();
	//Get target url to which we want to navigate to
	String targetUrl = document()->completeURL(url);
	/* START:: Added by Natraj to handle WML Device:Home  SFB00001083*/
	if ( url.endsWith(":home")) 
	{
		WMLDocument *doc = static_cast<WMLDocument *>(document()) ;
		doc->frame()->loader()->getFromHomepage() ; 
		return ; 
	}
	//It may be link like #card3, remove first character, get card id, card3 in this case

	int pos = url.find('#');
		if(pos == -1) // # not found in url, means it can be normal url like google.com
		{
			issueFetchRequest(url);
			if(m_tempEvent)   //do not pass the event to parent, as we handled the event here itself
			{
				m_tempEvent->setDefaultHandled();
			}
			return;
		}

	else if(pos == 0) // first char is '#', so it is local card
	{
		url.remove(0,1);
	}
	else   //For case like samepage.wml#card2, we should not call base class, should just set active id.
	{
		//Get current url from document
		String docUrl = document()->url();
		String docUrlPrefix = "";  //holds document url excluding card id
		int revpos;
		if((revpos = docUrl.reverseFind('#')) == -1)
		{
			docUrlPrefix = docUrl;
		}
		else
		{
			docUrlPrefix = docUrl.substring(0,revpos);
		}

		String targetUrlPrefix = "";  //holds target url excluding card id
		if((revpos = targetUrl.reverseFind('#')) == -1)
		{
			targetUrlPrefix = targetUrl;
		}
		else
		{
#if defined ( SFB_SUPPORT_WMLSCRIPT )
			Fragment = targetUrl.substring(revpos+1) ;
#endif//SFB_SUPPORT_WMLSCRIPT
			targetUrlPrefix = targetUrl.substring(0,revpos);
		}

		if(docUrlPrefix != targetUrlPrefix) //Check whether request is with in the page or not
		{
#if defined ( SFB_SUPPORT_WMLSCRIPT )
			if ( targetUrlPrefix.endsWith(".wmls"))
			{
				WMLDocument *doc = static_cast<WMLDocument *>(document()) ;
				Node *parent = parentNode() ;
				ASSERT (parent) ;
				if (parent->hasTagName(oneventTag)) // may be we need to add several tags here
				{
					doc->scriptInterface()->setWmlsecOnEventParams(targetUrlPrefix, Fragment) ;
				}
				else
				{
					
					doc->scriptInterface()->exeScript(targetUrlPrefix, Fragment);
				}
			}
			else
#endif//SFB_SUPPORT_WMLSCRIPT			
			{
				issueFetchRequest(url);				
			}
			if(m_tempEvent)   //do not pass the event to parent, as we handled the event here itself
			{
				m_tempEvent->setDefaultHandled();
			}
			return;
		}
		else //card in same page, set active id
		{
#if defined ( SFB_SUPPORT_WMLSCRIPT )
			if ( targetUrlPrefix.endsWith(".wmls"  ))
			{
				WMLDocument *doc = static_cast<WMLDocument *>(document()) ;
				Node *parent = parentNode() ;
				ASSERT (parent) ;
				if (parent->hasTagName(oneventTag)) // may be we need to add several tags here
				{
					doc->scriptInterface()->setWmlsecOnEventParams(targetUrlPrefix, Fragment) ;
				}
				else
				{
					doc->scriptInterface()->exeScript(targetUrlPrefix, Fragment); 
				}
			}
			else
#endif//SFB_SUPPORT_WMLSCRIPT
			{
			url = url.substring(pos+1);
		}
	}
	}

	//set active id first and start from event handling from that card
	document()->setActiveCardId(url);
	document()->frame()->loader()->processEventsAndFindDesiredCard(eventType, bForwardNavigation);

	if(m_tempEvent)   //do not pass the event to parent, as we handled the event here itself
	{
		m_tempEvent->setDefaultHandled();
	}
}


}//WebCore

#endif // SHP_SUPPORT_WML