/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2001 Dirk Mueller (mueller@kde.org)
 * Copyright (C) 2004, 2005, 2006, 2007 Apple Inc. All rights reserved.
 *           (C) 2006 Alexey Proskuryakov (ap@nypop.com)
 * Copyright (C) 2007 Samuel Weinig (sam@webkit.org)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

#if defined(SHP_SUPPORT_WML)
#include "config.h"
#include "WMLInputElement.h"

#include "BeforeTextInsertedEvent.h"
#include "CSSPropertyNames.h"
#include "Document.h"
#include "Editor.h"
#include "Event.h"
#include "EventHandler.h"
#include "EventNames.h"
#include "FocusController.h"
#include "FormDataList.h"
#include "Frame.h"
#include "HTMLFormElement.h"
#include "HTMLImageLoader.h"
#include "HTMLNames.h"
#include "KeyboardEvent.h"
#include "LocalizedStrings.h"
#include "MouseEvent.h"
#include "Page.h"
#include "RenderButton.h"
#include "RenderFileUploadControl.h"
#include "RenderImage.h"
#include "RenderText.h"
#include "RenderTextControl.h"
#include "RenderTheme.h"
#include "RenderSlider.h"
#include "SelectionController.h"
#include "TextBreakIterator.h"
#include "TextEvent.h"

#include "WMLDocument.h"
#include "FrameLoader.h"
#include "FrameLoaderClient.h"

#include "NativeEditField.h"

using namespace std;

namespace WebCore {

using namespace EventNames;
using namespace HTMLNames;

WMLInputElement::WMLInputElement(Document* doc, HTMLFormElement* f)
    : HTMLInputElement(doc, f),
	m_inputValueset(false)
{
 
}

void WMLInputElement::parseMappedAttribute(MappedAttribute *attr)
{
	WMLDocument *doc = static_cast<WMLDocument *>(document()) ;
	String varName; 
	String varValue = "";
    if (attr->name() == nameAttr) 
	{
		varName = attr->value();
		if(!(getAttribute(valueAttr).isEmpty()))
		{
			varValue = getAttribute(valueAttr);
			if(varValue.impl())
			{
				String *temp = document()->frame()->getVariableReplacedString (varValue, WMLBrowserContext::ENoEscape) ;
				if (temp)
					varValue = temp->copy();
			}
		}
		doc->BrowserContext()->AddVariable(varName,varValue,WMLBrowserContext::ENoEscape);
    } 
    else if (attr->name() == valueAttr) 
	{
		varValue = attr->value();
		if(varValue.impl())
		{
			String *temp = document()->frame()->getVariableReplacedString (varValue, WMLBrowserContext::ENoEscape) ;
			if (temp)
				varValue = temp->copy() ;
		}
		varName = getAttribute(nameAttr);
		if(!(varName.isEmpty()))
		{
			doc->BrowserContext()->SetVariableValue(varName,varValue);
		}
    } 
	else if(attr->name() == formatAttr)
	{
		varValue = attr->value();
		if(varValue.impl() && (!document()->validateInputFormatString(varValue)))
			setAttribute(formatAttr,String());
	}
    HTMLInputElement::parseMappedAttribute(attr);
}

void WMLInputElement::detach()
{
	HTMLInputElement::detach() ;
	m_inputValueset = false;
}

void WMLInputElement::attach()
{
	String VarName = getAttribute(nameAttr);
	if( !m_inputValueset )
	{
		if(!VarName.isEmpty())
		{
			WMLDocument* wmldoc = (WMLDocument*)document();
			String variableValue = wmldoc->BrowserContext()->GetVariableValue(VarName);
			if(variableValue.isEmpty())
			{
				variableValue = getAttribute(valueAttr);
			}
			setValue(variableValue);
			m_inputValueset = true;
		}
		else
		{
			m_inputValueset = false;
			return ;
		}
	}
	HTMLInputElement::attach();	
}

void WMLInputElement::defaultEventHandler(Event* evt)
{
	if (isTextField() && evt->type() == DOMFocusOutEvent)
	{
		NativeEditField* pNativeEdit = NativeEditField::getNativeEditFieldInstance();
		String val = value();
		if((getAttribute(emptyokAttr) == "false") && (val.isEmpty()))
		{
			if(!pNativeEdit->isKeyPadInFocus())
			{
			document()->frame()->loader()->client()->errorMsg("Input is must");
			}
			pNativeEdit->resetKeyPadFocusFlag();
			return;
		}
		const String format = getAttribute(formatAttr);
		if((format.length()) && (!VerifyInputFormat(value(),format)))
		{
			if((getAttribute(emptyokAttr) == "true") && (val.isEmpty()))
			{
				//Not valid as per format, but empty is ok as emptyok attrib val is true
			}
			else
			{
				//Report error
				setValue(String());
// Need to pickup the error messages from the resource strings.
#pragma message ("SDG - TODO - Need to pickup the error messages from the resource strings.")
				if(!pNativeEdit->isKeyPadInFocus())
				{
				document()->frame()->loader()->client()->errorMsg("The input is invalid.");
			}
		}
		}

		if(((format.length()) && (VerifyInputFormat(value(),format))) ||
			(!(format.length())))
		{
			//if name variable is present for this input element, update the new value in BrowserContext
			String varName = getAttribute(nameAttr);
			if(!(varName.isEmpty()))
			{
				WMLDocument *doc = static_cast<WMLDocument *>(document()) ;
				String varValue = value();
				if(varValue.impl())
				{
					String * temp = document()->frame()->getVariableReplacedString(varValue, WMLBrowserContext::ENoEscape);
					if (temp)
						varValue = temp->copy () ;
				}
				doc->BrowserContext()->SetVariableValue(varName, varValue);
			}
		}
		pNativeEdit->resetKeyPadFocusFlag();
	}
	HTMLInputElement::defaultEventHandler(evt);
}
    
} // namespace

#endif //SHP_SUPPORT_WML
