/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 * Copyright (C) 2004, 2005, 2006, 2007 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */
#if defined(SHP_SUPPORT_WML)
#include "config.h"
#include "Event.h"
#include "EventHandler.h"
#include "EventNames.h"
#include "HTMLNames.h"
#include "WMLRefreshElement.h"
#include "HTMLDocument.h"
#include "WMLDocument.h"
#include "WMLAnchorElement.h"
#include "WMLCardElement.h"
#include "WMLInputElement.h"
#include "WMLSelectElement.h"

namespace WebCore {

using namespace HTMLNames;
using namespace EventNames;

WMLRefreshElement::WMLRefreshElement(Document *doc)
:HTMLElement(refreshTag,doc),
m_RefreshTaskEnterEventSentToCard(false),
m_RefreshTaskExitEventSentToCard(false)

{

}
WMLRefreshElement::~WMLRefreshElement()
{

}

void WMLRefreshElement::handleEvent(EventType eventType, bool bForwardNavigation, bool & bIsEventHandled)
{
	// Refresh task is divided into 4 steps:
	// $$1. Stop timers and commit their values on refresh task enter.
	// $$2. Commit all the variables.
	// $$3. Restart the timer with the commited values.
	// $$4. Layout the document and reflect the updated variable values.

	WmlCardElement *parentCard = NULL;
	
	if(eventType != RefreshTaskEnterEvent && eventType != RefreshTaskExitEvent )
	{
		for(Node *pChild = firstChild();pChild;pChild = pChild->nextSibling())
			pChild->handleEvent(eventType,bForwardNavigation, bIsEventHandled);
		
		if(eventType == DoMenuEvent)
		{
			// Now for every child under the present <card> tag,except for the <do> 
			// & <timer> tags just check if their name matches with the variable 
			// names stored in the browser context.If they do match then update 
			// the element values with the variables present values.
			WMLDocument* wmldoc = (WMLDocument*)document();
			// Get the variables list present in browser context.
			AttributeMap attrMap = 	wmldoc->BrowserContext()->GetVariableAttributeMap();
			AttributeMap::iterator begin = attrMap.begin() ;
			AttributeMap::iterator end = attrMap.end() ;
			PassRefPtr<HTMLCollection> allNodes = document()->all();
			String activeCardId = wmldoc->getActiveCardId();
			bool foundActiveCard = false;

			// As we need to get only elements in the present card only
			// we would need to create a vector of nodes from the HTMLCollection
			for(int i = 0; i < allNodes->length(); i++)
			{
				Node* anode = allNodes->item(i);
				if(foundActiveCard && anode->localName()!= cardTag)
				{
					if(anode->localName()== doTag			||	anode->localName()== refreshTag || 
						anode->localName()== postfieldTag	||	anode->localName()== timerTag ||
						anode->localName()== setvarTag		||	anode->localName()== oneventTag)
					{
							continue;
					}
					else
					{
						Attribute* attr = anode->attributes()->getAttributeItem(nameAttr);
						if(!attr)
						{
							//if no variables to update then just layout them
							anode->detach();
							anode->attach();
							continue;
						}

						// Find elements which should reflect the variable values
						// and update them.
						for (AttributeMap::iterator it = begin; it != end ; ++it)	
						{
							String var = it->first;
							WMLVariable *varVal = it->second ;
							if(var == attr->value())
							{
								if(anode->localName() == inputTag)
								{
									WMLInputElement* elem = (WMLInputElement*)(anode);
									elem->setValue(varVal->getVariable());
								}
								else if(anode->localName() == selectTag)
								{
									WMLSelectElement* elem = (WMLSelectElement*)(anode);
									elem->setValue(varVal->getVariable());
								}
							}
						}//for
					}//else
				}//if(foundActiveCard && anode->localName()!= cardTag)
				else if(anode->localName()== cardTag)
				{
					//found card tag again,but we have already processed the 
					//active card,so we need not process the rest of the tags.
					if(foundActiveCard)
						break;
					WmlCardElement* wcard = (WmlCardElement*) anode;
					if(wcard->id() == activeCardId)
						foundActiveCard = true;
				}
			}//for(int i = 0; i < allNodes->length(); i++)
		} //if(DoMenuEvent)
		return;
	}//if(eventType != RefreshTaskEnterEvent && eventType != RefreshTaskExitEvent )

	//	
	// $$1 Refresh task handling enter
	//
	if(m_RefreshTaskEnterEventSentToCard == false)
	{
		m_RefreshTaskEnterEventSentToCard = true;
		Node *parent = parentNode() ;
		ASSERT (parent) ;
		while (parent)
		{
			if (parent->hasTagName(cardTag)) // may be we need to add several tags here
			{
				parentCard = static_cast<WmlCardElement *>(parent) ;
				parentCard->handleEvent(RefreshTaskEnterEvent,false, bIsEventHandled);
				break ;
			}
			else
				parent = parent->parentNode () ;
		}
		//m_RefreshTaskEnterEventSentToCard = false;
		m_RefreshTaskExitEventSentToCard = true;
	}
	// $$2
	if(eventType == RefreshTaskEnterEvent && (m_RefreshTaskEnterEventSentToCard))//After doing above tasks, do child tasks if any like setvar
	{
		for(Node *pChild = firstChild();pChild;pChild = pChild->nextSibling())
		{
			pChild->handleEvent(eventType,bForwardNavigation, bIsEventHandled);
		}
		m_RefreshTaskEnterEventSentToCard = false;

	}
	//	
	//$$3 Refresh task handling exit
	//
	if(m_RefreshTaskExitEventSentToCard)
	{
		parentCard->handleEvent(RefreshTaskExitEvent,false, bIsEventHandled);

		// $$4 Force the relayout of the complete tree now.
		/*document()->detach();
		document()->attach();*/
		document()->reRender();

		m_RefreshTaskEnterEventSentToCard = false;
		m_RefreshTaskExitEventSentToCard = false;
	}
}

RenderObject *WMLRefreshElement::createRenderer(RenderArena* arena, RenderStyle* style)
{
	RenderObject *obj = HTMLElement::createRenderer (arena, style) ;
	//for some miguided code here... :) for attributes

	Node *parent = parentNode() ;
	ASSERT (parent) ;
	while (parent)
	{
		if (parent->hasTagName(anchorTag)) // may be we need to add several tags here
		{
			WMLAnchorElement *anchor = static_cast<WMLAnchorElement *>(parent) ;
			anchor->setChildAttributes (this) ;
			m_isLink = true;
			break ;
		}
		else
			parent = parent->nextSibling () ;
	}
	return obj ;
	
}

void WMLRefreshElement::defaultEventHandler(Event* evt)
{
	if (m_isLink && (evt->type() == clickEvent)) {
		bool bIsEventHandled = false;
		handleEvent(RefreshTaskEnterEvent,true, bIsEventHandled);
		evt->setDefaultHandled();
	}
	else {
		HTMLElement::defaultEventHandler(evt)	;
	}
}

		
} //WebCore
#endif
