#if defined(SFB_SUPPORT_WMLSCRIPT)

#include "config.h"
#include "PlatformString.h"
#include "StringHash.h"
#include <Shp.h>
#include "ShpJasmine.h"
#include <wtf/HashMap.h>
#include "KURL.h"
#include "Wmlsec.h"
#include "WMLScriptInterface.h"
#include "CachedScript.h"
#include "WMLDocument.h"
#include "CString.h"
#include "Node.h"
#include "WMLBrowserContext.h"
#include "FrameLoader.h"
#include "Page.h"
#include "WMLGoElement.h"
#include "FrameLoader.h"
#include "Timer.h"
#include "WMLTimerElement.h"
#include "DocLoader.h"
#include "allocs.h"

#include "ChromeClient.h"

namespace WebCore {

static int RunWmlScriptAlert(void *closure, const char *message)
{
	return static_cast<WMLScript*>(closure)->WmlScriptAlert(message);
}

static int RunWmlScriptConfirm(void *closure, const char *message, const char *okText, const char *cancelText)
{
	return static_cast<WMLScript*>(closure)->WmlScriptConfirm(message,okText,cancelText);
}

static int RunWmlScriptPrompt(void *closure, const char *message, const char *defInp)
{
	return static_cast<WMLScript*>(closure)->WmlScriptPrompt(message,defInp);
}

static void ExtUrlDownload (void *closure, char *Value)
{
	//performs the download of a given external url
	return static_cast<WMLScript*>(closure)->ExtUrlDownload (Value) ;
}

static void DownloadUrl (void *closure, char *Value, char *contentType)
{
	//performs the download of a given url
	return static_cast<WMLScript*>(closure)->DownloadUrl (Value, contentType) ;
}

static void newBrowserContext (void *closure)
{
	//performs the same as newcontext attribute
	return static_cast<WMLScript*>(closure)->newBrowserContext () ;
}

static char* getCurrentCard (void *closure)
{
	//returns the current active card ID.
	return static_cast<WMLScript*>(closure)->getCurrentCard () ;
}

static char *getCurrentUrl (void *closure)
{
	//returns the current document url
	return static_cast<WMLScript*>(closure)->getCurrentUrl () ;
}

static char *getContextVariableValue (void *closure, char* name)
{
	//returns the value of the variable in the form of character string
	return static_cast<WMLScript*>(closure)->getContextVariableValue (name) ;
}
static BOOL isValidVariable(void *closure, char* name)
{
	//returns whether the variable is present in the browser context
	return static_cast<WMLScript*>(closure)->isValidVariable (name) ;
}

static void refreshBrowserDisplay (void *closure)
{
	//based on the current context, the browser display is updated
	return static_cast<WMLScript*>(closure)->refreshBrowserDisplay () ;
}

static void setVariableValue (void *closure, char* name, char * value)
{
	return static_cast<WMLScript*>(closure)->setVariableValue (name, value) ;
}

static void setGoElement(void *closure, char* Value )
{
	return static_cast<WMLScript*>(closure)->setGoElement (Value) ;
}
static void setPrevElement(void *closure, char *Value)
{
	return static_cast<WMLScript*>(closure)->setPrevElement (Value) ;
}

static void resumeWMLScriptExec(void *closure, char *Value)
{
	return static_cast<WMLScript*>(closure)->resumeScriptExec(Value) ;
}

static void InitializeCallbacks (WMLS_BRWFUNC_HANDLER *pHandle)
{
	pHandle->BrwCtx = newBrowserContext ;
	pHandle->BrwCurrCard = getCurrentCard;
	pHandle->BrwCurrUrl = getCurrentUrl ;
	pHandle->BrwgetVar = getContextVariableValue ;
	pHandle->BrwRefreshDoc = refreshBrowserDisplay ;
	pHandle->BrwsetVar = setVariableValue ;
	pHandle->BrwGoValue = setGoElement ;
	pHandle->BrwPrevValue = setPrevElement ;
	pHandle->BrwResScriptExecValue = resumeWMLScriptExec ;

	pHandle->BrwDownloadExtUrlValue = ExtUrlDownload ;
	pHandle->BrwDownloadUrlValue = DownloadUrl ;

	pHandle->BrwRunAlert = RunWmlScriptAlert;
	pHandle->BrwRunConfirm = RunWmlScriptConfirm;
	pHandle->BrwRunPrompt = RunWmlScriptPrompt;
	return ;
}

WMLScript::WMLScript(WMLDocument *doc)
:m_doc (doc)
{
	WMLSECINPUTPARAMS params ;
	WMLS_BRWFUNC_HANDLER callbacks ;
	ASSERT (m_doc) ;
	params.pWMLSIntfClass = this ;
	InitializeCallbacks (&callbacks) ;
	params.brwFunchandler = callbacks ;
	WmlsecInit(&params, &m_scriptHandle);	
}

WMLScript::~WMLScript()
{
}

bool WMLScript::WmlScriptAlert(const char *message)
{
	return m_doc->page()->chrome()->client()->runWmlScriptAlert(message);	
}

bool WMLScript::WmlScriptConfirm(const char *message,const char *okText,const char *cancelText)
{
	return m_doc->page()->chrome()->client()->runWmlScriptConfirm(message,okText,cancelText);		
}

bool WMLScript::WmlScriptPrompt(const char *message,const char *defInp)
{
	return m_doc->page()->chrome()->client()->runWmlScriptPrompt(message,defInp);
}

void WMLScript::setGoElement ( char* Value)
{
	goElt = Value ; 
	return ;
}

void WMLScript::setPrevElement (char *value)
{
	prevElt = value ;
	return ;
}

void WMLScript::newBrowserContext ()
{
	m_doc->BrowserContext()->ResetBrowserContext() ;
}

char *WMLScript::getCurrentCard ()
{
	String card = m_doc->getActiveCardId () ;
	char *tempCardId = (char *)card.utf8().data() ;
	char *cardId = 0 ;
	if (tempCardId)
		cardId = (char*)malloc(strlen(tempCardId)+1);
	if (cardId)
		AcStrcpy(cardId, tempCardId) ;
	return cardId ;
}

char *WMLScript::getCurrentUrl ()
{
	char *documentUrl = (char *)m_doc->url().ascii() ;
	char *url = 0 ;
	if (documentUrl)
		url = (char*)malloc(strlen(documentUrl)+1);
	if (url)
		AcStrcpy(url, documentUrl) ;
	return url ;
}

char *WMLScript::getContextVariableValue (char *name)
{
	String tempName(name) ;
	String value = m_doc->BrowserContext()->GetVariableValue(tempName) ;
	CString utf8Str = value.utf8() ;
	char *tempValue = (char *)(utf8Str.data()) ;
	char *pValue = 0 ;
	if (tempValue)
		pValue = (char*)malloc(strlen(tempValue)+1);
	if (pValue)
		AcStrcpy(pValue, tempValue) ;
	return pValue ;
}

bool WMLScript::isValidVariable (char *name)
{
	//not required to be implemented
	return TRUE ;
}

void WMLScript::refreshBrowserDisplay ()
{	
	refreshTaskValue = true ; 
	if ( refreshTaskValue )
	{
		HTMLCollection *pCardTimer = new HTMLCollection (m_doc->document(), HTMLCollection::timerTag) ;
		Node *pNode;
		for(pNode = pCardTimer->firstItem(); pNode; pNode = pCardTimer->nextItem())
		{
			WMLTimerElement *pCard = static_cast<WMLTimerElement *>(pNode);
			pCard->StopTimer() ; 
			delete pCardTimer ; 
		}
		
	}
		m_doc->document()->reRender();
}

void WMLScript::setVariableValue (char *name, char *value)
{
	String tempName(name) ;
	String tempValue(value) ;
	m_doc->BrowserContext()->AddVariable(tempName, tempValue, WMLBrowserContext::EDefault) ;
	m_doc->BrowserContext()->SetVariableValue(tempName, tempValue) ;
}
// It's for raw buffer
void WMLScript::bufferContent ( const String& url, char *buffer, int bufferLen )
{
	String charset = m_doc->charset() ;
	CompileContent( url, buffer, bufferLen,  charset ) ; 
}

void WMLScript::CompileContent(  const String& url, char* buffer, int bufferLen, String charset )
{
	WMLSRETCODE rCode ;
	int charsetVal = 0x6A ;
	rCode = WmlsecCompileScript(m_scriptHandle, buffer, bufferLen, charsetVal) ;
	if (RCODE_OK)
	{
		setcompiledBuffer ( url, ( char * )WMLS_GET_COMPILEDBUFF(m_scriptHandle), WMLS_GET_COMPILEDBUFFLEN(m_scriptHandle) ) ; 
	}
	return ;
}

void WMLScript::resumeScriptExec(char *fragment) 
{
	WMLSRETCODE rCode ;
	WMLSINPUTPARAMS WmlsInputParams = {0, } ;
	WmlsInputParams.InpBuff = app_buffer ;
	WmlsInputParams.InpBuffLen = p_buflen ;
	WmlsInputParams.pFragment = fragment ;
	if (contentType)
	{
		WmlsInputParams.urlContenttype = contentType ;
	}
	rCode = WmlsecResumeExecution(m_scriptHandle, &WmlsInputParams) ;
	if (RCODE_OK && (goElt || prevElt))
	{
		if (goElt)
		{		
			ResourceRequest request;
			String URLValue = String ( goElt );
			String targetgoUrl = m_doc->completeURL (URLValue ) ;
			String docUrl = m_doc->document()->url() ; 
	

			int pos = targetgoUrl.find('#');
			if ( pos == -1 )//# not found in url,
			{
				request.setURL(targetgoUrl.deprecatedString()) ; 
				m_doc->document()->frame()->loader()->urlSelected( request, String(), NULL, false, true) ;

			}
			else
			{
				String docUrl = m_doc->document()->url() ; 
				String docUrlPrefix = "";  //holds document url excluding card id
				int revpos;
				if((revpos = docUrl.reverseFind('#')) == -1)
				{
					docUrlPrefix = docUrl;
				}
				else
				{
					docUrlPrefix = docUrl.substring(0,revpos);
				}
				
				String targetUrlPrefix = "";  //holds target url excluding card id
				if((revpos = targetgoUrl.reverseFind('#')) == -1)
				{
					targetUrlPrefix = targetgoUrl;
				}
				else
				{
					targetUrlPrefix = targetgoUrl.substring(0,revpos);	
				}

				if(docUrlPrefix != targetUrlPrefix) //Check whether request is with in the page or not
				{
					//int revPos = targetUrlPrefix.reverseFind('#') ;
					int p = URLValue.reverseFind('#');
					String cardId = URLValue.substring(p+1) ; 
					m_doc->document()->setActiveCardId(cardId);
					m_doc->document()->frame()->loader()->processEventsAndFindDesiredCard(NavigationEvent, true);
					KURL url(targetUrlPrefix.deprecatedString()) ;
					request.setURL(url) ; 
					m_doc->document()->frame()->loader()->urlSelected( request, String(), NULL, false, true) ;
					m_doc->document()->reRender();
				}
				
				else //card in same page, set active id
				{	
					//set active id first and start from event handling from that card
					int revposition = docUrl.reverseFind('#') ; 
					String finalRequestUrl = "" ; 
					if ( revposition == -1 )
					{
						finalRequestUrl = docUrl ; 
					}
					else
					{
						finalRequestUrl = docUrl.substring(0, revposition);
					}
					int revPos = URLValue.reverseFind('#') ;
					String cardId = URLValue.substring(revPos+1) ; 
					m_doc->document()->setActiveCardId(cardId);
					m_doc->document()->frame()->loader()->processEventsAndFindDesiredCard(NavigationEvent, true);
					m_doc->document()->reRender();
				}
			}
		}
		else if(prevElt)
		{
			m_doc->document()->page()->goBack() ;
		}
	}
	//delete app_buffer ;

	return ;
}

void WMLScript::exeScript(const String& targetUrlPrefix, String &fragment ) 
{
	WMLSRETCODE rCode ;
	String DocumentUrl = m_doc->baseURL();
	CString frag = fragment.utf8() ;
	CString docURL = DocumentUrl.utf8() ;
	WMLSINPUTPARAMS WmlsInputParams = {0, } ;
	getcompiledBuffer ( targetUrlPrefix, &app_buffer, &p_buflen) ;
	/*	Note:p_buflen = 0 indicates script syntax error or an unfetched url error. 
		WML will continue with processing from the next element */
	if (p_buflen)
	{
		WmlsInputParams.InpBuff = app_buffer ;
		WmlsInputParams.InpBuffLen = p_buflen ;
		
		WmlsInputParams.pFragment = frag.data() ;
		WmlsInputParams.pDocUrl =  docURL.data() ;
		goElt = prevElt = 0 ;
		
		rCode = WmlsecExecuteScript(m_scriptHandle, &WmlsInputParams) ;
		if (RCODE_OK && (goElt || prevElt))
		{
			if (goElt)
			{
				ResourceRequest request;
				String URLValue = String ( goElt );
				String targetgoUrl = m_doc->completeURL (URLValue ) ;
				String docUrl = m_doc->document()->url() ; 
		

				int pos = targetgoUrl.find('#');
				if ( pos == -1 )//# not found in url,
				{
					request.setURL(targetgoUrl.deprecatedString()) ; 
					m_doc->document()->frame()->loader()->urlSelected( request, String(), NULL, false, true) ;

				}
				else
				{
					String docUrl = m_doc->document()->url() ; 
					String docUrlPrefix = "";  //holds document url excluding card id
					int revpos;
					if((revpos = docUrl.reverseFind('#')) == -1)
					{
						docUrlPrefix = docUrl;
					}
					else
					{
						docUrlPrefix = docUrl.substring(0,revpos);
					}
					
					String targetUrlPrefix = "";  //holds target url excluding card id
					if((revpos = targetgoUrl.reverseFind('#')) == -1)
					{
						targetUrlPrefix = targetgoUrl;
					}
					else
					{
						targetUrlPrefix = targetgoUrl.substring(0,revpos);	
					}

					if(docUrlPrefix != targetUrlPrefix) //Check whether request is with in the page or not
					{
						//int revPos = targetUrlPrefix.reverseFind('#') ;
						int p = URLValue.reverseFind('#');
						String cardId = URLValue.substring(p+1) ; 
						m_doc->document()->setActiveCardId(cardId);
						m_doc->document()->frame()->loader()->processEventsAndFindDesiredCard(NavigationEvent, true);
						KURL url(targetUrlPrefix.deprecatedString()) ;
						request.setURL(url) ; 
						m_doc->document()->frame()->loader()->urlSelected( request, String(), NULL, false, true) ;
						m_doc->document()->reRender();
					}
					
					else //card in same page, set active id
					{	
						//set active id first and start from event handling from that card
						int revposition = docUrl.reverseFind('#') ; 
						String finalRequestUrl = "" ; 
						if ( revposition == -1 )
						{
							finalRequestUrl = docUrl ; 
						}
						else
						{
							finalRequestUrl = docUrl.substring(0, revposition);
						}
						int revPos = URLValue.reverseFind('#') ;
						String cardId = URLValue.substring(revPos+1) ; 
						m_doc->document()->setActiveCardId(cardId);
						m_doc->document()->frame()->loader()->processEventsAndFindDesiredCard(NavigationEvent, true);
						m_doc->document()->reRender();
					}
				}
			}
			else if(prevElt)
			{
				m_doc->document()->page()->goBack() ;
			}
		}
		//delete app_buffer ;
	}
	return ;

}

void WMLScript::setcompiledBuffer( const String& url, char * buffer, int bufferLen )
{
	//if ( !m_cachedScriptMap.contains(url) ) 	
		//return ;
	WMLCachedScript* Urlvar = m_cachedScriptMap.get( url ) ;
	ASSERT (Urlvar ) ;
	if ( Urlvar )
		Urlvar->setBuffer (buffer, bufferLen) ;// it is for storing the buffer & it's length, ie one more copy

	app_buffer = buffer ;
	p_buflen = bufferLen ;
}

void WMLScript::getcompiledBuffer ( const String& url, char **app_buffer, int *p_buflen )
{
	//ASSERT(m_cachedScriptMap.contains(url)) ;
	WMLCachedScript *var = m_cachedScriptMap.get( url ) ;
	//ASSERT (var ) ;
	if ( var ) 
	{
		var->getBuffer (app_buffer, p_buflen ) ;
		//app_buffer = var->m_buffer ;
		//*p_buflen = var->m_bufferLen ;
	}
	else
	{
	}
	//	var->getBuffer (app_buffer, p_b;
}

void WMLScript::addCachedScript (const String &url, WMLCachedScript* cachedScript)	
{
	if (m_cachedScriptMap.contains(url))
		return ;
	m_cachedScriptMap.add(url, cachedScript ) ;
}

void WMLScript::setBuffer (char * buffer, int bufferLen )	
{
	app_buffer = buffer ;
	p_buflen = bufferLen ;
}

void WMLScript::ExtUrlDownload (char *UrltoDownload)
{
	String URLValue = String ( UrltoDownload );
	String targetgoUrl = m_doc->completeURL (URLValue ) ;
	char *l_app_buffer;
	int len;
	WMLCachedScript *var = m_cachedScriptMap.get( targetgoUrl ) ;
	//ASSERT (var ) ;
	if ( var ) 
	{
		var->getBuffer (&l_app_buffer, &len ) ;
		WMLS_SET_COMPILEDBUFF(m_scriptHandle,(unsigned char *)l_app_buffer);
		WMLS_SET_COMPILEDBUFFLEN(m_scriptHandle,len);
	}
	else
	{
		m_doc->document()->docLoader()->WmlrequestScript( targetgoUrl, String()) ;
	}
	return ;
}

void WMLScript::DownloadUrl (char *UrltoDownload, char *contentype)
{ 
	String URLValue = String ( UrltoDownload );
	ASSERT(NULL!=contentype) ;
	WMLSEC_SET_CONTENT_TYPE(m_scriptHandle, contentype) ;
	downloadUrl = URLValue ;
	/* load string ,ust provide a complete url */
	//WMLScript::contentType = contentype ;
	//String targetgoUrl = m_doc->completeURL (URLValue ) ;
	//String docUrl = m_doc->document()->URL() ; 

	m_doc->document()->docLoader()->WmlrequestScript( URLValue, String()) ;
	return ;
}

bool WMLScript::isWmlScriptSuspended()
{
	return IS_WMLSE_SUSPEND(m_scriptHandle) ;
}

bool WMLScript::isWmlScriptExecuting()
{
	return IS_WMLSE_EXECUTING(m_scriptHandle) ;
}

bool WMLScript::isWmlScriptIdle()
{
	return IS_WMLSE_IDLE(m_scriptHandle) ;
}

bool WMLScript::isWmlScriptOnEventExecuting()
{
	return IS_WMLSE_ONEVENTEXECUTING(m_scriptHandle) ;
}

void WMLScript::setWmlsecStateOnEventExecuting()
{
	SET_WMLSE_STATE_ONEVENT_ENCOUNTERED(m_scriptHandle);
	return;
}

void WMLScript::setWmlsecOnEventParams(String &url, String &fragment)
{
	/* memory will be allocated by the wmlsec controller 
	   no need to allocate fresh memory for url and fragment by the caller */
	onEventUrl = url ;
	onEventFragment = fragment ;
	/*Shall assert if the script enginr is in suspended/executing state */
	setWmlsecStateOnEventExecuting();
}

void WMLScript::setWmlsecExecOnEventScript(String &scriptUrl)
{
	/* 
	1. perform url compare
	2. if 1 is success, perform exeScript
	3. Otherwise, just return
	*/
	if (isWmlScriptOnEventExecuting() && (scriptUrl == onEventUrl))
	{
		exeScript(onEventUrl, onEventFragment) ;
		/*onEventUrl=0 ;
		onEventFragment = 0 ;*/
	}
	return ;

}
}//WebCore
#endif//SFB_SUPPORT_WMLSCRIPT