#if defined(SHP_SUPPORT_WML)
#include "config.h"
#include "Element.h"
#include "Timer.h"
#include "WMLTimerElement.h"
#include "AtomicString.h"
#include "CSSHelper.h"
#include "CSSPropertyNames.h"
#include "CSSValueKeywords.h"
#include "EventNames.h"
#include "HTMLDocument.h"
#include "HTMLNames.h"
#include "QualifiedName.h"
#include "WMLBrowserContext.h"
#include "WMLDocument.h"
#include "WmlCardElement.h"
#include "FrameLoader.h"

namespace WebCore {

using namespace HTMLNames;

WMLTimerElement::WMLTimerElement ( Document* doc)
:HTMLElement(timerTag, doc),
m_Timer(this, &WMLTimerElement::HandleTimerComplete),
m_TimerInterval(0),
m_AttributeName(NULL),
m_NameAttrPresent(false),
m_TimerFiredFirstTime(false)
{

}

WMLTimerElement::~WMLTimerElement()
{
	if(m_AttributeName)
	{
		delete m_AttributeName;
		m_AttributeName = NULL;
	}
}

String WMLTimerElement::id ( )const
{
    return getAttribute(idAttr);
}

void WMLTimerElement::setId(const String &value)
{
    setAttribute(idAttr, value);
}

String WMLTimerElement::Class() const
{
    return getAttribute(classAttr);
}

void WMLTimerElement::setClass(const String &value)
{
    setAttribute(classAttr, value);
}

String WMLTimerElement::Value() const
{
	return getAttribute(valueAttr);
}

void WMLTimerElement::setValue (const String &value)
{
	setAttribute(valueAttr,value);
}

void WMLTimerElement::HandleTimerComplete(Timer<WMLTimerElement>* aTimer)
{
	// Propagate this event to the card element.
	//parent()->handleEvent(TimerEvent,false);
	//m_bIsForwardNavigation = bIsForwardNavigation;

	String curActiveCardId = document()->getActiveCardId();
	HTMLCollection *pCardCollection = new HTMLCollection (document(), HTMLCollection::CardTag) ;
	document()->setIsCardChanged(false);
	bool bIsEventHandled = false;
	do
	{
		document()->setIsCardChanged(false);
		String activeId = document()->getActiveCardId();
		

		WmlCardElement *pCard = NULL;
		for(Node *pNode = pCardCollection->firstItem(); pNode; pNode = pCardCollection->nextItem())
		{
			pCard = static_cast<WmlCardElement *>(pNode);
			if(pCard->id() == activeId)
			{
				break;
			}
		}
		if(pCard) //This is card node, call handleEvent and check whether card id changed or not
		{
			pCard->handleEvent(TimerEvent,true, bIsEventHandled);
		}
	}while(document()->isCardChanged() && !bIsEventHandled);

	if(false == bIsEventHandled)
	{
		//Event flags will be set in the above loop.
		//Event flags decide whether to do tasks in template event elements or not.
		HTMLCollection *pTemplateCollection = new HTMLCollection (document(), HTMLCollection::TemplateTag) ;
		Node *pNode = pTemplateCollection->firstItem() ;
		if(pNode) //process template node
		{
			//Pass Timer event to Template only if current <timer> tag is present in active card
			WmlCardElement *pParent = static_cast<WmlCardElement *>(this->parent());
			if(pParent->id() == document()->getActiveCardId())
			{
				pNode->handleEvent(TimerEvent, true, bIsEventHandled);
			}
		}
		delete pTemplateCollection;
	}
	else 
	{
		document()->reRender();
		StopTimer();
	}


	//Reset bIsEventAlreadyHandled flag of all cards
	document()->resetCardEventFlags(pCardCollection);
	if ( curActiveCardId != document()->getActiveCardId() )
	{
		document()->reRender();
		StopTimer();
		document()->frame()->loader()->processEventsAndFindDesiredCard(NavigationEvent, true);
	}

	delete pCardCollection;
}

void WMLTimerElement::StartTimer(unsigned int aTimerInterval)
{
	//m_TimerInterval = aTimerInterval;
	//The timer timeout value is specified in units of one-tenth (1/10) of a second.
	if (!m_Timer.isActive())
	{
		double timerval = m_TimerInterval / 10.00;
		// 0 value disables the timer.
		if (timerval < 1.0 )
			timerval = 1.0 ;/* If timerval is less the 1 sec,we set timerval Minimum 1 second*/
		if (timerval != 0.0)
		    m_Timer.startOneShot(timerval);
	}

}

void WMLTimerElement::StopTimer()
{
	if (m_Timer.isActive())
		m_Timer.stop();
}

void WMLTimerElement::PauseTimer()
{
	//Not sure how to implement this one
	m_Timer.pause();
}

void WMLTimerElement::RestartTimer()
{
	m_Timer.restart();
}

String WMLTimerElement::Name() const
{
	return getAttribute(nameAttr);
}

// name attribute is holds the value that reflects the current value of the timer.
//
void WMLTimerElement::setName (const String &name)
{
	setAttribute(nameAttr,name);
}


void WMLTimerElement::parseMappedAttribute(MappedAttribute* attr)
{
	if(attr->name() == nameAttr)
	{
		m_NameAttrPresent = true;
		if(m_AttributeName)
		{
			delete m_AttributeName;
			m_AttributeName = NULL;
		}
		m_AttributeName = new String(attr->value().domString());
	}
	else if(attr->name() == valueAttr)
	{
		String temp(attr->value().domString().impl()) ;
		m_TimerInterval = temp.toInt() ;
		m_NameAttrPresent = false;

	}
	else
		HTMLElement::parseMappedAttribute(attr);
}

void WMLTimerElement::detach()
{
	StopTimer() ;
	HTMLElement::detach() ;
	m_TimerFiredFirstTime = false ;
}

void WMLTimerElement::attach()
{
	WmlCardElement* parCard= (WmlCardElement*)parent();
	//Start the timer the very first time
	if(!m_TimerFiredFirstTime)	{
		WMLDocument* wmldoc = (WMLDocument*)document();
		if(document()->getActiveCardId() == parCard->id())	{
			if ( m_TimerInterval < 0 )
				return ; 
			else	{
				String temp = Value() ;
				m_TimerInterval = temp.toInt() ;
			
				StartTimer(m_TimerInterval);
				m_TimerFiredFirstTime = true;
			}
		}
	}
}

void WMLTimerElement::handleEvent(EventType eventType, bool bForwardNavigation, bool & bIsEventHandled)
{
	// TODO_MAYUR:Need to check when should the NavigationEvent be
	//if(eventType == NavigationEvent && bForwardNavigation)
	//	StopTimer();	

	// Not sure what to do in this case of these events
/*	else if(eventType == PickEvent)
		i++;
*/
	if(eventType == RefreshTaskEnterEvent)
	{
		PauseTimer();
		m_TimerInterval = m_Timer.fireInterval()*10;
		WMLDocument* wmldoc = (WMLDocument*)document();
		String timerValue;
	}
	else if(eventType == RefreshTaskExitEvent)
	{
		String timeInterval ;
		StartTimer(m_TimerInterval);
	}
	m_TimerFiredFirstTime = false ;
}

}
#endif