/*
* ==============================================================================
*  Name         : FourWayNavigation.h
*  Part of      : WEBCORE / Page
*  Description  : Interface of 4-Way Navigation
*  Date         : April 19, 2008
*  Author       : Prashant Nevase (n.prashant@siso.com)
* ==============================================================================
*/

#ifndef __FOUR_WAY_NAVIGATION_H__
#define __FOUR_WAY_NAVIGATION_H__

#ifdef SUPPORT_FOUR_WAY_NAVIGATION //compile only if this flag is enabled

#if PLATFORM(SHP)
#include "CMMDefs.h"
#endif // PLATFORM(SHP)

#include "Navigation.h"
#include "Vector.h"

namespace WebCore {


struct NodeWithEdges
{
	NodeWithEdges() :	topEdge(0),
						bottomEdge(0),
						leftEdge(0),
						rightEdge(0),
						hCenter(0),
						vCenter(0),
						node(0)
	{
	}

	void reset()
	{
		topEdge = 0;
		bottomEdge = 0;
		leftEdge = 0;
		rightEdge = 0;
		hCenter = 0;
		vCenter = 0;
		node = 0;
	}
	
	int topEdge;
	int bottomEdge;
	int leftEdge;
	int rightEdge;

	int hCenter;
	int vCenter;

	Node* node;
};

class CFourWayNavigation : public CNavigation
{
public:
	//STATIC METHODS
	//for static global handling
	static void initFWNStatics();

	static void init();
	static void finalize();
	static CFourWayNavigation* createInstance();
	static void destroyInstance(CFourWayNavigation* fourWayNavigation);

	//OVERRIDDEN METHODS
	virtual void setFrameView(FrameView* frameView);

	//for new page load in the same window
	virtual void resetPage();
	virtual void pageLoadingFinished();

	virtual void updateVerticalStats()	{}
	virtual void updateHorizontalStats() {}

	virtual void handleNavigationEvent(int keyCode, bool isKeyUp);

	virtual void getPointerLocation(FrameView** frameView, int* x, int* y) {}
	virtual void setPointerLocation(int x, int y) {}

	virtual void scrollBy(int xOffset, int yOffset) {}

#ifdef SUPPORT_TOUCH_SCREEN
	virtual void touchPressed(IntPoint wndPoint) {}
	virtual void touchMoved(IntPoint wndPoint) {}
	virtual void touchReleased(IntPoint wndPoint) {}
#endif

	virtual void doPostPaintAction() {}
	virtual void doDefaultAction();

	virtual Node* getHoveredNode() { return m_hoveredNode; }

private:
	CFourWayNavigation();
	~CFourWayNavigation();

	bool findNextFocusableNode(ENavigationType direction);
	bool scrollView(ENavigationType direction);
	int getNextFocusableNode(ENavigationType direction);
	float getRelativeVerticalDistance(int left, int right, int v, int leftRef, int rightRef, int vRef);
	float getRelativeHorizontalDistance(int top, int bottom, int h, int topRef, int bottomRef, int hRef);
	
	void setDefaultFocusedNode();
	void focusCurrentNode();
	void activateCurrentNode();
	
	void collectFocusableNodesInVisibleRect();
	
	Vector<NodeWithEdges> m_nodeCollector;

	int m_frameVisibleBottom;
	int m_frameVisibleTop;
	int m_frameVisibleRight;
	int m_frameVisibleLeft;
	int m_scrollStep;

	bool m_startHandlingKeyEvents;

	NodeWithEdges m_currentFocusedNode;
};

} //namespace WebCore

#endif //SUPPORT_FOUR_WAY_NAVIGATION

#endif //__FOUR_WAY_NAVIGATION_H__