/*
 *  Copyright (c)  SISO, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics.
 */
/**
 * @file		WebSmartLinkMap.cpp
 * @brief		implement Smartlink
 * @author	Gnanasekar.s (Gnanasekar.s@samsung.com) 
 * @version	1.0
 * @date		2008.07.18	v1.0		Created			Gnanasekar.S				
 **/
#ifdef SFB_SMARTLINK_FEATURE 

#include "SmartLinkMap.h"
#include "SmartLinkCutLink.h"
#include "CString.h"
#include "Frame.h"
#include "ShpJasmine.h"
#include "SystemTime.h"

#define SMARTLINK_MAX_ENTRY				64
#define SMARTLINK_EVENT_TIME			2000


using namespace WebCore;

	// ============================ MEMBER FUNCTIONS ===============================

	// ----------------------------------------------------------------------------
	// SmartLink::New
	//
	// 
	// 
	// ----------------------------------------------------------------------------
	SmartLink* SmartLink::New(const String& aItem, const String& aText, const IntRect& aArea, const int& aType,  void* aNode)
	{
		SmartLink* self = new SmartLink(aItem, aText, aArea, aType, aNode);
		self->Construct(aItem, aText);
		return self;
	}
	    
	// ----------------------------------------------------------------------------
	// SmartLink::SmartLink
	//
	// 
	// 
	// ----------------------------------------------------------------------------
	SmartLink::SmartLink(const String& aItem, const String& aText,const IntRect& aArea, const int& aType,  void* aNode ) : iEntryArea(aArea), iEntryType(aType), iNode(aNode)
	{
	}

	// ----------------------------------------------------------------------------
	// SmartLink::Construct
	//
	// 
	// 
	// ----------------------------------------------------------------------------
	void SmartLink::Construct(const String& aItem, const String& aText)
	{
		iEntryText = new String(aText);
		iItem = new String(aItem);
	}
	         
	// ----------------------------------------------------------------------------
	// SmartLink::~CSmartLink
	//
	// 
	// 
	// ----------------------------------------------------------------------------
	SmartLink::~SmartLink()
	{ 
		delete iEntryText;
		delete iItem;
	}

	// ============================ MEMBER FUNCTIONS ===============================  
	  
	// ----------------------------------------------------------------------------
	// SmartLinkMap::InsertLink
	//
	// 
	// 
	// ----------------------------------------------------------------------------
	void SmartLinkMap::InsertLink(Vector<SmartLink*>& aSmartLinkArray, SmartLink* aSmartLink)
	{ 
		//remove duplicates 
		for(int i=0; i< aSmartLinkArray.size(); i++)
		{
			SmartLink* smartLink = aSmartLinkArray[i];
			if (smartLink->iEntryArea.intersects(aSmartLink->iEntryArea))
			{
				//if the y coords start on the same y axis and the heights are equal
				if (smartLink->iEntryArea.topLeftPoint().y() == aSmartLink->iEntryArea.topLeftPoint().y() && smartLink->iEntryArea.height() == aSmartLink->iEntryArea.height())
				{
					//if the width of the old one is less than or equal to the new one replace it
					if (smartLink->iEntryArea.width() <= aSmartLink->iEntryArea.width())
					{               
						aSmartLinkArray.remove(i);
						delete smartLink;
						aSmartLinkArray.insert( i, aSmartLink );
					}
					return;
				}   
			}
		}
		aSmartLinkArray.append(aSmartLink);             
	}  

	// ----------------------------------------------------------------------------
	// SmartLinkMap::IndexingCallback
	//
	// 
	// 
	// ----------------------------------------------------------------------------
	int SmartLinkMap::IndexingCallback( void* aSmartLinkMap)
	{
		BOOL bContinue = FALSE;

		if (aSmartLinkMap != NULL)
		{    
			if( GetEntryCount() > SMARTLINK_MAX_ENTRY ) 
			{
				return bContinue;
			}
			SmartLinkMap* smartLinkMap = static_cast<SmartLinkMap*>(aSmartLinkMap);
			void* listCutLink = SmartLinkUiCreateList();

			const String ptrBuffer(smartLinkMap->iFrame->GetNextRenderTextForSmartLinks(bContinue));   

			if (ptrBuffer.length() > 0)
			{
				//SysDebug(( MID_EXCEPTION, "SmartLink Parsing: Start" ));
				if(SmartLinkUiUtilParseTextByLink( listCutLink, (char*)(ptrBuffer.latin1().data()), 0, SMART_LINK_CUT_LINK_DIRECT ))
				{
					//SysDebug(( MID_EXCEPTION, "SmartLink Parsing: End" ));
					//SysDebug(( MID_EXCEPTION, "SmartLink GetRectArray for %d: Start", ListGetCount( listCutLink ) ));

					int entryParsedCount = SmartLinkUiGetListCount( listCutLink );

					if( (GetEntryCount() + entryParsedCount) >  SMARTLINK_MAX_ENTRY )
							entryParsedCount = SMARTLINK_MAX_ENTRY - GetEntryCount();

					for( int itemIndex=0; itemIndex < entryParsedCount; itemIndex++ )
					{     
						SmartLinkTextCutLinkObj* pTempObj = SmartLinkUiGetCurrentListObject(listCutLink);
						//String pItemToFind(ptrBuffer.substring(pTempObj->cutStartPoint, pTempObj->cutEndPoint ));
				
						String pItemToFind(ptrBuffer.substring(pTempObj->cutStartPoint, ( pTempObj->cutEndPoint - pTempObj->cutStartPoint ) + 1));

						Vector<IntRect> aEntryAreaArray;                
						smartLinkMap->iFrame->getRenderTextRectArray(smartLinkMap->iFrame->iSmartLinkNode, pItemToFind,aEntryAreaArray, 0);      
	                
						for (int arrayIndex=0; arrayIndex<aEntryAreaArray.size(); arrayIndex++)
						{ 
							if( GetEntryCount() > SMARTLINK_MAX_ENTRY ) 
							{
								SmartLinkUiMemFree(listCutLink);
								//SysDebug(( MID_EXCEPTION, "SmartLink GetRectArray: End" ));
								return FALSE;
							}
							SmartLink* smartLink = SmartLink::New(pItemToFind, String(pTempObj->pCutLinkContents,pTempObj->len)/*pItemToFind*/, aEntryAreaArray[arrayIndex], pTempObj->cutLinkType, smartLinkMap->iFrame->iSmartLinkNode);
							InsertLink(smartLinkMap->iSmartLinkArray, smartLink);                               
						}                                     

						SmartLinkUiTraverseNextNode(listCutLink);					
					}
					//SysDebug(( MID_EXCEPTION, "SmartLink GetRectArray: End" ));
				}
				//SysDebug(( MID_EXCEPTION, "SmartLink Parsing end" ));
			}
			SmartLinkUiMemFree(listCutLink);
		}
		return bContinue;
	}
	 
	// ----------------------------------------------------------------------------
	// SmartLinkMap::UpdateSmartLinkMap
	//
	// 
	// 
	// ----------------------------------------------------------------------------
	int SmartLinkMap::UpdateSmartLinkMap(void* aSmartLinkMap)
	{
		SmartLinkMap* smartLinkMap = static_cast<SmartLinkMap*>(aSmartLinkMap);
		
		double endTime = getCurrentTimeInMilliSec() + SMARTLINK_EVENT_TIME ;
		//Vector<IntRect> aEntryAreaArray; 
		while( ( iUpdateIndex < iSmartLinkArray.size() ) && ( getCurrentTimeInMilliSec() < endTime ) && !iStopIndexing )
		{			
			int arrayIndex = 0;
			Vector<IntRect> aEntryAreaArray; 
			SmartLink* textEntry = iSmartLinkArray[iUpdateIndex];

			smartLinkMap->iFrame->getRenderTextRectArray( static_cast<Node*>(textEntry->iNode), *(textEntry->iItem), aEntryAreaArray, 0);      
			
			if( 0 != aEntryAreaArray.size())
			{
				textEntry->iEntryArea = aEntryAreaArray[0];
			
				for ( arrayIndex=1; arrayIndex < aEntryAreaArray.size(); arrayIndex++)
				{ 
					iUpdateIndex++;
					SmartLink* smartLink = SmartLink::New(*(textEntry->iItem), *(textEntry->iEntryText), aEntryAreaArray[arrayIndex], textEntry->iEntryType, textEntry->iNode);
					iSmartLinkArray.insert( iUpdateIndex , smartLink );  
				}
				
				while( ( iSmartLinkArray.size() > (iUpdateIndex + 1) ) && ( iSmartLinkArray[iUpdateIndex + 1 ]->iNode == iSmartLinkArray[iUpdateIndex]->iNode ) 
					   && equalIgnoringCase( *(iSmartLinkArray[iUpdateIndex + 1]->iEntryText), *(iSmartLinkArray[iUpdateIndex]->iEntryText)) )
				{
					SmartLink* smartLink = 	iSmartLinkArray[iUpdateIndex + 1];
					iSmartLinkArray.remove( iUpdateIndex + 1 );
					delete smartLink;
				}
			}
			else
			{
				SmartLink* smartLink = 	iSmartLinkArray[iUpdateIndex];
				iSmartLinkArray.remove( iUpdateIndex );
				delete smartLink;
				iUpdateIndex--;
			}
			iUpdateIndex++;
		}

		if(iStopIndexing)
		{
			iUpdateIndex = 0;
			iStopIndexing = FALSE;
			return FALSE;
		}

		if( iSmartLinkArray.size() == iUpdateIndex )
		{
			iUpdateIndex = 0;
			return FALSE;
		}
		
		return TRUE;
	}

	// ----------------------------------------------------------------------------
	// CSmartLinkMap::NewL
	//
	// 
	// 
	// ----------------------------------------------------------------------------
	SmartLinkMap* SmartLinkMap::New(Frame* aFrame)
	{
		SmartLinkMap* self = new SmartLinkMap(aFrame);
		self->Construct();
		return self;
	}
	    
	    
	// ----------------------------------------------------------------------------
	// SmartLinkMap::SmartLinkMap
	//
	// 
	// 
	// ----------------------------------------------------------------------------
	SmartLinkMap::SmartLinkMap( Frame* aFrame) : iFrame(aFrame), iPhoneEntryIndex(0), iEmailEntryIndex(0)
	{
		iIndexingStatus = FALSE;
		iStopIndexing = FALSE;
		iUpdateIndex = 0;
	}

	// ----------------------------------------------------------------------------
	// SmartLinkMap::~SmartLinkMap
	//
	// 
	// 
	// ----------------------------------------------------------------------------
	SmartLinkMap::~SmartLinkMap()
	{     
		StopIndexingSmartLinks();
		//delete iIndexingCallback; 
	}

	// ----------------------------------------------------------------------------
	// CSmartLinkMap::ConstructL
	//
	// 
	// 
	// ----------------------------------------------------------------------------
	void SmartLinkMap::Construct()
	{
		//iIndexingCallback = NULL;
	}
	                


	// ----------------------------------------------------------------------------
	// SmartLinkMap::GetEntry
	//
	// 
	// 
	// ----------------------------------------------------------------------------
	SmartLink* SmartLinkMap::GetEntry(const IntRect& aRect)
	{ 
		for ( int i=0; i<iSmartLinkArray.size(); i++)
		{
			SmartLink* textEntry = iSmartLinkArray[i];                           
			if (textEntry->iEntryArea.intersects(aRect))
			{
				return iSmartLinkArray[i];        
			}
		}
		return NULL;
	}
	    

	// ----------------------------------------------------------------------------
	// SmartLinkMap::GetNextPhoneEntry
	// Gets the next phone entry and moves the position one forward
	// 
	// 
	// ----------------------------------------------------------------------------
	SmartLink* SmartLinkMap::GetNextPhoneEntry()
	{   
		if (iSmartLinkArray.size() > 0)
		{
			//if you reach the end of the list let other frames have a chance and return null
			if (iPhoneEntryIndex >= 0)
			{
				while (iPhoneEntryIndex<iSmartLinkArray.size())
				{     
					SmartLink* textEntry = iSmartLinkArray[iPhoneEntryIndex];                                    
					if (textEntry->iEntryType == SMARTLINKPHONE)        
					{
						++iPhoneEntryIndex;         
						return textEntry; 
					}
					++iPhoneEntryIndex;
				}
			}
		}
		iPhoneEntryIndex = 0;
		return NULL;  
	}

	// ----------------------------------------------------------------------------
	// SmartLinkMap::GetPrevPhoneEntry
	// Gets the previous email entry and moves the position one backwards
	// 
	// 
	// ----------------------------------------------------------------------------

	SmartLink* SmartLinkMap::GetPrevPhoneEntry()
	{ 
		if (iSmartLinkArray.size() > 0)
		{
			//if you reach the end of the list let other frames have a chance and return null
			if (iPhoneEntryIndex < iSmartLinkArray.size())
			{
				while (iPhoneEntryIndex>=0)
				{     
					SmartLink* textEntry = iSmartLinkArray[iPhoneEntryIndex];                                    
					if (textEntry->iEntryType == SMARTLINKPHONE)        
					{
						--iPhoneEntryIndex;         
						return textEntry; 
					}
					--iPhoneEntryIndex;
				}
			}
			iPhoneEntryIndex = iSmartLinkArray.size()-1;
		}
		return NULL;    
	}


	// ----------------------------------------------------------------------------
	// SmartLinkMap::GetNextEmailEntry
	// Gets the next email entry and moves the position one forward
	// 
	// 
	// ----------------------------------------------------------------------------
	SmartLink* SmartLinkMap::GetNextEmailEntry()
	{ 
		if (iSmartLinkArray.size() > 0)
		{
			//if you reach the end of the list let other frames have a chance and return null
			if (iEmailEntryIndex >= 0 )
			{
				while (iEmailEntryIndex<iSmartLinkArray.size())
				{     
					SmartLink* textEntry = iSmartLinkArray[iEmailEntryIndex];                                    
					if (textEntry->iEntryType == SMARTLINKEMAIL)        
					{
						++iEmailEntryIndex;         
						return textEntry; 
					}
					++iEmailEntryIndex;         
				}
			}
		}
		iEmailEntryIndex = 0;
		return NULL;  
	}

	// ----------------------------------------------------------------------------
	// SmartLinkMap::GetPrevEmailEntry
	// Gets the previous email entry and moves the position one backwards
	// 
	// 
	// ----------------------------------------------------------------------------
	SmartLink* SmartLinkMap::GetPrevEmailEntry()
	{ 
		if (iSmartLinkArray.size() > 0)
		{
       		//if you reach the end of the list let other frames have a chance and return null
			if (iEmailEntryIndex < iSmartLinkArray.size())
			{
				while (iEmailEntryIndex>=0)
				{     
					SmartLink* textEntry = iSmartLinkArray[iEmailEntryIndex];                                    
					if (textEntry->iEntryType == SMARTLINKEMAIL)        
					{
						--iEmailEntryIndex;         
						return textEntry; 
					}
					--iEmailEntryIndex;          
				}
			}
			iEmailEntryIndex = iSmartLinkArray.size()-1;
		}
		return NULL;    
	}

	// ----------------------------------------------------------------------------
	// SmartLinkMap::StartIndexingSmartLinks
	//
	// 
	// 
	// ----------------------------------------------------------------------------
	void SmartLinkMap::StartIndexingSmartLinks()
	{
		if(iIndexingStatus == TRUE)
		{       
			BOOL bStatus = UpdateSmartLinkMap(this);
			if(bStatus == FALSE)
			{
			    return;
			}
		}
		else
		{
			double endTime = getCurrentTimeInMilliSec() + SMARTLINK_EVENT_TIME ;
			while( getCurrentTimeInMilliSec() < endTime && !iStopIndexing)
		{
			BOOL bStatus = IndexingCallback(this);
			if(bStatus == FALSE)
				{
					iIndexingStatus = TRUE;
					return;
				}
			}

			if(iStopIndexing)
			{
				iStopIndexing = FALSE;
				return;
		}
	}

#ifdef MULTIPLE_INSTANCE_SUPPORT
		SFBalSendEvent(BAL_APP_BROWSER,BAL_EVENT_BROWSER_SMARTLINK, NULL, (int)iFrame, NULL, iFrame->getWebControlInstance());
#else
		SFBalSendEvent(BAL_APP_BROWSER,BAL_EVENT_BROWSER_SMARTLINK, NULL, (int)iFrame, NULL);
#endif//MULTIPLE_INSTANCE_SUPPORT

	}

	// ----------------------------------------------------------------------------
	// SmartLinkMap::StopIndexingSmartLinks
	//
	// 
	// 
	// ----------------------------------------------------------------------------
	void SmartLinkMap::StopIndexingSmartLinks()
	{
	  /*if (iIndexingCallback->IsActive())
		{   
		iIndexingCallback->Cancel();  
		}*/
#ifdef MULTIPLE_INSTANCE_SUPPORT		
		SFBalRemoveEventByID(BAL_APP_BROWSER,BAL_EVENT_BROWSER_SMARTLINK, iFrame->getWebControlInstance() );
#else
		SFBalRemoveEventByID(BAL_APP_BROWSER,BAL_EVENT_BROWSER_SMARTLINK, NULL);
#endif
		
		iIndexingStatus = FALSE;
		Vector<SmartLink*>::iterator endIterator = iSmartLinkArray.end();
		Vector<SmartLink*>::iterator beginIterator = iSmartLinkArray.begin();

		while(beginIterator != endIterator )
		{
			SmartLink* aSmartLink = *beginIterator;
			delete aSmartLink;
			++beginIterator;
		}
		iSmartLinkArray.clear();
	}

	// ----------------------------------------------------------------------------
	// SmartLinkMap::ReIndexSmartLinks
	//
	// 
	// 
	// ----------------------------------------------------------------------------
	void SmartLinkMap::ReIndexSmartLinks()
	{ 
		//iStopIndexing = TRUE;
	   /* if (!iIndexingCallback->IsActive())
		{       
		iSmartLinkArray.ResetAndDestroy();                
		iIndexingCallback->Start( TCallBack( &IndexingCallbackL, this ) );    
		} */
	}
#endif
