/*
 *  Copyright (c)  SISO, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics.
 */
/**
 * @file		WebSmartLinkMap.h
 * @brief		implement Smartlink
 * @author	Gnanasekar.s (Gnanasekar.s@samsung.com) 
 * @version	1.0
 * @date		2008.07.18	v1.0		Created			Gnanasekar.S				
 **/

#ifdef SFB_SMARTLINK_FEATURE 

#ifndef __SMARTLINKMAP_H__
#define __SMARTLINKMAP_H__

#include "Frame.h"
#include <wtf/Vector.h>

namespace WebCore {

class SmartLink 
{
  public:
	  static SmartLink* New(const String& aItem, const String& aText, const IntRect& aArea, const int& nType,  void* aNode);
      ~SmartLink();
  
  public:
    String* iItem;
    String* iEntryText;    
    IntRect iEntryArea;
    int	  iEntryType;      
	void* iNode;
    
  private:
        SmartLink(const String& aItem, const String& aText, const IntRect& aArea, const int& nType, void* aNode);
        void Construct(const String& aItem, const String& aText);      
};


class SmartLinkMap
{
public: // constructors and destructors
         static SmartLinkMap* New( Frame* aFrame);
        ~SmartLinkMap();
      
    void StartIndexingSmartLinks();
    void StopIndexingSmartLinks();
    void ReIndexSmartLinks();
    SmartLink* GetNextPhoneEntry();
    SmartLink* GetNextEmailEntry();
    SmartLink* GetPrevPhoneEntry();
    SmartLink* GetPrevEmailEntry();
    SmartLink* GetEntry(const IntRect& aRect);
    int GetEntryCount() { return iSmartLinkArray.size(); }
	void SetStopIndexing(){iStopIndexing = TRUE;};
        
private:    
    int IndexingCallback(void* aWebKitSmartTextMap);
    void InsertLink( Vector<SmartLink*>& aSmartLinkArray, SmartLink* aSmartLink);
	int UpdateSmartLinkMap(void* aWebKitSmartTextMap);
    SmartLinkMap( Frame* iFrame);
    void Construct();
      
private:          
    Frame* iFrame;       
    Vector<SmartLink*> iSmartLinkArray;    
	int iUpdateIndex;
    BOOL iIndexingStatus;    // this is required for update
 
    int iPhoneEntryIndex;
    int iEmailEntryIndex;    
	
	bool iStopIndexing;  // this is required on OOM

   
};

}
#endif//__SMARTLINKMAP_H__
#endif//#if( PLATFORM(SHP) && SFB_SMARTLINK_FEATURE )