/*
* ==============================================================================
*  Name         : VirtualCursor.h
*  Part of      : WEBCORE / Page
*  Description  : Interface of Virtual Cursor
*  Date         : February 29, 2008
*  Author       : Prashant Nevase
* ==============================================================================
*/

#ifndef __VIRTUAL_CURSOR_H__
#define __VIRTUAL_CURSOR_H__

#if PLATFORM(SHP)
#include "CMMDefs.h"
#endif // PLATFORM(SHP)

#include "Navigation.h"

#ifdef SUPPORT_VIRTUAL_CURSOR

namespace WebCore {

class CVCSAdjustment;


class CVirtualCursor : public CNavigation
{
public:
	//STATIC METHODS
	//for static global handling
	static void initVCStatics();

	static void init();
	static void finalize();
	static CVirtualCursor* createInstance();
	static void destroyInstance(CVirtualCursor* virtualCursor);

	//OVERRIDDEN METHODS
	virtual void setFrameView(FrameView* frameView);

	//for new page load in the same window
	virtual void resetPage();
	virtual void pageLoadingFinished();

	virtual void updateVerticalStats();
	virtual void updateHorizontalStats();

	virtual void handleNavigationEvent(int keyCode, bool isKeyUp);

	virtual void getPointerLocation(FrameView** frameView, int* x, int* y);
	virtual void setPointerLocation(int x, int y);

	virtual void scrollBy(int xOffset, int yOffset);

#ifdef SUPPORT_TOUCH_SCREEN
	virtual void touchPressed(IntPoint wndPoint);
	virtual void touchMoved(IntPoint wndPoint);
	virtual void touchReleased(IntPoint wndPoint);
#endif

	virtual void doPostPaintAction();
	virtual void doDefaultAction() {}

	virtual Node* getHoveredNode() { return m_hoveredNode; }

private:
	CVirtualCursor();
	~CVirtualCursor();

	static void drawCursor();
	static void scrollAdjustmentChangedCB(CVCSAdjustment*, void*);

	int getFrameMaxHeight(FrameView* view);
	int getFrameMaxWidth(FrameView* view);

	void smartMoveIFrame();

	//void scrollFrame(FrameView* frameView, ScrollDirection direction, float multiplier = 1.0);

	bool handleScrollUp(int wantToScrollBy, FrameView* view, bool scrollOnly = false);
	bool handleScrollDown(int wantToScrollBy, FrameView* view, bool scrollOnly = false);
	bool handleScrollLeft(int wantToScrollBy, FrameView* view, bool scrollOnly = false);
	bool handleScrollRight(int wantToScrollBy, FrameView* view, bool scrollOnly = false);

	bool setCursorInPage(ENavigationType scrollType, float scrollOffset, bool informWebCore = true);
		
	int getFrameTopHiddenHeight();
	int getFrameBottomHiddenHeight();
	int getFrameLeftHiddenWidth();
	int getFrameRightHiddenWidth();

	void setVerticalVisibleLimits();
	void setHorizontalVisibleLimits();

	int getFrameVisibleHeight();
	int getFrameVisibleWidth();

	void calculateVerticalLoc();
	void calculateHorizontalLoc();

#ifdef SMART_FOCUSING
	bool isFocusableNodeInJumpRange(ENavigationType scrollType, float& scrollOffset);
	bool isNextFocusableNodeFound(IntPoint documentPoint, float& scrollOffset);
#endif
	
	//horizontal and vertical scroll adjustments
	CVCSAdjustment* m_hAdj;
    CVCSAdjustment* m_vAdj;

	bool m_updatePointerX;
	bool m_updatePointerY;

	
	//frame left-top
	int m_frameTopX;
    int m_frameTopY;

	//min threshold for frameview size
	int m_minFrameViewHeight;
	int m_minFrameViewWidth;

	float m_scrollBy;

	//last cursor rect area
	IntRect m_lastCursorRectArea;

	//define LOC for cursor movement
	int m_topLoc;
	int m_bottomLoc;
	int m_leftLoc;
	int m_rightLoc;

	int m_frameMaxHeight;
	int m_frameMaxWidth;
};

} //namespace WebCore

#endif //SUPPORT_VIRTUAL_CURSOR - Compile only if this flag is enabled

#endif //__VIRTUAL_CURSOR_H__