/*
 * Copyright (C) 2004, 2005, 2006 Apple Computer, Inc.  All rights reserved.
 * Copyright (C) 2007 Alp Toker <alp@atoker.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "ShpJasmine.h"
#include "BitmapImage.h"

#include "NotImplemented.h"
#include "AffineTransform.h"
#include "FloatRect.h"
#include "GraphicsContext.h"

#include "GWES.h"
#include "RsrcMgr.h"

#include "ImageDecoder.h"
#include "OtherImageDecoder.h"


#ifdef __CMM__
#include <allocs.h>
#endif

#include "FileSystem.h"
#include "CString.h"

#define MAX_IMAGE_TILES 1000

#define RESOURCE_IMAGE_PATH     "/User/Br/"
#define MISSING_IMAGE_FILE      "missingImage.gif"
#define LOADING_IMAGE_FILE      "loadingimage.gif"
#define MAX_RESOURCE_SIZE       (1024U)

namespace WebCore {

#ifdef SFB_SUPPORT_IMAGE_ZOOM
    extern float g_zoomMultiplier;
#endif

void FrameData::clear()
{
    if (m_frame) {
        m_frame = 0;
        m_duration = 0.;
        m_hasAlpha = true;
    }

    notImplemented();
}

// ================================================
// Image Class
// ================================================

Image* Image::loadPlatformResource(const char *name)
{
    Image* img = new BitmapImage();
    PlatformFileHandle handle = invalidPlatformFileHandle;
    if(0 == AcStrcmp(name, "missingImage"))
    {
        openFile(RESOURCE_IMAGE_PATH, MISSING_IMAGE_FILE, FILE_MODE_READ, handle);
    }
    if(invalidPlatformFileHandle != handle)
    {
        char imgData[MAX_RESOURCE_SIZE];
        unsigned imgSize = 0;
        readFile(handle, imgData, MAX_RESOURCE_SIZE, &imgSize);
        RefPtr<SharedBuffer> buffer = new SharedBuffer(imgData, imgSize);
        img->setData(buffer, true);
        closeFile(handle);
    }
    return img;
}

void BitmapImage::initPlatformData()
{

}

void BitmapImage::invalidatePlatformData()
{
}

//Checking available memory is greater than allocated to draw scaled bitmap - Hema
static BOOL
CheckBufferSizeForBitmap(int height, int width)
{
	int availableSize=0, maxsizeAllocated=0 , datasize=0;

	//To calculate size of the image
	datasize=height*width*2;

	if (CMM_CURRENTLY_ALLOCATED_HEAP_SIZE>=(MAX_MEM_USAGE*CMM_TOTAL_HEAP_SIZE))
		return FALSE;

	//to check if the image size is not greater than maximage size which is 4MB
	if (datasize>MAX_IMAGE_SIZE)
		return FALSE;
	availableSize=SFBalMemGetStatEx();
		maxsizeAllocated=((MAX_ALLOWED_IMG_SIZE*availableSize)/100);

	if(maxsizeAllocated>datasize)
			return TRUE;
		else
			return FALSE;
	
}

// Drawing Routines

static BOOL
Jasmine_WmDrawTransparentBitmap( HGC hGC, 
								IntRect DestRect,
								IntRect BitmapRect,
								ImgCOLOR color, 
								WmBitmap* pbmBitmap, 
								USHORT* pAlpha)
{

	COLOR* pGCBuffer = NULL;

	if( DestRect.width() < 1 || DestRect.height() < 1 )
		return FALSE;

	if(!WmGetGCFrmbuf(  hGC,  &pGCBuffer ))
		return FALSE;	
	WmGC* pGC = WmGetGCPointer(  hGC );

	if(!pGCBuffer || !pGC)
		return FALSE;
	
	if( pbmBitmap->bitsPerPixel == 16)
	{
		unsigned int alpha = 0;
		unsigned int nr = 0, ng = 0, nb = 0, bR = 0, bB = 0, bG = 0 ;
		unsigned char r = 0, g = 0, b = 0;
		unsigned short *pBitmap = NULL, *pDestGC = NULL,*pTempAlpha = NULL ;
		 
		bR = color.R ;
		bG = color.G ;
		bB = color.B ;

		pBitmap = (unsigned short*)pbmBitmap->pBits;
		pDestGC = (unsigned short*)pGCBuffer;
		pTempAlpha = (unsigned short*)pGCBuffer;

		int startX = (BitmapRect.x() >= 0) ? 0 : -BitmapRect.x(); 
		int startY = (BitmapRect.y() >= 0) ? 0 : -BitmapRect.y(); 

		int pixelIndexX = (startY * BitmapRect.width()) + startX;

		// TODO : the coordinates are relative to the top left of the screen 
		//  not the top left of the GC rect 
		int GCpixelIndexX = ((DestRect.y() + pGC->rcGC.y)* pGC->rcFrmbuf.w) + (DestRect.x() + pGC->rcGC.x);

		for (int i = 0; i < DestRect.height(); i++ )
		{
			for (int j = 0; j < DestRect.width(); j++)
			{
				alpha = pAlpha [pixelIndexX + j] ;
				if( alpha > 0 && alpha < 255 )
				{
					if (pGCBuffer != NULL)
					{
						bR = (((pTempAlpha[GCpixelIndexX + j] & 0xF800) >> 8));
						bG = (((pTempAlpha[GCpixelIndexX + j] & 0x07C0) >> 3));
						bB = (((pTempAlpha[GCpixelIndexX + j] & 0x001F) << 3));
					}
									
					r = (((pBitmap[pixelIndexX + j] & 0xF800) >> 8));
					g = (((pBitmap[pixelIndexX + j] & 0x07C0) >> 3));
					b = (((pBitmap[pixelIndexX + j] & 0x001F) << 3));
	 
					/* Do Alpha blending */
					/**********************************************************************************
					/*     AlphaValue    Foreground    (MaxPixelvalue - alpha)     Background         *
					 *                   Color                                     Color              *
					 *                                                             eg: 255, 0, 0 for  *
					 *                                                                 red background */
					nr = ((alpha       * r )) + (255 - alpha) * bR ; // background r
					ng = ((alpha       * g )) + (255 - alpha) * bG ; // background g
					nb = ((alpha       * b )) + (255 - alpha) * bB ; // background b
					/**************************************************************************/

					r =  (nr >> 8) ;
					g =  (ng >> 8) ;
					b =  (nb >> 8) ;
			
					pDestGC[GCpixelIndexX + j] = ( ((r & 0xF8) << 8) | (((g & 0xF8) << 3)) | (((b & 0xF8) >> 3))) ;	
				}
				else 
				{
					if( alpha == 0 )
					{
						continue;
					}
					else
					{
						pDestGC[GCpixelIndexX + j] = pBitmap[pixelIndexX + j];
						continue;
					}
				}
			}
			pixelIndexX = pixelIndexX + BitmapRect.width();
			GCpixelIndexX = GCpixelIndexX + pGC->rcFrmbuf.w;
		}
	}	
	return TRUE;

}


void
BitmapImage::DrawPNG(GraphicsContext* context, 
					 WmBitmap* pbitmap, 
					 unsigned short *alphachannel, 
					 const FloatRect& dst, 
					 const FloatRect& src, 
					 BOOL ScaledBitmap)
{
    PlatformGraphicsContext* hGC = context->platformContext();
	FloatRect srcRect(src);
    FloatRect dstRect(dst);	
	int rectX = context->translateX(dstRect.x() - srcRect.x());
	int rectY = context->translateY(dstRect.y() - srcRect.y());
	if(alphachannel)
	{
		
		if(*(context->getWindowHandle()))
		{
			IntRect BitmapRect(rectX, rectY, pbitmap->rectW, pbitmap->rectH);
			WmRect	rectClient;
			WmGetWindowClientRect( *(context->getWindowHandle()), &rectClient );
			//TODO : read the scroll bar width through some function
			IntRect GCRect(0, 0, (rectClient.w - 7), (rectClient.h - 7));
			// GCRect has the intersection coordinates	
			GCRect.intersect(BitmapRect);
			ImgCOLOR color;
			color.R = 0x00;
			color.G = 0x00;
			color.B = 0x00;
				
			Jasmine_WmDrawTransparentBitmap(*hGC, 
											GCRect, 
											BitmapRect, 
											color, 
											pbitmap, 
											alphachannel);
		}

	}
	else
	{
		// If ScaledBitmap is true, then we should use the "WmDrawScaledBitmap" API
		// to draw the scaled image.
		if(ScaledBitmap)
		{
			WmDrawScaledBitmap( *hGC, 
									context->translateX(dstRect.x()), 
									context->translateY(dstRect.y()),
									dstRect.width(),
									dstRect.height(), 
									pbitmap );
		}
		else
		{
			WmDrawBitmap( *hGC, rectX, rectY, pbitmap );	
		}
	}		
}

void 
BitmapImage::draw(GraphicsContext* context, 
					   const FloatRect& ori_dst, 
					   const FloatRect& ori_src, 
					   CompositeOperator op)
{	
	PlatformGraphicsContext* hGC = context->platformContext();
	WmGC* pGC = WmGetGCPointer(  *hGC );

	if(pGC->gcAttrib.bitsPerPixel == 32)
	{
		SFBalDebug(MID_EXCEPTION,"BitmapImage::draw entered\n");
		draw32(context,ori_dst,ori_src,op);
		SFBalDebug(MID_EXCEPTION,"BitmapImage::draw done\n");
		return;
	}
	FloatRect dst = ori_dst;
	FloatRect src = ori_src;

	/*Checking for nrgative values of x andy coordinates for drawing PNG images*/
	

#ifdef SFB_GWES_TRANSFORM
	dst = context->getCTM().mapRect( dst );	
	//src = context->getCTM().mapRect( src );	
#endif
   
	ImgCodecType codecType;
	unsigned short *alphachannel = NULL;
	WmBitmap* pSrcBitmap = NULL;

	if(0 >= dst.width() || 0 >= dst.height())
    {
        return;
    }
	
    if (!m_source.initialized())
    {
#ifdef SFB_PROGRESSIVE_RENDERING
        showPlaceholderImage(context, dst);
#endif // SFB_PROGRESSIVE_RENDERING
        return;
    }

    NativeImagePtr image = frameAtIndex(m_currentFrame);
    if (!image || 0 == image->bytes() ) // If it's too early we won't have an image yet.
    {
#ifdef SFB_PROGRESSIVE_RENDERING
        showPlaceholderImage(context, dst);
#endif // SFB_PROGRESSIVE_RENDERING
        return;
    }

	WmBitmap bitmap;
	
    // Create WmBitmap from the decoded image data
    bitmap.compressType		= COMPRESS_NONE;
    bitmap.rectW			= size().width();
    bitmap.rectH			= size().height();
    bitmap.bitsPerPixel		= image->getBitsPerPixel();
    bitmap.bUseChromakey	= image->useChromakey();
    bitmap.colorChromakey	= image->getChromakey();
    bitmap.pBits			= image->bytes();
	bitmap.bUseReplacement  = FALSE;
    alphachannel			= image->getAlphachannel();
    codecType				= image->getCodecType();

    FloatRect srcRect(src);
    FloatRect dstRect(dst);
#if defined(_SFB_BACKINGSTORE)
	IntRect rect(dstRect);
	m_destinationRect = rect;
#endif
    float scaleX = srcRect.width() / dstRect.width();
    float scaleY = srcRect.height() / dstRect.height();

    // Set the compositing operation.
    if (op == CompositeSourceOver && !frameHasAlphaAtIndex(m_currentFrame))
        context->setCompositeOperation(CompositeCopy);
    else
        context->setCompositeOperation(op);

	
			
	if(scaleX != 1.0 || scaleY != 1.0)
	{
		//To check if sufficient memory is vailable to draw the image: Hema
		if(CheckBufferSizeForBitmap(dstRect.height(),dstRect.width()))
		{
			if((codecType == IMG_DEC_PNG) && (alphachannel != NULL)) 
			{
				if(scaleX > 1.0 || scaleY > 1.0)
				{
					if( !m_decodeOnce)
					{
						
					
							ImgDecodeImageScaled(IMG_DEC_PNG, (unsigned char*)m_data->data(), 
											 dstRect.width(), dstRect.height(),
			 								 m_image, m_data->size(), 
											 IMG_RGB_OUTPUT, IMG_RESIZE);
					
					
						m_pScaledBitmap->compressType = COMPRESS_NONE;
						m_pScaledBitmap->bUseChromakey = m_image->pHead->pbmBitmap->bChromaKeyFlag;
						m_pScaledBitmap->colorChromakey = m_image->pHead->pbmBitmap->chromaKey;
						m_pScaledBitmap->bitsPerPixel = m_image->pHead->pbmBitmap->bitsPerPixel;
						m_pScaledBitmap->rectH = m_image->pHead->pbmBitmap->height;
						m_pScaledBitmap->rectW = m_image->pHead->pbmBitmap->width;
						m_pScaledBitmap->pBits = m_image->pHead->pbmBitmap->pBits;
						alphachannel = (unsigned short*)m_image->pHead->pbmBitmap->pAlphaChannel;
						m_decodeOnce = true;
					}

					//DrawPNG(context, m_pScaledBitmap, (unsigned short*)m_image->pHead->pbmBitmap->pAlphaChannel, dstRect, srcRect, TRUE);
					
						WmDrawTransparentBitmapEx(*hGC,context->translateX(dstRect.x()), 
											context->translateY(dstRect.y()),m_pScaledBitmap,(unsigned short*)m_image->pHead->pbmBitmap->pAlphaChannel);
				}
				else
				{
					// If the value of scaleX and scaleY are not equal to 1
					// then, the iamge requred scaling (either upscaling or down scaling).
					// so we should pass the 6th parameter to DrawPNG( i.e ScaledBitmap) as TRUE.
					//DrawPNG(context, &bitmap, alphachannel, dst, src, TRUE);
				
					WmDrawTransparentBitmapEx(*hGC,context->translateX(dstRect.x()), 
											context->translateY(dstRect.y()),&bitmap,alphachannel);
				}
			}
			else
			{
				//Scaled Image is drawn only if sufficient memory is available
				WmDrawScaledBitmap( *hGC, 
									context->translateX(dstRect.x()), 
									context->translateY(dstRect.y()),
									dstRect.width(),
									dstRect.height(), 
									&bitmap );
			}

		}
	}
	else
	{
		WmRect newClipRect;
		WmRect bitmapRect;
		WmRect initialClipRect;

		// New Clip Rect
		newClipRect.x = context->translateX(dstRect.x());
		newClipRect.y = context->translateY(dstRect.y());
		newClipRect.w = dstRect.width();
		newClipRect.h = dstRect.height();

		// Bitmap Rect
		bitmapRect.h = bitmap.rectH;
		bitmapRect.w = bitmap.rectW;
		bitmapRect.x = context->translateX(dstRect.x() - srcRect.x());
		bitmapRect.y = context->translateY(dstRect.y() - srcRect.y());

		// If New Clip Rect and Bitmap Rect are same don't clip
		BOOL needClip = !(  (newClipRect.h == bitmapRect.h) &&
							(newClipRect.w == bitmapRect.w) &&
							(newClipRect.x == bitmapRect.x) &&
							(newClipRect.y == bitmapRect.y)
						  );
		WmClip clipRes;
		if(needClip)
		{
			//store the current clip rect
			WmGetClipRect(*hGC, &initialClipRect);

			// shkim modified start
			// intersect with original clipping area
			WebCore::IntRect originalclip( initialClipRect.x, initialClipRect.y, initialClipRect.w, initialClipRect.h );
			WebCore::IntRect newclip( context->translateX(dstRect.x()), context->translateY(dstRect.y()), dstRect.width(), dstRect.height() );
			newclip.intersect( originalclip );

			//Added for CQ 796,733,1040
			clipRes = WmSetClipRect(*hGC, 
									newclip.x(), 
									newclip.y(), 
									newclip.width(), 
									newclip.height());
                         // shkim modified end
			if(clipRes == CLIP_REJECT)
			{
				WmSetClipRect(*hGC, 
							  initialClipRect.x, 
							  initialClipRect.y, 
							  initialClipRect.w, 
							  initialClipRect.h);
				return;
			}
		}
		if((codecType == IMG_DEC_PNG) && (alphachannel != NULL))
		{
			//DrawPNG(context, &bitmap, alphachannel, dst, src, FALSE);
					
			WmDrawTransparentBitmapEx(*hGC,context->translateX(dstRect.x()), 
											context->translateY(dstRect.y()),&bitmap,alphachannel);
		}
		else
		{
			WmDrawBitmap( *hGC, 
						  context->translateX(dstRect.x() - srcRect.x()), 
						  context->translateY(dstRect.y() - srcRect.y()), 
						  &bitmap );
		}

		if(needClip)
		{
			//restore the save cliped rect
			WmSetClipRect(*hGC, 
						  initialClipRect.x, 
						  initialClipRect.y, 
						  initialClipRect.w, 
						  initialClipRect.h);
		}
	}
	
    startAnimation();
}


void 
BitmapImage::draw32(GraphicsContext* context, 
					   const FloatRect& ori_dst, 
					   const FloatRect& ori_src, 
					   CompositeOperator op)
{	
	BalFile hFile = INVALID_HOBJ;
	BalFile hFile1 = INVALID_HOBJ;
	SFBalDebug(MID_EXCEPTION,"BitmapImage::draw32 entered\n");
	FloatRect dst = ori_dst;
	FloatRect src = ori_src;

	/*Checking for nrgative values of x andy coordinates for drawing PNG images*/
	

#ifdef SFB_GWES_TRANSFORM
	dst = context->getCTM().mapRect( dst );	
	//src = context->getCTM().mapRect( src );	
#endif

    PlatformGraphicsContext* hGC = context->platformContext();

	ImgCodecType codecType;
	unsigned short *alphachannel = NULL;
	WmBitmap* pSrcBitmap = NULL;

	if(0 >= dst.width() || 0 >= dst.height())
    {
        return;
    }
	
    if (!m_source.initialized())
    {
#ifdef SFB_PROGRESSIVE_RENDERING
        showPlaceholderImage(context, dst);
#endif // SFB_PROGRESSIVE_RENDERING
        return;
    }

    NativeImagePtr image = frameAtIndex(m_currentFrame);
    if (!image || 0 == image->bytes() ) // If it's too early we won't have an image yet.
    {
#ifdef SFB_PROGRESSIVE_RENDERING
        showPlaceholderImage(context, dst);
#endif // SFB_PROGRESSIVE_RENDERING
        return;
    }
	FloatRect srcRect(src);
    FloatRect dstRect(dst);
#if defined(_SFB_BACKINGSTORE)
	IntRect rect(dstRect);
	m_destinationRect = rect;
#endif
    float scaleX = srcRect.width() / dstRect.width();
    float scaleY = srcRect.height() / dstRect.height();
	codecType				= image->getCodecType();
	alphachannel			= image->getAlphachannel();

	WmGC* pGC = WmGetGCPointer(  *hGC );
	WmBitmap bitmap;
		
	if(pGC->gcAttrib.bitsPerPixel == 32 &&(scaleX == 1.0 || scaleY == 1.0))
	{
		if(!m_decodeOnce)
		{
			if( FALSE == ImgDecodeImageScaled(codecType, (unsigned char*)m_data->data(), 
								 1000, 1000,
								 m_image, m_data->size(), 
								 IMG_BGRA_8888_OUTPUT_AND_ROTATE, IMG_RESIZE) )
			{
				SFBalDebug(MID_EXCEPTION,"BitmapImage::draw32 ImgDecodeImageScaled returned FALSE \n");
				if(m_image == 0 || m_image->pHead == 0 || m_image->pHead->pbmBitmap == 0 )
					SFBalDebug(MID_EXCEPTION,"BitmapImage::draw32 ImgDecodeImageScaled m_image is NULL \n");
				return;
			}
			SFBalDebug(MID_EXCEPTION,"BitmapImage::draw32 image decodede \n");
			m_decodeOnce = true;
			m_decodeScaledOnce = false;

		}
		bitmap.compressType			= COMPRESS_NONE;
		bitmap.rectW				= m_image->pHead->pbmBitmap->width;
		bitmap.rectH				= m_image->pHead->pbmBitmap->height;
		bitmap.bitsPerPixel			= m_image->pHead->pbmBitmap->bitsPerPixel;
		bitmap.bUseChromakey		= m_image->pHead->pbmBitmap->bChromaKeyFlag;
		bitmap.colorChromakey		= m_image->pHead->pbmBitmap->chromaKey;
		bitmap.pBits				= m_image->pHead->pbmBitmap->pBits;
		bitmap.bUseReplacement		= FALSE;
		alphachannel				= (unsigned short*)m_image->pHead->pbmBitmap->pAlphaChannel;
		//For debugging Please merge this 
		/*hFile = SFBalCreateFile("/User/Br/SFB/decodedImage.raw",FM_WRITE | FM_TRUNCATE) ;
		SFBalWriteFile(hFile,bitmap.pBits,bitmap.rectW*bitmap.rectH*4);
		SFBalCloseFile(hFile);*/
	}
	
    // Set the compositing operation.
    if (op == CompositeSourceOver && !frameHasAlphaAtIndex(m_currentFrame))
        context->setCompositeOperation(CompositeCopy);
    else
        context->setCompositeOperation(op);
			
	if(scaleX != 1.0 || scaleY != 1.0)
	{
		//To check if sufficient memory is vailable to draw the image: Hema
		if(CheckBufferSizeForBitmap(dstRect.height(),dstRect.width()))
		{
			if( !m_decodeScaledOnce)
			{
								
				ImgDecodeImageScaled(codecType, (unsigned char*)m_data->data(), 
									 dstRect.width(), dstRect.height(),
	 								 m_image, m_data->size(), 
									 IMG_BGRA_8888_OUTPUT_AND_ROTATE, IMG_RESIZE);
				SFBalDebug(MID_EXCEPTION,"BitmapImage::draw32 scaled image decoded\n");
				m_decodeOnce = false;
				m_decodeScaledOnce = true;
			}
			if((m_image->pHead->pbmBitmap->height != dstRect.height()) ||(m_image->pHead->pbmBitmap->width != dstRect.width()) )			
			{
							
				ImgDecodeImageScaled(codecType, (unsigned char*)m_data->data(), 
									 dstRect.width(), dstRect.height(),
	 								 m_image, m_data->size(), 
									 IMG_BGRA_8888_OUTPUT_AND_ROTATE, IMG_RESIZE);
				SFBalDebug(MID_EXCEPTION,"BitmapImage::draw32 scaled image decoded\n");
				
			}
			bitmap.compressType = COMPRESS_NONE;
			bitmap.bUseChromakey = m_image->pHead->pbmBitmap->bChromaKeyFlag;
			bitmap.colorChromakey = m_image->pHead->pbmBitmap->chromaKey;
			bitmap.bitsPerPixel = m_image->pHead->pbmBitmap->bitsPerPixel;
			bitmap.rectH = m_image->pHead->pbmBitmap->height;
			bitmap.rectW = m_image->pHead->pbmBitmap->width;
			bitmap.pBits = m_image->pHead->pbmBitmap->pBits;
			alphachannel = (unsigned short*)m_image->pHead->pbmBitmap->pAlphaChannel;

			//For debugging Please merge this as commeted
			/* hFile1 = SFBalCreateFile("/User/Br/SFB/scaledImage.raw",FM_WRITE | FM_TRUNCATE) ;
			SFBalWriteFile(hFile1,bitmap.pBits,bitmap.rectW*bitmap.rectH*4);
			SFBalCloseFile(hFile1);*/
					
		}
		if(codecType == IMG_DEC_PNG && alphachannel != NULL)
		{

		//DrawPNG(context, m_pScaledBitmap, (unsigned short*)m_image->pHead->pbmBitmap->pAlphaChannel, dstRect, srcRect, TRUE);
		if((dstRect.width() > srcRect.width()) || dstRect.height() > srcRect.height() )
			WmDrawScaledBitmap( *hGC, 
									context->translateX(dstRect.x()), 
									context->translateY(dstRect.y()),
									dstRect.width(),
									dstRect.height(), 
									&bitmap );
		else
			WmDrawTransparentBitmapEx(*hGC,context->translateX(dstRect.x()), 
								context->translateY(dstRect.y()),&bitmap,(unsigned short*)m_image->pHead->pbmBitmap->pAlphaChannel);
			SFBalDebug(MID_EXCEPTION,"BitmapImage::draw32 scaled image draw m_decodeOnce= %d\n",m_decodeOnce);		
		}
		else
		{
			if ( m_image->pHead->pbmBitmap->bitsPerPixel == 16 )   // nykang modified to fix some scroll problem 
			{
				//Scaled Image is drawn only if sufficient memory is available
				WmDrawScaledBitmap( *hGC, 
									context->translateX(dstRect.x()), 
									context->translateY(dstRect.y()),
									dstRect.width(),
									dstRect.height(), 
									&bitmap );
			}
			else
			{
				WmDrawTransparentBitmapEx(*hGC,context->translateX(dstRect.x()), 
						context->translateY(dstRect.y()),&bitmap,(unsigned short*)m_image->pHead->pbmBitmap->pAlphaChannel);
			}
			SFBalDebug(MID_EXCEPTION,"BitmapImage::draw32 scaled image draw m_decodeOnce = %d\n",m_decodeOnce);
		}

		
	}
	else
	{		
		if((codecType == IMG_DEC_PNG) && (alphachannel != NULL))
		{
			//DrawPNG(context, &bitmap, alphachannel, dst, src, FALSE);
					
			WmDrawTransparentBitmapEx(*hGC,context->translateX(dstRect.x()), 
											context->translateY(dstRect.y()),&bitmap,alphachannel);
		}
		else
		{
			WmDrawBitmap( *hGC, 
						  context->translateX(dstRect.x() - srcRect.x()), 
						  context->translateY(dstRect.y() - srcRect.y()), 
						  &bitmap );
			SFBalDebug(MID_EXCEPTION,"BitmapImage::draw32 image draw m_decodeOnce = %d\n",m_decodeOnce);
		}
	}

	startAnimation();
}



void Image::drawPattern(GraphicsContext* context, const FloatRect& tileRect, const AffineTransform& patternTransform,
                        const FloatPoint& phase, CompositeOperator op, const FloatRect& destRect)
{
    notImplemented();
}

#define SFB_RSRC_FILENAME "SFBrowserRsrc.rbm"
#include "SFBrowserRsrc.h"

// Priyanka this function can be used to load SHP resources
bool 
Image::LoadAndPaintPlatformResource(const char *name, 
									GraphicsContext* context, 
									const IntRect& rect)
{
    int rsrcId;
	WmBitmap* pBitmap = NULL;

	// temporary hack to get resource id

	if (!strcmp(name, "AudioPluginIcon"))
        rsrcId = IDB_SFB_SOUNDICON;
    else if (!strcmp(name, "AudioPluginPaused"))
        rsrcId = IDB_SFB_PAUSEICON;
    else if (!strcmp(name, "AudioPluginMuted"))
        rsrcId = IDB_SFB_MUTEICON;
    else if (!strcmp(name, "missingPlugin"))
			rsrcId = IDB_MISSING_FLASH; 
    else if (!strcmp(name, "FlashPluginIcon"))
        rsrcId = IDB_SFB_FLASHICON;
    else
        return 0;
	//Find Resource API
	//Load Resource API

	pBitmap = RsrcLoadBitmap( SFB_RSRC_FILENAME, rsrcId );
	if (!pBitmap)
        return 0;
	//Lock Resource API
	// scale the rect
#ifdef SFB_GWES_TRANSFORM
	IntRect dst = context->getCTM().mapRect(rect);
#else
	IntRect dst(rect);
#endif
#if 0
	WmDrawTransparentFilledRect (*(context->platformContext()), 
								SILVER,
								10,
								context->translateX(dst.x()), 
								context->translateY(dst.y()), 
								dst.width(), 
								dst.height());
#endif
	if (pBitmap)
	{
#ifdef SFB_GWES_TRANSFORM
		WmDrawScaledBitmap( *(context->platformContext()), 
						context->translateX( dst.x() + (dst.width()/2) - (pBitmap->rectW * context->getCTM().a()/2) ), 
						context->translateY( dst.y() + (dst.height()/2) - (pBitmap->rectH * context->getCTM().a()/2) ),
						pBitmap->rectW * context->getCTM().a(),
						pBitmap->rectH * context->getCTM().a(), 
					 pBitmap );
#else // below code is not tested
		WmDrawScaledBitmap( *(context->platformContext()), 
						context->translateX( dst.x() + (dst.width()/2)), 
						context->translateY( dst.y() + (dst.height()/2)),
						pBitmap->rectW ,
						pBitmap->rectH , 
					 pBitmap );

#endif
		__WsFreeBitmap(pBitmap);
	}

    return 1;
}

void BitmapImage::DrawPngTiles(const FloatRect& destRect, const FloatRect& tileRect,GraphicsContext* context, WmBitmap* subBitmapPng,unsigned short *alphachannel, WmBitmap& bitmap, int xPhaseVal, int yPhaseVal,const FloatPoint& phase)
{
	int xx= destRect.x();
	int yy = destRect.y();
	//if (xx < 0)
	//	xx = 0;
	//if(yy < 0 )
	//	yy = 0;
	PlatformGraphicsContext* hGC = context->platformContext();
	if(alphachannel!=NULL  )
	{
		if(subBitmapPng!=NULL)
    {
			for(int x = destRect.x(); x < ((int)(destRect.x()) + (int)(destRect.width()) - (int)(xPhaseVal) ); x += subBitmapPng->rectW )
        {
				for(int y = destRect.y();y < ((int)(destRect.y()) + ((int)destRect.height()) - ((int)yPhaseVal)); y += subBitmapPng->rectH )
				{
			//		if(xx >= 0 && yy >= 0)
					WmDrawTransparentBitmapEx(*hGC, context->translateX(x), 
											context->translateY(y), subBitmapPng,alphachannel);
			
				}
	}
}
		else if((destRect.height() == tileRect.height()  &&  destRect.width() == tileRect.width()) && (tileRect.height() == bitmap.rectH &&  tileRect.width()== bitmap.rectW))
		{
			for(int x = destRect.x(); x < ((int)(destRect.x()) + (int)(destRect.width()) - (int)(xPhaseVal) ); x += bitmap.rectW )
        {
				for(int y = destRect.y(); y < ((int)(destRect.y()) + ((int)destRect.height()) - ((int)yPhaseVal)); y += bitmap.rectH )
				{
			//if(xx >= 0 && yy >= 0)
			WmDrawTransparentBitmapEx(*hGC, context->translateX(x), 
											context->translateY(y), &bitmap,alphachannel);
		}
			}
		}
		else
		{
#ifdef _PATTERN_OPTIMIZATION
			drawPlatformBitmapTiles(context, destRect, tileRect, phase, xPhaseVal, yPhaseVal, &bitmap, false);
#else
			DrawSubBitmapTiles(destRect, phase, context, bitmap,xPhaseVal, yPhaseVal);
#endif
		}
	}
	else
	{
#ifdef _PATTERN_OPTIMIZATION
		drawPlatformBitmapTiles(context, destRect, tileRect, phase, xPhaseVal, yPhaseVal, &bitmap, false);
#else
		DrawSubBitmapTiles(destRect, phase, context, bitmap,xPhaseVal, yPhaseVal);
#endif
	}
}

#ifdef _PATTERN_OPTIMIZATION
void BitmapImage::drawPlatformBitmapTiles(GraphicsContext* context, const FloatRect& destRect, const FloatRect& tileRect, const FloatPoint& phase, int xPhaseVal, int yPhaseVal, WmBitmap* bitmap, bool isScaled)
{
	PlatformGraphicsContext* hGC = context->platformContext();
	
	//pattern rect
	register float dx = destRect.x();
	register float dy = destRect.y();
	register float dw = destRect.width();
	register float dh = destRect.height();
	
	register float cx = destRect.x();
	register float cy = destRect.y();
	register float cw;
	register float ch;

	if(isScaled)
	{
		cw = tileRect.width();
		ch = tileRect.height();
	}
	else
	{
		cw = bitmap->rectW;
		ch = bitmap->rectH;
	}
	
	if(((dx - phase.x()) >= 0) && ((dy - phase.y()) >= 0))
	{
		bool fc = true;
		bool ft = true;

		for(; cx < (dx + dw - xPhaseVal); )
		{
			if(fc)
			{
				for(; cy < (dy + dh - yPhaseVal); )
				{
					if(ft)
					{
						if(isScaled)
							WmDrawScaledBitmap(*hGC, context->translateX(cx), context->translateY(cy), dw, dh, bitmap);
						else
							WmDrawSubBitmap(*hGC, context->translateX(cx), context->translateY(cy), bitmap, dx - phase.x(), dy - phase.y(), cw, ch);

						ft = false;
					}
					else
					{
						if(int(cw) > 0 && int(ch) > 0)
						{
							WmCopyGC(*hGC, context->translateX(cx), context->translateY(cy),
								*hGC, context->translateX(dx), context->translateY(dy), cw, ch);
						}

						ch *= 2;
					}
					
					cy = dy + ch;
				}

				fc = false;
				cy = dy;
			}
			else
			{
				if(int(cw) > 0 && int(ch) > 0)
				{
					WmCopyGC(*hGC, context->translateX(cx), context->translateY(cy),
						*hGC, context->translateX(dx), context->translateY(dy), cw, ch);
				}

				cw *= 2;
			}

			cx = dx + cw;
		}
	}
}



#else //_PATTERN_OPTIMIZATION

void BitmapImage::DrawScaledSubBitmapTiles(GraphicsContext* context, const FloatRect& destRect, const FloatRect& tileRect, WmBitmap* subBitmap, const WmBitmap& bitmap, int xPhaseVal, int yPhaseVal)
{
	PlatformGraphicsContext* hGC = context->platformContext();
	for(int x = destRect.x(); (int)(x) < ((int)(destRect.x()) + (int)(destRect.width()) - (int)(xPhaseVal) ); x += tileRect.width() )
    {
		for(int y = destRect.y(); (int)(y) < ((int)(destRect.y()) + ((int)destRect.height()) - ((int)yPhaseVal)); y += tileRect.height() )
        {
			//checking for the positive value of context->translateX(x) and context->translateY(y) and not x and y(CQ BUG 2398) (modified for CQ3807)
			if(((context->translateX(x))>=0)&&((context->translateY(y))>=0))
			WmDrawScaledBitmap(*hGC, context->translateX(x), context->translateY(y), destRect.width(), destRect.height(), subBitmap);
		}
	}
}


void BitmapImage::DrawSubBitmapTiles(const FloatRect& destRect,const FloatPoint& phase, GraphicsContext* context, WmBitmap& bitmap, int xPhaseVal, int yPhaseVal)
{
	PlatformGraphicsContext* hGC = context->platformContext();
	int x = destRect.x();
	int y = destRect.y();
	
	if((x < destRect.x() + destRect.width() - xPhaseVal) &&
	   (y < destRect.y() + destRect.height() - yPhaseVal))
	{
		WmDrawSubBitmap(*hGC, context->translateX(x), context->translateY(y), &bitmap, destRect.x() - phase.x(), destRect.y() - phase.y(), destRect.width(), destRect.height());
		y += (bitmap.rectH - ( destRect.y() - phase.y()));
	}

	//checking for the positive value of (destRect.x() - phase.x()) and (destRect.y() - phase.y()) and not x and y(CQ BUG 2398): Hema
	if(((destRect.x() - phase.x())>=0)&&((destRect.y() - phase.y())>=0))
	{
		for(; x < destRect.x() + destRect.width() - xPhaseVal; )
		{
			for(; y < destRect.y() + destRect.height() - yPhaseVal; )
			{
				if(y == (int)destRect.y())
				{
					WmDrawSubBitmap(*hGC, context->translateX(x), context->translateY(y), &bitmap, 0, destRect.y() - phase.y(), destRect.width(), destRect.height());
				}
				else if(x == (int)destRect.x())
				{
					WmDrawSubBitmap(*hGC, context->translateX(x), context->translateY(y), &bitmap, destRect.x() - phase.x(), 0, destRect.width(), destRect.height());
				}
				else
				{
					WmDrawSubBitmap(*hGC, context->translateX(x), context->translateY(y), &bitmap, 0, 0, destRect.width(), destRect.height());
				}
				y += bitmap.rectH;
			}
			y = destRect.y();
			if(x == (int)destRect.x())
			{
        		x += (bitmap.rectW - (destRect.x() - phase.x()));
				continue;
			}
			x += bitmap.rectW;
		}
	}
}
#endif //_PATTERN_OPTIMIZATION

void BitmapImage::drawPattern(GraphicsContext* context, const FloatRect& ori_tileRect, const AffineTransform& patternTransform,
                        const FloatPoint& ori_phase, CompositeOperator op, const FloatRect& ori_destRect)
{
	PlatformGraphicsContext* hGC = context->platformContext();
	WmGC* pGC = WmGetGCPointer(  *hGC );
	if(pGC->gcAttrib.bitsPerPixel == 32)
	{
		SFBalDebug(MID_EXCEPTION,"BitmapImage::drawpattern entered\n");
		drawPattern32(context, ori_tileRect, patternTransform,
                        ori_phase, op,  ori_destRect);
		return;
	}


	FloatRect tileRect = ori_tileRect;
	FloatRect destRect = ori_destRect;
	FloatPoint phase = ori_phase;
#ifdef SFB_GWES_TRANSFORM
	tileRect = context->getCTM().mapRect( tileRect );	
	destRect = context->getCTM().mapRect( destRect );
	if( destRect.width() < 1 ) destRect.setWidth( 1.0 );
	if( destRect.height() < 1 ) destRect.setHeight( 1.0 );
	phase = context->getCTM().mapPoint( phase );
	//vamsi - We cant access webcontrol from here. So, calculate Zoom factor based on affine tranform
 	float g_zoomMultiplier = destRect.width() / ori_destRect.width() ;
#endif

    NativeImagePtr image = frameAtIndex(m_currentFrame);
    if (!image || 0 == image->bytes() ) // If it's too early we won't have an image yet.
        return;

    // Create WmBitmap from the decoded image data
	ImgCodecType codecType;
	unsigned short *alphachannel = NULL;
	WmBitmap* pSrcBitmap = NULL;
	

	WmBitmap bitmap;
	IntRect srcRect;

	WmBitmap* subBitmapPng = NULL;
	WmBitmap* subBitmap = NULL;

    bitmap.compressType = COMPRESS_NONE;
    bitmap.rectW = size().width();
    bitmap.rectH = size().height();
    bitmap.bitsPerPixel = image->getBitsPerPixel();
    bitmap.bUseChromakey = image->useChromakey();
    bitmap.colorChromakey = image->getChromakey();
    bitmap.pBits = image->bytes();
	bitmap.bUseReplacement  = FALSE;
    codecType = image->getCodecType();
    alphachannel = image->getAlphachannel();
	

if (codecType == IMG_DEC_PNG)
{
	//Skip small images/regions. Temp Hardcoding for target testing
	if( ((bitmap.rectW <= 5) || (bitmap.rectH <= 5) || (destRect.height() <= 5) || (destRect.width() <= 5)) )
		return;


	//Restict Total Number of Tiles to 10.
	int numH = (destRect.height() / bitmap.rectH);
	int numW = (destRect.width() / bitmap.rectW);

	if(numH < 1) numH = 1;
	if(numW < 1) numW = 1;


	if( ( (numH * numW) > 20)  )
		return;
	

	//Skip large scaling.
	
	if( ((((destRect.width() * destRect.height()) / (bitmap.rectW * bitmap.rectH)) > 20))  )
		return;
	

	//Skip large number of tiles.

	if( ((destRect.width() / bitmap.rectW) > 5) && ((destRect.height() / bitmap.rectH) > 5)  )
		return;
	
}

	WmRect clipRect;
	WmClip clipRes;
	WmGetClipRect(*hGC, &clipRect);

	// shkim modification start
	// intersect with original clipping area
	WebCore::IntRect originalclip( clipRect.x, clipRect.y, clipRect.w, clipRect.h );
	WebCore::IntRect newclip( context->translateX(destRect.x()), context->translateY(destRect.y()), destRect.width(), destRect.height() );
	newclip.intersect( originalclip );

	clipRes  = WmSetClipRect(*hGC, newclip.x(), newclip.y(), newclip.width(), newclip.height() );
	if( clipRes == CLIP_REJECT )
	{
		WmSetClipRect(*hGC, originalclip.x(), originalclip.y(), originalclip.width(), originalclip.height() );
			return;
	}

        // shkim modification end

	/*  While drawing the Bitmap ( using WmDrawBitmap ), the phase value (postion from where to draw the image)
		could be -ve. In that case add the value of phaseVal to destRect.
	*/
		int xPhaseVal = 0;
		int yPhaseVal = 0;

		if(phase.x() <0)
			xPhaseVal = phase.x();
		if(phase.y() < 0)
			yPhaseVal = phase.y();

	if(codecType == IMG_DEC_PNG && alphachannel!=NULL)
	{
		if(ori_destRect.height()!= bitmap.rectH  &&  ori_destRect.width() != bitmap.rectW && *alphachannel != 0)
		{
			if(CheckBufferSizeForBitmap(destRect.height(), destRect.width()))
					subBitmapPng = WmCaptureBitmapOnBitmap(ori_tileRect.x(), ori_tileRect.y(), 
													ori_destRect.width(), ori_destRect.height(), &bitmap);
			{
				if(!subBitmapPng)
				{
					//restore the save cliped rect
					WmSetClipRect(*hGC, clipRect.x, clipRect.y, clipRect.w, clipRect.h);
					return;
				}
			}
		}
	}

	
   	if(g_zoomMultiplier != 1.0f && !subBitmapPng)
    {
		if(CheckBufferSizeForBitmap(destRect.height(), destRect.width()))
			subBitmap = WmCaptureBitmapOnBitmap(ori_tileRect.x(), ori_tileRect.y(), ori_tileRect.width(), ori_tileRect.height(), &bitmap);
		else
		{
			//restore the save cliped rect
			WmSetClipRect(*hGC, clipRect.x, clipRect.y, clipRect.w, clipRect.h);
			return;
		 }
    }

	//removed the check for x and y CQ BUG 2398: Hema
    if((destRect.width()>0) && (destRect.height()>0))
    {
#ifdef _PATTERN_OPTIMIZATION
      if( !((codecType == IMG_DEC_PNG) &&  (((destRect.width() * destRect.height()) / (bitmap.rectW * bitmap.rectH)) > MAX_IMAGE_TILES)) ) /*check only for png*/
#else
        if(((destRect.width() * destRect.height()) / (bitmap.rectW * bitmap.rectH)) < MAX_IMAGE_TILES)
#endif
        {
			//new code
			if( !(destRect.x() - phase.x()) || !(destRect.y() - phase.y()))
			{
				xPhaseVal = 0;
				yPhaseVal = 0;
			}
			/*IF xPhaseVal or yPhaseVal is  < 0, we start drawing the pattern much ahead of clipping region.
              so add the xPhaseVal/yPhaseVal to destRect so tht total width/height to be drawn includes this
              extra xPhaseVal or yPhaseVal that we have drawn ahead of clipping region.*/
			
			if(subBitmap)
			{
#ifdef _PATTERN_OPTIMIZATION
				drawPlatformBitmapTiles(context, destRect, tileRect, phase, xPhaseVal, yPhaseVal, subBitmap, true);
#else
				DrawScaledSubBitmapTiles(context, destRect, tileRect, subBitmap, bitmap, xPhaseVal, yPhaseVal);
#endif
			}
			else if(codecType == IMG_DEC_PNG)
            {
				DrawPngTiles(destRect,tileRect,context,subBitmapPng,alphachannel, bitmap,xPhaseVal, yPhaseVal,phase);
			}
			else
			{
#ifdef _PATTERN_OPTIMIZATION
				drawPlatformBitmapTiles(context, destRect, tileRect, phase, xPhaseVal, yPhaseVal, &bitmap, false);
#else
				DrawSubBitmapTiles(destRect, phase, context, bitmap,xPhaseVal, yPhaseVal);
#endif
			}
			//end of new code
        }
    }

    if(subBitmap)
    {
        WmFreeCapturedBitmap(subBitmap);
    }

	if(subBitmapPng)
    {
        WmFreeCapturedBitmap(subBitmapPng);
    }
	
	//restore the save cliped rect
	WmSetClipRect(*hGC, clipRect.x, clipRect.y, clipRect.w, clipRect.h);
}


void BitmapImage::drawPattern32(GraphicsContext* context, const FloatRect& ori_tileRect, const AffineTransform& patternTransform,
                        const FloatPoint& ori_phase, CompositeOperator op, const FloatRect& ori_destRect)
{

	SFBalDebug(MID_EXCEPTION,"BitmapImage::drawpattern32 entered\n");
	ImgCodecType		codecType;
	WmBitmap			bitmap;
	IntRect				srcRect;
	unsigned short		*alphachannel	= NULL;
	WmBitmap*           pSrcBitmap		= NULL;
	WmBitmap*			subBitmapPng	= NULL;
	WmBitmap*			subBitmap		= NULL;

	PlatformGraphicsContext*	hGC			= context->platformContext();
	WmGC*						pGC			= WmGetGCPointer(  *hGC );	
	
	
	
	FloatRect					tileRect	= ori_tileRect;
	FloatRect					destRect	= ori_destRect;
	FloatPoint					phase		= ori_phase;

#ifdef SFB_GWES_TRANSFORM

	tileRect = context->getCTM().mapRect( tileRect );	
	destRect = context->getCTM().mapRect( destRect );

	if( destRect.width() < 1 ) destRect.setWidth( 1.0 );
	if( destRect.height() < 1 ) destRect.setHeight( 1.0 );

	phase = context->getCTM().mapPoint( phase );
	//vamsi - We cant access webcontrol from here. So, calculate Zoom factor based on affine tranform
 	float g_zoomMultiplier = destRect.width() / ori_destRect.width() ;

#endif

    NativeImagePtr image = frameAtIndex(m_currentFrame);
    if (!image || 0 == image->bytes() ) // If it's too early we won't have an image yet.
		return;
    codecType	= image->getCodecType();

	if(!m_decodeOnce)
	{
		ImgDecodeImageScaled(codecType, (unsigned char*)m_data->data(), 
							 1000, 1000,
							 m_image, m_data->size(), 
							 IMG_BGRA_8888_OUTPUT_AND_ROTATE, IMG_RESIZE);
		SFBalDebug(MID_EXCEPTION,"BitmapImage::drawpattern32 decode m_decodeOnce = %d\n",m_decodeOnce);
		m_decodeOnce = true;
		m_decodeScaledOnce = false;
	}

	bitmap.compressType		= COMPRESS_NONE;
	bitmap.rectW			= m_image->pHead->pbmBitmap->width;
	bitmap.rectH			= m_image->pHead->pbmBitmap->height;
	bitmap.bitsPerPixel		= m_image->pHead->pbmBitmap->bitsPerPixel;
	bitmap.bUseChromakey	= m_image->pHead->pbmBitmap->bChromaKeyFlag;
	bitmap.colorChromakey	= m_image->pHead->pbmBitmap->chromaKey;
	bitmap.pBits			= m_image->pHead->pbmBitmap->pBits;
	bitmap.bUseReplacement  = FALSE;
	alphachannel			= (unsigned short*)m_image->pHead->pbmBitmap->pAlphaChannel;
	
	if((ori_tileRect.height() == ori_destRect.height()) &&( ori_tileRect.width() == ori_destRect.width() ))
	{
		if ((ori_tileRect.height() != bitmap.rectH) || (ori_tileRect.width() != bitmap.rectW))
		{
			if(!m_decodeOnce)
			{
				ImgDecodeImageScaled(codecType, (unsigned char*)m_data->data(), 
								 ori_tileRect.width(), ori_tileRect.height(),
								 m_image, m_data->size(), 
								 IMG_BGRA_8888_OUTPUT_AND_ROTATE, IMG_RESIZE);
				SFBalDebug(MID_EXCEPTION,"BitmapImage::drawpattern32 decode m_decodeOnce = %d\n",m_decodeOnce);
				m_decodeOnce = true;
				m_decodeScaledOnce = false;
			}
			bitmap.compressType		= COMPRESS_NONE;
			bitmap.rectW			= m_image->pHead->pbmBitmap->width;
			bitmap.rectH			= m_image->pHead->pbmBitmap->height;
			bitmap.bitsPerPixel		= m_image->pHead->pbmBitmap->bitsPerPixel;
			bitmap.bUseChromakey	= m_image->pHead->pbmBitmap->bChromaKeyFlag;
			bitmap.colorChromakey	= m_image->pHead->pbmBitmap->chromaKey;
			bitmap.pBits			= m_image->pHead->pbmBitmap->pBits;
			bitmap.bUseReplacement  = FALSE;
			alphachannel			= (unsigned short*)m_image->pHead->pbmBitmap->pAlphaChannel;
		}
	}
	WmRect clipRect;
	WmClip clipRes;
	WmGetClipRect(*hGC, &clipRect);

	// shkim modification start
	// intersect with original clipping area
	WebCore::IntRect originalclip( clipRect.x, clipRect.y, clipRect.w, clipRect.h );
	WebCore::IntRect newclip( context->translateX(destRect.x()), context->translateY(destRect.y()), destRect.width(), destRect.height() );
	newclip.intersect( originalclip );

	clipRes  = WmSetClipRect(*hGC, newclip.x(), newclip.y(), newclip.width(), newclip.height() );
	if( clipRes == CLIP_REJECT )
	{
		WmSetClipRect(*hGC, originalclip.x(), originalclip.y(), originalclip.width(), originalclip.height() );
	}

        // shkim modification end

	/*  While drawing the Bitmap ( using WmDrawBitmap ), the phase value (postion from where to draw the image)
		could be -ve. In that case add the value of phaseVal to destRect.
	*/
	int xPhaseVal = 0;
	int yPhaseVal = 0;

	if(phase.x() <0)
		xPhaseVal = phase.x();
	if(phase.y() < 0)
		yPhaseVal = phase.y();

	if(codecType == IMG_DEC_PNG && alphachannel!=NULL)
	{
		if(ori_destRect.height()!= bitmap.rectH  &&  ori_destRect.width() != bitmap.rectW && *alphachannel != 0)
		{
			if(CheckBufferSizeForBitmap(destRect.height(), destRect.width()))
			{
					subBitmapPng = WmCaptureBitmapOnBitmap(ori_tileRect.x(), ori_tileRect.y(), 
													ori_destRect.width(), ori_destRect.height(), &bitmap);
				if(!subBitmapPng)
					return;
			}
		}

	}

	
   	if(g_zoomMultiplier != 1.0f && !subBitmapPng)
    {
		if(CheckBufferSizeForBitmap(destRect.height(), destRect.width()))
			 subBitmap = WmCaptureBitmapOnBitmap(ori_tileRect.x(), ori_tileRect.y(), ori_tileRect.width(), ori_tileRect.height(), &bitmap);
		else
			return;
    }

	//removed the check for x and y CQ BUG 2398: Hema
    if((destRect.width()>0) && (destRect.height()>0))
    {
        if(((destRect.width() * destRect.height()) / (bitmap.rectW * bitmap.rectH)) < MAX_IMAGE_TILES)
        {
			//new code
			if( !(destRect.x() - phase.x()) || !(destRect.y() - phase.y()))
			{
				xPhaseVal = 0;
				yPhaseVal = 0;
			}
			/*IF xPhaseVal or yPhaseVal is  < 0, we start drawing the pattern much ahead of clipping region.
              so add the xPhaseVal/yPhaseVal to destRect so tht total width/height to be drawn includes this
              extra xPhaseVal or yPhaseVal that we have drawn ahead of clipping region.*/
			
			if(subBitmap)
			{
				#ifdef _PATTERN_OPTIMIZATION
				drawPlatformBitmapTiles(context, destRect, tileRect, phase, xPhaseVal, yPhaseVal, subBitmap, true);
#else
				DrawScaledSubBitmapTiles(context, destRect, tileRect, subBitmap, bitmap, xPhaseVal, yPhaseVal);
#endif
			}
			else if(codecType == IMG_DEC_PNG && alphachannel!=NULL )
            {
				//sneha;please merge this also its a fix for widgets
				if(subBitmapPng != NULL)
				{

					for(int x = destRect.x(); x < ((int)(destRect.x()) + (int)(destRect.width()) - (int)(xPhaseVal) ); x += subBitmapPng->rectW )
					{
						for(int y = destRect.y();y < ((int)(destRect.y()) + ((int)destRect.height()) - ((int)yPhaseVal)); y += subBitmapPng->rectH )
						{
							
							WmDrawTransparentBitmapEx(*hGC, context->translateX(x), 
											context->translateY(y), subBitmapPng,alphachannel);
		
						}
					}
				}
				else
				{
					for(int x = destRect.x(); x < ((int)(destRect.x()) + (int)(destRect.width()) - (int)(xPhaseVal) ); x += bitmap.rectW )
					{
						for(int y = destRect.y();y < ((int)(destRect.y()) + ((int)destRect.height()) - ((int)yPhaseVal)); y += bitmap.rectH )
						{
							
							WmDrawTransparentBitmapEx(*hGC, context->translateX(x), 
											context->translateY(y), &bitmap,alphachannel);
		
						}
					}
				}
					//TODO:Will change to pattern optimization later//DrawPngTiles(destRect,tileRect,context,subBitmapPng,alphachannel, bitmap,xPhaseVal, yPhaseVal,phase);
			}
			else
			{
				#ifdef _PATTERN_OPTIMIZATION
				drawPlatformBitmapTiles(context, destRect, tileRect, phase, xPhaseVal, yPhaseVal, &bitmap, false);
#else
				DrawSubBitmapTiles(destRect, phase, context, bitmap,xPhaseVal, yPhaseVal);
#endif
			}
			//end of new code
        }
    }

    if(subBitmap)
    {
        WmFreeCapturedBitmap(subBitmap);
    }

	if(subBitmapPng)
    {
        WmFreeCapturedBitmap(subBitmapPng);
    }
	
	//restore the save cliped rect
	WmSetClipRect(*hGC, clipRect.x, clipRect.y, clipRect.w, clipRect.h);
}

void BitmapImage::checkForSolidColor()
{
    // FIXME: It's easy to implement this optimization. Just need to check the RGBA32 buffer to see if it is 1x1.
    m_isSolidColor = false;
}

#ifdef SFB_PROGRESSIVE_RENDERING
void BitmapImage::showPlaceholderImage(GraphicsContext* context, const FloatRect& rect)
{
    WmBitmap* image = context->getPlaceHolderImage();

    if(!image)
        return;

    PlatformGraphicsContext* hGC = context->platformContext();

    if(rect.width() < image->rectW || rect.height() < image->rectH)
    {
        WmDrawScaledBitmap( *hGC,
                            context->translateX(rect.x()),
                            context->translateY(rect.y()),
                            rect.width(),
                            rect.height(),
                            image );
    }
    else
    {
        WmDrawBitmap( *hGC,
                        context->translateX(rect.x()),
                        context->translateY(rect.y()),
                        image );
    }

    WmDrawRect( *hGC,
                context->translateX(rect.x()),
                context->translateY(rect.y()),
                rect.width(),
                rect.height() );

}
#endif // SFB_PROGRESSIVE_RENDERING

} // namespace WebCore

// End of File
