/*
 * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#define _WINGDI_
#define NOGDI

#include "GIFImageDecoder.h"

// Shp specific codec files
//#include "ImgCODEC.h"
#include "ImgGIFDecoder.h"

#include "MemCore.h"


#undef NOGDI
#undef _WINGDI_
#ifdef __CMM__
#include <allocs.h>
#endif

#include "OtherImageDecoder.h"
#include "MemCore.h"

/*Similar to as defined in SHP3\AVMS\imgcodec\ImgCodec.c*/
static const unsigned char	gJFIFMarker	 [] = { 0x4A, 0x46, 0x49, 0x46, 0x00};
static const unsigned char	gPNGHeader   [] = { 0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a };
static const unsigned char	gJPEGHeader  [] = { 0xFF, 0xD8 };
static const unsigned char	gGIFHeader	 [] = "GIF";
static const unsigned char	gTIFFHeader1 [] = { 0x49, 0x49, 0x2A, 0x00 };
static const unsigned char	gTIFFHeader2 [] = { 0x4D, 0x4D, 0x00, 0x2A };
static const unsigned char	gBMPHeader	 [] = { 0x42, 0x4D };

BOOL
SFBImgGetCodecType (ImgCodecType* pCodecType, unsigned char *pEncodedData, 
					 unsigned int size)
{
	ImgCodecType imgType ;
	unsigned int i=0, imageCount=0, x, y, width, height, imageWidth, imageHeight ;

	
		if (AcMemcmp (pEncodedData, gPNGHeader, 8) == 0)
		{
			/* Select PNG Decoder */
			imgType = IMG_DEC_PNG;
		}
		else
		if (AcMemcmp (pEncodedData, gJPEGHeader, 2) == 0)
		{
			/* Select JPEG Decoder */
			imgType = IMG_DEC_JPEG;
		}
		else
		if (AcMemcmp (pEncodedData, gGIFHeader, 3) == 0)
		{
			/* Verify whether image is GIF or AGIF */
			imageCount	= 0 ;
			imageWidth	= (pEncodedData[7] << 8) | pEncodedData[6];
			imageHeight	= (pEncodedData[9] << 8) | pEncodedData[8];
			//screen size : imageWidth, imageHeight
			for (i = 10; i < size; i ++)
			{
				if (pEncodedData [i] == 0x2c)
				{
					i ++;

					/* There is no way to identify whether 0x2c is actually
					 * image seperator byte or just a byte in LZW stream. To
					 * make sure (mostly) check for x, y, width and height
					 */
					x		= (pEncodedData[i + 1] << 8) | pEncodedData[i];
					y		= (pEncodedData[i + 3] << 8) | pEncodedData[i + 2];
					width	= (pEncodedData[i + 5] << 8) | pEncodedData[i + 4];
					height	= (pEncodedData[i + 7] << 8) | pEncodedData[i + 6];
					//screen size  ׻ frame width, height  ĿѴ.
					if ((x <= imageWidth) && (y <= imageHeight) &&
						((width )  <= imageWidth) && ((height ) <= imageHeight))
					{
						//SysDebug ((MID_IMGCODEC, "__ImgGetCodecType: W : %4d H : %4d X: %d Y: %d \n", 
						//	width, height, x, y));

						imageCount ++ ;
						if (imageCount > 1) break;
					}
				}
			}

			if (imageCount > 1)
			{
				imgType	= IMG_DEC_AGIF ;
			}
			else
			{
				/* Select GIF Decoder */
				imgType = IMG_DEC_GIF;
			}
		}
#if 1
		else
		if (AcMemcmp (pEncodedData, gTIFFHeader1, 2) == 0)
		{
			/* Select TIFF Decoder */
			imgType = IMG_DEC_TIF;
		}
		else
		if (AcMemcmp (pEncodedData, gTIFFHeader2, 2) == 0)
		{
			/* Select TIFF Decoder */
			imgType = IMG_DEC_TIF;
		}
#endif
		else
		if (AcMemcmp (pEncodedData, gBMPHeader, 2) == 0)
		{
			/* Select BMP Decoder */
			imgType = IMG_DEC_BMP;
		}
		else
		{
				//__ImgSetError (IMG_UNSUPPORTED_FORMAT);
				return FALSE;
	  		
		}
	*pCodecType = imgType ;
	return TRUE;
}

namespace WebCore {

GIFImageDecoder::GIFImageDecoder()
: m_frameCountValid(true),
  m_numOfDecodedFrames(0)
{
	m_image = (ImgImage*)malloc(sizeof(ImgImage));
    AcMemset (m_image, 0, sizeof(ImgImage));

    m_image->pGifFrame = (ImgGIFFrame*)malloc(sizeof(ImgGIFFrame));
    AcMemset (m_image->pGifFrame, 0, sizeof(ImgGIFFrame));
}

GIFImageDecoder::~GIFImageDecoder()
{
	if(m_image)
	{
		ImgFreeImage(m_image);
		free(m_image->pGifFrame);
		free(m_image);
		m_image = NULL;
	}

	for(unsigned int i = 0; i < m_frameBufferCache.size(); i++)
	{
		if(m_frameBufferCache[i].isDataInitialised())
		{
			m_frameBufferCache[i].setBytes(0);
			m_frameBufferCache[i].setDataInitialised(false);
		}
	}
}

// Take the data and store it.
void GIFImageDecoder::setData(SharedBuffer* data, bool allDataReceived)
{
	 ImgCodecType	pCodecType		;
    if (m_failed)
        return;

    // Cache our new data.
	ImageDecoder::setData(data, allDataReceived);

    // Our frame count is now unknown.
    m_frameCountValid = false;

	if(SFBImgGetCodecType (&pCodecType, ( unsigned char* )m_data->data(), m_data->size()))
	{
		if (pCodecType==IMG_DEC_GIF)
		{
			decode();
		}
		else
		{
			//Checking for the codec type as it will decode only if its an AGIF otherwise return : Hema
		if (pCodecType==IMG_DEC_AGIF)
			decodeFirstFrame();
			else
				return;
		
		}
	}
}

// Whether or not the size information has been decoded yet.
bool GIFImageDecoder::isSizeAvailable() const
{
    return m_sizeAvailable;
}

// The total number of frames for the image.  Will scan the image data for the answer
// (without necessarily decoding all of the individual frames).
int GIFImageDecoder::frameCount()
{
    if(!m_frameCountValid)
    {
        decode();
    }
    return m_frameBufferCache.size();
}

RGBA32Buffer* GIFImageDecoder::frameBufferAtIndex(size_t index)
{
    if (index < 0 )
        return NULL;

    if( 0 == m_frameBufferCache.size() )
    {
        decode();
    }

    RGBA32Buffer& frame = m_frameBufferCache[index];

	if(false == frame.isDataInitialised())
	{
		for(unsigned int i = 0; i < m_frameBufferCache.size(); i++)
		{
			if(m_frameBufferCache[i].isDataInitialised())
			{
				m_frameBufferCache[i].setBytes(0);
				m_frameBufferCache[i].setDataInitialised(false);
			}
		}

		decodeNextFrame(index);
	}


    return &frame;
}

void GIFImageDecoder::decode()
{
    ImgBitmapElem*	pBitmapElem		= NULL;
    m_frameCountValid = true;

    SysDebug(( MID_BROWSER, "GIFImageDecoder::decode() : entered\n" ));
    if ( NULL == m_image )
    {
        return;
    }
    if(SFBCheckBufferSize(IMG_DEC_GIF,m_image, ( unsigned char* )m_data->data(), m_data->size())!=TRUE)
    {
        MemoryManager::SetStatus(CMM_CHECKRESULT_COLLECTING);
		return ;
    }
	else
        {
	if ( !ImgDecodeImage( IMG_DEC_GIF, m_image, ( unsigned char* )m_data->data(), m_data->size() ) )
	{
		if(m_image != NULL)
		{
			if(m_image->pGifFrame != NULL)
				free(m_image->pGifFrame);
			free( m_image );
			m_image = NULL;
		}
		return;
	}
    m_frameBufferCache.resize( m_image->bitmapCount );
#ifdef _SHP_WM_COLOR_32
	int posIdx = 4;
#endif // _SHP_WM_COLOR_32

#ifdef _SHP_WM_COLOR_16
	int posIdx = 2;
#endif // _SHP_WM_COLOR_16
    int posWidth = m_image->width * posIdx;
    int dataSize = m_image->height * posWidth;

    // Set the image attributes
    pBitmapElem = m_image->pHead;
	for ( int i = 0; i < m_image->bitmapCount; pBitmapElem = pBitmapElem->pNext, i++ ) 
	{
        // update the size
        if(pBitmapElem->pbmBitmap->width && pBitmapElem->pbmBitmap->height && !m_sizeAvailable)
        {
            m_size.setWidth( pBitmapElem->pbmBitmap->width );
            m_size.setHeight( pBitmapElem->pbmBitmap->height );
            m_sizeAvailable = true;
        }

        // Add the raw data to the image structure
        m_frameBufferCache[i].setBytes((unsigned char*)pBitmapElem->pbmBitmap->pBits);

        // Inform that decode is complete
        m_frameBufferCache[i].setStatus( RGBA32Buffer::FrameComplete );

        m_frameBufferCache[i].setBitsPerPixel( pBitmapElem->pbmBitmap->bitsPerPixel );
        m_frameBufferCache[i].setUseChromakey( pBitmapElem->pbmBitmap->bChromaKeyFlag );
        m_frameBufferCache[i].setChromakey( pBitmapElem->pbmBitmap->chromaKey );
        m_frameBufferCache[i].ensureHeight( pBitmapElem->pbmBitmap->height );
		m_frameBufferCache[i].setDataInitialised(true);
		m_frameBufferCache[0].setAlphaChannel((unsigned short*)m_image->pHead->pbmBitmap->pAlphaChannel);
		m_frameBufferCache[0].setCodecType(IMG_DEC_GIF);

        // Set the frame rect
        IntRect frameRect( pBitmapElem->x, pBitmapElem->y,
                      pBitmapElem->pbmBitmap->width,
                      pBitmapElem->pbmBitmap->height);
        m_frameBufferCache[i].setRect( frameRect );

	}
}

    SysDebug(( MID_BROWSER, "GIFImageDecoder::decode() : done\n" ));
}

    
void GIFImageDecoder::decodeFirstFrame()
{
    if(SFBCheckBufferSize(IMG_DEC_AGIF, m_image, ( unsigned char* )m_data->data(), m_data->size())!=TRUE)
    {
        MemoryManager::SetStatus(CMM_CHECKRESULT_COLLECTING);
		return;
    }
	else
	{
	 BOOL bStatus = ImgGIFDecodeNextFrame( m_image, (unsigned char*)m_data->data(), m_data->size());

    // We haven't recieved the image data yet
    if(!m_image->pHead || bStatus == false)
        return;

    // set the offset for the first image so that we can
    // stop decoding at this condition
    m_FirstOffset = m_image->offset;

    m_repetitionCount = m_image->loopCount;

    // update the size
    if(m_image->width && m_image->height && !m_sizeAvailable)
    {
        m_size = IntSize(m_image->width, m_image->height);
        m_sizeAvailable = true;
    }

    // resize the cache to fit all the frames
    if( m_frameBufferCache.size() < m_numOfDecodedFrames + 1)
    {
        m_frameBufferCache.resize( m_numOfDecodedFrames + 1 );
        m_frameCountValid = true;
    }

	/* This is a temporary fix for gif images when the size of image is zero after decoding.
	 * The data size should ideally be set by the decoder.
	 */
	if(m_image->dataSize == 0)
	{
		m_image->dataSize = m_image->pHead->pbmBitmap->width * m_image->pHead->pbmBitmap->height * m_image->pHead->pbmBitmap->bitsPerPixel;
	}

    // Copy the decoded frame to cached buffer
    m_frameBufferCache[m_numOfDecodedFrames].setBytes((unsigned char*)m_image->pHead->pbmBitmap->pBits);
    m_frameBufferCache[m_numOfDecodedFrames].setDuration( m_image->pHead->pbmBitmap->delay );
    m_frameBufferCache[m_numOfDecodedFrames].setStatus( RGBA32Buffer::FrameComplete );
    m_frameBufferCache[m_numOfDecodedFrames].setBitsPerPixel( m_image->pHead->pbmBitmap->bitsPerPixel);
    m_frameBufferCache[m_numOfDecodedFrames].setUseChromakey( m_image->pHead->pbmBitmap->bChromaKeyFlag);
    m_frameBufferCache[m_numOfDecodedFrames].setChromakey( m_image->pHead->pbmBitmap->chromaKey );
    m_frameBufferCache[m_numOfDecodedFrames].ensureHeight(m_image->pHead->pbmBitmap->height);
	m_frameBufferCache[m_numOfDecodedFrames].setDataInitialised(true);
	m_frameBufferCache[0].setAlphaChannel((unsigned short*)m_image->pHead->pbmBitmap->pAlphaChannel);
	m_frameBufferCache[0].setCodecType(IMG_DEC_AGIF);


    // Set the frame rect
    IntRect frameRect(m_image->pHead->x, m_image->pHead->y,
                  m_image->pHead->pbmBitmap->width,
                  m_image->pHead->pbmBitmap->height);
    m_frameBufferCache[m_numOfDecodedFrames].setRect( frameRect );

    m_numOfDecodedFrames++;
}
	calculateFrameCount();
}





void GIFImageDecoder::calculateFrameCount()
{
// Calculating the image size and decoding for the specific number of frames only : Hema
	//added for Cq 2460 and 2524
	int imageWidth=0,imageHeight=0,nof=0,Imagesize=0;
Imagesize= ImgGetMemoryRequired ((unsigned char*)m_data->data(), m_data->size(), ( unsigned int* )&imageWidth,( unsigned int* )&imageHeight,( unsigned int* )&nof );
if( Imagesize > 0)
{
   while(m_numOfDecodedFrames < nof )
	{
	  BOOL bStatus = ImgGIFDecodeNextFrame( m_image, (unsigned char*)m_data->data(), m_data->size());
        // Data was not decoded properly
        if( !m_image->pHead || bStatus == false )
            return;

        // Break since we reached the first frame
        if(m_FirstOffset == m_image->offset)
        {
            break;
		}
		//Removed this line as it is not required to check for m_repetitionCount for all frames, it is already being done in decodeFirstFrame(): Hema
       // m_repetitionCount = m_image->loopCount;

        // update the size
        if(m_image->width && m_image->height && !m_sizeAvailable)
        {
            m_size = IntSize(m_image->width, m_image->height);
            m_sizeAvailable = true;
        }

        // resize the cache to fit all the frames
        if( m_frameBufferCache.size() < m_numOfDecodedFrames + 1)
        {
            m_frameBufferCache.resize( m_numOfDecodedFrames + 1 );
            m_frameCountValid = true;
        }

        m_frameBufferCache[m_numOfDecodedFrames].setDuration( m_image->pHead->pbmBitmap->delay );
        m_frameBufferCache[m_numOfDecodedFrames].setStatus( RGBA32Buffer::FrameComplete );
        m_frameBufferCache[m_numOfDecodedFrames].setBitsPerPixel( m_image->pHead->pbmBitmap->bitsPerPixel);
        m_frameBufferCache[m_numOfDecodedFrames].setUseChromakey( m_image->pHead->pbmBitmap->bChromaKeyFlag);
        m_frameBufferCache[m_numOfDecodedFrames].setChromakey( m_image->pHead->pbmBitmap->chromaKey );
        m_frameBufferCache[m_numOfDecodedFrames].ensureHeight(m_image->pHead->pbmBitmap->height);
		m_frameBufferCache[0].setAlphaChannel((unsigned short*)m_image->pHead->pbmBitmap->pAlphaChannel);
		m_frameBufferCache[0].setCodecType(IMG_DEC_AGIF);
	

        // Set the frame rect
        IntRect frameRect(m_image->pHead->x, m_image->pHead->y,
                      m_image->pHead->pbmBitmap->width,
                      m_image->pHead->pbmBitmap->height);
        m_frameBufferCache[m_numOfDecodedFrames].setRect( frameRect );

        m_numOfDecodedFrames++;
	}
}
	

}

void GIFImageDecoder::decodeNextFrame(unsigned int frameIndex)
{
	/*call next frame API sequentially(bug 1791)*/
	BOOL bStatus = ImgGIFDecodeNextFrame( m_image, (unsigned char*)m_data->data(), m_data->size());
    // Data was not decoded properly
    if(!m_image->pHead  || bStatus == false)
        return;
    // Copy the decoded frame to cached buffer
    m_frameBufferCache[frameIndex].setBytes((unsigned char*)m_image->pHead->pbmBitmap->pBits);
	m_frameBufferCache[frameIndex].setDataInitialised(true);

}

} // namespace WebCore

// End of File