/*
 * This file is part of the internal font implementation.  It should not be included by anyone other than
 * FontMac.cpp, FontWin.cpp and Font.cpp.
 *
 * Copyright (C) 2006, 2007 Apple Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

#ifndef FontPlatformData_H
#define FontPlatformData_H

#define _SFBAL_SUPPORT_VECTOR_FONT


#include "ShpJasmine.h"

#include "StringImpl.h"
#include "GWES.h"

#if PLATFORM(SHP)
#include "CMMDefs.h"
#endif // PLATFORM(SHP)




namespace WebCore {

class FontDescription;

class FontPlatformData
CMM_MODIFIED
{
public:
    class Deleted {};

    // Used for deleted values in the font cache's hash tables.
	FontPlatformData(Deleted) :
	m_size(0),
	m_syntheticBold(false),
	m_syntheticOblique(false)
    {
	}

	FontPlatformData():
	m_size(0),
	m_syntheticBold(false),
	m_syntheticOblique(false)
    {}

    FontPlatformData(const FontDescription&, const AtomicString& family);

    ~FontPlatformData();

    int size() const { return m_size; }
    bool syntheticBold() const { return m_syntheticBold; }
    bool syntheticOblique() const { return m_syntheticOblique; }


    static bool init();
	
	bool isFixedPitch();

	HFNT getFontHandle()const ;

    void setFont(HGC*) const;

    unsigned hash() const;

    bool operator==(const FontPlatformData& other) const;

	SFBalFontAttr m_fontAttr;

private:
    int m_size;
    bool m_syntheticBold;
    bool m_syntheticOblique;
};

}

#endif
