#ifdef USE_NATIVE_WIDGET_SUPPORT

#include "NativeComboField.h"
#include "NotImplemented.h"
#include "IntRect.h"
#include "FontPlatformData.h"
#include "FontDescription.h"
#include "RenderStyle.h"
#include "Document.h"
#include "Frame.h"
#include "Page.h"
#include "FocusController.h"
#include "EventHandler.h"
#include "PlatformKeyboardEvent.h"
#include "FrameView.h"

#ifdef SFB_GWES_TRANSFORM
#include "ChromeClient.h"
#endif

//forward declarations
extern HWIN gHWin;
#define SCROLLWIDTH 6
//Keeping the minimum width for combo box =28 as it is the minimum required to accomodate scroll, borders and one character: hema
#define MINIMUM_WIDTH_COMBOBOX 38  //modifying to 38 for Cq4294
#define MINIMUM_HEIGHT_COMBOBOX 12
namespace WebCore {

/* event handlers for combo box */
DECLARE_EVENT_MAP( WkNativeComboFieldEventMap );

DECLARE_HANDLE_EV_NOTIFY(WkNativeComboFieldEventNotifySelected);
DECLARE_HANDLE_EV_PRESSED(WkNativeComboFieldEventKeyPressed);

BEGIN_EVENT_MAP( WkNativeComboFieldEventMap, 2)
	HANDLE_EV_NOTIFY(WkNativeComboFieldEventNotifySelected)
	HANDLE_EV_PRESSED(WkNativeComboFieldEventKeyPressed)
END_EVENT_MAP()

static void _sendSelectionToBrowserEngine()
{
	NativeComboField* pNativeCombo = NativeComboField::getNativeComboFieldInstance();
	if(pNativeCombo && pNativeCombo->isActivated())
	{
		pNativeCombo->SetSelectedIndex();
	}
}

/* event callback */
BOOL WkNativeComboFieldEventKeyPressed(WmState* pState, WmEvent* pEvent)
{

	bool fEventAbsorbed = false;

	switch ( pEvent->key.keyCode )
	{
		case VK_HP_RIGHTEASY:
		{
			NativeComboField* pNativeCombo = NativeComboField::getNativeComboFieldInstance();
			if(pNativeCombo && pNativeCombo->isActivated())
			{
				pNativeCombo->unsetFocusedNode();
				pNativeCombo->deactivate();
				fEventAbsorbed = true;
			}
			break;
		}

		case VK_HP_LEFTEASY:
		{
			NativeComboField* pNativeCombo = NativeComboField::getNativeComboFieldInstance();
			if(pNativeCombo && pNativeCombo->isActivated())
			{
				pNativeCombo->unsetFocusedNode();
				pNativeCombo->deactivate();
				fEventAbsorbed = true;
				//BrAppControlCreateBrowsingOptionMenu();
			}
			break;
		}
		default:
			break;
	}//end case
	
	if(!fEventAbsorbed)
	{
		WmCallNextKeyPressedHandler(pState, pEvent);
	}

	return TRUE;
}

/* event callback */
BOOL WkNativeComboFieldEventNotifySelected(WmState* pState, WmEvent* pEvent)
{
	bool fEventAbsorbed = false;
         
	// HQ modification. Handling the combo-box notify close up event here and deactivating it
	if ( COMBOBOX_NOTIFY_ITEM_SELECTED == pEvent->notify.nType )
	{
        NativeComboField* pNativeCombo = NativeComboField::getNativeComboFieldInstance();
        if(pNativeCombo && pNativeCombo->isActivated())
        {
            _sendSelectionToBrowserEngine();
           
            fEventAbsorbed = true;
        }
    }

	if ( COMBOBOX_NOTIFY_CLOSEUP == pEvent->notify.nType)
	{
        NativeComboField* pNativeCombo = NativeComboField::getNativeComboFieldInstance();
        if(pNativeCombo && pNativeCombo->isActivated())
        {
               pNativeCombo->unsetFocusedNode();
               pNativeCombo->deactivate();
         
            fEventAbsorbed = true;
        }
    }


	if(!fEventAbsorbed)
	{
		WmCallNextNotifyHandler(pState, pEvent);
	}

	return TRUE;
}

static NativeComboField* g_pNativeField = NULL;
NativeComboField* NativeComboField::getNativeComboFieldInstance()
{
	if(NULL == g_pNativeField)
	{
		g_pNativeField = new NativeComboField();
	}

	return g_pNativeField;
}

void NativeComboField::destroyNativeComboFieldInstance()
{
	NativeComboField* pNativeCombo = NativeComboField::getNativeComboFieldInstance();
	if(pNativeCombo && pNativeCombo->isActivated())
	{
		pNativeCombo->deactivate();
	}
	delete g_pNativeField;
	g_pNativeField = NULL;
}

NativeComboField::NativeComboField() :
	m_handleWin(INVALID_HOBJ),
	m_pDocument(NULL),
	m_pSelectElement(NULL),
	m_nodeID(Node::ID_NONE)
{

}

NativeComboField::~NativeComboField()
{
	if(m_handleWin != INVALID_HOBJ)
	{
		deactivate();
	}
}

void NativeComboField::_createComboBox(const NativeComboBoxAttrib& attrib)
{
	int ComboFieldListCount = 0;
	IntRect rect(attrib.m_rect);
	

	m_pDocument->frame()->view()->scrollTo(rect);
	rect = m_pDocument->frame()->view()->contentsToWindow(rect);
	if(0 > rect.x())
	{
		m_pDocument->frame()->view()->scrollBy(rect.x(), 0);
		rect.setX(0);
	}
	int hWin = INVALID_HOBJ;
	int hGC = INVALID_HOBJ;

	getDocument()->frame()->page()->chrome()->client()->getWindowHandles(&hWin,&hGC);
	WmRect rectTemp;
	WmGetWindowClientRect( hWin, &rectTemp );
	int visibleWidth = rectTemp.w;	
	int visibleHeight =rectTemp.h;
	
	
#if defined(SFB_GWES_TRANSFORM)
	float zoomFactor = m_pDocument->page()->chrome()->client()->getZoomFactor();

	int frameRectX = m_pDocument->frame()->view()->contentsX();
	int frameRectY = m_pDocument->frame()->view()->contentsY();
	IntRect viewPortRect = m_pDocument->page()->chrome()->client()->viewportRect();

	rect.setX((rect.x() - (viewPortRect.x() - frameRectX)) * zoomFactor);
	rect.setY((rect.y() - (viewPortRect.y() - frameRectY)) * zoomFactor);

	rect.setWidth(rect.width() * zoomFactor);
	rect.setHeight(rect.height() * zoomFactor);

#endif
	int height= rect.height();
	int width = rect.width();

#if defined(SFB_GWES_TRANSFORM)
	
	if(rect.x() < 0)
		{
		 //reduce the width by the length which is hidden: CQ 3758
		  width = rect.width() + rect.x()- SCROLLWIDTH;
		  rect.setX(0);
		}
	if(rect.x() >= 0)
	{
	//considering scorllwidth into consideration.//CQ 3844
		if( (width + rect.x() ) > visibleWidth - SCROLLWIDTH )
		{
			width = visibleWidth - rect.x()- SCROLLWIDTH ;
		}
		else

		width = rect.width();
	}
	
#else

	if( (width + rect.x() ) > visibleWidth)
	{
		width = visibleWidth - rect.x() ;
	}
#endif
    // To calculate the list count for the combo box: Hema
	ComboFieldListCount= attrib.m_pSelectElement->listItems().size();
		
	if( (height + rect.y() ) > visibleHeight)
	{
		height = visibleHeight - rect.y();
	}
	//Keeping the minimum width for combo box =38 as it is the minimum required to accomodate scroll, borders and one character: Hema CQ 3987
	if(width < MINIMUM_WIDTH_COMBOBOX)
	{	
		width = MINIMUM_WIDTH_COMBOBOX;
		 rect.setWidth(MINIMUM_WIDTH_COMBOBOX);
	}

	if(height < MINIMUM_HEIGHT_COMBOBOX) //failsafe Minimum height 
	{
		height = MINIMUM_HEIGHT_COMBOBOX;
		rect.setHeight(MINIMUM_HEIGHT_COMBOBOX);
	}
	//modified for Cq 2726: Hema
	if((ComboFieldListCount>0) && (width > 0) &&(height > 0) &&(rect.height()>0) && (rect.width()>0))
	{
	// If list count is less than 4, the combo box is made of its orginal count otherwise its list count is restricted to 4. Cq bug no. 1909
	//Increasing the ComboFieldListCount to 6.
	if( ComboFieldListCount > 6 )
		ComboFieldListCount = 6;
	
	//vamsi - applied new comboxbox api's
	//Creating the combo box with double the height and width.
    //int hWin = INVALID_HOBJ;//moved to starting of this function
	//int hGC = INVALID_HOBJ;
	//getDocument()->frame()->page()->chrome()->client()->getWindowHandles(&hWin,&hGC);

	if(WmIsWindowValid (hWin ))
	{
	   m_handleWin = WsCreateComboBox ( hWin, COMBOBOX_STYLE_SIMPLE_DROP_DOWN_LIST, ComboFieldListCount, rect.x(), rect.y(), width, height, WmGetEventMap(WkNativeComboFieldEventMap) );
	}

	//RenderStyle* style = attrib.m_pStyle;
    //FontDescription fontDesc = style->fontDescription();
	//FontPlatformData fontData(fontDesc, fontDesc.family().family());

	//WsSetComboBoxTextItemFont(m_handleWin, fontData.getFontHandle());
    m_nodeID = Node::ID_SELECT;
	WmDrawWindow(hWin, TRUE );
	
	}

}

void NativeComboField::addItem(String value, unsigned index, bool bEnable)
{
	if (!value)
	{
		value.append(" ");
	}
	MCHAR * mVal = (MCHAR *) value.charactersWithNullTermination();
	//added to verify if the window (m_handleWin) is valid. 
	if(WmIsWindowValid (m_handleWin ))
	{
	WsAddComboBoxTextItem(m_handleWin, mVal);

	WsEnableComboBoxItem(m_handleWin, index, bEnable);
}
}

void NativeComboField::create(const NativeComboBoxAttrib& attrib)
{

	m_pDocument = attrib.m_pDocument;
	m_nodeID = attrib.m_nodeID;
	_createComboBox(attrib);
	
}

//vamsi - API for Engine to set Current selection to Winset
void NativeComboField::setCurSelItem(int index)
{
if(WmIsWindowValid (m_handleWin  ))
	WsSetComboBoxCurSelItem (m_handleWin, index);
}

void NativeComboField::activate(const NativeComboBoxAttrib& attrib)
{

	if(WmIsWindowValid (m_handleWin  ))
	{
	m_pSelectElement = attrib.m_pSelectElement;
	WsShowComboBoxDrop(m_handleWin, TRUE);
	}
	else
	unsetFocusedNode();

}

void NativeComboField::deactivate( bool bShow )
{
	if(WmIsWindowValid(m_handleWin))
	{
                int hWin = INVALID_HOBJ;
	        int hGC = INVALID_HOBJ;

        Frame* frame = getDocument() ? getDocument()->frame() : NULL;

        if(frame)
        {
            frame->page()->chrome()->client()->getWindowHandles(&hWin,&hGC);
        }

		WmDestroyWindow(m_handleWin);
		m_handleWin = INVALID_HOBJ;
		m_pDocument = NULL;
		m_pSelectElement = NULL;
		m_nodeID    = Node::ID_NONE;
		
		if( bShow )
		{
        if(INVALID_HOBJ != hWin)
        {
		WsDrawMainSoftKey(WmGetParentWindow(hWin));
		WsShowMainSoftKey(WmGetParentWindow(hWin));
		WmDrawWindow(hWin, TRUE );
		WmShowWindow(hWin);
	}
}
}
}

bool NativeComboField::isActivated() const
{
	return WmIsWindowValid(m_handleWin);
}

void NativeComboField::unsetFocusedNode()
{
	if(m_pDocument)
	{
		if(isActivated())
		{
			FrameView* view = m_pDocument->frame()->view();

			while (view->parent())
			{
				view = static_cast<FrameView*>(view->parent());
			}
			view->update();
		}

		m_pDocument->page()->focusController()->setFocusedNode(0, m_pDocument->frame());
	}
}

void NativeComboField::SetSelectedIndex()
{
	ASSERT(INVALID_HOBJ != m_handleWin);
	NativeComboField* pNativeCombo = NativeComboField::getNativeComboFieldInstance();
	if(isActivated())
	{
		int selectedIndex = WsGetComboBoxCurSelItemIndex(m_handleWin);

		if(WsGetComboBoxItemEnabled(m_handleWin, selectedIndex))
		{
			getElement()->setSelectedIndex(selectedIndex, true, true);
			
			// We send onChange() only for combo-box from setSelectedIndex
			// so explicitly call onChange for a list box
			if(isActivated() && !getElement()->usesMenuList())
			{
				getElement()->onChange();
			}
		}
	}

}

#ifdef SFB_GLOBAL_STATIC_HANDLING
void initNativeComboFieldGlobals()
{
    g_pNativeField = NULL;
}
#endif

}  // namespace WebCore


#endif //#ifdef USE_NATIVE_WIDGET_SUPPORT

