/*
 * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "SharedTimer.h"

#include "Page.h"
#include "SystemTime.h"
#include "Widget.h"
#include <wtf/Assertions.h>
#include "emtimer.h"

#define WEBKIT_SHARED_TIMER_NAME		("WebKitSharedTimer")

namespace WebCore {

static void (*sharedTimerFiredFunction)();
static HTimer hSharedTimer = INVALID_HOBJ;
static void	shared_timer_cb( HTimer hTimer, ULONG param );

void setSharedTimerFiredFunction(void (*f)())
{
	sharedTimerFiredFunction = f;
}

void setSharedTimerFireTime(double fireTime)
{
    ASSERT(sharedTimerFiredFunction);

    double interval = fireTime - currentTime();
    unsigned int intervalInMS;

    if (interval < 0)
	{
		//Shp timers do not support 0 millseconds timer thus taking it to 1 milliseconds
        intervalInMS = 1;
	}
    else
	{
        interval *= 1000;
        intervalInMS = (unsigned int)interval;

		//Shp timers do not support 0 millseconds timer thus taking it to 1 milliseconds
		if( 0 == intervalInMS)
		{
			intervalInMS = 1;
		}
    }

    stopSharedTimer();

	if(INVALID_HOBJ == hSharedTimer)
	{
	hSharedTimer = WmCreateTimer( (char *) WEBKIT_SHARED_TIMER_NAME);
	if(INVALID_HOBJ == hSharedTimer)
	{
		ASSERT(!"Creation of shared timer failed");
		return;
	}
	}
	WmStartTimer(hSharedTimer, intervalInMS, shared_timer_cb, 0);
}

bool isSharedTimerValid()
{
    return (hSharedTimer != INVALID_HOBJ);
}

void stopSharedTimer(bool destroy)
{
	if( hSharedTimer != INVALID_HOBJ )
	{
		WmCancelTimer( hSharedTimer );
		if(true == destroy)
		{
		WmDeleteTimer( hSharedTimer );		
		hSharedTimer = INVALID_HOBJ;
	}
}
}

static void shared_timer_cb( HTimer hTimer, ULONG param )
{
    if (sharedTimerFiredFunction)
        sharedTimerFiredFunction();
}

#ifdef SFB_GLOBAL_STATIC_HANDLING
void initSharedTimerShpGlobals()
{
    sharedTimerFiredFunction = 0;
    hSharedTimer = INVALID_HOBJ;
}
#endif

}
