/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		TextCodeSHP.cpp
 * @brief		Used by Jasmine Browser to encode and decode across different codecs and encodings.
 * @author		Zafar Hussain (zafar.h@samsung.com)
 * @version		1.0
 *
 */
#include "config.h"
#include "ShpJasmine.h"

#if USE(SHP_UNICODE)
#include "GWES.h"
#include "TextCodecSHP.h"
#include "PlatformString.h"
#include "CString.h"
#include "UnicodeSHP.h"
#include"TextEncodingRegistry.h" 
#include "allocs.h" 
 
namespace WebCore {

static std::auto_ptr<TextCodec> newTextCodecSHP(const TextEncoding& encoding, const void*)
{
	return std::auto_ptr<TextCodec>(new TextCodecSHP(encoding));
}		

void TextCodecSHP::registerBaseEncodingNames(EncodingNameRegistrar registrar)
{
    registrar("UTF-8", "UTF-8");

	registrar("ISO-8859-1", "ISO-8859-1");
	registrar("ISO-8859-2", "ISO-8859-2");
	registrar("ISO-8859-3", "ISO-8859-3");
	registrar("ISO-8859-4", "ISO-8859-4");
	registrar("ISO-8859-5", "ISO-8859-5");
	registrar("ISO-8859-7", "ISO-8859-7");
	registrar("ISO_8859-7:1987", "ISO-8859-7");
	registrar("ISO-8859-7", "ISO-8859-7");
	registrar("ISO-8859-8", "ISO-8859-8");
	registrar("ISO-8859-15", "ISO-8859-15");

	registrar("windows-1251", "windows-1251");
    registrar("windows-1253", "ISO-8859-7");
	registrar("windows-1255", "windows-1255");
	registrar("windows-1256", "windows-1256");
	registrar("koi8-r", "koi8-r");
	registrar("koi8-u", "koi8-u");
	registrar("UTF-32BE","UTF-32");
	registrar("UTF-32LE","UTF-32");
}

void TextCodecSHP::registerBaseCodecs(TextCodecRegistrar registrar)
{
    registrar("UTF-8", newTextCodecSHP, 0);
	registrar("ISO-8859-1", newTextCodecSHP, 0); 
	registrar("ISO-8859-2", newTextCodecSHP, 0); 
	registrar("ISO-8859-3", newTextCodecSHP, 0); 
	registrar("ISO-8859-4", newTextCodecSHP, 0); 
	registrar("ISO-8859-5", newTextCodecSHP, 0); 
	registrar("ISO-8859-7", newTextCodecSHP, 0); 
	registrar("ISO-8859-8", newTextCodecSHP, 0); 
	registrar("ISO-8859-15", newTextCodecSHP, 0);  
	registrar("windows-1251", newTextCodecSHP, 0);	 
	registrar("windows-1253", newTextCodecSHP, 0);	 
	registrar("windows-1255", newTextCodecSHP, 0);	 
	registrar("windows-1256", newTextCodecSHP, 0);	 
	registrar("koi8-r", newTextCodecSHP, 0);	 
	registrar("koi8-u", newTextCodecSHP, 0);	 	
	registrar("UTF-32LE", newTextCodecSHP, 0);	 	
	registrar("UTF-32BE", newTextCodecSHP, 0);	 	
		
		
}

static std::auto_ptr<TextCodec> registerExtendedCodecs(const TextEncoding& encoding, const void*)
{
	return std::auto_ptr<TextCodec>(new TextCodecSHP(encoding));
}

/*Currently SHP has limited codecs like for ex: UTF8 encoding etc;
  Check TextCodecICU.cpp for further encoding types. 
  If any new encoding type needs to be added, then it has to be registered below 
  and corresponding codec has to be defined in registerExtendedCodecs
*/
void TextCodecSHP::registerExtendedEncodingNames(EncodingNameRegistrar registrar)
{
		registrar("ISO-8859-8-I", "ISO-8859-8-I");
	//	registrar("ISO-8859-1", "ISO-8859-1");
	

}

void TextCodecSHP::registerExtendedCodecs(TextCodecRegistrar registrar)
{
	/*Check this one again. Currently I suppose newTextCodecSHP supports only UTF8 encoding.*/
	registrar("ISO-8859-8-I", newTextCodecSHP, 0);
//	registrar("ISO-8859-1", newTextCodecSHP, 0);
}


TextCodecSHP::TextCodecSHP(const TextEncoding& encoding)
    : m_encoding(encoding)
{
   	/*Implement me if reqd. Implementation in QT Unicode is as below*/
	// m_codec = QTextCodec::codecForName(m_encoding.name());
}

TextCodecSHP::~TextCodecSHP()
{
	/*Need to do something ??*/
}


String TextCodecSHP::decode(const char* bytes, size_t length, bool flush)
{
	const char* name = atomicCanonicalTextEncodingName(m_encoding.name());
	int len;
	if(length<=0||AcStrlen(bytes)<=0)
	{
		String Temp((UChar*)NULL);
		return Temp;
	}

    // Do a preck for both in text and out text + null chars
	OOM_PRE_CHECK( ((length+1) * sizeof(char)  ) + ( (sizeof(char)*(length+1) +1)* sizeof(MCHAR)*2),true )
    
	char *pszInText=(char*)calloc(1,(sizeof(char)*(length+1)));
    if(!pszInText || MemoryManager::IsOOMSent())	
    {
        free(pszInText);
	    String Temp((UChar*)NULL);
	    return Temp;
    }
	pszInText[length]='\0';
	memcpy((void*)pszInText,(void*)bytes,length);	
/**************************TO FIX NULL CHARACTER ISSUE*****************/
int tlen=AcStrlen( (char *)pszInText );
int k=1,j;
while((tlen+k)<length)
{
		//AcStrmcpy((char*)(pszInText+tlen),(char*)bytes, (tlen+k),(length-tlen-k));
		for(j=0;j<(length-tlen-k);j++)
		{
			pszInText[tlen+j]=bytes[tlen+j+k];	
		}
		tlen=AcStrlen( (char *)pszInText );
		k++;//becoz everytime this runs, the src string has one extra character that needs to be skipped.
}
/*****************************************************************************/





//	{	
		/****************IMPORTANT********************************************
		THIS CODE IS WRITTEN ASSUMING THAT THE LOCAL CODE IS UTF-8. 
		THIS ASSUMPTION IS TRUE ONLY IN UMTS VERSION PHONES.
		****************IMPORTANT********************************************/
		int ch=0;
		//len=WmGetPCodeSize((char *)bytes);
		//len=(length)*(3*sizeof(MCHAR));//len gives the number of bytes reserved.
		len=sizeof(MCHAR)*AcStrlen( (char *)pszInText );
		len=len+sizeof(MCHAR);//For NULL padding
		MCHAR *pszOutText = (MCHAR *) calloc(1,len);
        if(!pszOutText || MemoryManager::IsOOMSent())
	    {
			free(pszInText);
            free(pszOutText);
		    String Temp((UChar*)NULL);
		    return Temp;
	    }
		if(!AcStrcmp(name,"UTF-8"))//If the encoding is UTF-8, then enter the if loop.
			 ch=WmConvert2PCode(pszOutText, len,(char*)pszInText);
		else if(!AcStrcmp(name,"ISO-8859-1"))//If the encoding is UTF-8, then enter the if loop.
			 ch=WmConvertLatinCode2PCode(pszOutText, len,(char*)pszInText);
		else if(!AcStrcmp(name,"ISO-8859-2"))//If the encoding is UTF-8, then enter the if loop.
			 ch=WmConvertLatin2Code2PCode(pszOutText, len,(char*)pszInText);
		else if(!AcStrcmp(name,"ISO-8859-3"))//If the encoding is UTF-8, then enter the if loop.
			 ch=WmConvertLatin3Code2PCode(pszOutText, len,(char*)pszInText);
		else if(!AcStrcmp(name,"ISO-8859-4"))//If the encoding is UTF-8, then enter the if loop.
			 ch=WmConvertLatin4Code2PCode(pszOutText, len,(char*)pszInText);
		else if(!AcStrcmp(name,"ISO-8859-5"))//If the encoding is UTF-8, then enter the if loop.
			 ch=WmConvertLatin5Code2PCode(pszOutText, len,(char*)pszInText);
		else if(!AcStrcmp(name,"ISO-8859-7"))//If the encoding is UTF-8, then enter the if loop.
			 ch=WmConvertLatin7Code2PCode(pszOutText, len,(char*)pszInText);		
		else if(!AcStrcmp(name,"ISO-8859-8"))//If the encoding is UTF-8, then enter the if loop.
			 ch=WmConvertLatin8Code2PCode(pszOutText, len,(char*)pszInText);		
		else if(!AcStrcmp(name,"ISO-8859-15"))//If the encoding is UTF-8, then enter the if loop.
			 ch=WmConvertLatin15Code2PCode(pszOutText, len,(char*)pszInText);
		else if(!AcStrcmp(name,"windows-1251"))//If the encoding is UTF-8, then enter the if loop.
				ch=WmConvertWin1251Code2PCode(pszOutText, len,(char*)pszInText);
		else if(!AcStrcmp(name,"windows-1255"))//If the encoding is UTF-8, then enter the if loop.
			 ch=WmConvertWin1255Code2PCode(pszOutText, len,(char*)pszInText);
		else if(!AcStrcmp(name,"windows-1256"))//If the encoding is UTF-8, then enter the if loop.
			 ch=WmConvertWin1256Code2PCode(pszOutText, len,(char*)pszInText);
		
		else if(!AcStrcmp(name,"koi8-r"))//Russian encoding
			 ch=WmConvertKoi8rCode2PCode(pszOutText, len,(char*)pszInText);

		else if(!AcStrcmp(name,"koi8-u"))//ukranian encoding
			 ch=WmConvertKoi8uCode2PCode(pszOutText, len,(char*)pszInText);
#if 0
		else if(!AcStrcmp(name,"/*TBD*/"))//TBD. Dont know the name of the encoding
			 ch=WmConvertCyrillicCode2PCode(pszOutText, len,(char*)pszInText);
#endif 

		else
		{
			String Temp((UChar*)NULL);
			free(pszInText);
			free(pszOutText);
			return Temp;
		}	

		int i=WmStrlen(pszOutText);
		
		if(ch==0||i<=0)
		{
			String Temp((UChar*)NULL);
			free(pszInText);
			free(pszOutText);
			return Temp;
		}
	
	//HACK DONE TO SOLVE ibibo & Danske Bank issues:
	/**********************HACK BEGINS********************/
	if(!AcStrcmp(name,"UTF-8"))//If the encoding is UTF-8, then enter the if loop.
	{
		if(((int)*pszOutText)==65279)
		{		
			String Temp((UChar*)(pszOutText+1),i-1);
			free(pszInText);
			free(pszOutText);
			return Temp;
		}
	}
	/**********************HACK ENDS********************/

		
		String Temp((UChar*)pszOutText,i);
		free(pszInText);
		free(pszOutText);
		return Temp;
  OOM_POST_CHECK_FAILED( return String(); )	
}
	
CString TextCodecSHP::encode(const UChar* characters, size_t length, bool allowEntities)
{
	int len;

    if (!length)
        return "";

	len = WTF::Unicode::calcLengthForEncode((UChar *) characters, length);
	char *pszOutText = (char *) malloc(len);
	//WTF::Unicode::ConvertToSingleByteCharacterString(pszOutText,len,(UChar*)characters,(length));	
	WmConvertPCode2UTF((unsigned char *)pszOutText,len,(MCHAR*)characters,length);
	CString Temp(pszOutText,len-1);

	free(pszOutText);
	return Temp;
}

} // namespace WebCore
#endif 
