#ifndef CONTENTLISTENER_H
#define CONTENTLISTENER_H

#include "BridgeTypes.h"
#include "CMMDefs.h"

// INCLUDES

// CONSTANTS

// MACROS

// DATA TYPES

// FUNCTION PROTOTYPES

// FORWARD DECLARATIONS
namespace WebCore {
    class ResourceLoader;
};
class UrlResponseInfo;

// CLASS DECLARATION

/**
*  This is an interface class to pass transaction events
*  @lib resLoader.lib
*/
class IContentListener CMM_MODIFIED
    {
    public: // New functions

        /**
        * Destructor
        */
        virtual ~IContentListener() { }

        /**
        * This function returns the resource loader
        * @return WebCore::ResourceLoader returns the stored resource loader
        */
        virtual WebCore::ResourceLoader* GetResourceLoader() const { return 0; }

        /**
        * This function is called when the header information is available
        * @param _transactionId Observed transaction id.
        * @param _headers Header information
        * @return ErrNotSupported if it is not a native browser content
        */
        virtual eErrorCode Headers( int _transactionId, UrlResponseInfo* _headers ) = 0;

        /**
        * This function is called when a new partial body is available
        * @param _transactionId Observed transaction id.
        * @param _response Response information
        * @return
        */
        virtual void Response( int _transactionId, UrlResponseInfo* _response ) = 0;

        /**
        * This function is called when the transaction is complete
        * @param _transactionId Observed transaction id.
        * @param _error HTTP error (404) / cancel error
        * @return
        */
        virtual void Complete( int _transactionId, UrlResponseInfo* _response, eErrorCode _error ) = 0;

        /** 
        * This function is called when an error happens to the transaction.
        * @param _transactionId Observed transaction id.
        * @param _error HTTP error
        * @return
        */
        virtual void HandleError( int _transactionId, int _error ) = 0;

        //<TODO> : implement Ref counting
    };

#endif      // CONTENTLISTENER_H

// End of File

