// INCLUDE FILES
#include <wtf/Vector.h>
#include "PostDataParts.h"

// EXTERNAL DATA STRUCTURES

// EXTERNAL FUNCTION PROTOTYPES  

// CONSTANTS
// The post is transmitted in parts. A buffer is allocated of max size to
// contain the next part to be transmitted. 

// MACROS

// LOCAL CONSTANTS AND MACROS

// MODULE DATA STRUCTURES

// LOCAL FUNCTION PROTOTYPES

// FORWARD DECLARATIONS

// ============================= LOCAL FUNCTIONS ===============================


// ============================ MEMBER FUNCTIONS ===============================

// -----------------------------------------------------------------------------
// class members initialization
// -----------------------------------------------------------------------------
//
PostDataParts::PostDataParts(
	const Vector<WebCoreFormDataItem>& _data)
    : m_firstPart( true ),
      m_dataPart( NULL )
	{
	
	//iterate thru the post items and create PostDataItem instance for each data
	for(int index  = 0 ; index < _data.size(); index++)
		{
		PostDataItem* postDataItem = new PostDataItem(_data[index]);
		if(postDataItem)
			{
			m_postDataItems.append(postDataItem);
			// get the size of the data and add to the total size 
			m_totalSize += postDataItem->PendingContentSize();
			}
		}
	}

// -----------------------------------------------------------------------------
// destructor
// -----------------------------------------------------------------------------
//
PostDataParts::~PostDataParts()
	{
    for( int i = 0; i < m_postDataItems.size(); i++)
        {
        delete m_postDataItems[i];
        }
	m_postDataItems.clear();
	}


// -----------------------------------------------------------------------------
// returns total size of the content to be posted
// -----------------------------------------------------------------------------
//
int PostDataParts::TotalSize()
	{
	return m_totalSize ;
	}



// -----------------------------------------------------------------------------
// returns the next part of content to be posted, returns TRUE if it is the last content
// -----------------------------------------------------------------------------
//
bool PostDataParts::GetNextDataPart(
	char* _dataPart)
	{
	return m_lastPart;
	}


// -----------------------------------------------------------------------------
// resets the state of the object
// -----------------------------------------------------------------------------
void PostDataParts::Reset()
	{
	m_firstPart = true;
	m_postItemIndex = 0;
	m_lastPart = 0;
	//iterate thru the post items and create CPostDataItem instance for each data
	for(int index  = 0 ; index < m_postDataItems.size(); index++)
		{
		PostDataItem* postDataItem = m_postDataItems.at(index);		
		postDataItem->Reset();
		}
	Release();
	}

// -----------------------------------------------------------------------------
// invoked by HTTP post data supplier to release last posted part
// -----------------------------------------------------------------------------
void PostDataParts::Release()
	{
	delete m_dataPart;
	m_dataPart=NULL;
	}

// End of File

