    /*
* ==============================================================================
*  Name        : FileTransaction.cpp
*  Part of     : WebKit / ResourceLoader
*  Interface   : 
*  Description : Implementation of FileTransaction
*  Version     : %Version%
*/

// INCLUDE FILES
#include "FileTransaction.h"
#include "HttpLoader.h"
#include "UrlResponseHeaderInfo.h"
#include "TransactionManager.h"
#include "CString.h"
#ifdef __BROWSER_SDK
#include "webkitloader.h"
#endif

// EXTERNAL DATA STRUCTURES

// EXTERNAL FUNCTION PROTOTYPES  

// CONSTANTS

// MACROS

// LOCAL CONSTANTS AND MACROS

// MODULE DATA STRUCTURES

// LOCAL FUNCTION PROTOTYPES

// FORWARD DECLARATIONS

// ============================= LOCAL FUNCTIONS ===============================


// ============================ MEMBER FUNCTIONS ===============================

// -----------------------------------------------------------------------------
// FileTransaction::FileTransaction
// C++ default constructor 
// -----------------------------------------------------------------------------
//
FileTransaction::FileTransaction( 
    UrlRequestInfo* _urlRequestInfo,
    ITransactionObserver* _transactionObserver)
    : BaseTransaction( _urlRequestInfo, _transactionObserver ),
      m_transCompleted( FALSE ),
      m_isRedirected( FALSE ),
      m_handle( NULL )
    {
    }


// Destructor
FileTransaction::~FileTransaction()
    {    
    }

// -----------------------------------------------------------------------------
// FileTransaction::DoSubmit
// 
// 
// -----------------------------------------------------------------------------
// 
eErrorCode FileTransaction::DoSubmit()
    {
    // <TODO> : Do a cache check before submitting the transaction
    TransactionManager::sharedInstance()->add(this);
    return ERR_OK;
    }

// -----------------------------------------------------------------------------
// FileTransaction::CancelTransaction
// 
// 
// -----------------------------------------------------------------------------
//
void FileTransaction::CancelTransaction( 
    eErrorCode _error )
    {
    // cancel 
    TransactionManager::sharedInstance()->cancel(this);
    // 
    BaseTransaction::CancelTransaction( _error );
    }

// -----------------------------------------------------------------------------
// FileTransaction::CreateHeaderResponse
// <TODO>
// 
// -----------------------------------------------------------------------------
//
void FileTransaction::CreateHeaderResponse()
    {
    UrlResponseHeaderInfo* header = new UrlResponseHeaderInfo();      
    //
    header->SetTransId( m_transactionId );
    //
    header->SetTopLevel( m_urlRequestInfo->TopLevel() );    
    //
    header->SetChildFrameLoad( m_urlRequestInfo->ChildFrameLoad() );
    
    //add back the fragment if it exists
    if(header->ResponseUrl())
        {
        char* buf = strdup(header->ResponseUrl());
        if(m_fragment)
            {
            strcat(buf, "#");
            strcat(buf, m_fragment);
            }
        header->SetResponseUrl( buf );
        }
    header->SetRequestUrl( m_urlRequestInfo->Url() );
    
    //transfer the ownership to response info
    m_responseInfo->SetHeaderInfo(header);
    }
    
// -----------------------------------------------------------------------------
// FileTransaction::HandleGotResponseHeaders
// 
// Blank implementation since file requests doesn't get any header callbacks
// -----------------------------------------------------------------------------
//
void FileTransaction::HandleGotResponseHeaders( ) 
    {
    }

// -----------------------------------------------------------------------------
// FileTransaction::HandleResponseBodyData
// 
// 
// -----------------------------------------------------------------------------
//
void FileTransaction::HandleResponseBodyData(void* _body, int _len)
    {    
    // get it from the transaction
    //
    AddBodyData( _body, _len );
    }

// -----------------------------------------------------------------------------
// FileTransaction::AddRequestHeaders
// Add headers to the request as required by the WebKit.
// -----------------------------------------------------------------------------
//
void FileTransaction::AddRequestHeaders()
    {
    // referer header
    if( m_urlRequestInfo->SendRefererHeaderEnabled() && m_urlRequestInfo->RefererUrl() )
        {
        String ref("Referer");
        String refUrl(m_urlRequestInfo->RefererUrl());
        m_urlRequestInfo->SetRequestHeaderField( ref, refUrl);
        }

    String acceptHeaders( m_urlRequestInfo->AcceptHeader() );
    // accept header
    if( acceptHeaders.length() )
        {
        String strAccept("Accept");
        m_urlRequestInfo->SetRequestHeaderField( strAccept, acceptHeaders);
        }
    }

//  End of File  

