    /*
* ==============================================================================
*  Name        : HttpTransaction.cpp
*  Part of     : WebKit / ResourceLoader
*  Interface   :
*  Description : Implementation of HttpTransaction
*  Version     : %Version%
*/

// INCLUDE FILES
#include "types.h"
#include "HttpTransaction.h"
#include "HttpLoader.h"
#include "UrlResponseHeaderInfo.h"
#include "CString.h"

#include "TransactionManager.h"


#ifdef __CMM__
#include <AnsiCLib.h>
#include <allocs.h>
#else
#include "Shp.h"
#endif

#ifdef USE_CURL
typedef class TransactionManagerCURL TransactionManager;
#endif


#ifdef __BROWSER_SDK
#include "webkitloader.h"
#endif

// EXTERNAL DATA STRUCTURES

// EXTERNAL FUNCTION PROTOTYPES

// CONSTANTS

// MACROS

// LOCAL CONSTANTS AND MACROS

// MODULE DATA STRUCTURES

// LOCAL FUNCTION PROTOTYPES

// FORWARD DECLARATIONS

// ============================= LOCAL FUNCTIONS ===============================


// ============================ MEMBER FUNCTIONS ===============================

// -----------------------------------------------------------------------------
// HttpTransaction::HttpTransaction
// C++ default constructor
// -----------------------------------------------------------------------------
//
HttpTransaction::HttpTransaction(
    UrlRequestInfo* _urlRequestInfo,
    ITransactionObserver* _transactionObserver)
    : BaseTransaction( _urlRequestInfo, _transactionObserver ),
      m_handle( NULL )
    {
    }


// Destructor
HttpTransaction::~HttpTransaction()
    {
    }

// -----------------------------------------------------------------------------
// HttpTransaction::DoSubmit
//
//
// -----------------------------------------------------------------------------
//
eErrorCode HttpTransaction::DoSubmit()
    {
    // <TODO> : Do a cache check before submitting the transaction
    TransactionManager::sharedInstance()->add(this);
    return ERR_OK;
    }

// -----------------------------------------------------------------------------
// HttpTransaction::CancelTransaction
//
//
// -----------------------------------------------------------------------------
//
void HttpTransaction::CancelTransaction(
    eErrorCode _error )
    {
    // cancel
    TransactionManager::sharedInstance()->cancel(this);
    //
    BaseTransaction::CancelTransaction( _error );
    }

// -----------------------------------------------------------------------------
// HttpTransaction::CreateHeaderResponse
// <TODO>
//
// -----------------------------------------------------------------------------
//
void HttpTransaction::CreateHeaderResponse()
    {
    UrlResponseHeaderInfo* header = new UrlResponseHeaderInfo();
    //
    header->SetTransId( TransactionId() );
    //
    header->SetTopLevel( m_urlRequestInfo->TopLevel() );

    //add back the fragment if it exists
    if(header->ResponseUrl())
    {
        char* buf = (char *)malloc(AcStrlen(header->ResponseUrl()) + 1);
        AcStrcpy(buf, header->ResponseUrl());

        if(m_fragment)
            {
            AcStrcat(buf, "#");
            AcStrcat(buf, m_fragment);
            }
        header->SetResponseUrl( buf );
    }
	else
	{
        header->SetResponseUrl( m_urlRequestInfo->CurrentUrl() );
	}
    header->SetRequestUrl( m_urlRequestInfo->CurrentUrl() );

    //transfer the ownership to response info
    m_responseInfo->SetHeaderInfo(header);
    }

// -----------------------------------------------------------------------------
// HttpTransaction::HandleGotResponseHeaders
//
//
// -----------------------------------------------------------------------------
//
void HttpTransaction::HandleGotResponseHeaders( )
    {
    // create header object
    UrlResponseHeaderInfo* header = m_responseInfo->HeaderInfo();

    // set http status code
    header->SetHttpStatus( header->HttpStatus() );

    // set content type string
    String cT("Content-type");
    String contentType( header->GetResponseHeaderField(cT) );
    header->SetContentType( contentType.utf8().data() );

    // set the content location string
    String cL("Content-Location");
    String contentLocation( header->GetResponseHeaderField(cL));
    header->SetContentLocation( contentLocation.utf8().data() );

    // set content length
    String cLen("Content-Length");
    String contentLength( header->GetResponseHeaderField(cLen));
    header->SetContentLength( contentLength.toInt() );

    // get all the headers
    HTTPHeaderMap headers = header->ResponseHeaders();
    header->SetResponseHeaders(headers);

	// shkim
	// There are bug here, because BaseTransaction::HandleError always return FALSE.
	// So it will crashed in  TransactionComplete( ERR_CANCEL ).
	// If RM meet some error while handling HTTP 401, RM don't need to call header callback.
	// In this case RM just call error callback.

    //bool handled = FALSE;
    //if( ( m_responseInfo->HeaderInfo()->HttpStatus() == HTTP_UNAUTHORIZED ) ||
    //    ( m_responseInfo->HeaderInfo()->HttpStatus() == HTTP_PROXY_AUTH_REQUIRED ) )
    //    {
    //    handled = BaseTransaction::HandleError( m_responseInfo->HeaderInfo()->HttpStatus() );
    //    // trans is cancelled at this point. if
    //    // authentication is failed (not handled) then
    //    // just complete this transaction
    //    if( !handled )
    //        {
    //        //
    //        TransactionComplete( ERR_CANCEL );
    //        return;
    //        }
    //    }

    m_transactionObserver->Headers( this, m_responseInfo );
    }

// -----------------------------------------------------------------------------
// HttpTransaction::HandleResponseBodyData
//
//
// -----------------------------------------------------------------------------
//
void HttpTransaction::HandleResponseBodyData(void* _body, int _len)
{
    AddBodyData( _body, _len );
}

//  End of File

