#ifndef USE_CURL

#include "types.h"
#include "TransactionManager.h"
#include "CString.h"
#include "BaseTransaction.h"
#include "HttpTransaction.h"
#include "HTTPParsers.h"
#include "ShpJasmine.h"
//#include "BrPlDebug.h"//sfb_re_eng

#include "WebControl.h"

#ifdef __CMM__
#include <allocs.h>
#endif

#if defined(_SFB_PERF_LOG)		
#include "SFBalProfiler.h"
#endif

#ifdef HTTP_POST_UPLOAD_SUPPORT
#define POST_DATA_MAX_LENGTH (512*1024)
#endif

using namespace WebCore;

namespace WebCore
{
    String decodeURLEscapeSequences(const String& str);
    bool base64Decode(const char*, unsigned, Vector<char>&);
}


const double defaultPollTimeSec = 0.05;
const double retryPollTimeSec = 0.5; // Retry in case of queue full.

static TransactionManager* objSharedInstance = 0;

#ifdef SFB_GLOBAL_STATIC_HANDLING
void initTransactionManagerGlobals()
{
	objSharedInstance = NULL;
}
#endif

TransactionManager::TransactionManager()
    : m_downloadTimer(this, &TransactionManager::downloadTimerCallback)
    , m_TransactionListHead(0)
    , m_TransactionListTail(0)
    , m_pollTimeSec(defaultPollTimeSec)
{
}

TransactionManager::~TransactionManager()
{
}

// We need to move this code to the destructor once the code for 
// deleting the statics on exiting the browser is in place.
void TransactionManager::StopTimer()
{
	if (objSharedInstance)	
	{
		objSharedInstance->m_downloadTimer.restart() ;
		objSharedInstance->m_downloadTimer.stop() ;
	}
}

TransactionManager* TransactionManager::sharedInstance()
{
    if (!objSharedInstance)
        objSharedInstance = new TransactionManager();
    return objSharedInstance;
}

void TransactionManager::removeTransactionMgr()
{
    if (objSharedInstance)
	{
        delete objSharedInstance;
	}
    objSharedInstance = NULL;
}


// called with data after all headers have been processed via headerCallback
static void SFBRmHandleData( SFBRmHClient hClient, SFBRmReqId reqID,
                                      void * appData,
                                      char* respBody,
                                      int respBodyLen,
                                      BOOL bLastChunk)
{

    BaseTransaction* job = static_cast<BaseTransaction*>(appData);

    if (job)
    {

#if defined(_SFB_PERF_LOG)		
		KURL kurl = job->GetUrlRequestInfo()->CurrentUrl();
		SFBalProfileStart( "Transaction", (char*)kurl.string().latin1().data() );
#endif

		bool bTransCancelled = false;
		if((NULL != respBody) && (0 != respBodyLen))	//Ancil
        {
            // Lock the job
            job->Ref();
            // Send the response body
            job->HandleResponseBodyData(respBody, respBodyLen);

			// Get the current transaction status
			bTransCancelled = job->isTransCancelled();

            // Unlock the job
            job->Deref();
        }

        if(bLastChunk && !bTransCancelled)
        {
#ifdef SHP_NETWORK_LOGS
            BrPlAssertDebug( "SFBRmHandleData: Load Complete - reqId = %d, URL = %s\n", 
                                reqID, job->GetUrlRequestInfo()->CurrentUrl() );
#endif // #ifdef SHP_NETWORK_LOGS
            job->TransactionComplete();
        }

#if defined(_SFB_PERF_LOG)		
		//KURL kurl = job->GetUrlRequestInfo()->CurrentUrl();
		SFBalProfileEnd( "Transaction", (char*)kurl.string().latin1().data(), "DataCallback" );
#endif
    }
}

/*
 * This is being called for each HTTP header in the response. This includes '\r\n'
 * for the last line of the header.
 *
 * We will add each HTTP Header to the UrlResponseInfo and on the termination
 * of the header (\r\n) we will pass it to HandleGotResponseHeaders()
 * for parsing of Content-Type and Content-Disposition.
 */
static void SFBRmHandleHeaderWorker( SFBRmHClient hClient, SFBRmReqId reqID,
                                     void * appData,
                                     int statusCode,
                                     SFBRmResponseInd responseType,
                                     char* respHdr,
                                     int respHdrLen)
{

    BaseTransaction* job = static_cast<BaseTransaction*>(appData);

    if (!job) {
        return;
    }

    // Create the response header info
    if(!job->ResponseHeaders()) {
        job->CreateHeaderResponse();

    }

    String header(static_cast<const char*>(respHdr), respHdrLen);

    if ( '\r' == respHdr[0] && '\n' == respHdr[1] ) {
        UrlResponseHeaderInfo* responseHeader = job->ResponseHeaders();
		// shkim
		// Set status code
		responseHeader->SetHttpStatus( (eHttpStatus)statusCode );
        job->HandleGotResponseHeaders();
    }
    else
    {
        int splitPos = header.find(":");
        if (splitPos != -1)
        {
			// Check for redirection
			if( -1 != header.find( "X-SFB-RedirectedUrl" ) )
			{
				job->ResponseHeaders()->SetIsRedirected( true );
				job->ResponseHeaders()->SetRedirectedUrl( header.substring(splitPos+1).stripWhiteSpace().latin1().data() );

#ifdef SFB_UPDATE_URL_BAR  //Pramod - Bresson Model - update Url ComboBox bar
				// Here URL is updated whenever redirection happens
				if (!AcStrcmp(job->GetUrlRequestInfo()->CurrentUrl(), job->GetUrlRequestInfo()->Url()))
				{
					WebControl::WebControl *pWebControl = (WebControl::WebControl*)job->GetUrlRequestInfo()->GetWebControl();
					if (NULL != pWebControl)
					{
						WebControl::IContentsListener* pContentsListener = pWebControl->GetContentListener();
						if (NULL != pContentsListener)
						{
							pContentsListener->SetLinkInBrAppState(job->ResponseHeaders()->GetRedirectedUrl());
						}
					}
				}
#endif

			}
                        
                        /*Gnana Modification for Htttp refresh header */
			if( -1 != header.find( "refresh" ) )
			{
				job->ResponseHeaders()->SetRefresh(header.substring(splitPos+1).stripWhiteSpace().latin1().data());
			}
                        /* end of modification - gnana */
			if(-1 != header.left(splitPos).lower().find("content-type"))
			{
				job->ResponseHeaders()->SetResponseHeaderField(header.left(splitPos).lower(), header.substring(splitPos+1).stripWhiteSpace().lower());
				job->ResponseHeaders()->AddToResponseHeadersString( header.lower() );
			}
			else
			{
            job->ResponseHeaders()->SetResponseHeaderField(header.left(splitPos), header.substring(splitPos+1).stripWhiteSpace());
            job->ResponseHeaders()->AddToResponseHeadersString( header );
        }
    }
}
}



//static SFBResult SFBRmDivideHeaderIntoLines(int status, SFBRmEntity * pRmEntity)
static void SFBRmHandleHeader( SFBRmHClient hClient, SFBRmReqId reqID,
                                     void * appData,
                                     int statusCode,
                                     SFBRmResponseInd responseType,
                                     char* respHdr,
                                     int respHdrLen)
{
	char*	pByteFound				= NULL;
	char* 	tempBuffer				= NULL;
	char* 	ptrToTempBuffer 		= NULL;
	int		byteCount				= 0;
	char*	ptrToNewLine			= NULL;

	//chk if the header is empty
	SFBalDebug(BAL_NET, "SFBRmHandleHeader: Enter\n");

#if defined(_SFB_PERF_LOG)		
	    BaseTransaction* job = static_cast<BaseTransaction*>(appData);
		KURL kurl = job->GetUrlRequestInfo()->CurrentUrl();
		SFBalProfileStart( "Transaction", (char*)kurl.string().latin1().data() );
#endif

	//rohit-2281
	if ( NULL != respHdr )
	{
	ptrToNewLine		= respHdr;

	while(*ptrToNewLine != '\0')
	{
		//chk for \r\n, whether present or not

		pByteFound	= AcStrstr(ptrToNewLine, CRLF);

		if(NULL == pByteFound)
		{
			SFBalDebug(BAL_EXCEPTION, "SFBRmHandleHeader: CRLF not found\n");
			return;

		}

		byteCount	=  pByteFound - ptrToNewLine;

		tempBuffer	= (char*)SFBalMemCalloc(1, byteCount + 1 + 2 );// 1 additional byte for NULL character. 2 for CRLF

		if(NULL == tempBuffer)
		{
			SFBalDebug(BAL_EXCEPTION, "SFBRmHandleHeader: Memory Allocation Failed\n");
			return ;
		}

		AcMemcpy(tempBuffer, ptrToNewLine, byteCount );
		AcStrcat(tempBuffer, CRLF);

		tempBuffer[byteCount +2]	= '\0';

		SFBRmHandleHeaderWorker(	
			hClient,
			reqID, 
			appData,
			statusCode,
			responseType,
			tempBuffer,
			byteCount +2
			);

		SFBalMemFree(tempBuffer);

		tempBuffer = NULL;

		pByteFound		= pByteFound + 2;// pByteFound now points to the first character of the next line

		ptrToNewLine		= pByteFound;


	}	
	}
	tempBuffer = (char*)SFBalMemCalloc(1, AcStrlen(CRLF) + 1);
	AcStrcpy(tempBuffer, CRLF);

	SFBRmHandleHeaderWorker(	
		hClient,
		reqID, 
		appData,
		statusCode,
		responseType,
		tempBuffer,
		AcStrlen(CRLF)
		);

	SFBalMemFree(tempBuffer);

	tempBuffer = NULL;

	SFBalDebug(BAL_NET, "SFBRmHandleHeader: Exit\n");
	
#if defined(_SFB_PERF_LOG)		
		SFBalProfileEnd( "Transaction", (char*)kurl.string().latin1().data(), "HeaderCallback" );
#endif
	
}









static void SFBRmHadleError( SFBRmHClient hClient, SFBRmReqId reqID,
                                     void * appData,
                                     int errorCode)
{
    BaseTransaction* job = static_cast<BaseTransaction*>(appData);

#if defined(_SFB_PERF_LOG)		
		KURL kurl = job->GetUrlRequestInfo()->CurrentUrl();
		SFBalProfileStart( "Transaction", (char*)kurl.string().latin1().data() );
#endif
    job->HandleError((SFBStatus)errorCode);
#if defined(_SFB_PERF_LOG)		
		SFBalProfileEnd( "Transaction", (char*)kurl.string().latin1().data(), "ErrorCallback" );
#endif
    return;
}


static void SFBRmHandleAuthentication ( SFBRmHClient hClient,
											SFBRmReqId reqID,
											void* appData,
											SFBStatus nError,
											int prevFailCount,
											char* szProposedUser,
											char* szProposedPass,
											SFBRmProtectionSpace *pProtectionSpace,
											SFBRmAuthenticationCallbackResponse fpRespCallback
											)
{
	BaseTransaction* job = static_cast<BaseTransaction*>(appData);

	char* pszUrl = job->GetUrlRequestInfo()->CurrentUrl();

	SFBalDebug(BAL_COMMON, "SFBRmHandleAuthentication:RequestedUrl [%s] \n",pszUrl);

	WebControl::WebControl *pWebControl = (WebControl::WebControl*)job->GetUrlRequestInfo()->GetWebControl();

	pWebControl->ReceivedAuthenticationChallenge( hClient, reqID, job->TransactionId(),
					szProposedUser, szProposedPass, pProtectionSpace->pszRealm, pszUrl,fpRespCallback );
}


static void SFBRmHandleCookie ( SFBRmHClient hClient,
											SFBRmReqId reqID,
											void* appData,
											char* szHostName,
											BOOL bCookieExist,
											char* szCookieString,
											void* pCookie,
											SFBRmCookieCallbackResponse fpCallbackResponse
											)
{
	if( NULL != appData )
	{
	BaseTransaction* job = static_cast<BaseTransaction*>(appData);
	WebControl::WebControl *pWebControl = (WebControl::WebControl*)job->GetUrlRequestInfo()->GetWebControl();

	pWebControl->ReceivedCookieAcceptMethod( hClient, reqID, szHostName, bCookieExist, szCookieString, pCookie, fpCallbackResponse );
}
}

#ifdef HTTP_POST_UPLOAD_SUPPORT
static void getHandleForFile(const String& filePath, PlatformFileHandle& handle)
{
	
	String fileName;
	String dir;
	int slashIndex = filePath.reverseFind('/');
	if( -1 != slashIndex )
	{
		fileName = filePath.substring(slashIndex + 1);
		dir = filePath.substring(0, slashIndex + 1);
	}
	
	CString path = openFile(dir.latin1().data(), fileName.latin1().data(), FILE_MODE_READ, handle);
}
#endif

static void SFBRmSendNextPostChunk(void* appData, void* _pReqInfo, char** postBuffer, int* postDataLen)
{
	BaseTransaction* job = static_cast<BaseTransaction*>(appData);
	String postDataStr = job->GetUrlRequestInfo()->GetPostDataStr();
	SFBRmRequestInfo *pReqInfo = (SFBRmRequestInfo *)_pReqInfo;
	int remainLen = 0;
	
	const Vector<FormDataElement>& elements = job->GetUrlRequestInfo()->GetPostDataElemets();

	BaseTransaction::postDataInfo info = job->getPostDataInfo();

	if(info.elementNo >= elements.size())
	{
		return;
	}
	
	FormDataElement element = elements[info.elementNo];
	
	if (element.m_type == FormDataElement::encodedFile)
	{
		char* tempBuffer = NULL;
		unsigned int readBytes = 0;
		long long fileSizeResult = 0;

		PlatformFileHandle fileHandle;

		getHandleForFile(element.m_filename, fileHandle);
		if(fileHandle == INVALID_HOBJ)
		{
			job->HandleError((SFBStatus)SFB_BROWSER_FILE_ERROR);
			return ;
		}		

		bool retVal = getFileSize(element.m_filename, fileSizeResult);
		remainLen = fileSizeResult - info.seekPosition;

		
		if(remainLen > POST_DATA_MAX_LENGTH)
		{					
			*postBuffer = (char*)SFBalMemCalloc(1,POST_DATA_MAX_LENGTH);
			if(NULL == *postBuffer)
			{
				SFBalDebug(BAL_EXCEPTION, "Memory Allocation failure pPostData\n");
				return;
			}

			retVal = readFileWithSeek(fileHandle, *postBuffer, POST_DATA_MAX_LENGTH, &readBytes, info.seekPosition);

			*postDataLen = POST_DATA_MAX_LENGTH;
			info.seekPosition += POST_DATA_MAX_LENGTH;

			job->setPostDataInfo(info);
		}
		else
		{
			*postBuffer = (char*)SFBalMemCalloc(1, remainLen);
			if(NULL == *postBuffer)
			{
				SFBalDebug(BAL_EXCEPTION, "Memory Allocation failure pPostData\n");
				return;
			}

			retVal = readFileWithSeek(fileHandle, *postBuffer, remainLen, &readBytes, info.seekPosition);

			*postDataLen = remainLen;

			info.elementNo++;
			info.seekPosition = 0;
			job->setPostDataInfo(info);

			closeFile(fileHandle);
		}
	}
	else
	{
		if(element.m_data.size() > POST_DATA_MAX_LENGTH)
		{
			*postBuffer = (char*)SFBalMemCalloc(1,POST_DATA_MAX_LENGTH);
			if(NULL == *postBuffer)
			{
				SFBalDebug(BAL_EXCEPTION, "Memory Allocation failure pPostData\n");
				return;
			}
			
			AcStrncpy(*postBuffer, element.m_data.data() + info.seekPosition, POST_DATA_MAX_LENGTH);

			*postDataLen = POST_DATA_MAX_LENGTH;

			info.seekPosition += POST_DATA_MAX_LENGTH;
			job->setPostDataInfo(info);

		}
		else
		{
			*postBuffer = (char*)SFBalMemCalloc(1, element.m_data.size());
			if(NULL == *postBuffer)
			{
				SFBalDebug(BAL_EXCEPTION, "Memory Allocation failure pPostData\n");
				return;
			}

			AcStrncpy(*postBuffer, element.m_data.data(), element.m_data.size());
			
			*postDataLen = element.m_data.size();

			info.elementNo++;
			info.seekPosition = 0;
			job->setPostDataInfo(info);
		}
	}
}

//Anshul
/*
nLenPostedData: data posted in this shot
nLenTotalPostedData: data posted till now
nLenTotalPostData: total size of the file that needs to be posted

Note that earlier nLenTotalPostData was interpreted as the length of the data that remains to be posted


*/
static void SFBRmHandlePostData ( SFBRmHClient hClient,
											SFBRmReqId reqID,
											void* appData,
										//	int nLenPostedData,
											int nLenTotalPostedData,
											int nLenTotalPostData
											)
{
	BaseTransaction* job = static_cast<BaseTransaction*>(appData);
	WebControl::WebControl *pWebControl = (WebControl::WebControl*)job->GetUrlRequestInfo()->GetWebControl();

	const UrlRequestInfo* requestHeader = job->GetUrlRequestInfo();

	String cT("Content-type");
	String contentType( requestHeader->GetRequestHeaderField(cT) );

    if(AcStrstr(contentType.utf8().data(), "multipart/form-data") != NULL)
		pWebControl->ReceivedPostStatus( hClient, reqID, /*nLenPostedData,*/ nLenTotalPostedData, nLenTotalPostData );

}

static BOOL SFBRmHandleRedirection ( SFBRmHClient hClient,
										SFBRmReqId reqID,
										void* appData,
										SFBRmRequestInfo* pRedirectedReq,
										char* pResHeaders,
										int nLenResHeaders,
										SFBRmRedirectCallbackResponse fpCallbackResponse )
{
	int IsRedirect;

	BaseTransaction* job = static_cast<BaseTransaction*>(appData);
	WebControl::WebControl *pWebControl = (WebControl::WebControl*)job->GetUrlRequestInfo()->GetWebControl();

	IsRedirect = pWebControl->ReceivedRedirectionChallenge( hClient, reqID, job->TransactionId(),
					pRedirectedReq, pResHeaders, nLenResHeaders, fpCallbackResponse );

	return IsRedirect;
}


//This API is for SSL Confirmation Pop Up
//This calls the webcontrol function
static void SFBRmSSLConfirmationFunc( SFBRmHClient hClient, 
											SFBRmReqId reqID, 
											void* appData,
											char* pCertData,
											SFBalRmSSLCallbackResponse fpRespCallback
											)
{
	BaseTransaction* job = static_cast<BaseTransaction*>(appData);
	WebControl::WebControl *pWebControl = (WebControl::WebControl*)job->GetUrlRequestInfo()->GetWebControl();

	pWebControl->GetSSLConfirmation( hClient, reqID, job->TransactionId(), pCertData, fpRespCallback );
}

void TransactionManager::downloadTimerCallback(Timer<TransactionManager>* timer)
{
    bool started = startScheduledJobs(); // new jobs might have been added in the meantime

    if (!m_downloadTimer.isActive() && (started))
    {
        m_downloadTimer.startOneShot(m_pollTimeSec);
        // reset the poll timer
        m_pollTimeSec = defaultPollTimeSec;
    }
}

void TransactionManager::stopRMTransation(BaseTransaction* job)
{
    SFBRmReqId reqId = job->GetTransportHandle();
    if(reqId >= 0)
    {
		WebControl::WebControl *pWebControl = (WebControl::WebControl*)job->GetUrlRequestInfo()->GetWebControl();
        SFBRmRequestStop( (int)(pWebControl->GetLoadGroup()),reqId);
        job->SetTransportHandle(0);
    }
}

void TransactionManager::setupPUT(BaseTransaction* job)
{
    // <TODO> : PUT method not implemented
    return;
}

void TransactionManager::parseDataUrl(BaseTransaction* job)
{
    String data = job->GetUrlRequestInfo()->CurrentUrl();

    SysAssert(data.startsWith("data:", false));

    String header;
    bool base64 = false;

    int index = data.find(',');
    if (index != -1) 
    {
        header = data.substring(5, index - 5).lower();
        data = data.substring(index + 1);

        if (header.endsWith(";base64")) 
        {
            base64 = true;
            header = header.left(header.length() - 7);
        }
    } 
    else
        data = String();

    data = decodeURLEscapeSequences(data);

    size_t outLength = 0;
    char* outData = 0;
    if (base64 && !data.isEmpty()) 
    {
        // Need to Use the SHP Base64 if available, since WebCore's decoder isn't
        // general-purpose and fails on Acid3 test 97 (whitespace).
        Vector<char> out;
        if (base64Decode(data.latin1().data(), data.length(), out))
            data = String(out.data(), out.size());
        else
            data = String();
    }
    

    if (header.isEmpty())
        header = "text/plain;charset=US-ASCII";

    SFBRmHandleHeaderWorker( 0, 0, job, 200, SFB_HEADER_IND, (char *)header.latin1().data(), header.length());
    SFBRmHandleData( 0, 0, job, (char *)data.latin1().data(), data.length(), true);
}



void TransactionManager::setupGET(BaseTransaction* job, void* _pReqInfo)
{
	SFBRmRequestInfo *pReqInfo = (SFBRmRequestInfo *)_pReqInfo;

	pReqInfo->method = SFB_HTTP_METHOD_GET;
	AddReqHdrs(job, _pReqInfo);
}

#if 0
void TransactionManager::setupPOST(BaseTransaction* job, void* _pReqInfo)
{
    SFBRmRequestInfo *pReqInfo = (SFBRmRequestInfo *)_pReqInfo;
    SFBRmReqId reqId = job->GetTransportHandle();
    pReqInfo->method = SFB_HTTP_METHOD_POST;
    String postDataStr = job->GetUrlRequestInfo()->GetPostDataStr();

    pReqInfo->pPostData = (char*)calloc(1,postDataStr.length() + 1);
    if(NULL == pReqInfo->pPostData)
    {
        SFBalDebug(BAL_EXCEPTION, "Memory Allocation failure pPostData\n");
    }

#ifdef HTTP_POST_UPLOAD_SUPPORT
	// While uploading file to the server , we dont want to encode/decode the form data as 
	// the form data may contain raw file data( i.e raw data of an image).
	if(postDataStr.contains("filename"))
	{
		const UChar* chars = postDataStr.characters();
		for (unsigned i = 0; i != postDataStr.length(); ++i)
		{
			pReqInfo->pPostData[i] = chars[i];
		}
	}
	else
	{
#endif
    AcStrncpy(pReqInfo->pPostData,postDataStr.latin1().data(), postDataStr.length());
#ifdef HTTP_POST_UPLOAD_SUPPORT
	}
#endif

    pReqInfo->postDataLen = postDataStr.length();

	AddReqHdrs(job, _pReqInfo);
}
#endif

void TransactionManager::setupPOST(BaseTransaction* job, void* _pReqInfo)
{
    SFBRmRequestInfo *pReqInfo = (SFBRmRequestInfo *)_pReqInfo;
    SFBRmReqId reqId = job->GetTransportHandle();
    pReqInfo->method = SFB_HTTP_METHOD_POST;
    String postDataStr = job->GetUrlRequestInfo()->GetPostDataStr();

	const Vector<FormDataElement>& elements = job->GetUrlRequestInfo()->GetPostDataElemets();
	int numElements = elements.size();

	if(numElements > 1)
	{
		// Calculate the total size of all elements
		unsigned totalSize = 0;
		for (size_t i = 0; i < numElements; i++)
		{
			FormDataElement element = elements[i];
			if (element.m_type == FormDataElement::encodedFile)
			{
				long long fileSizeResult;
				if (getFileSize(element.m_filename, fileSizeResult))
				{
					totalSize += fileSizeResult;	
				}			
			}
			else
			{
				totalSize += elements[i].m_data.size();
			}
		}

		BaseTransaction::postDataInfo info;

		info.elementNo = 0;
		info.seekPosition = 0;

		job->setPostDataInfo(info);

		FormDataElement element = elements[info.elementNo];

		if(element.m_data.size() > POST_DATA_MAX_LENGTH)
		{
			pReqInfo->pPostData = (char*)calloc(1,POST_DATA_MAX_LENGTH);
			if(NULL == pReqInfo->pPostData)
			{
				SFBalDebug(BAL_EXCEPTION, "Memory Allocation failure pPostData\n");
				return;
			}

			AcStrncpy(pReqInfo->pPostData, element.m_data.data(), POST_DATA_MAX_LENGTH);

			pReqInfo->postDataLen = totalSize;//Total size of data
			pReqInfo->postDataReceivedLength = POST_DATA_MAX_LENGTH;// Size of data posted in one shot

			info.seekPosition += POST_DATA_MAX_LENGTH;
			job->setPostDataInfo(info);

		}
		else
		{
			pReqInfo->pPostData = (char*)calloc(1, element.m_data.size());
			if(NULL == pReqInfo->pPostData)
			{
				SFBalDebug(BAL_EXCEPTION, "Memory Allocation failure pPostData\n");
				return;
			}

			AcStrncpy(pReqInfo->pPostData, element.m_data.data(), element.m_data.size());

			pReqInfo->postDataLen = totalSize;//Total size of data
			pReqInfo->postDataReceivedLength = element.m_data.size();// Size of data posted in one shot
			
			info.elementNo++;
			job->setPostDataInfo(info);
		}
	}	
	else
	{
		pReqInfo->pPostData = (char*)calloc(1,postDataStr.length() + 1);
		if(NULL == pReqInfo->pPostData)
		{
			SFBalDebug(BAL_EXCEPTION, "Memory Allocation failure pPostData\n");
			return;
		}
		
		AcStrncpy(pReqInfo->pPostData,postDataStr.latin1().data(), postDataStr.length());
		
		pReqInfo->postDataLen = postDataStr.length();//Total size of data
		pReqInfo->postDataReceivedLength = postDataStr.length();// Size of data to be posted in one shot
	}

    AddReqHdrs(job, _pReqInfo);
}

void TransactionManager::AddReqHdrs(BaseTransaction* job, void* _pReqInfo)
{
	SFBRmRequestInfo *pReqInfo = (SFBRmRequestInfo *)_pReqInfo;
    HTTPHeaderMap headerFields = job->GetUrlRequestInfo()->GetRequestHeaders();
    HTTPHeaderMap::const_iterator end = headerFields.end();

    // VDJ: Hack to get aroung \r\n Unicode issues
    // Calculate total length
    String headerString;
    for (HTTPHeaderMap::const_iterator it = headerFields.begin(); it != end; ++it)
    {
        headerString = headerString + it->first + ": " + it->second + "\r\n";
    }

    // Copy data
	HTTPHeaderMap::const_iterator it1 = headerFields.begin();
	if(it1 != end)
	{
    pReqInfo->pRequestHeader = (char*)calloc(1, AcStrlen(headerString.ascii().data()) + 1);
    if(NULL == pReqInfo->pRequestHeader)
    {
        SFBalDebug(BAL_EXCEPTION, "Memory Allocation failure pRequestHeader\n");
    }
    for (HTTPHeaderMap::const_iterator it1 = headerFields.begin(); it1 != end; ++it1)
    {
        AcStrcat(pReqInfo->pRequestHeader, it1->first.latin1().data());
        AcStrcat(pReqInfo->pRequestHeader, ": ");
        AcStrcat(pReqInfo->pRequestHeader, it1->second.latin1().data());
        AcStrcat(pReqInfo->pRequestHeader, "\r\n");
    }
	}
}


void TransactionManager::add(BaseTransaction* job)
{
    TransactionList* tmpTL = new TransactionList(job);

    if(m_TransactionListTail)
        m_TransactionListTail->setNext(tmpTL);

    m_TransactionListTail = tmpTL;

    m_TransactionListTail->setNext(NULL);

    if(NULL == m_TransactionListHead )
        m_TransactionListHead = m_TransactionListTail;

    if (!m_downloadTimer.isActive())
        m_downloadTimer.startOneShot(m_pollTimeSec);
}

bool TransactionManager::removeScheduledJob(BaseTransaction* job)
{
    bool result = false;
   
    TransactionList* curr = m_TransactionListHead;
    TransactionList* prev = m_TransactionListHead;

    while (curr) {
        TransactionList* next = curr->next();

        if (job == curr->job()) 
        {
            if(curr == m_TransactionListHead)
            {
                m_TransactionListHead = next;
                if(!next)
                {
                    m_TransactionListTail = NULL;
                }
            }
            else
            {
                if(next == NULL)
                    m_TransactionListTail = prev;

                prev->setNext(next);
            }
            delete curr;
            result = true;
            break;
        }
        prev = curr;
        curr = next;
    }

    return result;
}

bool TransactionManager::startScheduledJobs()
{
    bool started = false;
    TransactionList* node = m_TransactionListHead;
    while (node) 
    {
        TransactionList* next = node->next();
        if( startJob(node->job()) )
        {
            started = true;
            removeScheduledJob(node->job());
        }
        else
        {
            // startJob failed so just set the list head 
            // we will retry later
            m_pollTimeSec = retryPollTimeSec;
            started = true; // work-around to start the download timer
            break;
        }
        node = next;
    }

    return started;
}

bool TransactionManager::startJob(BaseTransaction* job)
{
    SFBRmReqId reqId;
    SFBRmRequestInfo * pReqInfo = NULL;
    //For data URL we should check for current URL and not main URL CQ FIX:SFB00002661 
    KURL kurl = job->GetUrlRequestInfo()->CurrentUrl();

    if (kurl.protocol().contains("data")) {
        parseDataUrl(job);
        return true;
    }

#if defined(_SFB_PERF_LOG)
	SFBalProfileStart( "Transaction", (char*)kurl.string().latin1().data() );
#endif

    pReqInfo = (SFBRmRequestInfo *) calloc(1, sizeof(SFBRmRequestInfo));
    if(NULL == pReqInfo)
    {
        SFBalDebug(BAL_EXCEPTION, "Memory Allocation failure pReqInfo\n");
    }

    //Set the cache mode 
    if( ReloadIgnoringCacheData == job->GetUrlRequestInfo()->CacheMode() )
    {
        pReqInfo->cacheMode = SFB_CACHE_NOCACHE;
    }
    else if( ReturnCacheDataElseLoad == job->GetUrlRequestInfo()->CacheMode() )
    {
        pReqInfo->cacheMode = SFB_CACHE_HISTORY;
    }
    else if( ReturnCacheDataDontLoad == job->GetUrlRequestInfo()->CacheMode() )
    {
        pReqInfo->cacheMode = SFB_CACHE_ONLYCACHE;
    }
    else // UseProtocolCachePolicy or default case
    {
        pReqInfo->cacheMode = SFB_CACHE_NORMAL;
    }

    pReqInfo->szMainDocURL = (char *) calloc(1, (AcStrlen(job->GetUrlRequestInfo()->Url())+1));
    if(NULL == pReqInfo->szMainDocURL)
    {
        SFBalDebug(BAL_EXCEPTION, "Memory Allocation failure maindocURL\n");
    }
    AcStrcpy(pReqInfo->szMainDocURL, job->GetUrlRequestInfo()->Url());

    pReqInfo->szURL = (char *) calloc(1, (AcStrlen(job->GetUrlRequestInfo()->CurrentUrl()) +1));
    if(NULL == pReqInfo->szURL)
    {
        SFBalDebug(BAL_EXCEPTION, "Memory Allocation failure URL\n");
    }
    AcStrcpy(pReqInfo->szURL, job->GetUrlRequestInfo()->CurrentUrl());

    //<TODO> - Get allow cookies from settings
    pReqInfo->bAllowCookies = TRUE;

	WebControl::WebControl *pWebControl = (WebControl::WebControl*)job->GetUrlRequestInfo()->GetWebControl();

    if (URL_GET == job->GetUrlRequestInfo()->LoadMethod())
    {
		setupGET(job, pReqInfo);
		reqId = SFBRmRequestUrl((int)(pWebControl->GetLoadGroup()),
			pReqInfo,
			job,
			(SFBRmHeaderCallback)SFBRmHandleHeader,
			(SFBRmContentCallback)SFBRmHandleData,
			(SFBRmErrorCallback)SFBRmHadleError,
			NULL, 
			NULL,
			(SFBRmCookieCallback) SFBRmHandleCookie,
			(SFBRmAuthenticationCallback)SFBRmHandleAuthentication,
			(SFBRmSSLConfirmationCallback)SFBRmSSLConfirmationFunc,
			//(SFBRmPostCallback)SFBRmHandlePostData
			NULL,
			(SFBRmSendNextPostChunkCallback)SFBRmSendNextPostChunk
			);
    }
    else if (URL_POST == job->GetUrlRequestInfo()->LoadMethod())
    {
		setupPOST(job, pReqInfo);
		pWebControl->ReceivedPostInit();
		reqId = SFBRmRequestUrl((int)(pWebControl->GetLoadGroup()),
                            pReqInfo,
                            job,
                            (SFBRmHeaderCallback)SFBRmHandleHeader,
                            (SFBRmContentCallback)SFBRmHandleData,
                            (SFBRmErrorCallback)SFBRmHadleError,
			(SFBRmRedirectCallback)SFBRmHandleRedirection, 
							NULL, 
							(SFBRmCookieCallback) SFBRmHandleCookie,
							(SFBRmAuthenticationCallback)SFBRmHandleAuthentication,
							(SFBRmSSLConfirmationCallback)SFBRmSSLConfirmationFunc,
							(SFBRmPostCallback)SFBRmHandlePostData,
							(SFBRmSendNextPostChunkCallback)SFBRmSendNextPostChunk
                            );
	}

    bool retVal = false;

    if( -1 != reqId )
    {
        job->SetTransportHandle( reqId );
        retVal = true;
#ifdef SHP_NETWORK_LOGS
        BrPlAssertDebug( "TransactionManager::startJob : reqId = %d, URL = %s\n", 
                            reqId, job->GetUrlRequestInfo()->CurrentUrl() );
#endif // SHP_NETWORK_LOGS
    }
#ifdef SHP_NETWORK_LOGS
    else
    {

        BrPlAssertDebug( "TransactionManager::startJob : Queue full!!!\n" );
    }
#endif // SHP_NETWORK_LOGS

    if(pReqInfo)
    {
        if(pReqInfo->szMainDocURL)
        {
            free(pReqInfo->szMainDocURL);
            pReqInfo->szMainDocURL = NULL;
        }
        if(pReqInfo->szURL)
        {
            free(pReqInfo->szURL);
            pReqInfo->szURL = NULL;
        }
        if(pReqInfo->pPostData)
        {
            free(pReqInfo->pPostData);
            pReqInfo->pPostData = NULL;
        }
        if(pReqInfo->pRequestHeader)
        {
            free(pReqInfo->pRequestHeader);
            pReqInfo->pRequestHeader = NULL;
        }
        free(pReqInfo);
    }

#if defined(_SFB_PERF_LOG)
	SFBalProfileEnd( "Transaction", (char*)kurl.string().latin1().data(), "Requested" );
#endif
    return retVal;
}

void TransactionManager::cancel(BaseTransaction* job)
{
    if (removeScheduledJob(job))
        return;
    stopRMTransation(job);
    // FIXME: report an error?
}

#endif //USE_CURL
// End of File
