/*
 * Copyright (C) 2007 Holger Hans Peter Freyther
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Apple, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef WEBKIT_SHP_PAGE_H
#define WEBKIT_SHP_PAGE_H

#include "webkitshpdefines.h"
#include "GraphicsContext.h"
#include "BrowserControl_DataTypes.h"
#include "Settings.h"
#include "AtomicString.h"
#include "KURL.h"
#include "WmGrpType.h"//tahseen
#include "SFBRmMgr.h"
#ifdef __cplusplus
extern "C"
{
#endif
// Forward declaration
typedef struct _WebKitShpPagePrivate WebKitShpPagePrivate;

typedef enum {
    WEBKIT_SHP_ACCEPT_NAVIGATION_REQUEST,
    WEBKIT_SHP_IGNORE_NAVIGATION_REQUEST,
    WEBKIT_SHP_DOWNLOAD_NAVIGATION_REQUEST
} WEBKIT_SHP_NAVIGATION_REQUEST_RESPONSE;

// Added enum for file save api
typedef enum {
    WEBKIT_SHP_DIFF_RES,
	WEBKIT_SHP_SAME_NAME_RES_DIFF,
    WEBKIT_SHP_SAME_RES
} WEBKIT_SHP_RES_RESOLUTION;

#if PLATFORM(SHP) && defined(SFB_SSR_RENDERING)
// CELUNITE_SSR_Change_Begin
typedef enum {
    WEBKIT_SSR_MODE_OFF,
    WEBKIT_SSR_MODE_ON
} WEBKIT_SSR_MODES;
// CELUNITE_SSR_Change_End
// CELUNITE_SSR_Change_End
#endif

typedef struct
{
	UrlSchemeHandler	m_urlschemehandler;
	UIStateHandler		m_uistatehandler;	
}GlobalHandler;

typedef struct _WebKitShpPage
{
public:

	_WebKitShpPage() : m_progressHandler(0), m_titleChangedHandler(0), 
	m_linkHoveringHandler(0), m_statusbarTextChangedHandler(0), 
	m_iconLoadedHandler(0), m_selectionChangedHandler(0),isLoadingDone(false), privatePage(NULL)
	{
	}

	WebKitShpPagePrivate* privatePage;

	//------------------------------------------------------------------------
	// Callbacks for various signals.
	//------------------------------------------------------------------------
	ProgressHandler					m_progressHandler;
	TitleChangedHandler				m_titleChangedHandler;
	LinkHoveringHandler				m_linkHoveringHandler;
	StatusbarTextChangedHandler		m_statusbarTextChangedHandler;
	IconLoadedHandler				m_iconLoadedHandler;
	SelectionChangedHandler			m_selectionChangedHandler;

	bool isLoadingDone;
	WebCore::String					m_openerWindowUrl;

	// shkim
	SFBRmHClient m_hRmClient;
	SFBHCache m_hRmCache;
	SFBHCookie m_hRmCookie;

} _WebKitShpPage;

typedef struct _WebKitShpFrame WebKitShpFrame;
typedef struct _WebKitShpFrameData WebKitShpFrameData;
typedef struct _WebKitShpNetworkRequest WebKitShpNetworkRequest;
typedef struct _WebKitShpPage WebKitShpPage;

 void
webkit_shp_initialize(HWIN hWin);

 void
webkit_shp_finalize();

 WebKitShpPage*
webkit_shp_page_new (void);

 gboolean
webkit_shp_page_can_cut (WebKitShpPage* page);

 gboolean
webkit_shp_page_can_copy (WebKitShpPage* page);

 void
webkit_shp_page_paste (WebKitShpPage* page);

 gboolean
webkit_shp_page_can_go_backward (WebKitShpPage* page);

 gboolean
webkit_shp_page_can_go_forward (WebKitShpPage* page);

 void
webkit_shp_page_go_backward (WebKitShpPage* page);

 void
webkit_shp_page_go_forward (WebKitShpPage* page);

 void
webkit_shp_page_stop_loading (WebKitShpPage* page);

 void
webkit_shp_page_switch_tab (WebKitShpPage* page);

 void
webkit_shp_page_open (WebKitShpPage* page, const gchar* url);

 void
webkit_shp_page_reload (WebKitShpPage *page);

 void
webkit_shp_page_load_string (WebKitShpPage* page, const gchar* content, const gchar* content_mime_type, const gchar* content_encoding, const gchar* base_url);

 void
webkit_shp_page_load_html_string (WebKitShpPage* page, const gchar* content, const gchar* base_url);

 WebKitShpFrame*
webkit_shp_page_get_main_frame (WebKitShpPage* page);

#ifdef SFB_VISUAL_SUPPORT
 void
webkit_shp_page_visual_history(WebKitShpPage* page, WmBitmap** pWmBitmap,void* pUserData);
 //tahseen
#endif

#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
void
webkit_shp_page_set_preview(WebKitShpPage* page);

void
webkit_shp_page_is_preview(WebKitShpPage* page, bool* bPreview);

 void
webkit_shp_page_preview_key_event(WebKitShpPage* page, int keyCode,  bool isKeyUp);
#endif

 void
 webkit_shp_page_paint(WebKitShpPage* page, void* pUserData);

 void
 webkit_shp_page_set_hwnd(WebKitShpPage* page, HWIN hWin);
#if PLATFORM(SHP) && defined(SFB_SSR_RENDERING)
// CELUNITE_SSR_Change_Begin
 void
    webkit_shp_page_layoutChange(WebKitShpPage* page, WEBKIT_SSR_MODES sSRMode);
// CELUNITE_SSR_Change_End

// CELUNITE_SAVE_Change_Begin
 void
    webkit_shp_page_save(WebKitShpPage* page, const gchar* aPath);
// CELUNITE_SAVE_Change_End
#endif
void
webkit_shp_page_key_event(WebKitShpPage* page, int keyCode,  bool isKeyUp);

/**
 *   bs0425.lee  modified for touch events 
 */

void
webkit_shp_page_mouse_event(WebKitShpPage* page, int x, int y, int MouseEventType);

void
webkit_shp_page_ScrollBy(WebKitShpPage* page , int offsetX,int  offsetY);

gboolean
webkit_shp_page_IsFocusedByPoint(WebKitShpPage* page, int x, int y);

gboolean 
 webkit_shp_page_IsLinkedElement(WebKitShpPage* page, int x, int y);

gboolean 
 webkit_shp_page_IsImageElement(WebKitShpPage* page, int x, int y);

gboolean 
 webkit_shp_page_IsEditBox(void);

void
 webkit_shp_page_ExitEditBox(void);

void webkit_shp_page_GetElementInfoByPoint( WebKitShpPage* mPage, EPageItemType aType, ElementInfoBase* aInfo, int x, int y);


 /**
 *   bs0425.lee  modified for touch events 
 */

void
webkit_shp_page_virtual_cursor_init(WebKitShpPage* page);

void
webkit_shp_page_virtual_cursor_hide();

void
webkit_shp_page_finalize(WebKitShpPage* page);

void
webkit_shp_page_get_url(WebKitShpPage* page, gchar* aUrl, unsigned long* aSize);

void
webkit_shp_page_get_title(WebKitShpPage* page, gchar* aTitle, unsigned long* aSize);

void
webkit_shp_page_get_mime_type(WebKitShpPage* page, gchar* aType, unsigned long* aSize);

void
webkit_shp_page_get_size(WebKitShpPage* page, unsigned long* aSize);

#ifdef SFB_RSS_LINK //vamsi Jan 28 Rss Link Tag Support for SFB.
void
webkit_shp_page_get_rss_links(WebKitShpPage* page, SRssLink** pRssLink, int* rssCount, gchar** url);
#endif

void
webkit_shp_page_ZoomIn(WebKitShpPage* page, const int iZoom);

int        
webkit_shp_page_get_ZoomFactor( WebKitShpPage* page );

int        
webkit_shp_page_GetProperty( WebKitShpPage* mPage,
								const EBrowserProperty aBrowserProperty, 
								gboolean* aSettingValue );

int        
webkit_shp_page_SetProperty( WebKitShpPage* mPage,
								const EBrowserProperty aBrowserProperty, 
								gboolean aSettingValue );

void webkit_shp_page_SetHttpProxy( WebKitShpPage* mPage,const SProxyInfo* aProxyInfo);
void webkit_shp_page_SetIAP( WebKitShpPage* mPage,const IAPHANDLE aSelectedIapHandle);
void webkit_shp_page_SetCacheSize( WebKitShpPage* mPage,const guint aCacheSize);


void webkit_shp_page_GetHttpProxy( WebKitShpPage* mPage,SProxyInfo* aProxyInfo);
void webkit_shp_page_GetIAP( WebKitShpPage* mPage,IAPHANDLE* aSelectedIapHandle);
void webkit_shp_page_GetCacheSize( WebKitShpPage* mPage,guint* aCacheSize);


void webkit_shp_page_SetMinimumFontSize( WebKitShpPage* mPage,const int aSettingsValue);
void webkit_shp_page_SetMinimumLogicalFontSize( WebKitShpPage* mPage,const int aSettingsValue);
void webkit_shp_page_SetDefaultFontSize( WebKitShpPage* mPage,const int aSettingsValue);
void webkit_shp_page_SetDefaultFixedFontSize( WebKitShpPage* mPage,const int aSettingsValue);
	
void webkit_shp_page_SetDefaultTextEncodingName( WebKitShpPage* mPage,const gchar* aSettingsValue);
void webkit_shp_page_SetFtpDirectoryTemplatePath( WebKitShpPage* mPage,const gchar* aSettingsValue);
void webkit_shp_page_SetUserStyleSheetLocation( WebKitShpPage* mPage,const gchar*  aSettingsValue);
void webkit_shp_page_SetStandardFontFamily( WebKitShpPage* mPage,const gchar* aSettingsValue);
void webkit_shp_page_SetFixedFontFamily( WebKitShpPage* mPage,const gchar* aSettingsValue);
void webkit_shp_page_SetSerifFontFamily( WebKitShpPage* mPage,const gchar* aSettingsValue);
void webkit_shp_page_SetSansSerifFontFamily( WebKitShpPage* mPage,const gchar* aSettingsValue);
void webkit_shp_page_SetCursiveFontFamily( WebKitShpPage* mPage,const gchar* aSettingsValue);
void webkit_shp_page_SetFantasyFontFamily( WebKitShpPage* mPage,const gchar* aSettingsValue);
void webkit_shp_page_SetEditableLinkBehavior( WebKitShpPage* mPage,const WebCore::EditableLinkBehavior aSettingsValue);

void webkit_shp_page_GetMinimumFontSize( WebKitShpPage* mPage,int *  aSettingsValue);
void webkit_shp_page_GetMinimumLogicalFontSize( WebKitShpPage* mPage,int *  aSettingsValue);
void webkit_shp_page_GetDefaultFontSize( WebKitShpPage* mPage,int *  aSettingsValue);
void webkit_shp_page_GetDefaultFixedFontSize( WebKitShpPage* mPage,int *  aSettingsValue);

void webkit_shp_page_GetDefaultTextEncodingName(WebKitShpPage* page,gchar* aSettingsValue,gulong *aSize);
void webkit_shp_page_GetFtpDirectoryTemplatePath(WebKitShpPage* page,gchar* aSettingsValue,gulong *aSize);
void webkit_shp_page_GetUserStyleSheetLocation(WebKitShpPage* page,gchar* aSettingsValue,gulong *aSize);
void webkit_shp_page_GetStandardFontFamily(WebKitShpPage* page,gchar* aSettingsValue,gulong *aSize);
void webkit_shp_page_GetFixedFontFamily(WebKitShpPage* page,gchar* aSettingsValue,gulong *aSize);
void webkit_shp_page_GetSerifFontFamily(WebKitShpPage* page,gchar* aSettingsValue,gulong *aSize);
void webkit_shp_page_GetSansSerifFontFamily(WebKitShpPage* page,gchar* aSettingsValue,gulong *aSize);
void webkit_shp_page_GetCursiveFontFamily(WebKitShpPage* page,gchar* aSettingsValue,gulong *aSize);
void webkit_shp_page_GetFantasyFontFamily(WebKitShpPage* page,gchar* aSettingsValue,gulong *aSize);

void webkit_shp_page_GetEditableLinkBehavior( WebKitShpPage* mPage,WebCore::EditableLinkBehavior *aSettingsValue);

/**
 *   bs0425.lee  modified for touch events 
 */

void webkit_shp_page_GetSelectedElementTypeByPoint( WebKitShpPage* mPage,EPageItemType* aType , int pointX , int pointY );

/**
 *   bs0425.lee  modified for touch events 
 */

void webkit_shp_page_GetSelectedElementType( WebKitShpPage* mPage,EPageItemType* aType);
void webkit_shp_page_GetSelectedElementInfo( WebKitShpPage* mPage, EPageItemType aType, ElementInfoBase* aInfo);

void webkit_shp_element_save(WebKitShpPage* page, const gchar* aPath);

/**
 *   bs0425.lee  modified for touch events 
 */

void webkit_shp_element_saveByPoint(WebKitShpPage* page, const gchar* aPath, int pointX, int pointY);
/**
 *   bs0425.lee  modified for touch events 
 */
void webkit_shp_get_images(WebKitShpPage* page, bool bVisibleOnly, ImageSaveInfo*** imageList);
int webkit_shp_get_image_count(WebKitShpPage* page, bool bVisibleOnly);

void webkit_shp_page_GetItemCount(WebKitShpPage* page, const EPageItemType aType, guint* aCount);

void webkit_shp_page_GetLastModifiedTime(WebKitShpPage* page, gchar* aTime, gulong* aSize);

#ifdef _SFB_SEARCH_STRING_IN_PAGE //Jan-16 Vamsi/Pramod
bool webkit_shp_page_find(WebKitShpPage* page, const SFindInfo* pFindInfo);
#endif

#if defined SHP_SUPPORT_WML
void webkit_page_send_new_fetch_request (WebKitShpPage* page) ;
int webkit_document_get_count(WebKitShpPage *pageData);
int webkit_document_wmldoEvent(WebKitShpPage *pageData, int OptionId);
#endif //SHP_SUPPORT_WML

//vamsi relayout
void webkit_shp_page_relayout(WebKitShpPage* page);
#ifdef __cplusplus
} /*extern C*/
#endif

#endif
