/*
 * Copyright (C) 2007 Holger Hans Peter Freyther
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer. 
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution. 
 * 3.  Neither the name of Apple, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "ChromeClientShp.h"
#include "FloatRect.h"
#include "IntRect.h"
#include "PlatformString.h"
#include "CString.h"
#include "TextEncoding.h"
#include "webkitshppage.h"
#include "webkitshpprivate.h"
#include "NotImplemented.h"
//#include "BrAppWindowJavaScript.h" //REENG
#include "FrameLoadRequest.h"
#include "FrameLoader.h"
#include "WindowFeatures.h"

using namespace WebCore;

namespace WebKit {

ChromeClient::ChromeClient(WebKitShpPage* page)
    : m_webPage(page)
{
    notImplementedInGdk();
}

void ChromeClient::chromeDestroyed()
{
    delete this;
}

FloatRect ChromeClient::windowRect()
{
    notImplemented();
    return FloatRect();
}

void ChromeClient::setWindowRect(const FloatRect& r)
{
    notImplemented();
}

FloatRect ChromeClient::pageRect()
{
    notImplemented();
    return FloatRect();
}

float ChromeClient::scaleFactor()
{
    notImplemented();
    return 1.0;
}

void ChromeClient::focus()
{
    notImplemented();
}

void ChromeClient::unfocus()
{
    notImplemented();
}
    
Page* ChromeClient::createWindow(Frame* frame, const FrameLoadRequest& request, const WebCore::WindowFeatures& features)
{
    if (features.dialog) {
        notImplemented();
        return 0;
    } else {
    /* TODO: FrameLoadRequest is not used */
    if (!request.resourceRequest().url().isEmpty())
	{
		m_webPage->m_openerWindowUrl = frame->document()->url().latin1();
		frame->loader()->load(request, false, true, 0, 0, HashMap<String, String>());
	}

	return core(m_webPage);
	}
}

Page* ChromeClient::createModalDialog(Frame*, const FrameLoadRequest&)
{
    notImplemented();
    return 0;
}

void ChromeClient::show()
{
    notImplemented();
}

bool ChromeClient::canRunModal()
{
    notImplemented();
    return false;
}

void ChromeClient::runModal()
{
    notImplemented();
}

void ChromeClient::setToolbarsVisible(bool)
{
    notImplemented();
}

bool ChromeClient::toolbarsVisible()
{
    notImplemented();
    return false;
}

void ChromeClient::setStatusbarVisible(bool)
{
    notImplemented();
}

bool ChromeClient::statusbarVisible()
{
    notImplemented();
    return false;
}

void ChromeClient::setScrollbarsVisible(bool)
{
    notImplemented();
}

bool ChromeClient::scrollbarsVisible() {
    notImplemented();
    return false;
}

void ChromeClient::setMenubarVisible(bool)
{
    notImplemented();
}

bool ChromeClient::menubarVisible()
{
    notImplemented();
    return false;
}

void ChromeClient::setResizable(bool)
{
    notImplemented();
}

void ChromeClient::closeWindowSoon()
{
    webkit_shp_page_open( m_webPage, m_webPage->m_openerWindowUrl.latin1().data() );
}

bool ChromeClient::canTakeFocus(FocusDirection)
{
    notImplemented();
    return true;
}

void ChromeClient::takeFocus(FocusDirection)
{
    notImplemented();
}

bool ChromeClient::canRunBeforeUnloadConfirmPanel()
{
    notImplementedInGdk();
    return false;
}

bool ChromeClient::runBeforeUnloadConfirmPanel(const WebCore::String&, WebCore::Frame*)
{
    notImplemented();
    return false;
}

void ChromeClient::addMessageToConsole(const WebCore::String& message, unsigned int lineNumber, const WebCore::String& sourceId)
{
    notImplemented();
}

void ChromeClient::runJavaScriptAlert(Frame* frame, const String& message)
{
#ifndef NDEBUG /*ascii fnc is defined under NDEBUG flag only*/
    //sfb_re_eng
	//BrAppCreateJsAlertMsgbox(message.ascii().data());
#endif
}

bool ChromeClient::runJavaScriptConfirm(Frame* frame, const String& message)
{
#ifndef NDEBUG /*ascii fnc is defined under NDEBUG flag only*/
    //sfb_re_eng
	//return BrAppCreateJsConfirmPopup(message.ascii().data());
	return true;
#endif
}

bool ChromeClient::runJavaScriptPrompt(Frame* frame, const String& message, const String& defaultValue, String& result)
{
#ifndef NDEBUG /*ascii fnc is defined under NDEBUG flag only*/
	//sfb_re_eng
	//return BrAppCreateJsPromptPopup(message.ascii().data());
	return true;
#endif
}

void ChromeClient::setStatusbarText(const String& string)
{
    CString stringMessage = string.utf8();
	if(m_webPage->m_statusbarTextChangedHandler)
		(m_webPage->m_statusbarTextChangedHandler)(stringMessage.data());
}

bool ChromeClient::shouldInterruptJavaScript()
{
    notImplemented();
    return false;
}

bool ChromeClient::tabsToLinks() const
{
    notImplemented();
    return false;
}

IntRect ChromeClient::windowResizerRect() const
{
    notImplemented();
    return IntRect();
}

void ChromeClient::addToDirtyRegion(const IntRect&)
{
    notImplementedInGdk();
}

void ChromeClient::scrollBackingStore(int dx, int dy, const IntRect& scrollViewRect, const IntRect& clipRect)
{
    notImplemented();
}

void ChromeClient::updateBackingStore()
{
    notImplemented();
}

void ChromeClient::mouseDidMoveOverElement(const HitTestResult&, unsigned modifierFlags)
{
    notImplemented();
}

void ChromeClient::setToolTip(const String&)
{
    notImplemented();
}

void ChromeClient::print(Frame*)
{
    notImplemented();
}

void ChromeClient::exceededDatabaseQuota(Frame*, const String&)
{
    notImplemented();
}

void ChromeClient::populateVisitedLinks()
{
    notImplemented();
}

}
