/*
     File:       CFNetwork/CFURLConnectionPriv.h
     Copyright:  (c) 2007 by Apple Inc., all rights reserved
*/
#ifndef __CFURLCONNECTIONPRIV__
#define __CFURLCONNECTIONPRIV__

#ifndef __CFNETWORKDEFS__
#include <CFNetwork/CFNetworkDefs.h>
#endif

#ifndef __COREFOUNDATION__
#include <CoreFoundation/CoreFoundation.h>
#endif

#ifndef __CFURLREQUESTPRIV__
#include <CFNetwork/CFURLRequestPriv.h>
#endif

#ifndef __CFURLRESPONSEPRIV__
#include <CFNetwork/CFURLResponsePriv.h>
#endif

/* This is only needed for the stubs of CFURLAuthenticationChallenge and CFURLCachedResponse; it should be removed when those objects come available. */
#ifndef __CFURLPROTOCOLPRIV__
#include <CFNetwork/CFURLProtocolPriv.h>
#endif

#ifndef __CFURLAUTHCHALLENGEPRIV__
#include <CFNetwork/CFURLAuthChallengePriv.h>
#endif

#ifndef __CFURLCREDENTIALPRIV__
#include <CFNetwork/CFURLCredentialPriv.h>
#endif


/* Provide a compiler flag folks can check for to determine which version of the callbacks are the "default" */
#define ERRORS_AS_CFERRORS

#include <AvailabilityMacros.h>

#if PRAGMA_ONCE
#pragma once
#endif

#ifdef __cplusplus
extern "C" {
#endif

#pragma pack(push, 2)

#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint on
#endif


/*
 *  CFURLConnection
 *  
 *  Discussion:
 *    CFURLConnection is a CFType that represents a download attempt of
 *    a CFURLRequest.  CFURLConnection is inherently asynchronous,
 *    although a synchronous API does exist.  CFURLConnections are
 *    created from a request and a client.  The request describes the
 *    download to be attempted; the client is the object contacted as
 *    the download progresses. which It is used by CFURLConnection and
 *    CFURLDownload.
 */
typedef struct _CFURLConnection*        CFURLConnectionRef;
typedef CALLBACK_API_C( CFURLRequestRef , CFURLConnectionWillSendRequestCallback )(CFURLConnectionRef conn, CFURLRequestRef request, CFURLResponseRef redirectionResponse, const void *clientInfo);
typedef CALLBACK_API_C( void , CFURLConnectionDidReceiveResponseCallback )(CFURLConnectionRef conn, CFURLResponseRef response, const void *clientInfo);
typedef CALLBACK_API_C( void , CFURLConnectionDidReceiveDataCallback )(CFURLConnectionRef conn, CFDataRef data, CFIndex originalLength, const void *clientInfo);
typedef CALLBACK_API_C( void , CFURLConnectionDidStopBufferingDataCallback )(CFURLConnectionRef conn, CFDataRef bufferedData, CFIndex originalLength, const void *clientInfo);
typedef CALLBACK_API_C( void , CFURLConnectionDidFinishLoadingCallback )(CFURLConnectionRef conn, const void *clientInfo);
typedef CALLBACK_API_C( void , CFURLConnectionDidFailCallbackV0 )(CFURLConnectionRef conn, CFStreamError error, const void *clientInfo);
typedef CALLBACK_API_C( void , CFURLConnectionDidFailCallback )(CFURLConnectionRef conn, CFErrorRef error, const void *clientInfo);
typedef CALLBACK_API_C( CFCachedURLResponseRef , CFURLConnectionWillCacheResponseCallback )(CFURLConnectionRef conn, CFCachedURLResponseRef cachedResponse, const void *clientInfo);
typedef CALLBACK_API_C( void , CFURLConnectionDidReceiveAuthenticationChallengeCallback )(CFURLConnectionRef conn, CFURLAuthChallengeRef challenge, const void *clientInfo);
struct CFURLConnectionClient {
  CFIndex             version;                /* 0 or 1*/
  const void *        clientInfo;

  CFAllocatorRetainCallBack  retain;
  CFAllocatorReleaseCallBack  release;
  CFAllocatorCopyDescriptionCallBack  copyDescription;

  CFURLConnectionWillSendRequestCallback  willSendRequest; /* starting with v1, CFURLConnection will take ownership of the returned request (i.e. this callback should return a reference to the returned object)*/
  CFURLConnectionDidReceiveResponseCallback  didReceiveResponse;
  CFURLConnectionDidReceiveDataCallback  didReceiveData;
  CFURLConnectionDidStopBufferingDataCallback  didStopBuffering;
  CFURLConnectionDidFinishLoadingCallback  didFinishLoading;
  CFURLConnectionDidFailCallback  didFail;
  CFURLConnectionWillCacheResponseCallback  willCacheResponse; /* starting with v1, CFURLConnection will take ownership of the returned cached response (i.e. this callback should return a reference to the returned object)*/
  CFURLConnectionDidReceiveAuthenticationChallengeCallback  didReceiveAuthenticationChallenge;
};
typedef struct CFURLConnectionClient    CFURLConnectionClient;
/*
 *  CFURLConnectionGetTypeID()
 *  
 */
APPLE_WIN_API CFTypeID 
CFURLConnectionGetTypeID(void)                                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLConnectionCreate()
 *  
 */
APPLE_WIN_API CFURLConnectionRef 
CFURLConnectionCreate(
  CFAllocatorRef           alloc,
  CFURLRequestRef          request,
  CFURLConnectionClient *  client)                            AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLConnectionScheduleWithRunLoop()
 *  
 */
APPLE_WIN_API void 
CFURLConnectionScheduleWithRunLoop(
  CFURLConnectionRef   conn,
  CFRunLoopRef         rl,
  CFStringRef          mode)                                  AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLConnectionUnscheduleFromRunLoop()
 *  
 */
APPLE_WIN_API void 
CFURLConnectionUnscheduleFromRunLoop(
  CFURLConnectionRef   conn,
  CFRunLoopRef         rl,
  CFStringRef          mode)                                  AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
   If you want the actual download work to take on a thread other than where the connection itself is scheduled, call these.  
   Otherwise, the download will take place wherever the connection itself has been scheduled
*/
/*
 *  CFURLConnectionScheduleDownloadWithRunLoop()
 *  
 */
APPLE_WIN_API void 
CFURLConnectionScheduleDownloadWithRunLoop(
  CFURLConnectionRef   conn,
  CFRunLoopRef         rl,
  CFStringRef          mode)                                  AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLConnectionUnscheduleDownloadFromRunLoop()
 *  
 */
APPLE_WIN_API void 
CFURLConnectionUnscheduleDownloadFromRunLoop(
  CFURLConnectionRef   conn,
  CFRunLoopRef         rl,
  CFStringRef          mode)                                  AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


#if defined(__WIN32__)
/*
 *  CFURLConnectionScheduleWithCurrentMessageQueue()
 *  
 */
APPLE_WIN_API void 
CFURLConnectionScheduleWithCurrentMessageQueue(CFURLConnectionRef conn) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


#endif
/*
 *  CFURLConnectionStart()
 *  
 */
APPLE_WIN_API void 
CFURLConnectionStart(CFURLConnectionRef conn)                 AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLConnectionCancel()
 *  
 */
APPLE_WIN_API void 
CFURLConnectionCancel(CFURLConnectionRef conn)                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLConnectionHalt()
 *  
 */
APPLE_WIN_API void 
CFURLConnectionHalt(CFURLConnectionRef conn)                  AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLConnectionResume()
 *  
 */
APPLE_WIN_API void 
CFURLConnectionResume(CFURLConnectionRef conn)                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLConnectionUseCredential()
 *  
 */
APPLE_WIN_API void 
CFURLConnectionUseCredential(
  CFURLConnectionRef      conn,
  CFURLCredentialRef      creds,
  CFURLAuthChallengeRef   challenge)                          AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
   Synchronously sends the request and returns the data, plus the response and any error by-ref.  Caller must release the returned data, as well as 
   the response and error if they are requested.  No customization during the processing of the request is permitted.  
*/
/*
 *  CFURLConnectionSendSynchronousRequest()
 *  
 */
APPLE_WIN_API CFDataRef 
CFURLConnectionSendSynchronousRequest(
  CFURLRequestRef     request,
  CFURLResponseRef *  response,
  CFErrorRef *        err,
  CFTimeInterval      timeout)                                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint reset
#endif

#pragma pack(pop)

#ifdef __cplusplus
}
#endif

#endif /* __CFURLCONNECTIONPRIV__ */

