/*
     File:       CFNetwork/CFURLProtocolPriv.h
     Copyright:  (c) 2007 by Apple Inc., all rights reserved
*/
#ifndef __CFURLPROTOCOLPRIV__
#define __CFURLPROTOCOLPRIV__

#ifndef __CFNETWORKDEFS__
#include <CFNetwork/CFNetworkDefs.h>
#endif

#ifndef __COREFOUNDATION__
#include <CoreFoundation/CoreFoundation.h>
#endif

#ifndef __CFURLREQUESTPRIV__
#include <CFNetwork/CFURLRequestPriv.h>
#endif

#ifndef __CFURLRESPONSEPRIV__
#include <CFNetwork/CFURLResponsePriv.h>
#endif

#ifndef __CFURLAUTHCHALLENGEPRIV__
#include <CFNetwork/CFURLAuthChallengePriv.h>
#endif

#ifndef __CFURLCREDENTIALPRIV__
#include <CFNetwork/CFURLCredentialPriv.h>
#endif

#ifndef __CFURLCACHEPRIV__
#include <CFNetwork/CFURLCachePriv.h>
#endif

#include <AvailabilityMacros.h>

#if PRAGMA_ONCE
#pragma once
#endif

#ifdef __cplusplus
extern "C" {
#endif

#pragma pack(push, 2)

#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint on
#endif


/*
 *  CFURLProtocol
 *  
 *  Discussion:
 *    CFURLProtocol is a CFType that manages the download of a
 *    CFURLRequest, producing a CFURLResponse and data
 */
typedef const struct _CFURLProtocol*    CFURLProtocolRef;
/*
   Need to consider something like this, so we can confirm the final request before transmission
   typedef extern CFURLRequestRef (*CFURLProtocolWillSendRequestCallback)(CFURLProtocolRef prot, CFURLRequestRef newRequest, Boolean *shouldHalt, const void *client); 
*/
typedef CALLBACK_API_C( void , CFURLProtocolWasRedirectedCallback )(CFURLProtocolRef prot, CFURLRequestRef newRequest, CFURLResponseRef redirectResponse, const void *client);
typedef CALLBACK_API_C( void , CFURLProtocolCacheResponseIsValidCallback )(CFURLProtocolRef prot, CFCachedURLResponseRef cachedResponse, const void *client);
typedef CALLBACK_API_C( void , CFURLProtocolDidReceiveResponseCallback )(CFURLProtocolRef prot, CFURLResponseRef response, const void *client);
typedef CALLBACK_API_C( void , CFURLProtocolDidLoadDataCallback )(CFURLProtocolRef prot, CFDataRef data, SInt64 originalEncodedLength, const void *client);
typedef CALLBACK_API_C( void , CFURLProtocolDidFinishLoadingCallback )(CFURLProtocolRef prot, const void *client);
typedef CALLBACK_API_C( void , CFURLProtocolDidFailCallbackV0 )(CFURLProtocolRef prot, CFStreamError err, const void *client);
typedef CALLBACK_API_C( void , CFURLProtocolDidFailCallback )(CFURLProtocolRef prot, CFErrorRef err, const void *client);
typedef CALLBACK_API_C( void , CFURLProtocolDidReceiveAuthenticationChallengeCallback )(CFURLProtocolRef prot, CFURLAuthChallengeRef challenge, const void *client);
struct CFURLProtocolClient {
  CFIndex             version;                /* Should be 1; if it is 0, the v0 version of the didFail callback is assumed*/
  const void *        client;

                                              /* Memory management for the client pointer*/
  CFAllocatorRetainCallBack  retain;
  CFAllocatorReleaseCallBack  release;
  CFAllocatorCopyDescriptionCallBack  copyDescription;

                                              /* Called as the protocol is processed*/
  CFURLProtocolWasRedirectedCallback  wasRedirected;
  CFURLProtocolCacheResponseIsValidCallback  cacheResponseIsValid;
  CFURLProtocolDidReceiveResponseCallback  didReceiveResponse;
  CFURLProtocolDidLoadDataCallback  didLoadData;
  CFURLProtocolDidFinishLoadingCallback  didFinishLoading;
  CFURLProtocolDidFailCallback  didFail;
  CFURLProtocolDidReceiveAuthenticationChallengeCallback  didReceiveAuthenticationChallenge;
};
typedef struct CFURLProtocolClient      CFURLProtocolClient;
/*
 *  CFURLProtocolGetTypeID()
 *  
 */
APPLE_WIN_API CFTypeID 
CFURLProtocolGetTypeID(void)                                  AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/* "Class" functions*/
/*
 *  CFURLProtocolCanHandleRequest()
 *  
 */
APPLE_WIN_API Boolean 
CFURLProtocolCanHandleRequest(CFURLRequestRef request)        AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLProtocolCreateCanonicalRequest()
 *  
 */
APPLE_WIN_API CFURLRequestRef 
CFURLProtocolCreateCanonicalRequest(
  CFAllocatorRef    alloc,
  CFURLRequestRef   request)                                  AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLProtocolRequestsAreCacheEquivalent()
 *  
 */
APPLE_WIN_API Boolean 
CFURLProtocolRequestsAreCacheEquivalent(
  CFURLRequestRef   request1,
  CFURLRequestRef   request2)                                 AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/* Creating and using CFURLProtocol instances*/
/*
 *  CFURLProtocolCreate()
 *  
 */
APPLE_WIN_API CFURLProtocolRef 
CFURLProtocolCreate(
  CFAllocatorRef           alloc,
  CFURLRequestRef          request,
  CFCachedURLResponseRef   cachedResponse,
  CFURLProtocolClient *    client)                            AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLProtocolGetClient()
 *  
 */
APPLE_WIN_API CFURLProtocolClient * 
CFURLProtocolGetClient(CFURLProtocolRef prot)                 AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLProtocolGetRequest()
 *  
 */
APPLE_WIN_API CFURLRequestRef 
CFURLProtocolGetRequest(CFURLProtocolRef prot)                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLProtocolGetCachedResponse()
 *  
 */
APPLE_WIN_API CFCachedURLResponseRef 
CFURLProtocolGetCachedResponse(CFURLProtocolRef prot)         AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLProtocolScheduleWithRunLoop()
 *  
 */
APPLE_WIN_API void 
CFURLProtocolScheduleWithRunLoop(
  CFURLProtocolRef   prot,
  CFRunLoopRef       runLoop,
  CFStringRef        runLoopMode)                             AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLProtocolUnscheduleFromRunLoop()
 *  
 */
APPLE_WIN_API void 
CFURLProtocolUnscheduleFromRunLoop(
  CFURLProtocolRef   prot,
  CFRunLoopRef       runLoop,
  CFStringRef        runLoopMode)                             AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
   Do not use!  This is being staged out; use CFURLProtocolCopySchedulingArray, below, instead.
   Once CFSchedulingSet is available in CoreFoundation, there will be a scheduling set variant instead.
*/
/*
 *  CFURLProtocolGetSchedulingArray()
 *  
 */
APPLE_WIN_API CFArrayRef 
CFURLProtocolGetSchedulingArray(CFURLProtocolRef prot)        AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLProtocolCopySchedulingArray()
 *  
 */
APPLE_WIN_API CFArrayRef 
CFURLProtocolCopySchedulingArray(CFURLProtocolRef prot)       AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  _CFURLProtocolCachedResponseIsValid()
 *  
 */
APPLE_WIN_API Boolean 
_CFURLProtocolCachedResponseIsValid(CFURLProtocolRef prot)    AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLProtocolStartLoad()
 *  
 */
APPLE_WIN_API void 
CFURLProtocolStartLoad(CFURLProtocolRef prot)                 AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLProtocolCancelLoad()
 *  
 */
APPLE_WIN_API void 
CFURLProtocolCancelLoad(CFURLProtocolRef prot)                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLProtocolHaltLoad()
 *  
 */
APPLE_WIN_API void 
CFURLProtocolHaltLoad(CFURLProtocolRef prot)                  AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLProtocolResumeLoad()
 *  
 */
APPLE_WIN_API void 
CFURLProtocolResumeLoad(CFURLProtocolRef prot)                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
   Challenge must be the last authentication challenge sent from the protocol to its client; you cannot pre-emptively set a credential in the absence of a challenge
   Pass null credentials to continue without any credentials at all. 
*/
/*
 *  CFURLProtocolUseCredential()
 *  
 */
APPLE_WIN_API void 
CFURLProtocolUseCredential(
  CFURLProtocolRef        prot,
  CFURLCredentialRef      creds,
  CFURLAuthChallengeRef   challenge)                          AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


typedef CALLBACK_API_C( void , CFURLProtocolInstanceScheduleCallback )(CFURLProtocolRef protocol, CFRunLoopRef rl, CFStringRef rlMode, const void *info);
typedef CALLBACK_API_C( void , CFURLProtocolInstanceUnscheduleCallback )(CFURLProtocolRef protocol, CFRunLoopRef rl, CFStringRef rlMode, const void *info);
typedef CALLBACK_API_C( void , CFURLProtocolInstanceStartCallback )(CFURLProtocolRef protocol, const void *info);
typedef CALLBACK_API_C( void , CFURLProtocolInstanceStopCallback )(CFURLProtocolRef protocol, const void *info);
typedef CALLBACK_API_C( void , CFURLProtocolInstanceHaltCallback )(CFURLProtocolRef protocol, const void *info);
typedef CALLBACK_API_C( void , CFURLProtocolInstanceResumeCallback )(CFURLProtocolRef protocol, const void *info);
typedef CALLBACK_API_C( Boolean , CFURLProtocolInstanceCacheResponseIsValidCallback )(CFURLProtocolRef protocol, const void *info);
typedef CALLBACK_API_C( void , CFURLProtocolInstanceUseCredentialCallback )(CFURLProtocolRef protocol, CFURLCredentialRef creds, CFURLAuthChallengeRef challenge, const void *info);
struct CFURLProtocolInstanceCallbacks {
  CFIndex             version;                /* Should be 0*/

                                              /* Memory management for the info pointer*/
  CFAllocatorRetainCallBack  retain;
  CFAllocatorReleaseCallBack  release;
  CFAllocatorCopyDescriptionCallBack  copyDescription;
  CFURLProtocolInstanceScheduleCallback  schedule;
  CFURLProtocolInstanceUnscheduleCallback  unschedule;

  CFURLProtocolInstanceStartCallback  start;
  CFURLProtocolInstanceStopCallback  stop;

  CFURLProtocolInstanceHaltCallback  halt;
  CFURLProtocolInstanceResumeCallback  resume;

                                              /* If the protocol instance never issues authentication challenges, it may set this callback to NULL*/
  CFURLProtocolInstanceUseCredentialCallback  useCredential;

  CFURLProtocolInstanceCacheResponseIsValidCallback  cacheResponseIsValid;

};
typedef struct CFURLProtocolInstanceCallbacks CFURLProtocolInstanceCallbacks;
typedef CALLBACK_API_C( Boolean , CFURLProtocolImplCanHandleRequestCallback )(CFURLRequestRef request, const void *info);
typedef CALLBACK_API_C( CFURLRequestRef , CFURLProtocolImplCreateCanonicalRequestCallback )(CFAllocatorRef alloc, CFURLRequestRef request, const void *info);
typedef CALLBACK_API_C( Boolean , CFURLProtocolImplRequestsAreCacheEquivalentCallback )(CFURLRequestRef request1, CFURLRequestRef request2, const void *info);
typedef CALLBACK_API_C( void *, CFURLProtocolImplProtocolInstanceForProtocolCallback )(CFURLProtocolRef protocol, const CFURLProtocolInstanceCallbacks **callbacks, const void *info);
struct CFURLProtocolImplementation {
  CFIndex             version;                /* Should be 0*/
  const void *        info;

                                              /* Memory management for the info pointer*/
  CFAllocatorRetainCallBack  retain;
  CFAllocatorReleaseCallBack  release;
  CFAllocatorCopyDescriptionCallBack  copyDescription;

  CFURLProtocolImplCanHandleRequestCallback  canHandleRequest;
  CFURLProtocolImplCreateCanonicalRequestCallback  createCanonicalRequest;
  CFURLProtocolImplRequestsAreCacheEquivalentCallback  requestsAreCacheEquivalent;
  CFURLProtocolImplProtocolInstanceForProtocolCallback  protocolInstanceForProtocol;
};
typedef struct CFURLProtocolImplementation CFURLProtocolImplementation;
/*
 *  CFURLProtocolRegisterImplementation()
 *  
 */
APPLE_WIN_API void 
CFURLProtocolRegisterImplementation(CFURLProtocolImplementation * impl) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLProtocolUnregisterImplementation()
 *  
 */
APPLE_WIN_API void 
CFURLProtocolUnregisterImplementation(CFURLProtocolImplementation * impl) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/* Protocol implementations should use these to send messages to the client*/
/*
 *  _CFURLProtocolSendWasRedirectedCallback()
 *  
 */
APPLE_WIN_API void 
_CFURLProtocolSendWasRedirectedCallback(
  CFURLProtocolRef   prot,
  CFURLRequestRef    newRequest,
  CFURLResponseRef   redirectResponse)                        AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  _CFURLProtocolSendCacheResponseIsValidCallback()
 *  
 */
APPLE_WIN_API void 
_CFURLProtocolSendCacheResponseIsValidCallback(
  CFURLProtocolRef         prot,
  CFCachedURLResponseRef   cachedResponse)                    AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  _CFURLProtocolSendDidReceiveResponseCallback()
 *  
 */
APPLE_WIN_API void 
_CFURLProtocolSendDidReceiveResponseCallback(
  CFURLProtocolRef   prot,
  CFURLResponseRef   response)                                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  _CFURLProtocolSendDidLoadDataCallback()
 *  
 */
APPLE_WIN_API void 
_CFURLProtocolSendDidLoadDataCallback(
  CFURLProtocolRef   prot,
  CFDataRef          data,
  SInt64             originalEncodedLength)                   AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  _CFURLProtocolSendDidFinishLoadingCallback()
 *  
 */
APPLE_WIN_API void 
_CFURLProtocolSendDidFinishLoadingCallback(CFURLProtocolRef prot) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  _CFURLProtocolSendDidFailCallback()
 *  
 */
APPLE_WIN_API void 
_CFURLProtocolSendDidFailCallback(
  CFURLProtocolRef   prot,
  CFErrorRef         err)                                     AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  _CFURLProtocolSendDidReceiveAuthenticationChallengeCallback()
 *  
 */
APPLE_WIN_API void 
_CFURLProtocolSendDidReceiveAuthenticationChallengeCallback(
  CFURLProtocolRef        prot,
  CFURLAuthChallengeRef   challenge)                          AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/* Used by CFHTTPProtocol, Foundation and WebCore to construct the correct headers*/
/*
 *  _CFNetworkCopyPreferredLanguageCode()
 *  
 */
APPLE_WIN_API CFStringRef 
_CFNetworkCopyPreferredLanguageCode(void)                     AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint reset
#endif

#pragma pack(pop)

#ifdef __cplusplus
}
#endif

#endif /* __CFURLPROTOCOLPRIV__ */

