/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		ICacheStorage..h
 * @brief		Define interface of URILoader ICacheStorage
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.05	v1.0		Created
 */
#ifndef __ICacheStorage_Defined
#define __ICacheStorage_Defined


namespace URILoader
{

typedef long CacheAccessMode;
class ICacheEntry;

class ICacheStorage : public IUnknown
{
public:

	
	static const CacheAccessMode ACCESS_NONE = 0;
	static const CacheAccessMode ACCESS_READ = 1;
	static const CacheAccessMode ACCESS_WRITE = 2;
	static const CacheAccessMode ACCESS_READ_WRITE  = 3;

	virtual Result OpenCacheEntry( 
				String* aKey,
				CacheAccessMode accessMode,
				ICacheEntry **pEntry  ) = 0;


	/**
	* Clear all cache entries in this storage.
	*/
	virtual Result ClearStorage();
		
};

typedef long CacheStoragePolicy;

//Temporary comment
//CacheStoragePolicy STORE_ON_DISK = 0;
//CacheStoragePolicy STORE_IN_MEMORY = 1;

Result CreateCacheStorage( const char* szPath, CacheStoragePolicy storagePolicy );



} // namespace 

#endif //__ICacheStorage_Defined
