/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		ICookieStorage..h
 * @brief		Define interface of URILoader ICookieStorage
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.05	v1.0		Created
 */
#ifndef __ICookieStorage_Defined
#define __ICookieStorage_Defined

#include "URILoader.h"
#include "IURI.h"

namespace URILoader
{

class ICookieStorage : public IUnknown
{
public:

	/**
	* Get all cookies stored in the storage.
	* @param pVector [out] the vector containing ICookie 
	*/
	virtual Result GetAllCookies( UVector *pVector ) = 0;


	/**
	* Get all cookies for a specified URI
	*/
	virtual Result GetCookiesForURI( IURI* aURI, UVector *pVector  ) = 0;


	/**
	*
	*/
	//virtual Result DeleteCookie( ICookie* aCookie );

	/**
	*
	*/
	//virtual Result SetCookie( ICookie* aCookie );
	
	
};


/**
* Create a ICookieStorage for a cookie storage with specific szPath
* If two clients( ex. Browser and RSS ) create seperate ICookieStorage with same szPath,
* the clients will share same ICookieStorage.
*/
Result CreateCookieStorage( const char* szPath, ICookieStorage **ppStorageHandle );


} // namespace 

#endif //__ICookieStorage_Defined
