/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		IRequestObserver..h
 * @brief		Define interface of URILoader IRequestObserver
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.05	v1.0		Created
 */

#ifndef _IRequestObserver_Defined
#define _IRequestObserver_Defined

#include "URILoader.h"
#include "IRequest.h"

namespace URILoader
{

class IRequestObserver : public IUnknown
{
public:

	/***************************************************************************
	*
	* Funtions
	*
	****************************************************************************/
    /**
     * Called to signify the beginning of an asynchronous request.
     *
     * @param aRequest request being observed
     * @param aContext user defined context
     *
     * An exception thrown from onStartRequest has the side-effect of
     * causing the request to be canceled.
     */
    virtual void OnStartRequest( IRequest* aRequest,
                        void* aContext) = 0;

    /**
     * Called to signify the end of an asynchronous request.  This
     * call is always preceded by a call to onStartRequest.
     *
     * @param aRequest request being observed
     * @param aContext user defined context
     * @param aStatusCode reason for stopping (NS_OK if completed successfully)
     *
     * An exception thrown from onStopRequest is generally ignored.
     */
    virtual void OnStopRequest( IRequest* aRequest,
                       void* aContext,
                       Result aStatusCode) = 0;
};



} // namespace 

#endif //_IRequestObserver_Defined
