/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		ICertificate.h
 * @brief		Define interface ICertificate of WebControl
 * @author	Chaehwan Li (chaehwan.li@samsung.com)
 * @version	1.0
 * @date		2008.03.14	v1.0		Created
 */

#ifndef __ICertificate_Defined
#define __ICertificate_Defined

#include "WebDefs.h"
#include "CmCertTypes.h"

namespace	WebControl
{

class ICert
{
public:
	/*
	 * @desc gets total count
	 * @param 
	 * @return int count
	 */
	virtual int getTotalCount( void ) = 0;

	/*
	 * @desc gets CA count
	 * @param 
	 * @return int count
	 */
	virtual int getCACount( void ) = 0;

	/*
	 * @desc gets ROOT count
	 * @param 
	 * @return int count
	 */
	virtual int getRootCount( void ) = 0;

	/*
	 * @desc gets whether the given certi is root. 
	 * @param in_Index The Index is one of total count.
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT IsRootbyIndex( int in_Index ) = 0;

	/*
	 * @desc gets whether the given certi is CA. 
	 * @param in_Index The Index is one of total count.
	 * @return HRESULT TRUE/FALSE
	 */
	virtual int IsCAbyIndex( int in_Index ) = 0;

	/*
	 * @desc gets Subject Name
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @param in_Index The Index is one of total count.
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getSubjectNameByIndex( char* out_pszBuffer, int in_nBufferLen, int in_Index ) = 0;

	/*
	 * @desc gets Serial Nember
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @param in_Index The Index is one of total count.
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getSerialNumberByIndex( char* out_pszBuffer, int in_nBufferLen, int in_Index ) = 0;

	/*
	 * @desc gets Signature Algorithm
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @param in_Index The Index is one of total count.
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getSignatureAlgorithmByIndex( char* out_pszBuffer, int in_nBufferLen, int in_Index ) = 0;

	/*
	 * @desc gets Valid From
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @param in_Index The Index is one of total count.
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getValidFromByIndex( char* out_pszBuffer, int in_nBufferLen, int in_Index ) = 0;

	/*
	 * @desc gets Valid Until
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @param in_Index The Index is one of total count.
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getValidUntilByIndex( char* out_pszBuffer, int in_nBufferLen, int in_Index ) = 0;

	/*
	 * @desc gets Issuer Name
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @param in_Index The Index is one of total count.
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getIssuerNameByIndex( char* out_pszBuffer, int in_nBufferLen, int in_Index ) = 0;

	/*
	 * @desc gets Finger Print
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @param in_Index The Index is one of total count.
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getFingerPrintByIndex( char* out_pszBuffer, int in_nBufferLen, int in_Index ) = 0;

	/*
	 * @desc gets Location
	 * @param out_pszBuffer : pointer to buffer
	 * @param in_nBufferLen : length of buffer
	 * @param in_Index The Index is one of total count.
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getLocationByIndex( char* out_pszBuffer, int in_nBufferLen, int in_Index ) = 0;

};

/*
 * @desc
 */
class ICertificate
{
public:
	/*
	 * @desc gets Cert Class
	 * @param out_pICert : pointer to ICert class
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT GetCert( ICert* out_pICert ) = 0;

	/*
	 * @desc imports the certificates from Mocha to Browser Engine
	 * @param in_pCmRootCertList : pointer to the CmRootCertList ( SHP Root Certificates )
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT ImportCertList( CmRootCertList* in_pCmRootCertList ) = 0;
};

} // namespace WebControl

#endif // __ICertificate_Defined