/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		IDOM.h
 * @brief		Define interface IDOM of WebControl
 * @author	Chaehwan Li (chaehwan.li@samsung.com)
 * @version	1.0
 * @date		2008.03.14	v1.0		Created
 */

#ifndef __IDOM_Defined
#define __IDOM_Defined

#include "WebDefs.h"

namespace	WebControl
{

class IDOMAttr
{
public:
	/*
	 * @desc gets the value
	 * @param out_pszBuffer
	 * @param in_nBufferLen
	 * @return bool TRUE / FALSE
	 */
	virtual HRESULT getValue( char* out_pszBuffer, int in_nBufferLen ) = 0;
	/*
	 * @desc gets the name
	 * @param out_pszBuffer
	 * @param in_nBufferLen
	 * @return bool TRUE / FALSE
	 */
	virtual HRESULT getName( char* out_pszBuffer, int in_nBufferLen ) = 0;
};

class IDOMElement
{
public:
	/*
	 * @desc gets the tag name
	 * @param out_pszBuffer
	 * @param in_nBufferLen
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getTagName( char* out_pszBuffer, int in_nBufferLen ) = 0;
	/*
	 * @desc gets the DOM attribute
	 * @param out_pszBuffer
	 * @param in_nBufferLen
	 * @param in_pszName
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getAttribute( char* out_pszBuffer, int in_nBufferLen, char* in_pszName ) = 0;
	/*
	 * @desc get IDOMAttr
	 * @param in_pszName
	 * @return HRESULT TRUE/FALSE
	 */
	virtual IDOMAttr* getAttributeNode( char* in_pszName ) = 0;
};


class IDOMNode
{
public:
	/*
	 * @desc gets valud of node
	 * @param out_pszBuffer
	 * @param in_nBufferLen
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getValue( char* out_pszBuffer, int in_nBufferLen ) = 0;

	/*
	 * @desc gets name of node
	 * @param out_pszBuffer
	 * @param in_nBufferLen
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getName( char* out_pszBuffer, int in_nBufferLen ) = 0;

	/*
	 * @desc gets the type of node
	 * @param
	 * @return DomNodeType
	 */
	virtual DomNodeType getNodetype() = 0;

	/*
	 * @desc checks whether the node has attribute
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT hasAttribute() = 0;
};

} // namespace WebControl

#endif // __IDOM_Defined