/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		IIMEListener.h
 * @brief		Define interface IIMEListener of WebControl
 * @author	Chaehwan Li (chaehwan.li@samsung.com)
 * @version	1.0
 * @date		2008.03.14	v1.0		Created
 */

#ifndef __IIMEListener_Defined
#define __IIMEListener_Defined

#include "WebDefs.h"
#include "IIMESender.h"

namespace	WebControl
{

class IFontAttr
{
public:

	/*
	 * @desc gets the size of the font
	 * @param
	 * @return int font size
	 */
	virtual int getSize( void ) = 0;

	/*
	 * @desc gets whether the font is the bold or not
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT isBold( void ) = 0;

	/*
	 * @desc gets whether the font is the italic or not
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT isItalic( void ) = 0;

	/*
	 * @desc gets the name of font
	 * @param out_pszBuffer pointer to buffer
	 * @param in_nBufferLen length of buffer
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getFontName( char* out_pszBuffer, int in_nBufferLen ) = 0;

};

class IIMEInfo
{
public:
	/*
	 * @desc gets the charset for IME
	 * @param out_pszBuffer pointer to buffer
	 * @param in_nBufferLen length of buffer
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getCharset( char* out_pszBuffer, int in_nBufferLen ) = 0;

	/*
	 * @desc gets the length for IME to input buffer
	 * @param 
	 * @return int length
	 */
	virtual int getMaxLength( void ) = 0;

	/*
	 * @desc gets the number of WML Button
	 * @param out_pszBuffer pointer to buffer
	 * @param in_nBufferLen length of buffer
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getFormat( char* out_pszBuffer, int in_nBufferLen ) = 0;

	/*
	 * @desc gets the number of WML Button
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT isPassword( void ) = 0;

	/*
	 * @desc gets the type of window style
	 * @param 
	 * @return int window style
	 */
	virtual int getStyle( void ) = 0;

	/*
	 * @desc gets the rect of IME window
	 * @param out_x, out_y, out_w, out_h : x,y,w,h
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getRect( int* out_x, int* out_y, int* out_w, int* out_h ) = 0;

	/*
	 * @desc gets the number of WML Button
	 * @param out_pFontAttr pointer to IFontAttr
	 * @return int number
	 */
	virtual HRESULT getFontAttr( IFontAttr* out_pFontAttr ) = 0;

	/*
	 * @desc gets the usage
	 * @param 
	 * @return int usage
	 */
	virtual int getUsage( void ) = 0;

};

class IIMEListener
{
public:

	/*
	 * @desc listner function called when IME start is requested from browser engine
	 * @param in_pIMEInfo : pointer to IIMEInfo
	 * @param in_pIMESender : pointer to IIMESender
	 * @return bool TRUE / FALSE
	 */
	virtual HRESULT StartIMEListener( IIMEInfo* in_pIMEInfo, IIMESender* in_pIMESender ) = 0;

	/*
	 * @desc listner function called when IME end is requested from browser engine
	 * @param out_psBuffer : pointer to buffer
	 * @param out_BufferLen : length of buffer
	 * @return bool TRUE / FALSE
	 */
	virtual HRESULT EndIMEListener( char* out_psBuffer, int* out_BufferLen ) = 0;

	/*
	 * @desc listner function called when IME stop is requested from browser engine
	 * @param 
	 * @return bool TRUE / FALSE
	 */
	virtual HRESULT StopIMEListener( void ) = 0;

};

} // namespace WebControl

#endif // __IIMEListener_Defined