/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		ISetting.h
 * @brief		Define interface ISetting of WebControl
 * @author	Chaehwan Li (chaehwan.li@samsung.com)
 * @version	1.0
 * @date		2008.03.14	v1.0		Created
 */
#ifndef __ISetting_Defined
#define __ISetting_Defined

#include "WebDefs.h"

namespace	WebControl
{

class ISettingListener;

/*
 * @desc
 */
class ISetting {

public:
	/*
	 * @desc sets a property of Browser Engine.
	 * @param in_PropertyID : Property ID to setting
	 * @param in_PropertyValue : pointer to Property Value to setting
	 * @return bool TRUE / FALSE
	 */
	virtual HRESULT setProperty( int in_PropertyID, void* in_PropertyValue ) = 0;

	/*
	 * @desc gets a property of Browser Engine.
	 * @param in_PropertyID : Property ID to setting
	 * @param out_PropertyValue : pointer to Property Value to setting
	 * @return bool TRUE / FALSE
	 */
	virtual HRESULT getProperty( int in_PropertyID, void* out_PropertyValue ) = 0;

	/**
	* Add a listener to the listener list
	* All listeners in the list will be notified whenever the setting value change
	*/
	virtual HRESULT addListener( ISettingListener* pListener ) = 0;

	/**
	* Remove the listener from the listener list
	*/
	virtual HRESULT removeListener( ISettingListener* pListener ) = 0;
};


class ISettingListener
{
public:
	/**
	* called when the any values of pSettins is changed
	* @param pSetting the setting whose property is changed
	* @param in_PropertyID the property id whose value is changed
	*/
	virtual void OnSettingChanged( ISetting *pSetting, int in_PropertyID ) = 0;
};

} // namespace  WebControl

#endif // __ISetting_Defined




