/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics.
 */
/**
 * @file		WebBackingStore.cpp
 * @brief		
 * @author	Sangheon Kim (sangheon77.kim@samsung.com) @n
					
 * @version	1.0
 * @date		2008.07.23	v1.0		Created							shkim @n
 */

#if defined(_SFB_BACKINGSTORE) && !defined(SFB_GWES_TRANSFORM)

#include "WebBackingStore.h"
#include "WebControl.h"
#include "FrameView.h"
#include "GraphicsContext.h"
#include "FrameLoader.h"
#include "RenderView.h"

#define INFLATE_OFFSET 10
#define DEFAULT_BACKINGSTORE_MULTIPLIER	3.0 // backingstore = multiplier * viewsize
#define INITIAL_MIN_SCALE 0.33333
#define ORIGINAL_SCALE 1.0
#define BACKINGSTORE_CONTSIZE_DX 50
#define BACKINGSTORE_CONTSIZE_DY 50
// bs0425.lee modified for touch scrollbar. 
#define extendRange		40
#define extendBar	25

#ifdef SFB_GWES_TRANSFORM
#define MIN_MULTIPLIER 0.3
#endif


#define _KEEP_CHECKING_SIZE

static const int cRectThreshold = 10;
static const float cWastedSpaceThreshold = 0.75f;

#ifdef _BACKINGSTORE_DEBUG
#define DEBUG_FOLDER "/User/Br/Debug/"
#endif //_BACKINGSTORE_DEBUG

namespace WebControl
{

WebBackingStore::WebBackingStore( WebControl *pWebControl )
: //m_backingStoreRect( 0, 0, -1, -1 ),
m_prevContentsSize( -1, -1 ),
m_viewRect( 0, 0, pWebControl->GetWidth(), pWebControl->GetHeight() ),
m_zoomingPoint( -1, -1 ),
m_hBar( this, WebCore::HorizontalScrollbar, WebCore::RegularScrollbar ),
m_vBar( this, WebCore::VerticalScrollbar, WebCore::RegularScrollbar ),
m_scrollError( 0.0, 0.0 ),
m_scrollErrorCorrection( 0.0, 0.0 )
{
#ifdef SFB_GWES_TRANSFORM
	m_zoomMultiplier = MIN_MULTIPLIER;
#else
        m_zoomMultiplier = ORIGINAL_SCALE;
#endif
	m_pWebControl = pWebControl;
	m_pDirtyRegion = NULL;
	m_pPaintRegion = NULL;
	m_bCompleteDirty = false;
	//m_backingStoreGC = INVALID_HOBJ;
#ifdef SFB_GWES_TRANSFORM
	WebCore::IntSize backingStoreSize = CalculateBackingStoreSize( WebCore::IntSize() );

	if ( m_pWebControl->GetGCType() == BIT_32_GC  )
		m_backingStoreGC = WmCreateWindowSubMemGC32( m_pWebControl->GetHWIN(), 0, 0, backingStoreSize.width(), backingStoreSize.height()   );
	else
		m_backingStoreGC = WmCreateWindowSubMemGC( m_pWebControl->GetHWIN(), 0, 0, backingStoreSize.width(), backingStoreSize.height()   );
#else
	if ( m_pWebControl->GetGCType() == BIT_32_GC  )
		m_backingStoreGC = WmCreateWindowSubMemGC32( m_pWebControl->GetHWIN(), 0, 0, pWebControl->GetWidth(), pWebControl->GetHeight()  );
	else
		m_backingStoreGC = WmCreateWindowSubMemGC( m_pWebControl->GetHWIN(), 0, 0, pWebControl->GetWidth(), pWebControl->GetHeight()  );
#endif
	WebAssert( m_backingStoreGC != INVALID_HOBJ );
	m_bPageLoading = false;

	//WebCore::FrameView* frameView = NULL;
	//frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
	////frameView->setScrollbarsMode( WebCore::ScrollbarAlwaysOff );
}

WebBackingStore::~WebBackingStore( )
{
	if( m_backingStoreGC != INVALID_HOBJ )
		WmDeleteGC( m_backingStoreGC );
}


void WebBackingStore::SetBackingStoreMultiplier( float multiplier )
{
}

float WebBackingStore::GetBackingStoreMultiplier()
{
	return DEFAULT_BACKINGSTORE_MULTIPLIER;
}

void WebBackingStore::AddToDirtyRegion(const WebCore::IntRect& rect)
{
	WmBox box;
	WmRect rc;
	WebCore::FrameView* frameView = NULL;
	WebCore::IntRect tempRect;



	
	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
	WebCore::IntRect frameViewRect =  frameView->frameGeometry();

//debug
	//WebCore::IntRect viewRect  = frameView->frameGeometry();

	
	//if( m_bCompleteDirty == true )
	//	return;
	if( m_pDirtyRegion && (REGION_NUM_RECTS(m_pDirtyRegion) > cRectThreshold) )
	{
		tempRect = frameViewRect;
		//m_bCompleteDirty = true;
	}
	//else if( rect.contains( frameViewRect ) )
	//{
	//	tempRect = frameViewRect;
		//m_bCompleteDirty = true;
	//}
	else
	{
		tempRect = frameViewRect;
		tempRect.intersect( rect );
	}
	
	rc.x = tempRect.x();
	rc.y = tempRect.y();
	rc.w = tempRect.width();
	rc.h = tempRect.height();

	if( rc.w == 0 || rc.h == 0 )
		return;

#ifdef _PERFORMANCE_DEBUG
	m_addToDirtyCount++;
#endif

		
	CONVERT_RECT_TO_BOX(&box, &rc);

	if( m_pDirtyRegion == NULL )
	{
		m_pDirtyRegion = WmCreateRegion(&box, 0);
		if( m_pDirtyRegion == NULL )
			WebAssert( FALSE );
	}
	else
	{
		WmRegion region;

#if  defined(_BACKINGSTORE_DEBUG) && defined(_ADDTODIRTYREGION_DEBUG)
		HGC hDebugGC;
		if ( m_pWebControl->GetGCType() == BIT_32_GC  )
			hDebugGC = WmCreateWindowSubMemGC32( m_pWebControl->GetHWIN(), 0, 0, frameView->frameGeometry().width(), frameView->frameGeometry().height() );
		else
			hDebugGC = WmCreateWindowSubMemGC( m_pWebControl->GetHWIN(), 0, 0, frameView->frameGeometry().width(), frameView->frameGeometry().height() );

		CopyGC( hDebugGC, 0, 0, m_backingStoreGC, 0, 0,  frameView->frameGeometry().width(), frameView->frameGeometry().height()  );
#endif //_BACKINGSTORE_DEBUG

#if defined(_BACKINGSTORE_DEBUG) && defined(_ADDTODIRTYREGION_DEBUG)
							{
							HGC hGC = hDebugGC;
							WmBox* pBox;
							int nbBox, index;

							nbBox = REGION_NUM_RECTS(m_pDirtyRegion);
							pBox = REGION_RECTS(m_pDirtyRegion);

								//int dx, dy;
								//	WebCore::FrameView *frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
								//dx = frameView->contentsX() - m_backingStoreRect.x();
								//dy = frameView->contentsY() - m_backingStoreRect.y();

							WmSetFgColor( hGC, BLUE );
							WmSetLineWidth( hGC, 1 );
							for(index = 0; index < nbBox; index++, pBox++)
							{
								WmRect tempRect;
								CONVERT_BOX_TO_RECT(&tempRect, pBox);
								WmDrawRect( hGC, tempRect.x, tempRect.y, tempRect.w, tempRect.h );
							}

							WmSetFgColor(hGC, GREEN );
							WmSetLineWidth( hGC, 1 );
							WmDrawRect( hGC, rc.x, rc.y, rc.w, rc.h );
							//WmDrawTransparentFilledRect( hGC, YELLOW, 85, rc.x, rc.y, rc.w, rc.h );
							}
#endif //_BACKINGSTORE_DEBUG


		WmInitRegion( &region, &box, 0);
		WmUnionRegion( m_pDirtyRegion, m_pDirtyRegion, &region );

#if defined(_BACKINGSTORE_DEBUG) && defined(_ADDTODIRTYREGION_DEBUG)
							{
								HGC hGC = hDebugGC;
								WmBox* pBox;
								int nbBox, index;

								//int dx, dy;
								//	WebCore::FrameView *frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
								//dx = frameView->contentsX() - m_backingStoreRect.x();
								//dy = frameView->contentsY() - m_backingStoreRect.y();

								nbBox = REGION_NUM_RECTS(m_pDirtyRegion);
								pBox = REGION_RECTS(m_pDirtyRegion);

								WmSetFgColor( hGC, RED );
								WmSetLineWidth( hGC, 1 );
								for(index = 0; index < nbBox; index++, pBox++)
								{
									WmRect tempRect;
									CONVERT_BOX_TO_RECT(&tempRect, pBox);
									//WmDrawRect( hGC, tempRect.x, tempRect.y, tempRect.w, tempRect.h );
									WmDrawTransparentFilledRect( hGC, RED, 85, tempRect.x, tempRect.y, tempRect.w, tempRect.h );
								}

								SaveBackingStoreToImageFile( hDebugGC,frameView->frameGeometry().width(), frameView->frameGeometry().height(), "DirtyRegion" );
								WmDeleteGC( hDebugGC );

							} // debug code
#endif //_BACKINGSTORE_DEBUG

	}
}

WebCore::IntRect WebBackingStore::BackingStoreRect()
{
	WebCore::FrameView *frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

	WebCore::IntRect backingStoreRect = frameView->frameGeometry();
	backingStoreRect.setX( frameView->contentsX() );
	backingStoreRect.setY( frameView->contentsY() );

	return backingStoreRect;
}

void WebBackingStore::ScrollBy( int ori_dx, int ori_dy )
{
	int dx, dy;


	float floatdx, floatdy;
	floatdx = (float)ori_dx / (float)m_zoomMultiplier;
	floatdy = (float)ori_dy / (float)m_zoomMultiplier;


	dx = floatdx;
	dy = floatdy;


	if( dx == 0 && dy == 0 )
		return;

	int viewX, viewY;
	WebCore::FrameView *frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
	WebCore::IntSize contentsSize( frameView->contentsWidth(), frameView->contentsHeight() );

	WebCore::IntRect backingStoreRect =BackingStoreRect();

	WebCore::IntRect viewRect = m_viewRect;
	viewRect.setWidth( ((float)m_viewRect.width() / m_zoomMultiplier) );
	viewRect.setHeight( ((float)m_viewRect.height() / m_zoomMultiplier) );


	WebCore::IntSize scrollOffset( viewRect.x(), viewRect.y() );
	WebCore::IntSize maximumScroll = ( contentsSize - WebCore::IntSize(viewRect.width(), viewRect.height())) - scrollOffset;
    maximumScroll.clampNegativeToZero();
	WebCore::IntSize newScrollOffset = scrollOffset + WebCore::IntSize(dx, dy).shrunkTo(maximumScroll);
    newScrollOffset.clampNegativeToZero();

	viewRect.setX( newScrollOffset.width() );
	viewRect.setY( newScrollOffset.height() );
	m_viewRect.setX( newScrollOffset.width() );
	m_viewRect.setY( newScrollOffset.height() );

	if( backingStoreRect.contains( viewRect ) )
	{
		PaintToWindow( 0,0, m_viewRect.width(), m_viewRect.height(), true, true );
	}
	else
	{
		int newX = viewRect.x() - ((backingStoreRect.width() - viewRect.width()) / 2);
		int newY = viewRect.y() - ((backingStoreRect.height() - viewRect.height()) / 2);

		int newDx = newX - backingStoreRect.x();
		int newDy = newY - backingStoreRect.y();

		frameView->scrollBy( newDx, newDy );

	}
}


void WebBackingStore::ScrollBackingStore( int dx, int dy, const WebCore::IntRect& scrollViewRect, const WebCore::IntRect& clipRect)
{
	float dx2 ,dy2;
	WebCore::FrameView *frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
 
	WebCore::IntRect newBackingStore = BackingStoreRect();
	WebCore::IntRect oldBackingStore = newBackingStore;
	WebCore::IntRect intersects = newBackingStore;


	dx2 = (float)dx * (float)m_zoomMultiplier;
	dy2 = (float)dy * (float)m_zoomMultiplier;

	dx2 += m_scrollError.width();
	dy2 += m_scrollError.height();

	int intDx = dx2;
	int intDy = dy2;

	m_scrollError.setWidth( dx2 - (float)intDx );
	m_scrollError.setHeight(dy2 - (float)intDy );

	m_scrollErrorCorrection.setWidth( m_scrollError.width() / m_zoomMultiplier );
	m_scrollErrorCorrection.setHeight(  m_scrollError.height() / m_zoomMultiplier );

	oldBackingStore.move( intDx, intDy );

	intersects.intersect( oldBackingStore );

	//
	CopyGC( m_backingStoreGC, intersects.x() - newBackingStore.x(), intersects.y() - newBackingStore.y(),
		m_backingStoreGC, intersects.x() - oldBackingStore.x(), intersects.y() - oldBackingStore.y(), intersects.width(), intersects.height() );


#ifdef _BACKINGSTORE_DEBUG
	SaveBackingStoreToImageFile( m_backingStoreGC,frameView->frameGeometry().width(), frameView->frameGeometry().height(), "ScrollGC" );

	HGC hDebugGC;

	if ( m_pWebControl->GetGCType() == BIT_32_GC  )
		hDebugGC= WmCreateWindowSubMemGC32( m_pWebControl->GetHWIN(), 0, 0, frameView->frameGeometry().width(), frameView->frameGeometry().height() );
	else
		hDebugGC= WmCreateWindowSubMemGC( m_pWebControl->GetHWIN(), 0, 0, frameView->frameGeometry().width(), frameView->frameGeometry().height() );

	WebCore::GraphicsContext gc(&hDebugGC);
	WebCore::IntRect paintRect = frameView->frameGeometry();
	gc.save();

#if defined(SFB_GWES_TRANSFORM)
			gc.scale( WebCore::FloatSize( m_zoomMultiplier, m_zoomMultiplier) );
			gc.translate( -m_scrollErrorCorrection.width(), -m_scrollErrorCorrection.height() );
			paintRect.move( m_scrollErrorCorrection.width(), m_scrollErrorCorrection.height()  );
#endif
	gc.clip(paintRect);
	frameView->paint(&gc, paintRect);
	gc.restore();

	SaveBackingStoreToImageFile( hDebugGC,frameView->frameGeometry().width(), frameView->frameGeometry().height(), "ScrollGCPaint" );
	WmDeleteGC( hDebugGC );
#endif



	oldBackingStore = newBackingStore;
	intersects = newBackingStore;


	oldBackingStore.move( dx, dy ); 

	intersects.intersect( oldBackingStore );

	// Dirty region
	WmBox box;
	WmRect rc;
	WmRegion region, updateRegion;

	rc.x = newBackingStore.x();
	rc.y = newBackingStore.y();
	rc.w = newBackingStore.width();
	rc.h = newBackingStore.height();
		
	CONVERT_RECT_TO_BOX(&box, &rc);

	WmInitRegion( &updateRegion, &box, 0 );

	rc.x = intersects.x();
	rc.y = intersects.y();
	rc.w = intersects.width();
	rc.h = intersects.height();

	CONVERT_RECT_TO_BOX(&box, &rc);

	WmInitRegion( &region, &box, 0 );

	WmSubtractRegion( &updateRegion, &updateRegion, &region);

	WmBox* pBox;
	int nbBox, index;

	nbBox = REGION_NUM_RECTS(&updateRegion);
	pBox = REGION_RECTS(&updateRegion);

	for(index = 0; index < nbBox; index++, pBox++)
	{
		WebCore::IntRect tempRect( pBox->x1 - newBackingStore.x(), pBox->y1 - newBackingStore.y(), pBox->x2-pBox->x1, pBox->y2 - pBox->y1 );

		//tempRect.inflate( INFLATE_OFFSET );

		AddToDirtyRegion( tempRect );
	}

	AddToPaintRegion( WebCore::IntRect( 0, 0, (float)(m_viewRect.width())/m_zoomMultiplier, (float)(m_viewRect.height())/m_zoomMultiplier) );

	UpdateBackingStore( false );

}


void WebBackingStore::UpdateBackingStore( bool bCompleteDirty )
{
	if( m_backingStoreGC == INVALID_HOBJ )
		return;

	if( m_pDirtyRegion == NULL  && bCompleteDirty == false )
		return;

#ifdef _PERFORMANCE_DEBUG
	m_updateBackingStoreCount++;
#endif

	WebCore::FrameView *frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

#ifdef _BACKINGSTORE_DEBUG
		HGC hDebugGC;

		if ( m_pWebControl->GetGCType() == BIT_32_GC  )
			hDebugGC = WmCreateWindowSubMemGC32( m_pWebControl->GetHWIN(), 0, 0, frameView->frameGeometry().width(), frameView->frameGeometry().height() );
		else
			hDebugGC = WmCreateWindowSubMemGC( m_pWebControl->GetHWIN(), 0, 0, frameView->frameGeometry().width(), frameView->frameGeometry().height() );

		//WmCopyGC( hDebugGC, 0, 0, m_backingStoreGC, 0, 0,  frameView->frameGeometry().width(), frameView->frameGeometry().height() );
		WmDrawFilledRect( hDebugGC, YELLOW, 0, 0, frameView->frameGeometry().width(), frameView->frameGeometry().height() );

		{
			HGC hGC = hDebugGC;
			WmBox* pBox;
			int nbBox, index;

			if( m_pDirtyRegion )
			{
				//int dx, dy;
				//	WebCore::FrameView *frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
				//dx = frameView->contentsX() - m_backingStoreRect.x();
				//dy = frameView->contentsY() - m_backingStoreRect.y();

				nbBox = REGION_NUM_RECTS(m_pDirtyRegion);
				pBox = REGION_RECTS(m_pDirtyRegion);

				//WmClearGC( hGC );
				WmSetFgColor( hGC, YELLOW );

				for(index = 0; index < nbBox; index++, pBox++)
				{
					WmRect tempRect;
					CONVERT_BOX_TO_RECT(&tempRect, pBox);
					WmDrawRect( hGC, tempRect.x, tempRect.y, tempRect.w, tempRect.h );
				}
			}
		}
#endif //_BACKINGSTORE_DEBUG


	frameView->layoutIfNeededRecursive();


	WTF::Vector<WebCore::IntRect> paintRects;
	if( !bCompleteDirty )
	{
		WmBox * regionBox;
		regionBox = WmRegionExtents( m_pDirtyRegion );
		WebCore::IntRect regionRect( regionBox->x1, regionBox->y1, regionBox->x2-regionBox->x1, regionBox->y2-regionBox->y1 );
		GetUpdateRects( m_pDirtyRegion, regionRect, paintRects );
	}
	else
	{
		paintRects.append( frameView->frameGeometry() );
	}


	

	WebAssert( paintRects.size() > 0 );

	//WmClearGC( m_backingStoreGC );
	for( unsigned i = 0; i < paintRects.size(); i++ )
	{
		if (frameView && frameView->frame() && frameView->frame()->renderer()) 
		{
#ifdef _BACKINGSTORE_DEBUG
			WebCore::IntRect paintRectDebug = paintRects[i];
#endif
			WebCore::IntRect paintRect = paintRects[i];
			
			AddToPaintRegion( paintRect );

			WebCore::GraphicsContext gc(&m_backingStoreGC);
			gc.save();


#if defined(SFB_GWES_TRANSFORM)

			gc.scale( WebCore::FloatSize( m_zoomMultiplier, m_zoomMultiplier) );
			gc.translate( -m_scrollErrorCorrection.width(), -m_scrollErrorCorrection.height() );
			paintRect.move( m_scrollErrorCorrection.width(), m_scrollErrorCorrection.height()  );
#endif
			gc.clip(paintRect);
			frameView->paint(&gc, paintRect);
			gc.restore();

#ifdef _BACKINGSTORE_DEBUG
					{
					WebCore::GraphicsContext DebugGC(&hDebugGC);
					DebugGC.save();
#if defined(SFB_GWES_TRANSFORM)
					DebugGC.scale( WebCore::FloatSize( m_zoomMultiplier, m_zoomMultiplier) );
					DebugGC.translate(  -m_scrollErrorCorrection.width(), -m_scrollErrorCorrection.height() );
					paintRectDebug.move(  m_scrollErrorCorrection.width(), m_scrollErrorCorrection.height()  );	

	
#endif
					DebugGC.clip(paintRectDebug);
					frameView->paint(&DebugGC,paintRectDebug);

					DebugGC.setStrokeColor( WebCore::Color("red") );
					DebugGC.setFillColor( WebCore::Color( 0xff, 0xff, 0xff, 0x00 ) );
					DebugGC.drawRect( paintRectDebug );
					
					DebugGC.restore();

					//WmSetFgColor( hDebugGC, RED );
					//WmDrawRect( hDebugGC, paintRects[i].x(), paintRects[i].y(), paintRects[i].width(), paintRects[i].height() );
					}
#endif

		}
		else
		{
			WebAssert( FALSE );
		}
	}

#ifdef _BACKINGSTORE_DEBUG
	SaveBackingStoreToImageFile( hDebugGC,frameView->frameGeometry().width(), frameView->frameGeometry().height(), "Update" );
	SaveBackingStoreToImageFile( m_backingStoreGC,frameView->frameGeometry().width(), frameView->frameGeometry().height(), "Backing" );
	WmDeleteGC( hDebugGC );
#endif

	WmDestroyRegion( m_pDirtyRegion );
	m_pDirtyRegion = NULL;
	m_bCompleteDirty = false;
}

void WebBackingStore::AddToPaintRegion( const WebCore::IntRect & paintRect )
{
	WmBox box;
	WmRect rc;
	WebCore::FrameView* frameView = NULL;
	WebCore::IntRect tempRect;
	WebCore::IntRect rect = paintRect;

	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();	

	rect.move(  -(m_viewRect.x() - frameView->contentsX()), -(m_viewRect.y() - frameView->contentsY()) );
	rect.scale( m_zoomMultiplier);

	tempRect = m_viewRect;
	tempRect.setX(0);
	tempRect.setY(0);

	tempRect.intersect( rect );

	rc.x = tempRect.x();
	rc.y = tempRect.y();
	rc.w = tempRect.width();
	rc.h = tempRect.height();

	if( rc.w == 0 || rc.h == 0 )
		return;

	CONVERT_RECT_TO_BOX(&box, &rc);

	if( m_pPaintRegion == NULL )
	{
		m_pPaintRegion = WmCreateRegion(&box, 0);
		if( m_pPaintRegion == NULL )
			WebAssert( FALSE );
	}
	else
	{
		WmRegion region;

		WmInitRegion( &region, &box, 0);
		WmUnionRegion( m_pPaintRegion, m_pPaintRegion, &region );
	}

}

void WebBackingStore::GetUpdateRects( WmRegion* region, const WebCore::IntRect& dirtyRect, WTF::Vector<WebCore::IntRect> &rects )
{
	WmBox* pBox;
	int nBox;

	nBox = REGION_NUM_RECTS(region);
	pBox = REGION_RECTS(region);

	rects.clear();

	if( nBox > cRectThreshold )
	{
		rects.append( dirtyRect );
		return;
	}

	double singlePixels = 0.0;
	unsigned index;

	for(index = 0; index < nBox; index++, pBox++)
	{
		singlePixels += (pBox->x2 - pBox->x1 ) * (pBox->y2 - pBox->y1 );
	}

	double unionPixels = dirtyRect.width() * dirtyRect.height();
	double wastedSpace = 1.0 - (singlePixels/unionPixels);
	if( wastedSpace <= cWastedSpaceThreshold )
	{
		rects.append( dirtyRect );
		return;
	}

	pBox = REGION_RECTS(region);

	for(index = 0; index < nBox; index++, pBox++)
	{
		WebCore::IntRect rect;
		rect.setX( pBox->x1 );
		rect.setY( pBox->y1 );
		rect.setWidth( pBox->x2 - pBox->x1 );
		rect.setHeight( pBox->y2 - pBox->y1 );

		rects.append( rect );
	}

}

void WebBackingStore::AdjustBackingStorePos( const WebCore::IntRect& viewRect )
{
	//int dx = (m_backingStoreRect.width() - viewRect.width()) / 2;
	//int dy = (m_backingStoreRect.height() - viewRect.height()) / 2;

	//int x, y;

	//x = viewRect.x() - dx;
	//y = viewRect.y() - dy;

	//if( x < 0 ) x = 0;
	//if( y < 0 ) y = 0;

	//m_backingStoreRect.setX( x );
	//m_backingStoreRect.setY( y );
}

void WebBackingStore::RotateBackingStore( int w, int h )
{
	m_viewRect.setWidth( w );
	m_viewRect.setHeight( h );

	//if( ValidateBackingStore() == false )
	//{
	//	WebCore::FrameView* frameView = NULL;
	//
	//	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
	//	// force to relayout
	//	//ChangeBackingStoreSize( frameView->frameGeometry().size() );
	//}

	WebCore::FrameView* frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
	WebCore::IntSize contentSize;
	contentSize.setWidth( frameView->contentsWidth() );
	contentSize.setHeight( frameView->contentsHeight() );

	WebCore::IntSize newBackingStoreSize = CalculateBackingStoreSize(  contentSize  );

	ChangeBackingStoreSize( newBackingStoreSize );
}


bool WebBackingStore::ValidateBackingStore()
{
	WebCore::IntSize contentSize;
	WebCore::IntSize backingStoreSize;
	WebCore::FrameView* frameView = NULL;
	WebCore::IntSize currBackingStoreSize;

	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

	contentSize.setWidth( frameView->contentsWidth() );
	contentSize.setHeight( frameView->contentsHeight() );


	if( m_prevContentsSize == contentSize )
		return false;

	SysDebug(( MID_EXCEPTION, "ValidateBackingStore : LayoutDebug prevContSize( %d, %d ), curContSize( %d, %d )\n", 
		m_prevContentsSize.width(), m_prevContentsSize.height(), contentSize.width(), contentSize.height() ));

	m_prevContentsSize = contentSize;


	backingStoreSize = CalculateBackingStoreSize(  contentSize  );


	currBackingStoreSize = frameView->frameGeometry().size();

	SysDebug(( MID_EXCEPTION, "ValidateBackingStore : LayoutDebug prevBackingStoreSize( %d, %d ), currBackingStoreSize( %d, %d )\n", 
		currBackingStoreSize.width(), currBackingStoreSize.height(), backingStoreSize.width(), backingStoreSize.height() ));


	// validate view
	int backingStoreX = frameView->contentsX();
	int backingStoreY = frameView->contentsY();
	WebCore::IntSize maxPos( backingStoreSize - m_viewRect.size() );
	WebCore::IntSize desiredOffset( m_viewRect.x() - backingStoreX, m_viewRect.y() - backingStoreY );
	WebCore::IntSize viewoffset = desiredOffset.shrunkTo(maxPos);
	viewoffset.clampNegativeToZero();

	m_viewRect.setX( backingStoreX + viewoffset.width() );
	m_viewRect.setY( backingStoreY + viewoffset.height() );


	if( currBackingStoreSize == backingStoreSize )
	{
		
		return false;
	}
	else // need to change
	{
		//m_backingStoreRect.setSize( backingStoreSize );
		//AdjustBackingStorePos( viewRect );

		ChangeBackingStoreSize( backingStoreSize );



		return true;
		//return false;
	}

}

void WebBackingStore::ChangeBackingStoreSize( const WebCore::IntSize &newSize )
{
	WebCore::Frame *frame =  m_pWebControl->GetMainWebFrame()->GetFrame();
	WebCore::IntSize oldSize = frame->view()->frameGeometry().size();


#if  defined(_KEEP_CHECKING_SIZE) && !defined(SFB_GWES_TRANSFORM)
	int dx = oldSize.width() - newSize.width();
	int dy = oldSize.height() - newSize.height();

	if( dx < 0 ) dx = -dx;
	if( dy < 0 ) dy = -dy;


	if( dx < BACKINGSTORE_CONTSIZE_DX && dy < BACKINGSTORE_CONTSIZE_DY   )
	{
		SysDebug(( MID_EXCEPTION, "ChangeBackingStoreSize : LayoutDebug , The (dx(%d), dy(%d)) is not enough to change size\n", dx, dy ));
		return;
	}

#else

	if( oldSize == newSize )
	{
		SysDebug(( MID_EXCEPTION, "ChangeBackingStoreSize : LayoutDebug , The (dx, dy) is not enough to change size\n" ));
		return;
	}
#endif


#ifdef _PERFORMANCE_DEBUG
	m_resizeBackingStoreCount++;
#endif


	if( m_backingStoreGC != INVALID_HOBJ )
		WmDeleteGC( m_backingStoreGC );

	if ( m_pWebControl->GetGCType() == BIT_32_GC  )
		m_backingStoreGC = WmCreateWindowSubMemGC32( m_pWebControl->GetHWIN(), 0, 0, newSize.width(), newSize.height() );
	else
		m_backingStoreGC = WmCreateWindowSubMemGC( m_pWebControl->GetHWIN(), 0, 0, newSize.width(), newSize.height() );

	WebAssert( m_backingStoreGC != INVALID_HOBJ );


	SysDebug(( MID_EXCEPTION, "ChangeBackingStoreSize : LayoutDebug oldSize(%d,%d) newSize( %d, %d )\n", 
		oldSize.width(), oldSize.height(),
		newSize.width(), newSize.height() ));


	// change view size
    frame->view()->resize( newSize.width() , newSize.height() );

    //frame->forceLayout();
    //frame->view()->adjustViewSize();
	//frame->view()->setNeedsLayout();
    //frame->sendResizeEvent();


}


void WebBackingStore::PaintToWindow( int x, int y, int w, int h, bool bDraw, bool bShow )
{
	HGC hGC;
	WebCore::FrameView* frameView = NULL;
	WebCore::FloatRect viewFloatRect;

	if( m_backingStoreGC == INVALID_HOBJ )
		return;

	

#ifdef _PERFORMANCE_DEBUG
	m_showToWindowCount++;
#endif


	int dx, dy;


	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

	// test
	//x = 0; y = 0;
	//w = m_viewRect.width();
	//h = m_viewRect.height();

	hGC = WmCreateClientGC( m_pWebControl->GetHWIN() );

	// draw area which has been updated
	if( x ==0 && y == 0 && w == 0 && h == 0 && m_pPaintRegion )
	{
		WmBox* pBox;
		int nbBox, index;

		nbBox = REGION_NUM_RECTS(m_pPaintRegion);
		pBox = REGION_RECTS(m_pPaintRegion);

		for(index = 0; index < nbBox; index++, pBox++)
		{
			WmRect tempRect;
			CONVERT_BOX_TO_RECT(&tempRect, pBox);
			dx = m_viewRect.x() - frameView->contentsX() + tempRect.x;
			dy = m_viewRect.y() - frameView->contentsY() + tempRect.y;
			if( bDraw )
				CopyGC( hGC, tempRect.x, tempRect.y, m_backingStoreGC, dx, dy, tempRect.w, tempRect.h );
			if( bShow )
				WmShowGCRectArea( hGC, tempRect.x, tempRect.y, tempRect.w, tempRect.h ); 
		}

		if( bShow )
		{
			WmDestroyRegion( m_pPaintRegion );
			m_pPaintRegion = NULL;
		}

		PaintScrollbar( hGC, bShow );
	}
	else // draw only the specified area
	{
		dx = m_viewRect.x() - frameView->contentsX() + x;
		dy = m_viewRect.y() - frameView->contentsY() + y;
		if( bDraw )
			CopyGC( hGC, x, y, m_backingStoreGC, dx, dy, w, h );

		if( bShow )
			WmShowGCRectArea( hGC, x, y, w, h ); 

		PaintScrollbar( hGC, bShow );
	}



	WmDeleteGC( hGC );
}


void WebBackingStore::PaintScrollbar( int hGC, bool bShow )
{
	WebCore::FrameView *frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
	int contentsWidth, contentsHeight;
	contentsWidth = frameView->contentsWidth();
	contentsHeight = frameView->contentsHeight();

	if( contentsHeight > m_viewRect.height() )
	{
		int hBarH = 0;
		if( contentsWidth > m_viewRect.width() )
		hBarH = m_hBar.height();

		if( m_viewRect.height() != m_vBar.height() || m_vBar.x() != m_viewRect.width() - m_vBar.width()  )
		{
			WebCore::IntRect vBarRect = WebCore::IntRect( m_viewRect.width() - m_vBar.width(),
                                   0,
                                   m_vBar.width(),
                                   m_viewRect.height() - hBarH );
			m_vBar.setRect( vBarRect );
		}
		m_vBar.setProportion( m_viewRect.height() - hBarH, (float)frameView->contentsHeight()*m_zoomMultiplier);
		m_vBar.setValue( m_viewRect.y() );


		WebCore::GraphicsContext gc(&hGC);
		gc.save();
		m_vBar.paint(&gc, m_viewRect);
		gc.restore();

	if( bShow )
		WmShowGCRectArea( hGC, m_vBar.x(), m_vBar.y(), m_vBar.width(), m_vBar.height() ); 
	}

	if( contentsWidth > m_viewRect.width() )
	{
		int vBarW = 0;
		if( contentsHeight > m_viewRect.height() )
		vBarW = m_vBar.width();

		if( m_viewRect.width() != m_hBar.width() || m_hBar.y() != m_viewRect.height() - m_hBar.height()  )
		{
			WebCore::IntRect hBarRect = WebCore::IntRect( 0,
                                   m_viewRect.height() - m_hBar.height(),
                                   m_viewRect.width() - vBarW,
                                   m_hBar.height());
			m_hBar.setRect( hBarRect );
		}
		m_hBar.setProportion( m_viewRect.width() - vBarW, (float)frameView->contentsWidth() * m_zoomMultiplier);
		m_hBar.setValue( m_viewRect.x() );


		WebCore::GraphicsContext gc(&hGC);
		gc.save();
		m_hBar.paint(&gc, m_viewRect);
		gc.restore();

	if( bShow )
		WmShowGCRectArea( hGC, m_hBar.x(), m_hBar.y(), m_hBar.width(), m_hBar.height() ); 
	}




}


void WebBackingStore::PageLoadStarted()
{
	m_bPageLoading = true;

	

	//WebCore::FrameView* frameView = NULL;
	//frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

	//m_viewRect.setX(frameView->contentsX());
	//m_viewRect.setY(frameView->contentsY());

#ifdef _BACKINGSTORE_DEBUG
	FmRemoveDir( DEBUG_FOLDER );
	FmCreateDir( DEBUG_FOLDER );
	m_filecount = 0;
#endif

#ifdef _PERFORMANCE_DEBUG
	m_addToDirtyCount = 0;
	m_adjustInitialScaleCount =0;
	m_updateBackingStoreCount = 0;
	m_showToWindowCount = 0;
	m_resizeBackingStoreCount = 0;
#endif //_BACKINGSTORE_DEBUG
}

void WebBackingStore::PageLoadFinished()
{
	m_bPageLoading = false;

#ifdef _PERFORMANCE_DEBUG
	SysDebug(( MID_EXCEPTION, "PageLoadFinished : LayoutDebug AddToDirty(%d), AdjustScale(%d), UpdateStore(%d), ShowStore(%d), ResizeStore(%d)\n", 
		m_addToDirtyCount,	m_adjustInitialScaleCount,	m_updateBackingStoreCount,	m_showToWindowCount , m_resizeBackingStoreCount ));
#endif
}

bool WebBackingStore::AdjustInitialScale()
{

	if( m_bPageLoading == false )
		return false; // scale is not changed

	WebCore::FrameView* frameView = NULL;
	WebCore::IntSize contentSize, prevContentsSize;
	WebCore::IntSize viewSize;
	float curScale, w, h;


	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

	//curScale = frameView->frame()->zoomFactor();

	w = frameView->contentsWidth();
	//w = w / curScale;
	h = frameView->contentsHeight();
	//h = h / curScale;

	contentSize.setWidth( w );
	contentSize.setHeight( h );

	viewSize = m_viewRect.size();//frameView->frameGeometry().size();

#ifndef _KEEP_CHECKING_SIZE
	SysDebug(( MID_EXCEPTION, "AdjustInitialScale : LayoutDebug backingstoreSize(%d %d) , curContSize( %d, %d )\n", 
	frameView->size().width(), frameView->size().height(), contentSize.width(), contentSize.height() ));
#endif

	if( m_prevContentsSize == contentSize )
		return false;
	else if( viewSize.width() >= contentSize.width() )
		return false;  // scale is not changed
	else
	{
		WebCore::Frame *frame = frameView->frame();
		float viewW = viewSize.width();
		float contentW = contentSize.width();
		float prevScale = frame->zoomFactor();

		contentW = contentW / prevScale;

		float newScale = viewW / contentW;


		if( newScale < INITIAL_MIN_SCALE )
			newScale = INITIAL_MIN_SCALE;


		if( newScale == prevScale )
			return false;
		else 
		{
#ifdef _PERFORMANCE_DEBUG
			m_adjustInitialScaleCount++;
#endif

				SysDebug(( MID_EXCEPTION, "AdjustInitialScale : LayoutDebug backingstoreSize(%d %d) , curContSize( %d, %d ), scale(%0.5f)\n", 
		frameView->size().width(), frameView->size().height(), contentSize.width(), contentSize.height(), newScale ));
			
			//frame->setZoomFactor( newScale , false);
			m_prevContentsSize = contentSize;

			Zoom( newScale );
			return true;
		}
	}
}

void WebBackingStore::AdjustViewPos()
{
	WebCore::FrameView* frameView = NULL;
	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

	int backingStoreX = frameView->contentsX();
	int backingStoreY = frameView->contentsY();

	WebCore::IntSize backingStoreSize = frameView->frameGeometry().size();

	WebCore::IntSize maxPos( backingStoreSize - m_viewRect.size() );
	WebCore::IntSize desiredOffset( m_viewRect.x() - backingStoreX, m_viewRect.y() - backingStoreY );
	WebCore::IntSize viewoffset = desiredOffset.shrunkTo(maxPos);
	viewoffset.clampNegativeToZero();

	m_viewRect.setX( backingStoreX + viewoffset.width() );
	m_viewRect.setY( backingStoreY + viewoffset.height() );

}

WebCore::IntPoint WebBackingStore::WindowToBackingStore( const WebCore::IntPoint& point) 
{
	WebCore::FrameView* frameView = NULL;
	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

	WebCore::IntPoint windowPoint = point;

	int dx, dy;

	dx = m_viewRect.x() - frameView->contentsX();
	dy = m_viewRect.y() - frameView->contentsY();

	windowPoint.move( dx, dy );

#if defined(SFB_GWES_TRANSFORM)
	windowPoint.setX( (float)windowPoint.x() / m_zoomMultiplier );
	windowPoint.setY( (float)windowPoint.y() / m_zoomMultiplier );
#endif


	return windowPoint;
}

WebCore::IntPoint WebBackingStore::BackingStoreToWindow( const WebCore::IntPoint& point) 
{
	//WebCore::FrameView* frameView = NULL;
	//frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

	int x, y;
	x = point.x();
	y = point.y();

	x = x - m_viewRect.x();
	y = y - m_viewRect.y();


	return WebCore::IntPoint( x, y );
}


void
Memcpy16(void* dest, const void* src, size_t length )
{
	char *pDest = (char*)dest;
	char *pSrc = (char*)src;

	if( pDest > pSrc && (pDest-pSrc) < length*2 )
	{
		int overlaplen = length*2 - (pDest-pSrc);
		int empty = length*2 - overlaplen;

		int i = 0;
		while(1)
		{
			i++;
			char *pD = pDest + length*2 - i*empty;
			char *pS = pSrc + length*2 - i*empty;

			if( pD < pDest )
			{
				AcMemcpy16( pDest, pSrc, (length*2 - (i-1)*empty)/2 );
				break;
			}
			
			AcMemcpy16( pD, pS, empty/2 );
		}
	}
	else
		AcMemcpy16( pDest, pSrc, length );
}


/**
* WmCopyGC create WmBitamp and draw the bitmap to dest GC.
* So WmCopyGC have to copy the area 2 times.
*
* This method move the area to dest GC directly.
*/
void WebBackingStore::CopyGC( HGC hDestGC, int destX, int destY, HGC hSrcGC, int srcX, int srcY, int srcW, int srcH )
{
	COLOR *srcframeBuf;
	COLOR *destframeBuf;
	int y, destframeBufW, srcframeBufW;
	WmGC *pSrcGC, *pDestGC;

	pSrcGC = WmGetGCPointer( hSrcGC );
	pDestGC = WmGetGCPointer( hDestGC );

	srcframeBuf = pSrcGC->pMem;
	srcframeBufW = pSrcGC->rcFrmbuf.w;

	destframeBuf = pDestGC->pMem;
	destframeBufW = pDestGC->rcFrmbuf.w;

	if ( !pDestGC->bMemGC )
	{
		destX += pDestGC->rcGC.x;
		destY += pDestGC->rcGC.y;
	}
	

	if( hDestGC != hSrcGC || destY <= srcY )
	{
		for ( y = srcY; y < srcY + srcH; y++, destY++ )
			Memcpy16( (destframeBuf + destY * destframeBufW + destX), (srcframeBuf + y * srcframeBufW + srcX), srcW );     
	}
	else
	{
		int dy = destY + srcH -1;
		for ( y = srcY+srcH-1; y >= srcY; y--, dy-- )
			Memcpy16( (destframeBuf + dy * destframeBufW + destX), (srcframeBuf + y * srcframeBufW + srcX), srcW );     
		
	}
}

WebCore::IntSize WebBackingStore::CalculateBackingStoreSize( const WebCore::IntSize & contentsSize )
{
#if !defined(SFB_GWES_TRANSFORM)
	WebCore::FrameView *frameView =  m_pWebControl->GetMainWebFrame()->GetFrame()->view();

	WebCore::IntSize backingStoreSize;

	backingStoreSize.setWidth( (float)m_viewRect.width() * GetBackingStoreMultiplier() );
	backingStoreSize.setHeight( (float)m_viewRect.height() * GetBackingStoreMultiplier()   );



	if( backingStoreSize.width() > contentsSize.width() )
		backingStoreSize.setWidth( contentsSize.width() );

	if( backingStoreSize.height() > contentsSize.height() )
		backingStoreSize.setHeight( contentsSize.height() );

	if( backingStoreSize.width() < m_viewRect.width() )
		backingStoreSize.setWidth( m_viewRect.width() );
	if( backingStoreSize.height() < m_viewRect.height() )
		backingStoreSize.setHeight( m_viewRect.height() );

	SysDebug(( MID_EXCEPTION, "CalculateBackingStoreSize : LayoutDebug : candiBackingStore(%d,%d), contSize( %d,%d ), backingStore(%d,%d)\n",
		(int)((float)m_viewRect.width() * GetBackingStoreMultiplier()), (int)((float)m_viewRect.height() * GetBackingStoreMultiplier()),
		contentsSize.width(), contentsSize.height(),
		backingStoreSize.width(), backingStoreSize.height() ));

	return backingStoreSize;
#else

	float backingStoreWidth = m_viewRect.width();
	float backingStoreHeight = m_viewRect.height();

	backingStoreWidth = backingStoreWidth / MIN_MULTIPLIER;
	backingStoreHeight = backingStoreHeight / MIN_MULTIPLIER;

	WebCore::IntSize backingStoreSize( backingStoreWidth, backingStoreHeight );
	return backingStoreSize;

#endif
}

float WebBackingStore::GetZoom()
{
#if defined(SFB_GWES_TRANSFORM)
	return m_zoomMultiplier;
#else
	return m_pWebControl->GetMainWebFrame()->GetFrame()->zoomFactor();
#endif
}


WebCore::FloatRect WebBackingStore::CalculateZoomArea( float newMultiplier, WebCore::IntPoint& newViewPoint )
{
	WebCore::FloatRect zoomArea( m_viewRect );
	WebCore::FloatPoint center;
	float w, h;

	float multiplier = GetZoom();
	multiplier = newMultiplier / multiplier;


	center.setX( zoomArea.x() + (zoomArea.width() / 2) );
	center.setY( zoomArea.y() + (zoomArea.height() / 2) );

	w = zoomArea.width() / multiplier;
	h = zoomArea.height() / multiplier;

	
	zoomArea.setX( center.x() - w/2 );
	zoomArea.setY( center.y() - h/2 );
	zoomArea.setWidth( w );
	zoomArea.setHeight( h );

	newViewPoint.setX(  zoomArea.x() * multiplier );
	newViewPoint.setY( zoomArea.y() * multiplier );
	//newViewPoint.clampNegativeToZero();

	return zoomArea;
}

/**
* Show zooming effect
*/
void WebBackingStore::Zooming( float newMultiplier )
{
	int dx = 0, dy = 0;
	WebCore::IntRect backingStore( BackingStoreRect() );
	WebCore::FloatRect zoomArea( m_viewRect );
	WebCore::FloatPoint center;
	float w, h;
	WebCore::IntPoint dest;

	zoomArea = CalculateZoomArea( newMultiplier, dest );

	m_zoomingPoint.setX( dest.x() );
	m_zoomingPoint.setY( dest.y() );

	WebCore::IntRect zoomRect = enclosingIntRect( zoomArea );
	
	

	HGC hZoomGC;
	if ( m_pWebControl->GetGCType() == BIT_32_GC  )
		hZoomGC = WmCreateWindowSubMemGC32( m_pWebControl->GetHWIN(), 0, 0, zoomRect.width(), zoomRect.height() );
	else
		hZoomGC = WmCreateWindowSubMemGC( m_pWebControl->GetHWIN(), 0, 0, zoomRect.width(), zoomRect.height() );

	WebCore::IntRect intersectRect( backingStore );
	intersectRect.intersect( zoomRect );

	CopyGC( hZoomGC, intersectRect.x() - zoomRect.x(),  intersectRect.y() - zoomRect.y(),
					m_backingStoreGC, intersectRect.x() - backingStore.x(),  intersectRect.y() - backingStore.y(), intersectRect.width(), intersectRect.height() );

	COLOR *frameBuf;
	WmGetGCFrmbuf( hZoomGC, &frameBuf );

	WmBitmap bitmap;
	bitmap.compressType  = COMPRESS_NONE;
	bitmap.bUseChromakey = FALSE;
	bitmap.bitsPerPixel	 = BITS_PER_PIXEL;	
	bitmap.rectW		 = zoomRect.width();
	bitmap.rectH		 = zoomRect.height();
	bitmap.pBits = frameBuf;


	HGC hGC = WmCreateClientGC( m_pWebControl->GetHWIN() );


	WmDrawScaledBitmap( hGC, 0, 0, m_viewRect.width(), m_viewRect.height(), &bitmap );

//#ifdef _BACKINGSTORE_DEBUG
//	SaveBackingStoreToImageFile( hGC, m_viewRect.width(), m_viewRect.height(), "zooming" );
//#endif

	WmShowGC( hGC );
	WmDeleteGC( hZoomGC );
	WmDeleteGC( hGC );
}

void WebBackingStore::Zoom( float newMultiplier )
{

#if !defined(SFB_GWES_TRANSFORM)
	WebCore::FrameView *frameView =  m_pWebControl->GetMainWebFrame()->GetFrame()->view();
	float oldcontentsWidth = frameView->contentsWidth();
	float oldcontentsHeight = frameView->contentsHeight();

	//contentsWidth *= newMultiplier;
	//contentsHeight *= newMultiplier;

	//WebCore::IntSize backingStoreSize = CalculateBackingStoreSize( WebCore::IntSize( contentsWidth, contentsHeight ) );
	WebCore::IntSize backingStoreSize = CalculateBackingStoreSize( m_viewRect.size() );

	frameView->frame()->view()->resize( backingStoreSize.width() , backingStoreSize.height() );
 //   frameView->frame()->sendResizeEvent();
	//ChangeBackingStoreSize( backingStoreSize );


	m_pWebControl->GetMainWebFrame()->GetFrame()->setZoomFactor( newMultiplier , false);


	int contentsWidth = frameView->contentsWidth();
	int contentsHeight = frameView->contentsHeight();

	WebCore::RenderView* root = static_cast<WebCore::RenderView*>(frameView->frame()->renderer());

	SysDebug(( MID_EXCEPTION, "Zoom : LayoutDebug oldContSize(%d,%d), calContSize(%d,%d), newContSize(%d,%d), docSize(%d,%d)\n",
		(int)oldcontentsWidth, (int)oldcontentsHeight,
		(int)(oldcontentsWidth*newMultiplier), (int)(oldcontentsHeight*newMultiplier),
		contentsWidth, contentsHeight,
		root->overflowWidth(false), root->overflowHeight(false) ));

	backingStoreSize = CalculateBackingStoreSize( WebCore::IntSize( contentsWidth, contentsHeight ) );
	ChangeBackingStoreSize( backingStoreSize );



	if( m_zoomingPoint.x() >= 0 )
	{
		m_viewRect.setLocation( m_zoomingPoint );
		m_zoomingPoint.setX( -1 );
		m_zoomingPoint.setY( -1 );
	}

#ifndef _KEEP_CHECKING_SIZE
	AdjustViewPos();
#endif


#else //SFB_GWES_TRANSFORM

	if( newMultiplier < MIN_MULTIPLIER )
		newMultiplier = MIN_MULTIPLIER;

	if( m_zoomMultiplier == newMultiplier )
		return;

	WebCore::IntPoint newViewPoint;
	WebCore::FloatRect zoomArea = CalculateZoomArea( newMultiplier, newViewPoint );

	m_zoomMultiplier = newMultiplier;

	WebCore::FrameView *frameView =  m_pWebControl->GetMainWebFrame()->GetFrame()->view();
	WebCore::FloatRect dirtyRect( frameView->frameGeometry() );
	dirtyRect.scale( 1/newMultiplier );
	AddToDirtyRegion( enclosingIntRect( dirtyRect) );
	UpdateBackingStore( false );

	WebCore::IntSize offset;
	offset.setWidth( newViewPoint.x() - m_viewRect.x() );
	offset.setHeight( newViewPoint.y() - m_viewRect.y() );

	ScrollBy( offset.width(), offset.height() );

#endif

}


void WebBackingStore::ResetViewState()
{
	SysDebug(( MID_EXCEPTION, "ResetViewState : LayoutDebug started\n" ));
	m_viewRect.setX(0);
	m_viewRect.setY(0);

	WebCore::Frame* frame = NULL;
	frame = m_pWebControl->GetMainWebFrame()->GetFrame();


	//frame->setZoomFactor( ORIGINAL_SCALE,  false);

#if !defined( SFB_GWES_TRANSFORM )
	WebCore::IntSize backingStoreSize = CalculateBackingStoreSize( m_viewRect.size() );
	ChangeBackingStoreSize( backingStoreSize );
#endif

	WmClearGC( m_backingStoreGC );
	
	m_bCompleteDirty = false;

	SysDebug(( MID_EXCEPTION, "ResetViewState : LayoutDebug finished\n" ));
}

void WebBackingStore::SaveViewState(WebCore::HistoryItem* item)
{
	WebCore::HistoryItemViewState state;

	state.m_viewPos.setX( m_viewRect.x() );
	state.m_viewPos.setY( m_viewRect.y() );

	item->setViewState( state );
}

void WebBackingStore::RestoreViewState()
{
	WebCore::HistoryItem* currentItem = m_pWebControl->GetMainWebFrame()->GetFrame()->loader()->currentHistoryItem();
    WebAssert(currentItem);

	WebCore::HistoryItemViewState state = currentItem->viewState();

	m_viewRect.setX( state.m_viewPos.x() );
	m_viewRect.setY( state.m_viewPos.y() );
}

void WebBackingStore::DispatchDidFirstLayout()
{

#if !defined(_KEEP_CHECKING_SIZE) && !defined(SFB_GWES_TRANSFORM)
	SysDebug(( MID_EXCEPTION, "DispatchDidFirstLayout : LayoutDebug started\n" ));
	AdjustInitialScale();
	SysDebug(( MID_EXCEPTION, "DispatchDidFirstLayout : LayoutDebug finished\n" ));
#endif

}


void WebBackingStore::DispatchDidLoadMainResource(WebCore::DocumentLoader*) 
{
	SysDebug(( MID_EXCEPTION, "DispatchDidLoadMainResource : LayoutDebug started\n" ));
}

void WebBackingStore::Paint( int x, int y, int w, int h )
{

	bool bCompleteDirty = false;

#if defined(_KEEP_CHECKING_SIZE) && !defined(SFB_GWES_TRANSFORM)
	bCompleteDirty = AdjustInitialScale();

	if( bCompleteDirty )
		return;

	bCompleteDirty = ValidateBackingStore();

	if( bCompleteDirty )
		return;
#endif


	UpdateBackingStore( bCompleteDirty );

	PaintToWindow( x, y, w, h, true, false );
}



void WebBackingStore::valueChanged(WebCore::Scrollbar*)
{
}

// Used to obtain a window clip rect.
WebCore::IntRect WebBackingStore::windowClipRect() const
{
	return m_viewRect;
}

// FIXME: It would be nice to set this state on the scroll bar instead of
// having to ask for it from the client at paint time.
bool WebBackingStore::isActive() const
{
	return true;
}

WebCore::IntRect WebBackingStore::GetViewRect()
{
	WebCore::IntRect viewRect = m_viewRect;

	viewRect.setWidth( ((float)viewRect.width()) / m_zoomMultiplier );
	viewRect.setHeight( ((float)viewRect.height()) / m_zoomMultiplier );

	return viewRect;
}

// bs0425.lee modified for touch scroll. 
WebCore::IntRect WebBackingStore::getBackingScrollbarData(const WebCore::IntPoint& mousePosition )
{
	//WebCore::PlatformScrollbar* c_Bar;
	WebCore::IntRect c_thumbRect;

	/*if (m_hBar)
	{
		c_Bar=m_hBar;
		c_thumbRect=c_Bar.thumbRect();
		m_trackRect=c_Bar.trackRect();

		// extend the range of touch scroll. 
		m_trackRect.setY(m_trackRect.y()-extendRange);
		c_thumbRect.setY(c_thumbRect.y()-extendRange);

		m_trackRect.setHeight(m_trackRect.height()+extendRange);
		c_thumbRect.setHeight(c_thumbRect.height()+extendRange);

		if(c_thumbRect.contains(mousePosition))
			return c_thumbRect;
	}
*/

	
		c_thumbRect=m_hBar.thumbRect();
		m_trackRect=m_hBar.trackRect();

		// extend the range of touch scroll. 
		m_trackRect.setY(m_trackRect.y()-extendRange);
		c_thumbRect.setY(c_thumbRect.y()-extendRange);

		m_trackRect.setHeight(m_trackRect.height()+extendRange);
		c_thumbRect.setHeight(c_thumbRect.height()+extendRange);

		// extend scrollBar range. 
		c_thumbRect.setX(c_thumbRect.x()-extendBar);
		if(c_thumbRect.x()<=0)
			c_thumbRect.setX(0);

		c_thumbRect.setWidth(c_thumbRect.x()+c_thumbRect.width()+extendBar);
		

		if(c_thumbRect.contains(mousePosition))
			return c_thumbRect;
//	} 
		else
		{
		c_thumbRect=m_vBar.thumbRect();
		m_trackRect=m_vBar.trackRect();

		m_trackRect.setX(m_trackRect.x()-extendRange);
		c_thumbRect.setX(c_thumbRect.x()-extendRange);
	
		m_trackRect.setWidth(m_trackRect.width()+extendRange);
		c_thumbRect.setWidth(c_thumbRect.width()+extendRange);

		// extend scrollBar range. 
		c_thumbRect.setY(c_thumbRect.y()-extendBar);
		if(c_thumbRect.y()<=0)
			c_thumbRect.setY(0);

		c_thumbRect.setHeight(c_thumbRect.y()+c_thumbRect.height()+extendBar);

		if(c_thumbRect.contains(mousePosition))
            		  return c_thumbRect;
		}
	
    return WebCore::IntRect(0, 0, 0, 0);    
}

WebCore::IntRect WebBackingStore::getBackingTrackRect() const
  {   
	  return m_trackRect;
 }

#ifdef _BACKINGSTORE_DEBUG

void WebBackingStore::SaveBackingStoreToImageFile( HGC hGC, int w, int h, char* namePrefix )
{
		WmBitmap *pCaptureBitmap = NULL;
		ImgImage img;
		char szFilePath[100] = {0x00, };
		COLOR *frameBuf;

		if( hGC == INVALID_HOBJ )
			return;


		WmGetGCFrmbuf( hGC, &frameBuf );



	AcMemset(&img, 0x00, sizeof(ImgImage));

	img.pHead = (ImgBitmapElem*)MemAlloc( sizeof(ImgBitmapElem) );

	if (!img.pHead )
	{
		WebAssert( FALSE );
	}
	
	img.pHead->pbmBitmap = (ImgBitmap*)MemAlloc( sizeof(ImgBitmap) );

	if (!(img.pHead->pbmBitmap))
	{
		WebAssert( FALSE );
	}

		unsigned char* pEncodedData = NULL;
		ULONG	uFileSize = 0;
		HFILE hFile = INVALID_HOBJ;
		//pCaptureBitmap = WmCaptureBitmap( hGC, 0 , 0 , w , h ); 
		//		fileCount++;

		AcMemset(img.pHead->pbmBitmap, 0x00, sizeof(ImgBitmap));

		img.bitmapCount = 1;
		img.pHead->pbmBitmap->width  = w;   //width
		img.pHead->pbmBitmap->height =  h;   //height
		img.pHead->pbmBitmap->pBits  = frameBuf;
		img.pHead->pbmBitmap->bitsPerPixel = 16;
		//img.pHead->pbmBitmap->bChromaKeyFlag = pCaptureBitmap->bUseChromakey;
		//img.pHead->pbmBitmap->chromaKey = pCaptureBitmap->colorChromakey;

		 //Bmp ==> Jpg
		if (FALSE == ImgEncodeImage(IMG_ENC_JPEG, &img, &pEncodedData, &uFileSize, 5))
		{
			WebAssert( FALSE );
		}
		
		m_filecount++;
		AcSprintfS( szFilePath, sizeof(szFilePath), "%s%05d%s.jpg", (char*)DEBUG_FOLDER, m_filecount, namePrefix );

		FmRemoveFile( szFilePath );
		// Create File
		hFile = FmCreateFile( szFilePath, FM_WRITE);  //  //?? |FM_TRUNCATE
		if( hFile == INVALID_HOBJ )
		{
			WebAssert( FALSE );
		}

		//FmWriteFile( hFile, frameBuf, w*h*2);

		if (FmWriteFile( hFile, pEncodedData, uFileSize ) == FALSE)
		{
			WebAssert( FALSE );
			
		}

		if (pEncodedData)
		{
			MemFree(pEncodedData);
		}

		//if (pCaptureBitmap)
		//{
		//	WmFreeCapturedBitmap(pCaptureBitmap);
		//}

		FmCloseFile( hFile );

		if ( (img.pHead) && (img.pHead->pbmBitmap) )
		{
			MemFree(img.pHead->pbmBitmap);
			img.pHead->pbmBitmap = NULL;
		}

		if (img.pHead)
		{
			MemFree(img.pHead);
			img.pHead = NULL;
		}

}
#endif // _BACKINGSTORE_DEBUG


} // namespace WebControl


#endif //#ifndef SFB_GWES_TRANSFORM