/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics.
 */
/**
 * @file		WebBackingStore.cpp
 * @brief		
 * @author	Sangheon Kim (sangheon77.kim@samsung.com) @n
					
 * @version	1.0
 * @date		2008.07.23	v1.0		Created							shkim @n
 */
#ifdef SFB_GWES_TRANSFORM

#include "WebBackingStore.h"
#include "WebControl.h"
#include "FrameView.h"
#include "GraphicsContext.h"
#include "FrameLoader.h"
#include "RenderView.h"
//#if defined(_PERFORMANCE_DEBUG)
//#include "WebPerformanceLog.h"
//#endif

#if defined(_PERFORMANCE_DEBUG)
#include "SFBalProfiler.h"
#endif

#define BACKINGSTORE_MULTIPLIER	3.0 // backingstore = multiplier * viewsize
#define ORIGINAL_SCALE 1.0
#define DEFAULT_MINZOOM	0.33333
#define DEFAULT_MAXZOOM	2.0
#define UPDATECONTENTS_TIMEOUT 5000 //5sec

#ifdef DYNAMIC_SCROLLBAR
#define DYNAMICSCROLLBAR_TIMEOUT 2000 //2Sec
#endif

#define __CreateTimer	WmCreateTimer 
#define __StartTimer WmStartTimer
#define __DeleteTimer WmDeleteTimer

// bs0425.lee modified for touch scrollbar. 
#define extendRange		40
#define extendBar	25

#define INFLATE_DIRTY_RECT	2

static const int cRectThreshold = 10;
static const float cWastedSpaceThreshold = 0.75f;

#ifdef _BACKINGSTORE_DEBUG
#define DEBUG_FOLDER "/User/Br/Debug/"
#endif //_BACKINGSTORE_DEBUG

#define _SFB_BACKINGSTORE_IMPROVE_LOADPERF
#define _USE_UPDATECONTENTS_TIMER

namespace WebControl
{

WebBackingStore::WebBackingStore( WebControl *pWebControl, int id )
: 
m_hBar( this, WebCore::HorizontalScrollbar, WebCore::RegularScrollbar ),
m_vBar( this, WebCore::VerticalScrollbar, WebCore::RegularScrollbar ),
m_scrollError( 0.0, 0.0 ),
m_scrollErrorCorrection( 0.0, 0.0 )
{
	m_zoomMultiplier = ORIGINAL_SCALE;
	m_minZoomMultiplier = DEFAULT_MINZOOM;
	m_maxZoomMultiplier = DEFAULT_MAXZOOM;
	m_pWebControl = pWebControl;
	m_pDirtyRegion = NULL;
	m_pPaintRegion = NULL;
	m_bDidUserZoomed = false;
	m_bEnableInitialOverviewMode = false;
	m_hUpdateContentsTimer = INVALID_HOBJ;
	m_bNeedToUpdateContents = false;
        m_bUseOldBackingStore = false;
//	SysDebug(( MID_EXCEPTION, "WebBackingStore::WebBackingStore : Width(%d), Height(%d) \n", pWebControl->GetWidth(), pWebControl->GetHeight()));
#ifdef DYNAMIC_SCROLLBAR
	m_hTouchScrollBarTimer = INVALID_HOBJ;
#endif

	m_hWindowGC = WmCreateClientGC( m_pWebControl->GetHWIN() );
	WebAssert( m_hWindowGC != INVALID_HOBJ );

	if ( m_pWebControl->GetGCType() == BIT_32_GC )
	{
		WmSetWindowBgColor( m_pWebControl->GetHWIN(), BLACK );
		m_backingStoreGC = WmCreateWindowSubMemGC32( m_pWebControl->GetHWIN(), 0, 0, pWebControl->GetWidth(), pWebControl->GetHeight()  );
	}
	else
		m_backingStoreGC = WmCreateWindowSubMemGC( m_pWebControl->GetHWIN(), 0, 0, pWebControl->GetWidth(), pWebControl->GetHeight()  );

	m_rcView = GetWindowRect();
	m_rcBackingStore = GetWindowRect();

	WebAssert( m_backingStoreGC != INVALID_HOBJ );
	m_bPageLoading = false;
	m_bHideScrollbar = false;

#ifdef _BACKINGSTORE_DEBUG
	m_WebControlID = id;
	m_filecount = 0;
#endif
}

WebBackingStore::~WebBackingStore( )
{
	if( WmVerifyGC(m_backingStoreGC) == TRUE )
		WmDeleteGC( m_backingStoreGC );

	if( WmVerifyGC(m_hWindowGC) == TRUE )
		WmDeleteGC( m_hWindowGC );

#ifdef DYNAMIC_SCROLLBAR
	if(m_hTouchScrollBarTimer != INVALID_HOBJ)
	{
		__DeleteTimer( m_hTouchScrollBarTimer);
		m_hTouchScrollBarTimer = INVALID_HOBJ;
	}
#endif

	if( m_hUpdateContentsTimer != INVALID_HOBJ )// delete timer	
	{
		__DeleteTimer( m_hUpdateContentsTimer );
		m_hUpdateContentsTimer = INVALID_HOBJ;
	}
}


float WebBackingStore::GetBackingStoreMultiplier()
{
	if( m_bEnableInitialOverviewMode )
		return (float)BACKINGSTORE_MULTIPLIER;
	else
	{
		//if( IsLoading() )
		return (float)1.0;
		//else
		//	return (float)BACKINGSTORE_MULTIPLIER;
	}
}

void  WebBackingStore::SetInitialOverviewMode( bool bEnable )
{
	m_bEnableInitialOverviewMode = bEnable;
}

void WebBackingStore::AddToDirtyRegion(const WebCore::IntRect& rect)
{

#if defined(_PERFORMANCE_DEBUG)
	SFBalProfileStart( "WebControl0", "AddToDirtyRegion" );
#endif

	WmBox box;
	WmRect rc;
	WebCore::FrameView* frameView = NULL;
	WebCore::IntRect tempRect, dirtyTempRect;



	
	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
	WebCore::IntRect frameViewRect =  frameView->frameGeometry();

//debug
	//WebCore::IntRect viewRect  = frameView->frameGeometry();

	

	//if( m_pDirtyRegion && (REGION_NUM_RECTS(m_pDirtyRegion) > cRectThreshold) )
	//{
	//	tempRect = frameViewRect;
	//}
	//else
	//{
			dirtyTempRect = rect;
			dirtyTempRect.setWidth( dirtyTempRect.width() + INFLATE_DIRTY_RECT );
			dirtyTempRect.setHeight( dirtyTempRect.height() + INFLATE_DIRTY_RECT );
			tempRect = frameViewRect;
			tempRect.intersect( dirtyTempRect );
//	}

//#if defined( _SFB_BACKINGSTORE_IMPROVE_LOADPERF )
//	if( IsLoading() )
//	{
//		WebCore::IntRect rcView = GetViewRect();
//		WebCore::IntRect rcFrameView = GetFrameViewRect();
//		rcView.move( -rcFrameView.x(), -rcFrameView.y() );
//		tempRect.intersect( rcView );
//	}
//#endif
	
	rc.x = tempRect.x();
	rc.y = tempRect.y();
	rc.w = tempRect.width();
	rc.h = tempRect.height();

	if( rc.w == 0 || rc.h == 0 )
	{
#if defined(_PERFORMANCE_DEBUG)
		SFBalProfileEnd( "WebControl0", "AddToDirtyRegion", NULL );
#endif
		return;
	}

#ifdef _PERFORMANCE_DEBUG
	m_addToDirtyCount++;
#endif

		
	CONVERT_RECT_TO_BOX(&box, &rc);

	if( m_pDirtyRegion == NULL )
	{
		m_pDirtyRegion = WmCreateRegion(&box, 0);
		if( m_pDirtyRegion == NULL )
			WebAssert( FALSE );
	}
	else
	{
		WmRegion region;

#if  defined(_BACKINGSTORE_DEBUG) && defined(_ADDTODIRTYREGION_DEBUG)
		HGC hDebugGC;
		if ( m_pWebControl->GetGCType() == BIT_32_GC  )
			 hDebugGC = WmCreateWindowSubMemGC32( m_pWebControl->GetHWIN(), 0, 0, GetBackingStoreSize().width(), GetBackingStoreSize().height() );
		else
			 hDebugGC = WmCreateWindowSubMemGC( m_pWebControl->GetHWIN(), 0, 0, GetBackingStoreSize().width(), GetBackingStoreSize().height() );
		
		CopyGC( hDebugGC, 0, 0, m_backingStoreGC, 0, 0,  GetBackingStoreSize().width(), GetBackingStoreSize().height()  );
#endif //_BACKINGSTORE_DEBUG

//#if defined(_BACKINGSTORE_DEBUG) && defined(_ADDTODIRTYREGION_DEBUG)
//							{
//							HGC hGC = hDebugGC;
//							WmBox* pBox;
//							int nbBox, index;
//
//							nbBox = REGION_NUM_RECTS(m_pDirtyRegion);
//							pBox = REGION_RECTS(m_pDirtyRegion);
//
//							WmSetFgColor( hGC, BLUE );
//							WmSetLineWidth( hGC, 1 );
//							for(index = 0; index < nbBox; index++, pBox++)
//							{
//								WmRect tempRect;
//								CONVERT_BOX_TO_RECT(&tempRect, pBox);
//								WmDrawRect( hGC, tempRect.x, tempRect.y, tempRect.w, tempRect.h );
//							}
//
//							WmSetFgColor(hGC, GREEN );
//							WmSetLineWidth( hGC, 1 );
//							WmDrawRect( hGC, rc.x, rc.y, rc.w, rc.h );
//							//WmDrawTransparentFilledRect( hGC, YELLOW, 85, rc.x, rc.y, rc.w, rc.h );
//							}
//#endif //_BACKINGSTORE_DEBUG


		WmInitRegion( &region, &box, 0);
		WmUnionRegion( m_pDirtyRegion, m_pDirtyRegion, &region );

#if defined(_BACKINGSTORE_DEBUG) && defined(_ADDTODIRTYREGION_DEBUG)
							{
								HGC hGC = hDebugGC;
								WmBox* pBox;
								int nbBox, index;

								nbBox = REGION_NUM_RECTS(m_pDirtyRegion);
								pBox = REGION_RECTS(m_pDirtyRegion);

								for(index = 0; index < nbBox; index++, pBox++)
								{
									WmRect tempRect;
									CONVERT_BOX_TO_RECT(&tempRect, pBox);

									int hDebugWin = m_pWebControl->GetHWIN();
									WebCore::IntRect paintRectDebug(  tempRect.x, tempRect.y, tempRect.w, tempRect.h );
									WebCore::GraphicsContext DebugGC(&hDebugGC, &hDebugWin);
									DebugGC.save();

									DebugGC.scale( WebCore::FloatSize( m_zoomMultiplier, m_zoomMultiplier) );
									DebugGC.translate(  -m_scrollErrorCorrection.width(), -m_scrollErrorCorrection.height() );
									paintRectDebug.move(  m_scrollErrorCorrection.width(), m_scrollErrorCorrection.height()  );	

									DebugGC.translate( -(GetBackingStoreRect().x() - GetFrameViewRect().x()), -(GetBackingStoreRect().y() - GetFrameViewRect().y()) );
									//paintRectDebug.move( (GetBackingStoreRect().x() - GetFrameViewRect().x()), (GetBackingStoreRect().y() - GetFrameViewRect().y())  );
					

	//								DebugGC.clip(paintRectDebug);

									DebugGC.setStrokeColor( WebCore::Color("red") );
									DebugGC.setFillColor( WebCore::Color( 0xff, 0xff, 0xff, 0x00 ) );
									DebugGC.drawRect( paintRectDebug );
									
									DebugGC.restore();

									//WmDrawRect( hGC, tempRect.x, tempRect.y, tempRect.w, tempRect.h );
									//WmDrawTransparentFilledRect( hGC, RED, 85, tempRect.x, tempRect.y, tempRect.w, tempRect.h );
									
								}

								SaveBackingStoreToImageFile( hDebugGC,GetBackingStoreSize().width(), GetBackingStoreSize().height(), "DirtyRegion" );
								WmDeleteGC( hDebugGC );

							} // debug code
#endif //_BACKINGSTORE_DEBUG

	}

#if defined(_PERFORMANCE_DEBUG)
	SFBalProfileEnd( "WebControl0", "AddToDirtyRegion", NULL );
#endif
}

WebCore::IntSize WebBackingStore::GetMaxBackingStoreSize()
{
	WebCore::IntSize size = GetWindowRect().size();

	size.setWidth( (float)size.width() * GetBackingStoreMultiplier() );
	size.setHeight( (float)size.height() * GetBackingStoreMultiplier() );

	return size;
}

WebCore::IntRect WebBackingStore::GetBackingStoreRect()
{
	WebCore::IntRect rc = m_rcBackingStore;

	WebCore::IntSize size;



	//WebCore::IntSize backingStoreSize;
	size.setWidth( (float)m_rcBackingStore.width() / m_zoomMultiplier );
	size.setHeight( (float)m_rcBackingStore.height() / m_zoomMultiplier );

	WebCore::IntRect frameViewRect = GetFrameViewRect();
	WebCore::IntSize frameViewSize = frameViewRect.size();


	frameViewSize.setWidth( (float)frameViewSize.width()  );
	frameViewSize.setHeight( (float)frameViewSize.height()  );


	//WebCore::IntSize maxBackingStoreSize = m_rcBackingStore.size();
	//maxBackingStoreSize.setWidth( (float)maxBackingStoreSize.width() * m_zoomMultiplier );
	//maxBackingStoreSize.setHeight( (float)maxBackingStoreSize.height() * m_zoomMultiplier );

	size = size.shrunkTo( frameViewSize );

	rc.setSize( size );


	//SysAssert( frameViewRect.contains( rc ) );

	return rc;
}


WebCore::IntSize WebBackingStore::GetBackingStoreSize()
{
	return m_rcBackingStore.size();
}



WebCore::IntRect WebBackingStore::GetFrameViewRect()
{
	WebCore::IntRect rc;
	WebCore::FrameView *frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

	rc.setX( frameView->contentsX() );
	rc.setY( frameView->contentsY() );
	rc.setWidth( frameView->frameGeometry().width() );
	rc.setHeight( frameView->frameGeometry().height() );

	return rc;
}

WebCore::IntRect WebBackingStore::GetWindowRect()
{
	WebCore::IntRect rc( 0, 0, m_pWebControl->GetWidth(), m_pWebControl->GetHeight() );
	return rc;
}


void WebBackingStore::SetViewPos(  const WebCore::IntPoint & pos )
{
	m_rcView.setLocation( pos );
}
void WebBackingStore::SetViewSize( const WebCore::IntSize & size )
{
	m_rcView.setSize( size );
}
void WebBackingStore::SetBackingStorePos(  const WebCore::IntPoint & pos )
{
	m_rcBackingStore.setLocation( pos );

	//debug
	WebCore::IntRect rcBackingStore = GetBackingStoreRect();
	WebCore::IntRect rcView = GetViewRect();

	SysAssert( rcBackingStore.contains(rcView) );
}
void WebBackingStore::SetBackingStoreSize( const WebCore::IntSize & size )
{
	m_rcBackingStore.setSize( size );
}




WebCore::IntRect WebBackingStore::CalculateRectContaingCenterRect( const WebCore::IntRect & centerRect, const WebCore::IntRect & boundaryRect, const WebCore::IntSize & rectSize )
{
	int newX = centerRect.x() - ((rectSize.width() - centerRect.width()) / 2);
	int newY = centerRect.y() - ((rectSize.height() - centerRect.height()) / 2);

	WebCore::IntRect newRect( newX, newY, rectSize.width(), rectSize.height() );

	WebCore::IntRect intersect = boundaryRect;
	intersect.intersect( newRect );

	int dx = rectSize.width() - intersect.width();
	int dy = rectSize.height() - intersect.height();

	SysAssert( dx <= rectSize.width()/2 );
	SysAssert( dy <= rectSize.height()/2 );

	if( dx != 0 )
	{
		if( boundaryRect.x() > newRect.x() )
			newRect.move( dx, 0 );
		else
			newRect.move( -dx, 0 );
	}

	if( dy != 0 )
	{
		if( boundaryRect.y() > newRect.y() )
			newRect.move( 0, dy );
		else
			newRect.move( 0, -dy );
	}

	SysAssert( boundaryRect.contains( newRect ) );
	SysAssert( newRect.contains( centerRect ) );

	return newRect;

}

#ifdef DYNAMIC_SCROLLBAR
void WebBackingStore_ScrollBarTimerCB(HTimer hTimer, ULONG param)
{
	WebBackingStore *pBackingStore = (WebBackingStore*)param;

	__DeleteTimer( pBackingStore->m_hTouchScrollBarTimer );
	pBackingStore->m_hTouchScrollBarTimer = INVALID_HOBJ;

	pBackingStore->RequestPaintToWindow( 0, 0, pBackingStore->GetWindowRect().width(), pBackingStore->GetWindowRect().height() );
}
#endif

void WebBackingStore::ScrollBy( int ori_dx, int ori_dy )
{
	int dx, dy;

	dx = (float)ori_dx / (float)m_zoomMultiplier;
	dy = (float)ori_dy / (float)m_zoomMultiplier;

	if( dx == 0 && dy == 0 )
		return;

#ifdef DYNAMIC_SCROLLBAR
	if( !m_bHideScrollbar )
	{
	if(  m_hTouchScrollBarTimer == INVALID_HOBJ  )
	{
		m_hTouchScrollBarTimer = __CreateTimer( "WebBackingStoreScrollBar" );
				WebAssert( m_hTouchScrollBarTimer != INVALID_HOBJ );

		BOOL bResult = __StartTimer( m_hTouchScrollBarTimer, DYNAMICSCROLLBAR_TIMEOUT, 
			WebBackingStore_ScrollBarTimerCB, (ULONG)this );
		WebAssert( bResult );
	}
	}
#endif

	__ScrollBy( dx, dy );

}

void WebBackingStore::__ScrollBy( int dx, int dy )
{
	if( dx == 0 && dy == 0 )
		return;


	int viewX, viewY;
	WebCore::FrameView *frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
	WebCore::IntSize contentsSize( frameView->contentsWidth(), frameView->contentsHeight() );

	WebCore::IntRect frameViewRect = GetFrameViewRect();
	WebCore::IntRect backingStoreRect = GetBackingStoreRect();
	WebCore::IntRect viewRect = GetViewRect();

	//WebAssert( frameViewRect.contains( backingStoreRect ) );
	//WebAssert( backingStoreRect.contains( viewRect ) );


//#ifdef _BACKINGSTORE_DEBUG
//		HGC hDebugGC = WmCreateWindowSubMemGC( m_pWebControl->GetHWIN(), 0, 0, GetBackingStoreRect().width(), GetBackingStoreRect().height() );
//		CopyGC( hDebugGC, 0, 0, m_backingStoreGC, 0, 0,   GetBackingStoreRect().width(), GetBackingStoreRect().height()r );
//		WebCore::IntRect debugRect = viewRect;
//#endif


	WebCore::IntSize scrollOffset( viewRect.x(), viewRect.y() );
	WebCore::IntSize maximumScroll = ( contentsSize - WebCore::IntSize(viewRect.width(), viewRect.height())) - scrollOffset;
    maximumScroll.clampNegativeToZero();
	WebCore::IntSize newScrollOffset = scrollOffset + WebCore::IntSize(dx, dy).shrunkTo(maximumScroll);
    newScrollOffset.clampNegativeToZero();

//#ifdef _BACKINGSTORE_DEBUG
//					WebCore::GraphicsContext DebugGC(&hDebugGC);
//					DebugGC.save();
////#if defined(SFB_GWES_TRANSFORM)
//					DebugGC.scale( WebCore::FloatSize( m_zoomMultiplier, m_zoomMultiplier) );
//					DebugGC.translate(  -m_scrollErrorCorrection.width(), -m_scrollErrorCorrection.height() );
//					debugRect.move(  m_scrollErrorCorrection.width(), m_scrollErrorCorrection.height()  );	
////#endif //SFB_GWES_TRANSFORM
//					debugRect.move( -backingStoreRect.x(), -backingStoreRect.y() );
//					DebugGC.setStrokeColor( WebCore::Color("red") );
//					DebugGC.setFillColor( WebCore::Color( 0xff, 0xff, 0xff, 0x00 ) );
//					DebugGC.drawRect( debugRect );
//					//DebugGC.restore();
//#endif //_BACKINGSTORE_DEBUG

	viewRect.setX( newScrollOffset.width() );
	viewRect.setY( newScrollOffset.height() );
	SetViewPos( viewRect.location() );

	if( frameViewRect.contains( viewRect) )
	{
		if( backingStoreRect.contains( viewRect ) )
		{
#if defined(_SFB_BACKINGSTORE_IMPROVE_LOADPERF)
			if( IsLoading() )
			{
				UpdateBackingStore( false );
			}
#endif
			//PaintToWindow( 0,0, GetWindowRect().width(), GetWindowRect().height(), true, true );
			RequestPaintToWindow( 0,0, GetWindowRect().width(), GetWindowRect().height() );
		}
		else // scroll backingstore
		{

			WebCore::IntRect newRect = CalculateRectContaingCenterRect( viewRect, frameViewRect, backingStoreRect.size() );
			int newDx = newRect.x() - backingStoreRect.x();
			int newDy = newRect.y() - backingStoreRect.y();

			__ScrollBackingStore( -newDx, -newDy );
			// priyanka : to refresh the plugin rectangle locations, the plugins which are out of backing sotre
			frameView->geometryChanged();
			//PaintToWindow( 0,0,0, 0, true, true );
			RequestPaintToWindow( 0,0,0, 0 );

		}
	}
	else // scroll frameview
	{
		int newX = viewRect.x() - ((frameViewRect.width() - viewRect.width()) / 2);
		int newY = viewRect.y() - ((frameViewRect.height() - viewRect.height()) / 2);

		int newDx = newX - frameViewRect.x();
		int newDy = newY - frameViewRect.y();

		frameView->scrollBy( newDx, newDy );
		AdjustViewPos();
	}



//#ifdef _BACKINGSTORE_DEBUG
//					debugRect = viewRect;
//					debugRect.move(  m_scrollErrorCorrection.width(), m_scrollErrorCorrection.height()  );	
//					debugRect.move( -backingStoreRect.x(), -backingStoreRect.y() );
//					DebugGC.setStrokeColor( WebCore::Color("yellow") );
//					DebugGC.setFillColor( WebCore::Color( 0xff, 0xff, 0xff, 0x00 ) );
//					DebugGC.drawRect( debugRect );
//					DebugGC.restore();
//					SaveBackingStoreToImageFile( hDebugGC,frameView->frameGeometry().width()*m_zoomMultiplier, frameView->frameGeometry().height()*m_zoomMultiplier, "ScrollBy" );
//					WmDeleteGC( hDebugGC );
//#endif

		// debug
		WebCore::IntRect rcDbgFrameView, rcDbgBackingStore, rcDbgView;
		rcDbgFrameView = GetFrameViewRect();
		rcDbgBackingStore = GetBackingStoreRect();
		rcDbgView = GetViewRect();
		SysAssert(( rcDbgFrameView.contains( rcDbgBackingStore ) ));
		SysAssert(( rcDbgBackingStore.contains( rcDbgView ) ));

}


void WebBackingStore::ScrollBackingStore( int dx, int dy, const WebCore::IntRect& scrollViewRect, const WebCore::IntRect& clipRect)
{
	WebCore::IntRect viewRect = GetViewRect();
	WebCore::IntRect backingStoreRect = GetBackingStoreRect();
	WebCore::IntRect frameViewRect = GetFrameViewRect();

	if( frameViewRect.contains( viewRect ) == false )
	{
		WebCore::FrameView *frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
		WebCore::IntSize contentsSize( frameView->contentsWidth(), frameView->contentsHeight() );

		if( frameView->contentsWidth() == 0 || frameView->contentsHeight() == 0 )
	{
		// This exception nomally happen when the first bytes of page comes and webcore reset the content size of frameview as 0x0
//		SysDebug(( MID_EXCEPTION, "WebBackingStore::ScrollBackingStore : frameviewrect does not contains viewrect\n" ));
		return;
	}
		else // adjust view pos
		{
			WebCore::IntRect rcIntersect = frameViewRect;
			rcIntersect.intersect( viewRect );

			if( rcIntersect.isEmpty() )
			{
				SetViewPos( frameViewRect.location() );
			}
			else
			{
				WebCore::IntRect rcNewView = CalculateRectContaingCenterRect( rcIntersect, frameViewRect, viewRect.size() );
				SetViewPos( rcNewView.location() );
			}
		}
	}

	viewRect = GetViewRect();

	WebCore::IntRect newRect = CalculateRectContaingCenterRect( viewRect,frameViewRect, backingStoreRect.size() );
	int newDx = newRect.x() - backingStoreRect.x();
	int newDy = newRect.y() - backingStoreRect.y();

	__ScrollBackingStore( -newDx, -newDy );

	// debug
	WebCore::IntRect rcDbgFrameView = GetFrameViewRect();
	WebCore::IntRect rcDbgBackingStore = GetBackingStoreRect();
	WebCore::IntRect rcDbgView = GetViewRect();
	SysAssert(( rcDbgFrameView.contains( rcDbgBackingStore ) ));
	SysAssert(( rcDbgBackingStore.contains( rcDbgView ) ));

}



void WebBackingStore::__ScrollBackingStore( int dx, int dy )
{
	float dx2 ,dy2;
	WebCore::FrameView *frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

	//WebCore::IntPoint backingStorePos( GetBackingStoreRect().location() );
	//backingStorePos.move( -dx, -dy );
	//SetBackingStorePos(  backingStorePos );

 
	WebCore::IntRect newBackingStore = GetBackingStoreRect();
	newBackingStore.setSize( GetBackingStoreSize() );
	WebCore::IntRect oldBackingStore = newBackingStore;
	WebCore::IntRect intersects = newBackingStore;

	dx2 = (float)dx * (float)m_zoomMultiplier;
	dy2 = (float)dy * (float)m_zoomMultiplier;

	dx2 += m_scrollError.width();
	dy2 += m_scrollError.height();

	int intDx = dx2;
	int intDy = dy2;

	m_scrollError.setWidth( dx2 - (float)intDx );
	m_scrollError.setHeight(dy2 - (float)intDy );

	m_scrollErrorCorrection.setWidth( m_scrollError.width() / m_zoomMultiplier );
	m_scrollErrorCorrection.setHeight(  m_scrollError.height() / m_zoomMultiplier );

	oldBackingStore.move( intDx, intDy );

	intersects.intersect( oldBackingStore );

	if( intersects.width() != 0 && intersects.height() != 0 )
		CopyGC( m_backingStoreGC, intersects.x() - newBackingStore.x(), intersects.y() - newBackingStore.y(),
			 m_backingStoreGC, intersects.x() - oldBackingStore.x(), 	intersects.y() - oldBackingStore.y(), intersects.width(),intersects.height() );

	WebCore::IntPoint backingStorePos( GetBackingStoreRect().location() );
	backingStorePos.move( -dx, -dy );
	SetBackingStorePos(  backingStorePos );

	newBackingStore = GetBackingStoreRect();
	oldBackingStore = newBackingStore;
	intersects = oldBackingStore;


	oldBackingStore.move( dx, dy ); 

	intersects.intersect( oldBackingStore );




#ifdef _BACKINGSTORE_DEBUG
	SaveBackingStoreToImageFile( m_backingStoreGC,GetBackingStoreSize().width(), GetBackingStoreSize().height(), "ScrollGC" );
	HGC hDebugGC;
	if ( m_pWebControl->GetGCType() == BIT_32_GC )
		hDebugGC = WmCreateWindowSubMemGC32( m_pWebControl->GetHWIN(), 0, 0, GetBackingStoreSize().width(), GetBackingStoreSize().height() );
	else
		hDebugGC = WmCreateWindowSubMemGC( m_pWebControl->GetHWIN(), 0, 0, GetBackingStoreSize().width(), GetBackingStoreSize().height() );
	
	int hDebugWin = m_pWebControl->GetHWIN();
	WebCore::GraphicsContext gc(&hDebugGC, &hDebugWin);
	WebCore::IntRect paintRect = frameView->frameGeometry();
	gc.save();

	gc.scale( WebCore::FloatSize( m_zoomMultiplier, m_zoomMultiplier) );
	gc.translate( -m_scrollErrorCorrection.width(), -m_scrollErrorCorrection.height() );
	paintRect.move( m_scrollErrorCorrection.width(), m_scrollErrorCorrection.height()  );

	gc.translate( -(GetBackingStoreRect().x() - GetFrameViewRect().x()), -(GetBackingStoreRect().y() - GetFrameViewRect().y()) );
	//paintRect.move( (GetBackingStoreRect().x() - GetFrameViewRect().x()), (GetBackingStoreRect().y() - GetFrameViewRect().y())  );

	gc.clip(paintRect);
	frameView->paint(&gc, paintRect);
	gc.restore();

	SaveBackingStoreToImageFile( hDebugGC,GetBackingStoreSize().width(), GetBackingStoreSize().height(), "ScrollGCPaint" );
	WmDeleteGC( hDebugGC );
#endif





	if( intersects.width() != 0 && intersects.height() != 0 )
	{
		// Dirty region
		WmBox box;
		WmRect rc;
		WmRegion region, updateRegion;

		rc.x = newBackingStore.x();
		rc.y = newBackingStore.y();
		rc.w = newBackingStore.width();
		rc.h = newBackingStore.height();
			
		CONVERT_RECT_TO_BOX(&box, &rc);

		WmInitRegion( &updateRegion, &box, 0 );

		rc.x = intersects.x();
		rc.y = intersects.y();
		rc.w = intersects.width();
		rc.h = intersects.height();

		CONVERT_RECT_TO_BOX(&box, &rc);

		WmInitRegion( &region, &box, 0 );

		WmSubtractRegion( &updateRegion, &updateRegion, &region);

		WmBox* pBox;
		int nbBox, index;

		nbBox = REGION_NUM_RECTS(&updateRegion);
		pBox = REGION_RECTS(&updateRegion);

		for(index = 0; index < nbBox; index++, pBox++)
		{
			WebCore::IntRect tempRect( pBox->x1 - GetFrameViewRect().x(), pBox->y1 - GetFrameViewRect().y(), pBox->x2-pBox->x1, pBox->y2 - pBox->y1 );

			//tempRect.inflate( INFLATE_OFFSET );

			AddToDirtyRegion( tempRect );
		}
		
		UpdateBackingStore( false );
	}
	else
	{
		UpdateBackingStore( true );
	}

	AddToPaintRegion( WebCore::IntRect( GetViewRect().x() - GetFrameViewRect().x(), GetViewRect().y() - GetFrameViewRect().y() , GetViewRect().width(), GetViewRect().height()) );


		// debug
		WebCore::IntRect rcDbgFrameView, rcDbgBackingStore, rcDbgView;
		rcDbgFrameView = GetFrameViewRect();
		rcDbgBackingStore = GetBackingStoreRect();
		rcDbgView = GetViewRect();
		SysAssert(( rcDbgFrameView.contains( rcDbgBackingStore ) ));
		SysAssert(( rcDbgBackingStore.contains( rcDbgView ) ));

}


void WebBackingStore::UpdateBackingStore( bool bCompleteDirty )
{
	if( m_backingStoreGC == INVALID_HOBJ )
		return;

	if( m_pDirtyRegion == NULL  && bCompleteDirty == false )
		return;

#ifdef _PERFORMANCE_DEBUG
	m_updateBackingStoreCount++;
#endif

#if defined(_PERFORMANCE_DEBUG)
	SFBalProfileStart( "WebControl0", "UpdateBackingStore-Calc" );
#endif

	WebCore::FrameView *frameView = NULL;
	if( m_pWebControl->GetMainWebFrame()->GetFrame() == NULL )
		return;

	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

#ifdef _BACKINGSTORE_DEBUG
		HGC hDebugGC;
		if ( m_pWebControl->GetGCType() == BIT_32_GC )	
			hDebugGC = WmCreateWindowSubMemGC32( m_pWebControl->GetHWIN(), 0, 0, GetBackingStoreSize().width(), GetBackingStoreSize().height() );
		else
			hDebugGC = WmCreateWindowSubMemGC( m_pWebControl->GetHWIN(), 0, 0, GetBackingStoreSize().width(), GetBackingStoreSize().height() );
		
		//WmCopyGC( hDebugGC, 0, 0, m_backingStoreGC, 0, 0,  frameView->frameGeometry().width(), frameView->frameGeometry().height() );
		WmDrawFilledRect( hDebugGC, YELLOW, 0, 0, GetBackingStoreSize().width(), GetBackingStoreSize().height() );

		{
			HGC hGC = hDebugGC;
			WmBox* pBox;
			int nbBox, index;

			if( m_pDirtyRegion )
			{
				//int dx, dy;
				//	WebCore::FrameView *frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
				//dx = frameView->contentsX() - m_backingStoreRect.x();
				//dy = frameView->contentsY() - m_backingStoreRect.y();

				nbBox = REGION_NUM_RECTS(m_pDirtyRegion);
				pBox = REGION_RECTS(m_pDirtyRegion);

				//WmClearGC( hGC );
				WmSetFgColor( hGC, YELLOW );

				for(index = 0; index < nbBox; index++, pBox++)
				{
					WmRect tempRect;
					CONVERT_BOX_TO_RECT(&tempRect, pBox);
					WmDrawRect( hGC, tempRect.x, tempRect.y, tempRect.w, tempRect.h );
				}
			}
		}
#endif //_BACKINGSTORE_DEBUG


	frameView->layoutIfNeededRecursive();


	WTF::Vector<WebCore::IntRect> paintRects;
	if( !bCompleteDirty )
	{
		if( m_pDirtyRegion == NULL )
			return;
	
		WmRegion *pDirtyRegion = m_pDirtyRegion;
#if defined(_SFB_BACKINGSTORE_IMPROVE_LOADPERF)
		WmRegion newReg;

		if( IsLoading() )
		{
			WebCore::IntRect rcView = GetViewRect();
			WebCore::IntRect rcFrameView = GetFrameViewRect();
			rcView.move( -rcFrameView.x(), -rcFrameView.y() );
			WmBox box;
			WmRect rc;

			rc.x = rcView.x();
			rc.y = rcView.y();
			rc.w = rcView.width();
			rc.h = rcView.height();

			CONVERT_RECT_TO_BOX(&box, &rc);
			WmInitRegion( &newReg, &box, 0 );

			BOOL bResult = WmIntersectRegion( &newReg, &newReg, m_pDirtyRegion );
			WebAssert( bResult );

			pDirtyRegion = &newReg;
		}
#endif //_SFB_BACKINGSTORE_IMPROVE_LOADPERF

		WmBox * regionBox = NULL;
		if(pDirtyRegion != NULL)
			regionBox = WmRegionExtents( pDirtyRegion );
		
		if(regionBox == NULL)
			return;

		WebCore::IntRect regionRect( regionBox->x1, regionBox->y1, regionBox->x2-regionBox->x1, regionBox->y2-regionBox->y1 );
		GetUpdateRects( pDirtyRegion, regionRect, paintRects );
	}
	else
	{
		WebCore::IntRect rcPaintRect =  frameView->frameGeometry() ;
#if defined(_SFB_BACKINGSTORE_IMPROVE_LOADPERF)
		if( IsLoading() )
		{
			WebCore::IntRect rcView = GetViewRect();
			WebCore::IntRect rcFrameView = GetFrameViewRect();
			rcView.move( -rcFrameView.x(), -rcFrameView.y() );
			rcPaintRect = rcView;
		}
#endif //_SFB_BACKINGSTORE_IMPROVE_LOADPERF
		paintRects.append(rcPaintRect);

	}

	//WebAssert( paintRects.size() > 0 );
	if(  paintRects.size() <= 0 )
	{
#if defined(_PERFORMANCE_DEBUG)
		SFBalProfileEnd( "WebControl0", "UpdateBackingStore-Calc", NULL );
#endif
		return;
	}

#if defined(_SFB_BACKINGSTORE_IMPROVE_LOADPERF)
	if( IsLoading() )
	{
		if( m_pDirtyRegion == NULL )
		{
			AddToDirtyRegion(frameView->frameGeometry());
		}

		for( unsigned i = 0; i < paintRects.size(); i++ )
		{
			WebCore::IntRect paintRect = paintRects[i];
			WmBox box;
			WmRect rc;
			WmRegion region;

			rc.x = paintRect.x();
			rc.y = paintRect.y();
			rc.w = paintRect.width();
			rc.h = paintRect.height();
			
			CONVERT_RECT_TO_BOX(&box, &rc);

			WmInitRegion( &region, &box, 0 );
			BOOL bResult = WmSubtractRegion( m_pDirtyRegion, m_pDirtyRegion, &region );
			WebAssert( bResult );
		}
	}
	else
	{
#endif
		WmDestroyRegion( m_pDirtyRegion );
		m_pDirtyRegion = NULL;
#if defined(_SFB_BACKINGSTORE_IMPROVE_LOADPERF)
	}
#endif


#if defined(_PERFORMANCE_DEBUG)
	SFBalProfileEnd( "WebControl0", "UpdateBackingStore-Calc", NULL );
#endif
	
#if defined(_PERFORMANCE_DEBUG)
	SFBalProfileStart( "WebControl0", "UpdateBackingStore-Paint" );
#endif


	//WmClearGC( m_backingStoreGC );
	for( unsigned i = 0; i < paintRects.size(); i++ )
	{
		if (frameView && frameView->frame() && frameView->frame()->renderer()) 
		{
#ifdef _BACKINGSTORE_DEBUG
			WebCore::IntRect paintRectDebug = paintRects[i];
#endif
			WebCore::IntRect paintRect = paintRects[i];
			int hWin = m_pWebControl->GetHWIN();
			
			AddToPaintRegion( paintRect );

			WebCore::GraphicsContext gc(&m_backingStoreGC,&hWin);
			gc.save();

			gc.scale( WebCore::FloatSize( m_zoomMultiplier, m_zoomMultiplier) );
			gc.translate( -m_scrollErrorCorrection.width(), -m_scrollErrorCorrection.height() );
			paintRect.move( m_scrollErrorCorrection.width(), m_scrollErrorCorrection.height()  );

			gc.translate( -(GetBackingStoreRect().x() - GetFrameViewRect().x()), -(GetBackingStoreRect().y() - GetFrameViewRect().y()) );
			//paintRect.move( (GetBackingStoreRect().x() - GetFrameViewRect().x()), (GetBackingStoreRect().y() - GetFrameViewRect().y())  );

			gc.clip(paintRect);
			frameView->paint(&gc, paintRect);
			gc.restore();

#ifdef _BACKINGSTORE_DEBUG
					{
					int hDebugWin = m_pWebControl->GetHWIN();
					WebCore::GraphicsContext DebugGC(&hDebugGC, &hDebugWin);
					DebugGC.save();

					DebugGC.scale( WebCore::FloatSize( m_zoomMultiplier, m_zoomMultiplier) );
					DebugGC.translate(  -m_scrollErrorCorrection.width(), -m_scrollErrorCorrection.height() );
					paintRectDebug.move(  m_scrollErrorCorrection.width(), m_scrollErrorCorrection.height()  );	

					DebugGC.translate( -(GetBackingStoreRect().x() - GetFrameViewRect().x()), -(GetBackingStoreRect().y() - GetFrameViewRect().y()) );
					//paintRectDebug.move( (GetBackingStoreRect().x() - GetFrameViewRect().x()), (GetBackingStoreRect().y() - GetFrameViewRect().y())  );
	

					DebugGC.clip(paintRectDebug);
					frameView->paint(&DebugGC,paintRectDebug);

					DebugGC.setStrokeColor( WebCore::Color("red") );
					DebugGC.setFillColor( WebCore::Color( 0xff, 0xff, 0xff, 0x00 ) );
					DebugGC.drawRect( paintRectDebug );
					
					DebugGC.restore();

					//WmSetFgColor( hDebugGC, RED );
					//WmDrawRect( hDebugGC, paintRects[i].x(), paintRects[i].y(), paintRects[i].width(), paintRects[i].height() );
					}
#endif

		}
		else
		{
//			WebAssert( FALSE );
			return;
		}
	}

#ifdef _BACKINGSTORE_DEBUG
	SaveBackingStoreToImageFile( hDebugGC,GetBackingStoreSize().width(), GetBackingStoreSize().height(), "Update" );
	SaveBackingStoreToImageFile( m_backingStoreGC,GetBackingStoreSize().width(), GetBackingStoreSize().height(), "Backing" );
	WmDeleteGC( hDebugGC );
#endif

#if defined(_PERFORMANCE_DEBUG)
	SFBalProfileEnd( "WebControl0", "UpdateBackingStore-Paint", NULL );
#endif

}

void WebBackingStore::AddToPaintRegion( const WebCore::IntRect & paintRect )
{
	WmBox box;
	WmRect rc;
	WebCore::FrameView* frameView = NULL;
	WebCore::IntRect tempRect;
	WebCore::IntRect rect = paintRect;

	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();	

	rect.move(  -(GetViewRect().x() - frameView->contentsX()), -(GetViewRect().y() - frameView->contentsY()) );
	rect.scale( m_zoomMultiplier);

	tempRect = GetWindowRect();

	tempRect.intersect( rect );

	rc.x = tempRect.x();
	rc.y = tempRect.y();
	rc.w = tempRect.width();
	rc.h = tempRect.height();

	if( rc.w == 0 || rc.h == 0 )
		return;

	CONVERT_RECT_TO_BOX(&box, &rc);

	if( m_pPaintRegion == NULL )
	{
		m_pPaintRegion = WmCreateRegion(&box, 0);
		if( m_pPaintRegion == NULL )
			WebAssert( FALSE );
	}
	else
	{
		WmRegion region;

		WmInitRegion( &region, &box, 0);
		WmUnionRegion( m_pPaintRegion, m_pPaintRegion, &region );
	}

}

void WebBackingStore::GetUpdateRects( WmRegion* region, const WebCore::IntRect& dirtyRect, WTF::Vector<WebCore::IntRect> &rects )
{
	WmBox* pBox;
	int nBox;

	nBox = REGION_NUM_RECTS(region);
	pBox = REGION_RECTS(region);

	rects.clear();

	if( nBox > cRectThreshold )
	{
		rects.append( dirtyRect );
		return;
	}

	double singlePixels = 0.0;
	unsigned index;

	for(index = 0; index < nBox; index++, pBox++)
	{
		singlePixels += (pBox->x2 - pBox->x1 ) * (pBox->y2 - pBox->y1 );
	}

	double unionPixels = dirtyRect.width() * dirtyRect.height();
	double wastedSpace = 1.0 - (singlePixels/unionPixels);
	if( wastedSpace <= cWastedSpaceThreshold )
	{
		rects.append( dirtyRect );
		return;
	}

	pBox = REGION_RECTS(region);

	for(index = 0; index < nBox; index++, pBox++)
	{
		WebCore::IntRect rect;
		rect.setX( pBox->x1 );
		rect.setY( pBox->y1 );
		rect.setWidth( pBox->x2 - pBox->x1 );
		rect.setHeight( pBox->y2 - pBox->y1 );

		rects.append( rect );
	}

}



void WebBackingStore::AdjustViewPos()
{
	WebCore::IntRect rcFrameView, rcBackingStore, rcView;

	rcFrameView = GetFrameViewRect();
	rcBackingStore = GetBackingStoreRect();
	rcView = GetViewRect();

	if( rcFrameView.contains( rcBackingStore ) == false || 
		rcFrameView.contains( rcView ) == false ||
		rcBackingStore.contains( rcView ) == false )
	{
		WebCore::IntRect rcIntersect = rcFrameView;
		rcIntersect.intersect( rcView );

		if( rcIntersect.isEmpty() )
			SetViewPos( rcFrameView.location() );
		else
		{
			WebCore::IntRect rcNewView = CalculateRectContaingCenterRect( rcIntersect, rcFrameView, rcView.size() );
			SetViewPos( rcNewView.location() );
		}

		rcView = GetViewRect();


		WebCore::IntRect rcNewBackingStore = CalculateRectContaingCenterRect( rcView, rcFrameView, rcBackingStore.size() );
		//SetBackingStorePos( rcNewBackingStore.location() );
		int newDx = rcNewBackingStore.x() - rcBackingStore.x();
		int newDy = rcNewBackingStore.y() - rcBackingStore.y();

		__ScrollBackingStore( -newDx, -newDy );


		// debug
		WebCore::IntRect rcDbgFrameView, rcDbgBackingStore, rcDbgView;
		rcDbgFrameView = GetFrameViewRect();
		rcDbgBackingStore = GetBackingStoreRect();
		rcDbgView = GetViewRect();
		SysAssert(( rcDbgFrameView.contains( rcDbgBackingStore ) ));
		SysAssert(( rcDbgBackingStore.contains( rcDbgView ) ));
	}
}

void WebBackingStore::RotateBackingStore( int width, int height )
{
	//WebCore::IntRect rcFrameView, rcView;

	//rcFrameView = GetFrameViewRect();
	//rcView = GetViewRect();
	//if( rcFrameView.contains( rcView ) == false )
	//{
	//	WebCore::IntRect rcBackingStore = GetBackingStoreRect();
	//	WebCore::IntRect rcIntersect = rcFrameView;
	//	rcIntersect.intersect( rcView );

	//	WebCore::IntRect rcNewView = CalculateRectContaingCenterRect( rcIntersect, rcFrameView, GetViewRect().size() );
	//	int dx = rcNewView.x() - rcView.x();
	//	int dy = rcNewView.y() - rcView.y();

	//	__ScrollBy( dx, dy );
	//}

	if( m_hWindowGC != INVALID_HOBJ )
		WmDeleteGC( m_hWindowGC );

	m_hWindowGC = WmCreateClientGC( m_pWebControl->GetHWIN() );
	WebAssert( m_hWindowGC != INVALID_HOBJ );

	ChangeBackingStoreSize( GetWindowRect().size() );
	ChangeFrameViewSize( GetWindowRect().size(), true );

	WebCore::FrameView* frameView = NULL;
	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();


	WebCore::IntSize contentsSize;
	contentsSize.setWidth( frameView->contentsWidth() );
	contentsSize.setHeight( frameView->contentsHeight() );
	
	int w = (float)GetWindowRect().width() / (float)GetMinZoomFactor() * (float)GetBackingStoreMultiplier();
	int h = (float)GetWindowRect().height() / (float)GetMinZoomFactor() * (float)GetBackingStoreMultiplier();
	WebCore::IntSize frameViewSize( w, h );
	frameViewSize = frameViewSize.shrunkTo( contentsSize );
	frameViewSize = frameViewSize.expandedTo( GetWindowRect().size() );


	w = (float)frameViewSize.width() * m_zoomMultiplier;
	h = (float) frameViewSize.height() * m_zoomMultiplier;
	WebCore::IntSize backingStoreSize( w, h );
	backingStoreSize = backingStoreSize.shrunkTo( GetMaxBackingStoreSize() );	
	backingStoreSize = backingStoreSize.expandedTo( GetWindowRect().size() );

	bool bBackingStoreChanged = ChangeBackingStoreSize( backingStoreSize );
	bool bFrameViewChanged = ChangeFrameViewSize( frameViewSize, true );

	AdjustViewPos();
// This is commented to avoid white screen when changed to full screen from normal mode
//	if( bBackingStoreChanged /*&& bFrameViewChanged == false*/ )
	{
		UpdateBackingStore( true );
	}
}



//bool WebBackingStore::ValidateBackingStore()
//{
//	WebCore::IntSize contentSize;
//	WebCore::IntSize backingStoreSize;
//	WebCore::FrameView* frameView = NULL;
//	WebCore::IntSize currBackingStoreSize;
//
//	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
//
//	contentSize.setWidth( frameView->contentsWidth() );
//	contentSize.setHeight( frameView->contentsHeight() );
//
//
//	if( m_prevContentsSize == contentSize )
//		return false;
//
//	SysDebug(( MID_EXCEPTION, "ValidateBackingStore : LayoutDebug prevContSize( %d, %d ), curContSize( %d, %d )\n", 
//		m_prevContentsSize.width(), m_prevContentsSize.height(), contentSize.width(), contentSize.height() ));
//
//	m_prevContentsSize = contentSize;
//
//
//	backingStoreSize = CalculateBackingStoreSize(  contentSize  );
//
//
//	currBackingStoreSize = frameView->frameGeometry().size();
//
//	SysDebug(( MID_EXCEPTION, "ValidateBackingStore : LayoutDebug prevBackingStoreSize( %d, %d ), currBackingStoreSize( %d, %d )\n", 
//		currBackingStoreSize.width(), currBackingStoreSize.height(), backingStoreSize.width(), backingStoreSize.height() ));
//
//
//	// validate view
//	int backingStoreX = frameView->contentsX();
//	int backingStoreY = frameView->contentsY();
//	WebCore::IntSize maxPos( backingStoreSize - m_viewRect.size() );
//	WebCore::IntSize desiredOffset( m_viewRect.x() - backingStoreX, m_viewRect.y() - backingStoreY );
//	WebCore::IntSize viewoffset = desiredOffset.shrunkTo(maxPos);
//	viewoffset.clampNegativeToZero();
//
//	m_viewRect.setX( backingStoreX + viewoffset.width() );
//	m_viewRect.setY( backingStoreY + viewoffset.height() );
//
//
//	if( currBackingStoreSize == backingStoreSize )
//	{
//		
//		return false;
//	}
//	else // need to change
//	{
//		//m_backingStoreRect.setSize( backingStoreSize );
//		//AdjustBackingStorePos( viewRect );
//
//		ChangeBackingStoreSize( backingStoreSize );
//
//
//
//		return true;
//		//return false;
//	}
//
//}

bool WebBackingStore::ChangeFrameViewSize( const WebCore::IntSize &newSize, bool bForce )
{
	WebCore::Frame *frame =  m_pWebControl->GetMainWebFrame()->GetFrame();
	WebCore::IntSize oldSize = frame->view()->frameGeometry().size();

	if( oldSize == newSize && bForce == false)
		return false;

#ifdef _PERFORMANCE_DEBUG
	m_resizeBackingStoreCount++;
#endif

	if( m_bUseOldBackingStore == true )
		frame->view()->resize( oldSize.width() , oldSize.height() );
	else
	// change view size
    frame->view()->resize( newSize.width() , newSize.height() );
   
	//fix for BUG ID 3918 Some times layout is not updated in resize event 
# ifdef  SFB_WIDGET
	if(!SFBalIsExposeEventPresent() || ("Widget" == m_pWebControl->GetInstanceName()))
#else
	if(!SFBalIsExposeEventPresent())
#endif
	{
		WebCore::FrameView *frameView = NULL;

		if( m_pWebControl->GetMainWebFrame()->GetFrame() == NULL )
			return true;

		frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

		frameView->setNeedsLayout();
		frameView->layoutIfNeededRecursive();
	}

	// debug
	//WebCore::IntRect rcFrameView, rcBackingStore, rcView;
	//rcFrameView = GetFrameViewRect();
	//rcBackingStore = GetBackingStoreRect();
	//rcView = GetViewRect();
	//SysAssert(( rcFrameView.contains( rcBackingStore ) ));
	//SysAssert(( rcBackingStore.contains( rcView ) ));


	return true;
   
}

bool WebBackingStore::ChangeBackingStoreSize( const WebCore::IntSize &newSize, bool bForce  )
{
	WebCore::Frame *frame =  m_pWebControl->GetMainWebFrame()->GetFrame();
	WebCore::IntSize oldSize = GetBackingStoreSize();

	if( oldSize == newSize && bForce == false )
		return false;

	if( newSize.width() * newSize.height() * 2 > SFBalMemGetStatEx() )
	{
		m_zoomMultiplier = ORIGINAL_SCALE;
		m_bUseOldBackingStore = true;
		return true;
	}

	m_bUseOldBackingStore = false;

//	if( m_backingStoreGC != INVALID_HOBJ )
	if( WmVerifyGC(m_backingStoreGC) == TRUE )
		WmDeleteGC( m_backingStoreGC );

//	SysDebug(( MID_EXCEPTION, "WebBackingStore::ChangeBackingStoreSize : Width(%d), Height(%d) \n", newSize.width(), newSize.height()));

	if ( m_pWebControl->GetGCType() == BIT_32_GC  )
	{
		WmSetWindowBgColor( m_pWebControl->GetHWIN(), BLACK );
		m_backingStoreGC = WmCreateWindowSubMemGC32( m_pWebControl->GetHWIN(), 0, 0, newSize.width(), newSize.height() );
	}
	else
		m_backingStoreGC = WmCreateWindowSubMemGC( m_pWebControl->GetHWIN(), 0, 0, newSize.width(), newSize.height() );

	WebAssert( m_backingStoreGC != INVALID_HOBJ );

	SetBackingStoreSize( newSize );


	//// debug
	//WebCore::IntRect rcFrameView, rcBackingStore, rcView;
	//rcFrameView = GetFrameViewRect();
	//rcBackingStore = GetBackingStoreRect();
	//rcView = GetViewRect();
	//SysAssert(( rcFrameView.contains( rcBackingStore ) ));
	//SysAssert(( rcBackingStore.contains( rcView ) ));

	return true;

}


void WebBackingStore::PaintToWindow( int x, int y, int w, int h, bool bDraw, bool bShow )
{
	//HGC hGC;
	WebCore::FrameView* frameView = NULL;
	WebCore::FloatRect viewFloatRect;


	if( m_backingStoreGC == INVALID_HOBJ )
		return;

/* Start:Fix for CQ 1955, 4285  and all overlapp UI issues in Breson. */
    HWIN hFocusedWnd = WmGetFocusWindow();
	if(hFocusedWnd != m_pWebControl->GetHWIN())
	{
		int winType = 0;
		if(WmGetWindowType(hFocusedWnd,&winType))
		{
			if((WINDOW_EDITAREA == winType) || (WINDOW_EDITFIELD == winType)
					|| (WINDOW_MESSAGEBOX == winType) ||(WINDOW_COMBOBOX == winType) || (WINDOW_LISTEX == winType) )
			{
				if((m_pWebControl->GetHWIN() == WmGetParentWindow(hFocusedWnd)) ||
					(WINDOW_MESSAGEBOX == winType) ||
					(WINDOW_EDITFIELD == winType) || (WINDOW_EDITAREA == winType ) ||
					(WINDOW_LISTEX == winType) /*|| (WINDOW_COMBOBOX == winType)*/ )//CQ4450 fix
				{
					goto paint_to_window;
				}
			}
		}
		return;
	}
paint_to_window:
/* End:Fix for CQ 1955 and all overlapp UI issues in Breson. */
	


#ifdef _PERFORMANCE_DEBUG
	m_showToWindowCount++;
#endif

#if defined(_PERFORMANCE_DEBUG)
	SFBalProfileStart( "WebControl0", "PaintToWindow" );
#endif
	int dx, dy;


	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

	WebCore::IntRect validRect = GetWindowRect();
	if( frameView->contentsHeight()*m_zoomMultiplier > GetWindowRect().height() )
	{
#ifdef DYNAMIC_SCROLLBAR
			if( m_hTouchScrollBarTimer != INVALID_HOBJ)
#endif
		validRect.setWidth( validRect.width() - m_vBar.width() );
#ifdef DYNAMIC_SCROLLBAR
			else
			{

				validRect.setWidth( validRect.width() );

			}
#endif
	}
	if( frameView->contentsWidth() *m_zoomMultiplier > GetWindowRect().width() )
	{
#ifdef DYNAMIC_SCROLLBAR
		if( m_hTouchScrollBarTimer != INVALID_HOBJ)
#endif
		validRect.setHeight( validRect.height() - m_hBar.height() );
#ifdef DYNAMIC_SCROLLBAR
		else
		{

			validRect.setHeight( validRect.height() );
		}
#endif

	}


	// test
	//x = 0; y = 0;
	//w = m_viewRect.width();
	//h = m_viewRect.height();

	//hGC = WmCreateClientGC( m_pWebControl->GetHWIN() );

	// draw area which has been updated
	if( x ==0 && y == 0 && w == 0 && h == 0 && m_pPaintRegion )
	{
		WmBox* pBox;
		int nbBox, index;

		nbBox = REGION_NUM_RECTS(m_pPaintRegion);
		pBox = REGION_RECTS(m_pPaintRegion);

		for(index = 0; index < nbBox; index++, pBox++)
		{
			//WmRect tempRect;
			//CONVERT_BOX_TO_RECT(&tempRect, pBox);

			WebCore::IntRect updateRect( pBox->x1, pBox->y1, pBox->x2 - pBox->x1, pBox->y2 - pBox->y1 );

			if( !m_bHideScrollbar )
				updateRect.intersect( validRect );

			dx = GetViewRect().x() - GetBackingStoreRect().x();// + tempRect.x;
			dy = GetViewRect().y() - GetBackingStoreRect().y();// + tempRect.y;
			dx = (float)dx * m_zoomMultiplier;
			dy = (float)dy * m_zoomMultiplier;
			dx += updateRect.x();//tempRect.x;
			dy += updateRect.y();//tempRect.y;

			if( bDraw )
				CopyGC( m_hWindowGC, updateRect.x(), updateRect.y(), m_backingStoreGC, dx, dy, updateRect.width(), updateRect.height() );
			if( bShow )
				WmShowGCRectArea( m_hWindowGC, updateRect.x(), updateRect.y(), updateRect.width(), updateRect.height() ); 
		}

		if( bShow )
		{
			WmDestroyRegion( m_pPaintRegion );
			m_pPaintRegion = NULL;
		}
#ifdef DYNAMIC_SCROLLBAR
		if( !m_bHideScrollbar && m_hTouchScrollBarTimer != INVALID_HOBJ)
#else
		if( !m_bHideScrollbar )
#endif			 
			PaintScrollbar( m_hWindowGC, bShow );
	}
	else // draw only the specified area
	{
		WebCore::IntRect updateRect( x, y, w, h );

		if( !m_bHideScrollbar )
			updateRect.intersect( validRect );

		dx = GetViewRect().x() - GetBackingStoreRect().x();
		dy = GetViewRect().y() - GetBackingStoreRect().y();
		dx = (float)dx * m_zoomMultiplier;
		dy = (float)dy * m_zoomMultiplier;
		dx += updateRect.x();
		dy += updateRect.y();

		if( bDraw )
			CopyGC( m_hWindowGC, x, y, m_backingStoreGC, dx, dy, updateRect.width(), updateRect.height() );

		if( bShow )
			WmShowGCRectArea( m_hWindowGC, updateRect.x(), updateRect.y(), updateRect.width(), updateRect.height() );

#ifdef DYNAMIC_SCROLLBAR
		if( !m_bHideScrollbar && m_hTouchScrollBarTimer != INVALID_HOBJ)
#else
		if( !m_bHideScrollbar )
#endif	
			PaintScrollbar( m_hWindowGC, bShow );
	}



	//WmDeleteGC( m_hGC );

#if defined(_PERFORMANCE_DEBUG)
	SFBalProfileEnd( "WebControl0", "PaintToWindow", NULL );
#endif
}

void WebBackingStore::RequestPaintToWindow( int x, int y, int w, int h )
{
	IContentsListener* pContentListener = NULL;
	if( m_pWebControl == NULL )
		return;

	pContentListener = m_pWebControl->GetContentListener();


	if( pContentListener )
		pContentListener->updateScreen( x, y, w, h );
}

void WebBackingStore::PaintScrollbar( int hGC, bool bShow )
{
	WebCore::FrameView *frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
	int hWin = m_pWebControl->GetHWIN();
	int contentsWidth, contentsHeight;
	contentsWidth = frameView->contentsWidth();
	contentsHeight = frameView->contentsHeight();

	if( contentsHeight*m_zoomMultiplier > GetWindowRect().height() )
	{
		int hBarH = 0;
		if( contentsWidth >  GetWindowRect().width() )
		hBarH = m_hBar.height();

		if(  GetWindowRect().height() != m_vBar.height() || m_vBar.x() !=  GetWindowRect().width() - m_vBar.width()  )
		{
			WebCore::IntRect vBarRect = WebCore::IntRect(  GetWindowRect().width() - m_vBar.width(),
                                   0,
                                   m_vBar.width(),
                                    GetWindowRect().height() - hBarH );
			m_vBar.setRect( vBarRect );
		}
		m_vBar.setProportion(  GetWindowRect().height() - hBarH, (float)frameView->contentsHeight()*m_zoomMultiplier);
		m_vBar.setValue( (float)GetViewRect().y() *m_zoomMultiplier  );


		WebCore::GraphicsContext gc(&hGC,&hWin);
		gc.save();
		m_vBar.paint(&gc, GetWindowRect());
		gc.restore();

	if( bShow )
		WmShowGCRectArea( hGC, m_vBar.x(), m_vBar.y(), m_vBar.width(), m_vBar.height() ); 
	}

	if( contentsWidth *m_zoomMultiplier > GetWindowRect().width() )
	{
		int vBarW = 0;
		if( contentsHeight > GetWindowRect().height() )
		vBarW = m_vBar.width();

		if( GetWindowRect().width() != m_hBar.width() || m_hBar.y() != GetWindowRect().height() - m_hBar.height()  )
		{
			WebCore::IntRect hBarRect = WebCore::IntRect( 0,
                                   GetWindowRect().height() - m_hBar.height(),
                                   GetWindowRect().width() - vBarW,
                                   m_hBar.height());
			m_hBar.setRect( hBarRect );
		}
		m_hBar.setProportion( GetWindowRect().width() - vBarW, (float)frameView->contentsWidth() * m_zoomMultiplier);
		m_hBar.setValue( (float)GetViewRect().x() * m_zoomMultiplier  );

		int hWin = m_pWebControl->GetHWIN();

		WebCore::GraphicsContext gc(&hGC,&hWin);
		gc.save();
		m_hBar.paint(&gc, GetWindowRect());
		gc.restore();

	if( bShow )
		WmShowGCRectArea( hGC, m_hBar.x(), m_hBar.y(), m_hBar.width(), m_hBar.height() ); 
	}




}


void WebBackingStore::PageLoadStarted()
{

#if defined(_PERFORMANCE_DEBUG)
	SFBalProfileCleanAll();
	SFBalProfileStart( "WebControl0", "TotalLoadingTime" );
#endif
	

	//WebCore::FrameView* frameView = NULL;
	//frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

	//m_viewRect.setX(frameView->contentsX());
	//m_viewRect.setY(frameView->contentsY());

#ifdef _BACKINGSTORE_DEBUG
	FmRemoveDir( DEBUG_FOLDER );
	FmCreateDir( DEBUG_FOLDER );
	m_filecount = 0;
#endif

#ifdef _PERFORMANCE_DEBUG
	m_addToDirtyCount = 0;
	m_adjustInitialScaleCount =0;
	m_updateBackingStoreCount = 0;
	m_showToWindowCount = 0;
	m_resizeBackingStoreCount = 0;
#endif //_BACKINGSTORE_DEBUG
}

void WebBackingStore::PageLoadFinished()
{

	m_bPageLoading = false;

	if( m_bEnableInitialOverviewMode == false )
	{
		// prevent AdjustInitialScale from changing zoom value
		m_bDidUserZoomed = true;
	}

#if defined( _SFB_BACKINGSTORE_IMPROVE_LOADPERF )	
		AdjustInitialScale(false);
		UpdateBackingStore( false );
#else
		AdjustInitialScale(false);
#endif

	//}
	//WebCore::FrameView* frameView = NULL;
	//frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();


	//WebCore::IntSize contentsSize;
	//contentsSize.setWidth( frameView->contentsWidth() );
	//contentsSize.setHeight( frameView->contentsHeight() );

	//
	//int w = (float)GetWindowRect().width() / (float)m_minZoomMultiplier * (float)BACKINGSTORE_MULTIPLIER;
	//int h = (float)GetWindowRect().height() / (float)m_minZoomMultiplier * (float)BACKINGSTORE_MULTIPLIER;
	//WebCore::IntSize frameViewSize( w, h );
	//frameViewSize = frameViewSize.shrunkTo( contentsSize );
	//frameViewSize = frameViewSize.expandedTo( GetWindowRect().size() );


	//w = (float)frameViewSize.width() * m_zoomMultiplier;
	//h = (float) frameViewSize.height() * m_zoomMultiplier;
	//WebCore::IntSize backingStoreSize( w, h );
	//backingStoreSize = backingStoreSize.shrunkTo( GetMaxBackingStoreSize() );

	//ChangeBackingStoreSize( backingStoreSize );
	//ChangeFrameViewSize( frameViewSize );

#if defined(_PERFORMANCE_DEBUG)
	SFBalProfileEnd( "WebControl0", "TotalLoadingTime", NULL );
	SFBalProfilePrintAll();
	SFBalProfileCleanAll();
#endif


#ifdef _PERFORMANCE_DEBUG
	SysDebug(( MID_EXCEPTION, "PageLoadFinished : LayoutDebug AddToDirty(%d), AdjustScale(%d), UpdateStore(%d), ShowStore(%d), ResizeStore(%d)\n", 
		m_addToDirtyCount,	m_adjustInitialScaleCount,	m_updateBackingStoreCount,	m_showToWindowCount , m_resizeBackingStoreCount ));
#endif
}

bool WebBackingStore::AdjustInitialScale(bool bCheckCondition )
{
	if( bCheckCondition )
{
	if( IsLoading() == false || m_bDidFirstLayout == false || m_bDidLoadMainResource == false )
		return false; // scale is not changed
	}

	WebCore::FrameView* frameView = NULL;
	WebCore::IntSize contentsSize;

	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

	contentsSize.setWidth( frameView->contentsWidth() );
	contentsSize.setHeight( frameView->contentsHeight() );

	WebCore::IntSize windowSize = GetWindowRect().size();
	contentsSize = contentsSize.expandedTo( windowSize );

	float scale;

	scale = (float)windowSize.width() / (float)contentsSize.width();

	if( (contentsSize.width() > contentsSize.height()) && ((scale * contentsSize.height()) < windowSize.height()) && ((scale * contentsSize.width()) >= windowSize.width()) )
		scale = (float)windowSize.height() / (float)contentsSize.height();
		
	
	//float heightSclae = (float)contentSize.height() / (float)windowSize.height();

	if( scale < GetMinZoomFactor() )
		scale = GetMinZoomFactor();

	if( m_zoomMultiplier > scale || bCheckCondition == false )
	{
#ifdef _PERFORMANCE_DEBUG
	m_adjustInitialScaleCount++;
#endif
		
		AddToPaintRegion( GetWindowRect() );

		//m_minZoomMultiplier = scale;
		if(m_bDidUserZoomed == false)
			m_zoomMultiplier = scale;

		
		int w = (float)GetWindowRect().width() / (float)GetMinZoomFactor() * (float)GetBackingStoreMultiplier();
		int h = (float)GetWindowRect().height() / (float)GetMinZoomFactor() * (float)GetBackingStoreMultiplier();
		WebCore::IntSize frameViewSize( w, h );
		frameViewSize = frameViewSize.shrunkTo( contentsSize );
		frameViewSize = frameViewSize.expandedTo( GetWindowRect().size() );


		w = (float)frameViewSize.width() * scale;
		h = (float) frameViewSize.height() * scale;
		WebCore::IntSize backingStoreSize( w, h );
		backingStoreSize = backingStoreSize.shrunkTo( GetMaxBackingStoreSize() );
		backingStoreSize = backingStoreSize.expandedTo( GetWindowRect().size() );

		ChangeBackingStoreSize( backingStoreSize, true );
		ChangeFrameViewSize( frameViewSize );

		if( WmVerifyGC(m_hWindowGC) == TRUE )
			WmClearGC( m_hWindowGC );

		AdjustViewPos();

		return true;
	}
	else
		return false;
}

float WebBackingStore::GetMinZoomFactor()
{
	if( m_bEnableInitialOverviewMode )
	return m_minZoomMultiplier;
	else
		return (float)1.0;
}



void WebBackingStore::SetMinZoomFactor( float factor )
{
	m_minZoomMultiplier = factor;

}
void WebBackingStore::SetMaxZoomFactor( float factor )
{
	m_maxZoomMultiplier = factor;
}
float WebBackingStore::GetMaxZoomFactor()
{
	return m_maxZoomMultiplier;
}

void WebBackingStore::MaximizeBackingStore(bool bForce)
{
	WebCore::IntSize size = GetFrameViewRect().size();

	size.setWidth( (float)size.width() * m_zoomMultiplier );
	size.setHeight( (float)size.height() * m_zoomMultiplier );

	size = size.shrunkTo( GetMaxBackingStoreSize() );
	size = size.expandedTo( GetWindowRect().size() );
	ChangeBackingStoreSize( size, bForce );

	UpdateBackingStore( true );
}

void WebBackingStore::MinimizeBackingStore()
{
	SetBackingStorePos( GetViewRect().location() );
	ChangeBackingStoreSize( GetWindowRect().size() );

	//UpdateBackingStore( true );
	WebCore::IntRect rect;
	rect.setX( GetViewRect().x() - GetFrameViewRect().x() );
	rect.setY( GetViewRect().y() - GetFrameViewRect().y() );
	rect.setWidth( (float)GetViewRect().width() / m_zoomMultiplier );
	rect.setHeight( (float)GetViewRect().height() / m_zoomMultiplier );
	AddToDirtyRegion( rect );
	UpdateBackingStore( false );
}

float WebBackingStore::GetPossibleMinZoomMultiplier()
{
	float minZoom = GetMinZoomFactor() ;
	WebCore::IntSize frameViewSize = GetFrameViewRect().size();
	WebCore::IntSize windowSize = GetWindowRect().size();

	if( frameViewSize.width() * minZoom >= windowSize.width() &&
		frameViewSize.height() * minZoom >= windowSize.height() )
	{
		return minZoom;
	}
	else
	{
		float wScale, hScale;
		WebCore::FrameView* frameView = NULL;
		frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

		WebCore::IntSize contentsSize( frameView->contentsWidth(), frameView->contentsHeight() );

		if( frameViewSize.width() >= contentsSize.width() )
			wScale = minZoom;
		else
		{
			wScale = (float)windowSize.width() / (float)frameViewSize.width();
		}

		if( frameViewSize.height() >= contentsSize.height() )
			hScale = minZoom;
		else
		{
			hScale = (float)windowSize.height() / (float)frameViewSize.height();
		}

		if( wScale > hScale )
			return wScale;
		else
			return hScale;
	}

}

//void WebBackingStore::AdjustViewPos()
//{
//	WebCore::FrameView* frameView = NULL;
//	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();
//
//	int backingStoreX = frameView->contentsX();
//	int backingStoreY = frameView->contentsY();
//
//	WebCore::IntSize backingStoreSize = frameView->frameGeometry().size();
//
//	WebCore::IntSize maxPos( backingStoreSize - m_viewRect.size() );
//	WebCore::IntSize desiredOffset( m_viewRect.x() - backingStoreX, m_viewRect.y() - backingStoreY );
//	WebCore::IntSize viewoffset = desiredOffset.shrunkTo(maxPos);
//	viewoffset.clampNegativeToZero();
//
//	m_viewRect.setX( backingStoreX + viewoffset.width() );
//	m_viewRect.setY( backingStoreY + viewoffset.height() );
//
//}

WebCore::IntPoint WebBackingStore::WindowToBackingStore( const WebCore::IntPoint& point) 
{
	WebCore::IntPoint windowPoint = point;

	int dx, dy;


	windowPoint.setX( (float)windowPoint.x() / m_zoomMultiplier );
	windowPoint.setY( (float)windowPoint.y() / m_zoomMultiplier );


	dx = GetViewRect().x() - GetFrameViewRect().x();
	dy = GetViewRect().y() - GetFrameViewRect().y();

	windowPoint.move( dx, dy );


	return windowPoint;
}

WebCore::IntPoint WebBackingStore::BackingStoreToWindow( const WebCore::IntPoint& point) 
{
	//Gnana Modified For Drag and Drop
	WebCore::FrameView* frameView = NULL;
	frameView = m_pWebControl->GetMainWebFrame()->GetFrame()->view();

	int x, y;
	int dx, dy;


	dx = GetViewRect().x() - GetFrameViewRect().x();
	dy = GetViewRect().y() - GetFrameViewRect().y();

	x = (point.x() - dx)*m_zoomMultiplier;
	y = (point.y() - dy)*m_zoomMultiplier;

	return WebCore::IntPoint( x, y );
}


void
Memcpy16(void* dest, const void* src, size_t length )
{
	char *pDest = (char*)dest;
	char *pSrc = (char*)src;

	if( pDest > pSrc && (pDest-pSrc) < length*2 )
	{
		int overlaplen = length*2 - (pDest-pSrc);
		int empty = length*2 - overlaplen;

		int i = 0;
		while(1)
		{
			i++;
			char *pD = pDest + length*2 - i*empty;
			char *pS = pSrc + length*2 - i*empty;

			if( pD < pDest )
			{
				AcMemcpy16( pDest, pSrc, (length*2 - (i-1)*empty)/2 );
				break;
			}
			
			AcMemcpy16( pD, pS, empty/2 );
		}
	}
	else
		AcMemcpy16( pDest, pSrc, length );
}


/**
* WmCopyGC create WmBitamp and draw the bitmap to dest GC.
* So WmCopyGC have to copy the area 2 times.
*
* This method move the area to dest GC directly.
*/
void WebBackingStore::CopyGC( HGC hDestGC, int destX, int destY, HGC hSrcGC, int srcX, int srcY, int srcW, int srcH )
{
//	SysDebug(( MID_EXCEPTION, "WebBackingStore::CopyGC : DestGC(%d), destX(%d), destY(%d), SrcGC(%d), SrcX(%d), SrcY(%d), SrcW(%d), SrcH(%d) \n", hDestGC, destX, destY, hSrcGC, srcX, srcY, srcW, srcH));

	if ( srcW <= 0 || srcH <= 0 )
		return;
	//WmCopyGC( hDestGC, destX, destY, hSrcGC, srcX, srcY, srcW, srcH );

	if ( m_pWebControl->GetGCType() == BIT_32_GC  )
	{
		if ( srcW <= 0 || srcH <= 0 )
			return;
	
		WmCopyGC( hDestGC, destX, destY, hSrcGC, srcX, srcY, srcW, srcH );
	}
	else
	{

#if defined(_PERFORMANCE_DEBUG)
	SFBalProfileStart( "WebControl0", "CopyGC" );
#endif
		COLOR *srcframeBuf;
		COLOR *destframeBuf;
		int y, destframeBufW, srcframeBufW;
		WmGC *pSrcGC, *pDestGC;
	
		pSrcGC = WmGetGCPointer( hSrcGC );
		pDestGC = WmGetGCPointer( hDestGC );
	

		//debug
		//SysAssert(( destX >= 0 && destY >= 0 && srcX >= 0 && srcY >= 0 ));
		//SysAssert(( pSrcGC->rcClipGC.x <= srcX && pSrcGC->rcClipGC.y <= srcY && srcX + srcW <= pSrcGC->rcClipGC.w && srcY + srcH <= pSrcGC->rcClipGC.h ));
		//SysAssert(( destX < pDestGC->rcClipGC.w && destY < pDestGC->rcClipGC.h && destX + srcW <= pDestGC->rcClipGC.w && destY + srcH <= pDestGC->rcClipGC.h ));
		
		if( ( destX >= 0 && destY >= 0 && srcX >= 0 && srcY >= 0 ) == false )
			return;
		
		if( ( pSrcGC->rcClipGC.x <= srcX && pSrcGC->rcClipGC.y <= srcY && srcX + srcW <= pSrcGC->rcClipGC.w && srcY + srcH <= pSrcGC->rcClipGC.h ) == false )
		{

			SysDebug(( MID_EXCEPTION, "WebBackingStore::CopyGC : srcX(%d), srcY(%d), srcW(%d), srcH(%d)  \n", srcX, srcY, srcW, srcH ));
			
			SysDebug(( MID_EXCEPTION, "WebBackingStore::CopyGC : srcClipX(%d), srcClipY(%d), srcClipW(%d), srcClipH(%d)  \n", pSrcGC->rcClipGC.x, pSrcGC->rcClipGC.y, pSrcGC->rcClipGC.w, pSrcGC->rcClipGC.h ));

			int tempX, tempY;

			if( pSrcGC->rcClipGC.x > srcX )
			{
				tempX = pSrcGC->rcClipGC.x - srcX;
				srcX = pSrcGC->rcClipGC.x;
				srcW = srcW - tempX;
				destX = destX + tempX;
			}

			if( pSrcGC->rcClipGC.y > srcY )
			{
				tempY = pSrcGC->rcClipGC.y - srcY;
				srcY = pSrcGC->rcClipGC.y;
				srcH = srcH - tempY;
				destY = destY + tempY;
			}

			if( srcX + srcW > pSrcGC->rcClipGC.w )
			{
				destX = destX + ( (srcX + srcW) - pSrcGC->rcClipGC.w );
				srcW = srcW - ( (srcX + srcW) - pSrcGC->rcClipGC.w );
			}

			if( srcY + srcH > pSrcGC->rcClipGC.h )
			{
				destY = destY + ( (srcY + srcH) - pSrcGC->rcClipGC.h );
				srcH = srcH - ( (srcY + srcH) - pSrcGC->rcClipGC.h );
			}

		}
			

		if( ( destX < pDestGC->rcClipGC.w && destY < pDestGC->rcClipGC.h && destX + srcW <= pDestGC->rcClipGC.w && destY + srcH <= pDestGC->rcClipGC.h ) == false )
			return;

	
		srcframeBuf = pSrcGC->pMem;
		srcframeBufW = pSrcGC->rcFrmbuf.w;
	
		destframeBuf = pDestGC->pMem;
		destframeBufW = pDestGC->rcFrmbuf.w;
	
		if ( !pDestGC->bMemGC )
		{
			destX += pDestGC->rcGC.x;
			destY += pDestGC->rcGC.y;
		}
		
	
		if( hDestGC != hSrcGC || destY <= srcY )
		{
			for ( y = srcY; y < srcY + srcH; y++, destY++ )
				Memcpy16( (destframeBuf + destY * destframeBufW + destX), (srcframeBuf + y * srcframeBufW + srcX), srcW );     
		}
		else
		{
			int dy = destY + srcH -1;
			for ( y = srcY+srcH-1; y >= srcY; y--, dy-- )
				Memcpy16( (destframeBuf + dy * destframeBufW + destX), (srcframeBuf + y * srcframeBufW + srcX), srcW );     
			
		}
#if defined(_PERFORMANCE_DEBUG)
	SFBalProfileEnd( "WebControl0", "CopyGC", NULL );
#endif
	}
}

//WebCore::IntSize WebBackingStore::CalculateBackingStoreSize( const WebCore::IntSize & contentsSize )
//{
//#if !defined(SFB_GWES_TRANSFORM)
//	WebCore::FrameView *frameView =  m_pWebControl->GetMainWebFrame()->GetFrame()->view();
//
//	WebCore::IntSize backingStoreSize;
//
//	backingStoreSize.setWidth( (float)m_viewRect.width() * GetBackingStoreMultiplier() );
//	backingStoreSize.setHeight( (float)m_viewRect.height() * GetBackingStoreMultiplier()   );
//
//
//
//	if( backingStoreSize.width() > contentsSize.width() )
//		backingStoreSize.setWidth( contentsSize.width() );
//
//	if( backingStoreSize.height() > contentsSize.height() )
//		backingStoreSize.setHeight( contentsSize.height() );
//
//	if( backingStoreSize.width() < m_viewRect.width() )
//		backingStoreSize.setWidth( m_viewRect.width() );
//	if( backingStoreSize.height() < m_viewRect.height() )
//		backingStoreSize.setHeight( m_viewRect.height() );
//
//	SysDebug(( MID_EXCEPTION, "CalculateBackingStoreSize : LayoutDebug : candiBackingStore(%d,%d), contSize( %d,%d ), backingStore(%d,%d)\n",
//		(int)((float)m_viewRect.width() * GetBackingStoreMultiplier()), (int)((float)m_viewRect.height() * GetBackingStoreMultiplier()),
//		contentsSize.width(), contentsSize.height(),
//		backingStoreSize.width(), backingStoreSize.height() ));
//
//	return backingStoreSize;
//#else
//
//	float backingStoreWidth = m_viewRect.width();
//	float backingStoreHeight = m_viewRect.height();
//
//	backingStoreWidth = backingStoreWidth / MIN_MULTIPLIER;
//	backingStoreHeight = backingStoreHeight / MIN_MULTIPLIER;
//
//	WebCore::IntSize backingStoreSize( backingStoreWidth, backingStoreHeight );
//	return backingStoreSize;
//
//#endif
//}

float WebBackingStore::GetZoom()
{
	return m_zoomMultiplier;
}

WebCore::FloatRect WebBackingStore::CalculateZoomArea( float newMultiplier, WebCore::IntPoint& centerPoint )
{
	WebCore::FloatRect zoomArea( GetViewRect() );
	//zoomArea.setSize( GetWindowRect().size() );
	WebCore::FloatPoint center;
	float w, h;

	float multiplier = GetZoom();
	multiplier = newMultiplier;// / multiplier;


	if( centerPoint.x() <  0 && centerPoint.y() < 0 )
	{
	center.setX( zoomArea.x() + (zoomArea.width() / 2) );
	center.setY( zoomArea.y() + (zoomArea.height() / 2) );
	}
	else
	{
		center.setX( centerPoint.x() );
		center.setY( centerPoint.y() );
	}

	w = (float)(GetWindowRect().width()) / multiplier;
	h = (float)(GetWindowRect().height()) / multiplier;

	
	zoomArea.setX( center.x() - w/2 );
	zoomArea.setY( center.y() - h/2 );
	zoomArea.setWidth( w );
	zoomArea.setHeight( h );

	//newCenterPoint.setX(  zoomArea.x() /* multiplier*/ );
	//newCenterPoint.setY( zoomArea.y() /* multiplier*/ );
	//newViewPoint.clampNegativeToZero();

	centerPoint.setX( center.x() );
	centerPoint.setY( center.y() );

	return zoomArea;
}

/**
* Show zooming effect
*/
void WebBackingStore::Zooming( float newMultiplier, WebCore::IntPoint centerPoint )
{
	int dx = 0, dy = 0;
	//WebCore::IntRect backingStore( GetBackingStoreRect() );
	WebCore::FloatRect zoomArea;
	WebCore::FloatPoint center;
	float w, h;
	//WebCore::IntPoint dest( -1, -1 );

#ifdef DYNAMIC_SCROLLBAR
	if(m_hTouchScrollBarTimer != INVALID_HOBJ)
	{
		__DeleteTimer( m_hTouchScrollBarTimer);
		m_hTouchScrollBarTimer = INVALID_HOBJ;
	}
#endif


	if( newMultiplier == m_zoomMultiplier )
		return;

	zoomArea = CalculateZoomArea( newMultiplier, centerPoint );

//	SysDebug(( MID_EXCEPTION, "WebBackingStore::Zooming : m(%0.5f), centerPoint(%d, %d)\n", newMultiplier, centerPoint.x(), centerPoint.y() ));


	// scroll backing store
	WebCore::IntRect newRect = CalculateRectContaingCenterRect( GetViewRect(), GetFrameViewRect(), GetBackingStoreRect().size() );
	int newDx = newRect.x() - GetBackingStoreRect().x();
	int newDy = newRect.y() - GetBackingStoreRect().y();
	__ScrollBackingStore( -newDx, -newDy );



	WebCore::IntRect zoomRect = enclosingIntRect( zoomArea );
	
	
#ifdef _BACKINGSTORE_DEBUG
	HGC hDebugGC;
	
	if ( m_pWebControl->GetGCType() == BIT_32_GC )
		hDebugGC = WmCreateWindowSubMemGC32( m_pWebControl->GetHWIN(), 0, 0, GetBackingStoreSize().width(), GetBackingStoreSize().height() );
	else
		hDebugGC = WmCreateWindowSubMemGC( m_pWebControl->GetHWIN(), 0, 0, GetBackingStoreSize().width(), GetBackingStoreSize().height() );
		
		CopyGC( hDebugGC, 0, 0, m_backingStoreGC, 0, 0,  GetBackingStoreSize().width(), GetBackingStoreSize().height() );
		
		WmSetFgColor( hDebugGC, RED );
		WmDrawRect( hDebugGC, (zoomArea.x() - GetBackingStoreRect().x())*m_zoomMultiplier, (zoomArea.y() - GetBackingStoreRect().y())*m_zoomMultiplier, 
			zoomArea.width()*m_zoomMultiplier, zoomArea.height()*m_zoomMultiplier );

		SaveBackingStoreToImageFile( hDebugGC,GetBackingStoreSize().width(), GetBackingStoreSize().height(), "zoomingpos" );
		WmDeleteGC( hDebugGC );
#endif


	HGC hZoomGC;

	int zoomWidth = (zoomRect.width() * m_zoomMultiplier);
	int zoomHeight = (zoomRect.height() * m_zoomMultiplier);
//	SysDebug(( MID_EXCEPTION, "WebBackingStore::Zooming : Width(%d), Height(%d) \n", zoomWidth, zoomHeight));

	if(WEB_CURRENTLY_ALLOCATED_PLATFORM_HEAP_SIZE >= (WEB_MAX_MEM_USAGE * WEB_TOTAL_PLATFORM_HEAP_SIZE))
		return;

	if(zoomWidth * zoomHeight * 2 > SFBalMemGetStatEx())
		return;


	if ( m_pWebControl->GetGCType() == BIT_32_GC  )
	{
		WmSetWindowBgColor( m_pWebControl->GetHWIN(), BLACK );
		hZoomGC = WmCreateWindowSubMemGC32( m_pWebControl->GetHWIN(), 0, 0,  zoomRect.width()*m_zoomMultiplier, zoomRect.height()*m_zoomMultiplier );
	}
	else
		hZoomGC = WmCreateWindowSubMemGC( m_pWebControl->GetHWIN(), 0, 0, zoomRect.width()*m_zoomMultiplier, zoomRect.height()*m_zoomMultiplier );

	WebCore::IntRect intersectRect( zoomRect );
	//intersectRect.setWidth( intersectRect.width() / m_zoomMultiplier );
	//intersectRect.setHeight( intersectRect.height() / m_zoomMultiplier );
	//intersectRect.scale( m_zoomMultiplier );
	//WebCore::IntRect tempRect = zoomRect;
	//tempRect.scale( m_zoomMultiplier );
	intersectRect.intersect( GetBackingStoreRect() );



	CopyGC( hZoomGC, (intersectRect.x() - zoomRect.x())*m_zoomMultiplier,  (intersectRect.y() - zoomRect.y())*m_zoomMultiplier,
					m_backingStoreGC, 
					(intersectRect.x() - GetBackingStoreRect().x())*m_zoomMultiplier,  
					(intersectRect.y() - GetBackingStoreRect().y())*m_zoomMultiplier, 
					(intersectRect.width())*m_zoomMultiplier, 
					(intersectRect.height())*m_zoomMultiplier);


#ifdef _BACKINGSTORE_DEBUG
	SaveBackingStoreToImageFile( hZoomGC,zoomRect.width()*m_zoomMultiplier, zoomRect.height()*m_zoomMultiplier, "zoomingimg" );
#endif

	COLOR *frameBuf;
	WmGetGCFrmbuf( hZoomGC, &frameBuf );

	WmBitmap bitmap;
	bitmap.compressType  = COMPRESS_NONE;
	bitmap.bUseChromakey = FALSE;
	bitmap.bitsPerPixel	 = BITS_PER_PIXEL;	
	bitmap.rectW		 = zoomRect.width()*m_zoomMultiplier;
	bitmap.rectH		 = zoomRect.height()*m_zoomMultiplier;
	bitmap.pBits = frameBuf;


	HGC hGC = WmCreateClientGC( m_pWebControl->GetHWIN() );


	WmDrawScaledBitmap( hGC, 0, 0, GetWindowRect().width(), GetWindowRect().height(), &bitmap );

//#ifdef _BACKINGSTORE_DEBUG
//	SaveBackingStoreToImageFile( hGC,GetWindowRect().width(), GetWindowRect().height(), "zooming" );
//#endif

	WmShowGC( hGC );
	WmDeleteGC( hZoomGC );
	WmDeleteGC( hGC );
}

void WebBackingStore::Zoom( float newMultiplier, WebCore::IntPoint newcenterPoint )
//void WebBackingStore::Zoom( float newMultiplier )
{
	if( newMultiplier < GetPossibleMinZoomMultiplier() )
		newMultiplier = GetPossibleMinZoomMultiplier();

	if( newMultiplier > GetMaxZoomFactor() )
		newMultiplier = GetMaxZoomFactor();

	//if( m_zoomMultiplier == newMultiplier )
		//return;

	//WebCore::IntPoint newViewPoint;
	//WebCore::FloatRect zoomArea = CalculateZoomArea( newMultiplier, newViewPoint );

	WebCore::IntSize size = GetFrameViewRect().size();

	size.setWidth( (float)size.width() * newMultiplier );
	size.setHeight( (float)size.height() * newMultiplier );

	size = size.shrunkTo( GetMaxBackingStoreSize() );
	size = size.expandedTo( GetWindowRect().size() );
	int requiredSize = (size.width() * size.height() * 2);

	if(WEB_CURRENTLY_ALLOCATED_PLATFORM_HEAP_SIZE + requiredSize >= (WEB_MAX_MEM_USAGE * WEB_TOTAL_PLATFORM_HEAP_SIZE))
	{
		if( m_zoomMultiplier < ORIGINAL_SCALE )
		{
			if( newMultiplier > ORIGINAL_SCALE)
			{
				newMultiplier = ORIGINAL_SCALE;
			}
		}
		else if( newMultiplier > ORIGINAL_SCALE)
		{
			RequestPaintToWindow( 0,0, GetWindowRect().width(), GetWindowRect().height() );
			return;
		}
	}

	WebCore::IntSize frameSize = GetFrameViewRect().size();

	frameSize.setWidth( (float)frameSize.width() * newMultiplier );
	frameSize.setHeight( (float)frameSize.height() * newMultiplier );

	frameSize = frameSize.shrunkTo( GetMaxBackingStoreSize() );
	frameSize = frameSize.expandedTo( GetWindowRect().size() );
	requiredSize = (frameSize.width() * frameSize.height() * 2);

	if( requiredSize > SFBalMemGetStatEx())
	{
		RequestPaintToWindow( 0,0, GetWindowRect().width(), GetWindowRect().height() );
		return;
	}

	m_bDidUserZoomed = true;

	if( newcenterPoint.x() <  0 && newcenterPoint.y() < 0 )
		CalculateZoomArea( newMultiplier, newcenterPoint );

	m_zoomMultiplier = newMultiplier;

	WebCore::FrameView *frameView =  m_pWebControl->GetMainWebFrame()->GetFrame()->view();
	//WebCore::FloatRect dirtyRect( frameView->frameGeometry() );
	//dirtyRect.scale( 1/newMultiplier );


	WebCore::IntSize offset;
	WebCore::IntPoint center;
	SetViewPos( GetFrameViewRect().location() );
	SetBackingStorePos( GetFrameViewRect().location() );
	center.setX( GetViewRect().x() + GetViewRect().width() / 2 );
	center.setY( GetViewRect().y() + GetViewRect().height() / 2 );
	offset.setWidth( newcenterPoint.x() - center.x() );
	offset.setHeight( newcenterPoint.y() - center.y() );

	//AddToDirtyRegion( enclosingIntRect( dirtyRect) );
	//UpdateBackingStore( true );

	// change backingstore size
	MaximizeBackingStore(true);

 
//	SysDebug(( MID_EXCEPTION, "WebBackingStore::Zoom : oldcenter(%d, %d)\n", 
//		center.x(), center.y() ));

	__ScrollBy( offset.width(), offset.height() );

	center.setX( GetViewRect().x() + GetViewRect().width() / 2 );
	center.setY( GetViewRect().y() + GetViewRect().height() / 2 );

#if defined(_SFB_BACKINGSTORE_IMPROVE_LOADPERF)
	if( IsLoading() || (offset.width() == 0 && offset.height() == 0) )
#else
	if( offset.width() == 0 && offset.height() == 0 )
#endif
	{
		IContentsListener* pContentListener = NULL;
		pContentListener = m_pWebControl->GetContentListener();
		if( pContentListener )
			pContentListener->updateScreen( 0, 0, GetWindowRect().width(), GetWindowRect().height());
	}

//	SysDebug(( MID_EXCEPTION, "WebBackingStore::Zoom : newcenter(%d, %d), offset(%d,%d), scroll(%d,%d)\n", 
//		newcenterPoint.x(), newcenterPoint.y(),
//		offset.width(), offset.height(),
//		center.x(), center.y() ));
}


void WebBackingStore::SetPageOverview( const WebCore::IntPoint & centerpoint )
{
	WebCore::IntPoint center = WindowToBackingStore( centerpoint );
	center.move( GetFrameViewRect().x(), GetFrameViewRect().y() );

	if( m_zoomMultiplier == GetPossibleMinZoomMultiplier() )
	{
		//center.setX( (float)center.x() * (float)ORIGINAL_SCALE / m_zoomMultiplier  );
		//center.setY( (float)center.y() * (float)ORIGINAL_SCALE / m_zoomMultiplier  );
		Zooming( ORIGINAL_SCALE, center );
		Zoom( ORIGINAL_SCALE, center );
	}
	else
	{
		//center.setX( (float)center.x() * (float)m_minZoomMultiplier / m_zoomMultiplier  );
		//center.setY( (float)center.y() * (float)m_minZoomMultiplier / m_zoomMultiplier  );
		Zooming( GetPossibleMinZoomMultiplier(), center );
		Zoom( GetPossibleMinZoomMultiplier(), center );
	}
}


void WebBackingStore::ResetViewState()
{
//	SysDebug(( MID_EXCEPTION, "ResetViewState : LayoutDebug started\n" ));


	m_bPageLoading = true;
	m_bDidLoadMainResource = false;
	m_bDidFirstLayout = false;
	m_bDidUserZoomed = false;

	WebCore::Frame* frame = NULL;
	frame = m_pWebControl->GetMainWebFrame()->GetFrame();

	m_zoomMultiplier = ORIGINAL_SCALE;
	//m_minZoomMultiplier = MIN_MULTIPLIER;
	m_rcView = GetWindowRect();
	m_rcBackingStore = GetWindowRect();

	WebCore::IntSize size = GetWindowRect().size();
	ChangeFrameViewSize( size );
	ChangeBackingStoreSize( size, true );

	AddToPaintRegion( WebCore::IntRect( GetViewRect().x() - GetFrameViewRect().x(), GetViewRect().y() - GetFrameViewRect().y() , GetViewRect().width(), GetViewRect().height()) );


//	SysDebug(( MID_EXCEPTION, "ResetViewState : LayoutDebug finished\n" ));
}

class WebBackingStoreViewState : public WebCore::HistoryItemViewState
{
public:
	WebCore::IntPoint		m_viewPos;
	~WebBackingStoreViewState() {};
};

void WebBackingStore::SaveViewState(WebCore::HistoryItem* item)
{
	WebBackingStoreViewState *pState = (WebBackingStoreViewState*)item->viewState();
	if( pState == NULL )
	{
		pState = new WebBackingStoreViewState();
	}

	pState->m_viewPos = GetViewRect().location();

	item->setViewState( pState );
}

void WebBackingStore::RestoreViewState()
{
	WebCore::HistoryItem* currentItem = m_pWebControl->GetMainWebFrame()->GetFrame()->loader()->currentHistoryItem();
	//WebAssert(currentItem);

	if(currentItem)
	WebBackingStoreViewState* pState = (WebBackingStoreViewState*)currentItem->viewState();
	//WebAssert( pState );

	//m_restoredViewPos = pState->m_viewPos;
}

void WebBackingStore::DispatchDidFirstLayout()
{
//	SysDebug(( MID_EXCEPTION, "DispatchDidFirstLayout : LayoutDebug started\n" ));
	m_bDidFirstLayout = true;
	WebCore::HistoryItem* currentItem = NULL;
	WebBackingStoreViewState* pState = NULL;

	currentItem = m_pWebControl->GetMainWebFrame()->GetFrame()->loader()->currentHistoryItem();
//	WebAssert(currentItem);
	
	if(currentItem)
		pState = (WebBackingStoreViewState*)currentItem->viewState();

	// backward/forward patch
	WebCore::IntRect rcFrameView = GetFrameViewRect();
	WebCore::IntRect rcView = GetViewRect();
	WebCore::IntRect rcRestoredView = GetViewRect();
	if( pState )
		rcRestoredView.setLocation( pState->m_viewPos );


	int dx = rcRestoredView.x() - rcView.x();
	int dy = rcRestoredView.y() - rcView.y();
	__ScrollBy( dx, dy );

}


void WebBackingStore::DispatchDidLoadMainResource(WebCore::DocumentLoader* loader) 
{
	m_bDidLoadMainResource = true;
//	SysDebug(( MID_EXCEPTION, "DispatchDidLoadMainResource : LayoutDebug started\n" ));
}

void WebBackingStore::Paint()
{
	bool bCompleteDirty = false;

	if( m_bEnableInitialOverviewMode )
	if( AdjustInitialScale() )
	{
		UpdateBackingStore( true );
		return; // relayout
	}

	UpdateBackingStore( bCompleteDirty );

	//PaintToWindow( x, y, w, h, true, false );
}



void WebBackingStore::valueChanged(WebCore::Scrollbar*)
{
}

// Used to obtain a window clip rect.
WebCore::IntRect WebBackingStore::windowClipRect() const
{
	WebCore::IntRect rect; /*= GetViewRect();
	rect.setWidth( GetWindowRect().width() );
	rect.setHeight( GetWindowRect().height() );*/
	return rect;
}

// FIXME: It would be nice to set this state on the scroll bar instead of
// having to ask for it from the client at paint time.
bool WebBackingStore::isActive() const
{
	return true;
}

WebCore::IntRect WebBackingStore::GetViewRect()
{
	WebCore::IntRect rc = m_rcView;

	rc.setWidth( (float)GetWindowRect().width() / m_zoomMultiplier );
	rc.setHeight( (float)GetWindowRect().height() / m_zoomMultiplier );

	WebCore::FrameView *frameView =  m_pWebControl->GetMainWebFrame()->GetFrame()->view();

	WebCore::IntSize viewSize = rc.size();
	WebCore::IntSize frameViewSize = GetFrameViewRect().size();
	viewSize = viewSize.shrunkTo( frameViewSize );

	rc.setSize( viewSize );

	return rc;

}

void WebBackingStore::CopyScreen( int destGC, int destx, int desty, int srcx, int srcy, int srcw, int srch )
{
	int dx, dy;

	dx = (float)(GetViewRect().x() - GetBackingStoreRect().x()) * m_zoomMultiplier;
	dy = (float)(GetViewRect().y() - GetBackingStoreRect().y()) * m_zoomMultiplier;

	dx += srcx;
	dy += srcy;

	WmCopyGC( destGC, destx, desty, m_backingStoreGC, dx, dy, srcw, srch );

//#ifdef _BACKINGSTORE_DEBUG
//	SaveBackingStoreToImageFile( destGC, srcw, srch,  );
//#endif
}


bool WebBackingStore::ScrollTo( WebCore::IntPoint point )
{
	WebCore::IntPoint currPoint = GetViewRect().location();

	int dx = point.x() - currPoint.x();
	int dy = point.y() - currPoint.y();

	__ScrollBy( dx, dy );

	return true;
}

void WebBackingStore_UpdateContentsTimerCB(HTimer hTimer, ULONG param)
{
	WebBackingStore *pBackingStore = (WebBackingStore*)param;

	if( pBackingStore->m_bNeedToUpdateContents )
		pBackingStore->UpdateContents( WebCore::IntRect( 0,0,0,0 ) ); 

	__DeleteTimer( pBackingStore->m_hUpdateContentsTimer );
	pBackingStore->m_hUpdateContentsTimer = INVALID_HOBJ;

	pBackingStore->m_bNeedToUpdateContents = false;
}


void WebBackingStore::UpdateContents( WebCore::IntRect rect )
{
		IContentsListener* pContentListener = NULL;

		if(m_pWebControl->isExitInstanceCalled())
			return;
		// update backingstore
		// The update area is automatically caculated by the BackingStore
		Paint();

		pContentListener = m_pWebControl->GetContentListener();
		if( pContentListener && !m_pWebControl->isZooming() )
			pContentListener->updateScreen(  0, 0, 0, 0 );
}


void WebBackingStore::HandleUpdateContents( WebCore::IntRect rect )
{
#if defined(_SFB_BACKINGSTORE_IMPROVE_LOADPERF) && defined(  _USE_UPDATECONTENTS_TIMER )

	if( IsLoading() == false )
	{
		UpdateContents( rect );
		return;
	}

	if(  m_hUpdateContentsTimer == INVALID_HOBJ  )
	{
		UpdateContents( rect );

			m_bNeedToUpdateContents = false;

			m_hUpdateContentsTimer = __CreateTimer( "WebBackingStoreUpdate" );
			WebAssert( m_hUpdateContentsTimer != INVALID_HOBJ );

			BOOL bResult = __StartTimer( m_hUpdateContentsTimer, UPDATECONTENTS_TIMEOUT, 
				WebBackingStore_UpdateContentsTimerCB, (ULONG)this );
			WebAssert( bResult );
	}
	else
	{
		m_bNeedToUpdateContents = true;
	}
#else
	UpdateContents( rect );
#endif

}

// bs0425.lee modified for touch scroll. 
WebCore::IntRect WebBackingStore::getBackingScrollbarData(const WebCore::IntPoint& mousePosition ) 
{
	//WebCore::PlatformScrollbar* c_Bar;
	WebCore::IntRect c_thumbRect;

	/*if (m_hBar)
	{
		c_Bar=m_hBar;
		c_thumbRect=c_Bar.thumbRect();
		m_trackRect=c_Bar.trackRect();

		// extend the range of touch scroll. 
		m_trackRect.setY(m_trackRect.y()-extendRange);
		c_thumbRect.setY(c_thumbRect.y()-extendRange);

		m_trackRect.setHeight(m_trackRect.height()+extendRange);
		c_thumbRect.setHeight(c_thumbRect.height()+extendRange);

		if(c_thumbRect.contains(mousePosition))
			return c_thumbRect;
	}
*/

	
		c_thumbRect=m_hBar.thumbRect();
		m_trackRect=m_hBar.trackRect();

		// extend the range of touch scroll. 
		m_trackRect.setY(m_trackRect.y()-extendRange);
		c_thumbRect.setY(c_thumbRect.y()-extendRange);

		m_trackRect.setHeight(m_trackRect.height()+extendRange);
		c_thumbRect.setHeight(c_thumbRect.height()+extendRange);

		// extend scrollBar range. 
		c_thumbRect.setX(c_thumbRect.x()-extendBar);
		if(c_thumbRect.x()<=0)
			c_thumbRect.setX(0);

		c_thumbRect.setWidth(c_thumbRect.x()+c_thumbRect.width()+extendBar);

		if(c_thumbRect.contains(mousePosition))
			return c_thumbRect;
//	} 
		else
		{


		c_thumbRect=m_vBar.thumbRect();
		m_trackRect=m_vBar.trackRect();

		m_trackRect.setX(m_trackRect.x()-extendRange);
		c_thumbRect.setX(c_thumbRect.x()-extendRange);
	
		m_trackRect.setWidth(m_trackRect.width()+extendRange);
		c_thumbRect.setWidth(c_thumbRect.width()+extendRange);

		// extend scrollBar range. 
		c_thumbRect.setY(c_thumbRect.y()-extendBar);
		if(c_thumbRect.y()<=0)
			c_thumbRect.setY(0);

		c_thumbRect.setHeight(c_thumbRect.y()+c_thumbRect.height()+extendBar);

		if(c_thumbRect.contains(mousePosition))
              return c_thumbRect;
		}
	
    return WebCore::IntRect(0, 0, 0, 0);    
}

WebCore::IntRect WebBackingStore::getBackingTrackRect() const
  {   
	  return m_trackRect;
 }


#ifdef _BACKINGSTORE_DEBUG

void WebBackingStore::SaveBackingStoreToImageFile( HGC hGC, int w, int h, char* namePrefix )
{
		WmBitmap *pCaptureBitmap = NULL;
		ImgImage img;
		char szFilePath[100] = {0x00, };
		COLOR *frameBuf;
		WmGC *pGC;

		if( hGC == INVALID_HOBJ )
			return;


		//WmGetGCFrmbuf( hGC, &frameBuf );

	pGC = WmGetGCPointer( hGC );

	frameBuf = pGC->pMem;

	if ( !pGC->bMemGC )
	{
		frameBuf += pGC->rcGC.y * pGC->rcFrmbuf.w + pGC->rcGC.x ;
	}



	AcMemset(&img, 0x00, sizeof(ImgImage));

	img.pHead = (ImgBitmapElem*)MemAlloc( sizeof(ImgBitmapElem) );

	if (!img.pHead )
	{
		WebAssert( FALSE );
	}
	
	img.pHead->pbmBitmap = (ImgBitmap*)MemAlloc( sizeof(ImgBitmap) );

	if (!(img.pHead->pbmBitmap))
	{
		WebAssert( FALSE );
	}

		unsigned char* pEncodedData = NULL;
		ULONG	uFileSize = 0;
		HFILE hFile = INVALID_HOBJ;
		//pCaptureBitmap = WmCaptureBitmap( hGC, 0 , 0 , w , h ); 
		//		fileCount++;

		AcMemset(img.pHead->pbmBitmap, 0x00, sizeof(ImgBitmap));

		img.bitmapCount = 1;
		img.pHead->pbmBitmap->width  = w;   //width
		img.pHead->pbmBitmap->height =  h;   //height
		img.pHead->pbmBitmap->pBits  = frameBuf;
		img.pHead->pbmBitmap->bitsPerPixel = 16;
		//img.pHead->pbmBitmap->bChromaKeyFlag = pCaptureBitmap->bUseChromakey;
		//img.pHead->pbmBitmap->chromaKey = pCaptureBitmap->colorChromakey;

		 //Bmp ==> Jpg
		if (FALSE == ImgEncodeImage(IMG_ENC_JPEG, &img, &pEncodedData, &uFileSize, 5))
		{
			WebAssert( FALSE );
		}

		AcSprintfS( szFilePath, sizeof(szFilePath), "%sWebControl%d/", (char*)DEBUG_FOLDER, m_WebControlID );

		HDir hDir;
		if( (hDir = FmOpenDir( szFilePath )) == INVALID_HOBJ )
		{
			FmCreateDir( szFilePath );
		}
		else
			FmCloseDir( hDir );
		
		m_filecount++;
		AcSprintfS( szFilePath, sizeof(szFilePath), "%sWebControl%d/%05d%s.jpg", (char*)DEBUG_FOLDER, m_WebControlID, m_filecount, namePrefix );

		FmRemoveFile( szFilePath );
		// Create File
		hFile = FmCreateFile( szFilePath, FM_WRITE);  //  //?? |FM_TRUNCATE
		if( hFile == INVALID_HOBJ )
		{
			WebAssert( FALSE );
		}

		//FmWriteFile( hFile, frameBuf, w*h*2);

		if (FmWriteFile( hFile, pEncodedData, uFileSize ) == FALSE)
		{
			WebAssert( FALSE );
			
		}

		if (pEncodedData)
		{
			MemFree(pEncodedData);
		}

		//if (pCaptureBitmap)
		//{
		//	WmFreeCapturedBitmap(pCaptureBitmap);
		//}

		FmCloseFile( hFile );

		if ( (img.pHead) && (img.pHead->pbmBitmap) )
		{
			MemFree(img.pHead->pbmBitmap);
			img.pHead->pbmBitmap = NULL;
		}

		if (img.pHead)
		{
			MemFree(img.pHead);
			img.pHead = NULL;
		}

}
#endif // _BACKINGSTORE_DEBUG


} // namespace WebControl

#endif