/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		WebControl.h
 * @brief		implement interface IControl of WebControl
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.18	v1.0		Created
 */
#ifndef __WebControl_Defined
#define __WebControl_Defined

#include "IControl.h"
#include "WebControlConfig.h"
#include "WebFrame.h"
#include "WebPreferences.h"
#include "IDecisionListener.h"

#include "Page.h"
#include "Frame.h"
#include "PlatformMouseEvent.h"

#include "ShpJasmine.h"


////////////////////////////////////////////////
//// [todo] check if this is required
//extern "C" 
//{
//	typedef struct _StateChangeInfo{
//		int		m_stateChange;
//		BOOL					m_prevHttpState;
//		BOOL					m_prevHttpInclusionState;
//	}StateChangeInfo;
//}
/////////////////////////////////////////////////

namespace	WebControl
{

class WebContentsSender;
#if defined(_SFB_BACKINGSTORE)
class WebBackingStore;
#endif

class WebControl : public IControl, public ISettingListener
{
public:

	virtual HRESULT setDecisionListener( IDecisionListener *in_pIDecisionListener );
	virtual HRESULT setDownloadListener( URILoader::IStreamListener *in_pIStreamListener );


	/*
	 * @desc registers with Content's listner to a Browser Engine.
	 * @param in_pIContentsListener : pointer to contents listeners
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT setContentListener( IContentsListener* in_pIContentsListener );
	virtual HRESULT getContentListener( IContentsListener** out_pIContentsListener );

	/*
	 * @desc registers with IME's listner to a Browser Engine.
	 * @param in_pIIMEListener : pointer to a IME listeners
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT setIMEListener( IIMEListener* in_pIIMEListener ) ;

	/*
	 * @desc registers with Window's listner to a Browser Engine.
	 * @param in_pIWindowListner : pointer to a Window listeners
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT setWindowListener( IWindowListener* in_pIWindowListener );

	/*
	 * @desc registers with PDPContext's listner to a Browser Engine.
	 * @param in_pIPDPContextListener : pointer to a PDPContext listeners
	 * @return HRESULT TRUE/FALSE
	 */
	//virtual HRESULT setPDPContextListner( IPDPContextListener* in_pIPDPContextListener ) = 0;

	/*
	 * @desc suspends a Browser Engine
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT suspend( void );

	/*
	 * @desc resumes a Browser Engine
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT resume( void );

	/*
	 * @desc aborts page request and page actions of a Browser Engine
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT abort( void );

	/*
	 * @desc creates a thumbnail image for displayed page currently.
	 * @param
	 * @return IImage pointer to IImage
	 */
	virtual HRESULT createPageThumbnail( IImage** out_image );

	/*
	 * @desc gets an image informatioin on focused.
	 * @param out_pImage : pointer to IImage
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getCurrentImageInfo( IImage** out_pImage );

	/*
	 * @desc gets an image informatioin with position.
	 * @param out_pImage : pointer to IMAGEINFO
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getCurrentImageInfoByPoint( IImage** out_pImage, int x, int y );

	/*
	 * @desc gets a current page informatioin.
	 * @param out_pImageInfo : pointer to IPageInfo
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getCurrentPageInfo( IPageInfo** out_pPageInfo );

	/*
	 * @desc checks to exist a displayed contents currently.
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT isExistDisplayedContents() ;

	/*
	 * @desc gets state whether plugin is enabled or not with given mime type.
	 * @param in_PlugInType : plugin types
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getPlugInEnable( char* in_pszPlugInMime ); 

	/*
	 * @desc gets a position of virtual pointer
	 * @param out_x, out_y : x, y
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getVirtualPointerCurrentPos( int* out_x, int* out_y );

	/*
	 * @desc loads URL with POST method
	 * @param in_pszUrl : posted url
	 * @param in_pszPostData : posted data
	 * @param in_PostDataLen : length of the posted data
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT postURL( char* in_pszUrl, char* in_pszPostData, int in_PostDataLen );

	/*
	 * @desc loads URL
	 * @param In_pszUrl : URL
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT loadURL(const char* in_pszUrl );

	/*
	 * @desc gets current document URL
	 * @param none
	 * @return const char* docUrl
	 */
	virtual const char* getCurrentDocURL();

	/*
	 * @desc check mode that a browser engine is image save mode or not
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT isImageSaveMode( void );

	/*
	 * @desc check mode that a browser engine is page pilot mode or not
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	//#ifdef _SFB_PAGE_PREVIEW

	virtual HRESULT isMapViewMode( void );

	//#endif

	/*
	 * @desc check image is on the point or not
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT isPointOnImage( int x, int y );

	/*
	 * @desc check mode that a browser engine is search text mode or not
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT isSearchTextMode( void );

	/*
	 * @desc check mode that a browser engine is virtual pointer mode or not
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT isVirtualPointerMode( void );

	/*
	 * @desc passes key pressed event to browser engine
	 * @param in_KeyValue : key value is based on platform
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT KeyPressed( int in_KeyValue );

	/*
	 * @desc passes key released event to browser engine
	 * @param in_KeyValue : key value is based on platform
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT KeyReleased( int in_KeyValue );

	/*
	 * @desc loads contents with contents data
	 * @param in_pszUrl : URL
	 * @param in_pszData : contents
	 * @param in_DataLen : length
	 * @param in_pszMimeType : mimetype of contents
	 * @param in_pszCharset : charset of contents
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT loadData( char* in_pszUrl, char* in_pszData, int in_DataLen, char* in_pszMimeType, char* in_pszCharset );

	/*
	 * @desc loads contents with file name
	 * @param in_pszPathFileName : File name included path. ( without filescheme )
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT loadFile( char* in_pszPathFileName );

	/*
	 * @desc loads contents with file name that is saved by a browser engine and is located in saved page directory.
	 * @param in_pszPathFileName : File Name included path
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT loadSavedPage( char* in_pszPathFileName );

	/*
	 * @desc moves focus to next text
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT nextSearchText( char* in_pszSearchText );

	/*
	 * @desc gets a title of current page
	 * @param out_pszTitle : pointer to title
	 * @param in_TitleLen : length of title
	 * @return HRESULT TRUE/FALSE
	 */
	// removed
	//virtual HRESULT getPageInfoTitle( char* out_pszTitle, int in_TitleLen );

	/*
	 * @desc gets a URL of current page
	 * @param out_pszUrl : pointer to URL
	 * @param in_UrlLen : length of URL
	 * @return HRESULT TRUE/FALSE
	 */
	// removed
	//virtual HRESULT getPageInfoURL( char* out_pszUrl, int in_UrlLen );

	/*
	 * @desc scrolls by given x, y
	 * @param x,y : 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT scrollPane( int dx, int dy );



   //  2008.08.05  bs0425.lee modified for flick scroll. 
	/*
	 * @desc flickScrolls by given x, y
	 * @param x,y : 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT flickScrollPane( int dx, int dy );

       //  2008.08.05  bs0425.lee modified for long press scroll. 
       /*
	 * @desc zoom by given x, y
	 * @param x,y : 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT PressZoomMove(int zoomX, int zoomY );

	/*
	 * @desc moves focus to previous text
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT prevSearchText( char* in_pszSearchText ) ;

	/*
	 * @desc reloads current contents
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT reloadPage( void );

	/*
	 * @desc saves a current contents to given path
	 * @param in_pszPath : path to indicate for saved page.
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT saveCurrentPage( char* in_pszPath );

	/*
	 * @desc sets page rendering mode to browser engine
	 * @param in_nRenderingMode : page rendering mode
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT setPageRenderingMode( int in_nRenderingMode );

		/*
	 * @desc get page rendering mode to browser engine
	 * @param in_nRenderingMode : page rendering mode
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getPageRenderingMode( int *out_nRenderingMode );

	/*
	 * @desc sets virtual pointer mode to browser engine
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT setVirtualPointerMode( BOOL in_bOnOff );

	/*
	 * @desc sets save image mode to browser engine
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT setImageSaveMode( BOOL in_bOnOff );

	/*
	 * @desc sets page Map View mode to browser engine
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT isSearchMode( BOOL bMode );
#if defined(JS_MULTIPLE_WINDOW_SUPPORT)/* Anjana - UI changes for JS New Window */
	virtual HRESULT setSecureMode( BOOL bMode );

	virtual BOOL getSecureMode(void);
#endif
	/*
	 * @desc sets one click mode to browser engine
	 * @param 
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT setOneClickMode( BOOL bMode );

	BOOL isOneClickMode();

	//#ifdef _SFB_PAGE_PREVIEW
	virtual HRESULT setMapViewMode( BOOL in_bOnOff );
	virtual bool pagePilotScroll( int pos_X, int pos_Y);
	//#endif

#ifdef SFB_DRAG_DROP
	//hema
	virtual HRESULT getCurrentDragElementRect(int*height, int*width,int* actualX,int* actualY, int x, int y,  int* ctElement, int* elementType );
	virtual HRESULT getCurrentDragElementInfo(void* ctElement, char* srcUrl, char* hrefUrl,char** textString );
#endif

	/*
	 * @desc sets zoom value of contents to browser engine
	 * @param in_Zoom : Zoom Value ( 0 ~ 100 : 50 means 100% , 100 means 200% )
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT setZoom( unsigned int in_Zoom ) ;


	/*
	 * @desc 
	 * @param in_Zoom : Zoom Value ( 0 ~ 100 : 50 means 100% , 100 means 200% )
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT zooming( unsigned int in_Zoom );

	/*
	 * @desc gets zoom value of contents to browser engine
	 * @param in_Zoom : Zoom Value ( 0 ~ 100 : 50 means 100% , 100 means 200% )
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getZoom( unsigned int *out_Zoom );

	/*
	 * @desc passes click event by touch to browser engine
	 * @param x, y  : position
	 * @return HRESULT TRUE/FALSE
	 */
	//virtual HRESULT TouchClick( int x, int y );

	/*
	 * @desc passes focus move event by touch to browser engine
	 * @param x, y  : position
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT TouchFocusMove( int x, int y );

	/*
	 * @desc passes touch pressed event to browser engine
	 * @param x, y  : position
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT TouchPressed( int x, int y ) ;

	/*
	 * @desc passes touch released event to browser engine
	 * @param x, y  : position
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT TouchReleased( int x, int y, bool bHandleDoubleTab = false );

	/*
	 * @desc passes touch longPressed event to browser engine
	 * @param x, y  : position
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT TouchLongPressed( int x, int y );
	
	/*
	 * @desc passes Draw Zoom Bitmap event to browser engine
	 * @param x, y  : position
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT DrawZoomBitmap( int hGC, int x, int y );


	/*
	 * @desc gets total count of images in current contents
	 * @param 
	 * @return int Count of images
	 */
	virtual HRESULT getCountImages( int *out_Count );

	virtual void getLastViewedImageUrl( char * );
	virtual HRESULT getImagesInfo(bool bVisibleOnly ,ImageSaveInfo*** imageList);

	/*
	 * @desc searches text on current contents and if the text is found then enter the search mode
	 * @param in_pszSearchText : pointer to text to search
	 * @param out_pCount : pointer to the searched text if is found.
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT beginSearchText( char* in_pszSearchText, int* out_pCount );

	/*
	 * @desc finishes the search mode
	 * @param
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT endSearchText( void ) ;

	/*
	 * @desc draws Browser Engin's screen
	 * @param in_x, in_y, in_w, in_h : x, y, w, h
	 * @return
	 */
	virtual HRESULT drawScreen( int in_x, int in_y, int in_w, int in_h ) ;

	/*
	 * @desc changes Browser screen's area
	 * @param w, h : weight, height
	 * @return
	 */
	virtual HRESULT changeScreenSize( int w, int h ) ;

	/*
	 * Signifies the end of page load
	*/
	//bool IsLoadingDone();

	/*
	 * @desc rotates Browser screen's area
	 * @param in_x, in_y, in_w, in_h : x, y, w, h
	 * @return
	 */
	virtual HRESULT rotateScreen( int in_x, int in_y, int in_w, int in_h );

	/*
	 * @desc shows Browser Engin's screen
	 * @param in_x, in_y, in_w, in_h : x, y, w, h
	 * @return
	 */
	virtual HRESULT showScreen( int in_x, int in_y, int in_w, int in_h );

	/*
	* @desc copy the content of screen to destination gc
	*
	*/
	virtual HRESULT copyScreen( int destGC, int destx, int desty, int srcx, int srcy, int srcw, int srch );

	/*
	* @desc get the graphic context which WebControl is using
	*/
	virtual HRESULT getGC( int* out_gc );

	/*
	 * @desc updates page without reload.
	 * @param
	 * @return
	 */
	virtual HRESULT updatePage( void );

	/*
	 * @desc checks whether the focus is assigned on the point or not.
	 * @param x, y : point
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT	isFocusedByPoint( int x, int y );

	/*
	 * @desc checks Divide this function with UI API. whether the focus is assigned on the point or not.
	 * @param x, y : point
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT	linkFocusedByPoint( int x, int y );

	/*
	 * @desc checks whether the focus is possible on the point or not.
	 * @param x, y : point
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT isFocusableByPoint( int x, int y );

	/*
	 * @desc checks whether the anchor tag is on the point or not.
	 * @param x, y : point
	 * @return HRESULT TRUE/FALSE
	 */
	// removed
	//virtual HRESULT isAnchorByPoint( int x, int y );

	/*
	 * @desc gets DOM Node with point.
	 * @param out_pIPropertyMap : element property map
	 * @param in_x : x
	 * @param in_y : y
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getElementByPoint( IPropertyMap** out_pIPropertyMap, int x, int y );

	/*
	 * @desc gets DOM Node with focus
	 * @param out_pIPropertyMap : element property map
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getElementByFocus( IPropertyMap** out_pIPropertyMap );


	/*
	 * @desc sets account for proxy
	 * @param in_Account : account handle of Mocha
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT setAccessPoint( int in_Account );

	/*
	 * @desc sets a request header property of Browser Engine.
	 * @param in_pszHttpHeader : pointer to the HttpHeader
	 * @param in_pszHeader : pointer to the HttpHeader Value
	 * @return HRESULT TRUE / FALSE
	 */
	//virtual HRESULT setHttpHeaderProperty( char* in_pszHttpHeader, char* in_pszHeaderBody );

	/*
	 * @desc goes to forward or backward history
	 * @param 
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT goBackwardHistory( void ) ;

#ifdef SFB_UPDATE_URL_BAR  //Pramod - Bresson Model - update Url ComboBox bar
	/* 
	 * @desc gets the current processing url
	 * @param  : None
	 * @return : char* - Url string
	 */
	virtual void getCurrentUrlFromHistoryList( char* szUrl ); //Pramod
	virtual void getUrlFromHistoryList( int nDirect, char* szUrl );/* Fix:CQ 2813:Anjana, check for forward/backward url available */
#endif

	/*
	 * @desc goes to forward or backward history
	 * @param 
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT goForwardHistory( void );

	/**
	* @desc check if gobackward is possible
	* @return HRESULT TRUE / FALSE
	*/
	virtual HRESULT canGoBackwardHistory();

	/**
	* @desc check if goforward is possible
	* @return HRESULT TRUE / FALSE
	*/
	virtual HRESULT canGoForwardHistory();

	/*
	 * @desc clears history
	 * @param 
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT clearHistory( void );

	/*
	 * @desc clears cache
	 * @param 
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT clearCache( void );

	/*
	 * @desc clears cache
	 * @param 
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT InitCache( void );
	/*
	 * @desc emptiess cookie
	 * @param 
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT emptyCookie( void ) ;

	/*
	 * @desc gets IWMLDo
	 * @param 
	 * @return pointer to the IWMLDo
	 */
	// removed
	//virtual HRESULT getIWMLDo( IWMLDo** out_WMLDo ) ;

	/*
	 * @desc gets ICertificate
	 * @param 
	 * @return pointer to the ICertificate
	 */
	virtual HRESULT getICertificate( ICertificate** out_Certificate ) ;

	/*
	 * @desc gets ISetting
	 * If clients does not set any specific ISetting, the default ISetting will be returned.
	 * The default ISetting is shared by all IControl instance.
	 * @param 
	 * @return pointer to the ISetting
	 */
	virtual  HRESULT getISetting( ISetting** out_Setting );

	/*
	 * @desc sets ISetting
	 * @param 
	 * @return pointer to the ISetting
	 */
	virtual  HRESULT setISetting( ISetting* in_Setting );

	/*
	 *@param drawsBackround YES to cause the receiver to draw a default white background, NO otherwise.
	*@desc Sets whether the receiver draws a default white background when the loaded page has no background specified.
	*/
	virtual HRESULT setDrawBackground( BOOL in_bDraw );

	/**
	* called when the any values of pSettins is changed
	* @param pSetting the setting whose property is changed
	* @param in_PropertyID the property id whose value is changed
	*/
	virtual void OnSettingChanged( ISetting *pSetting, int in_PropertyID );

	// bs0425.lee 2008.04.29. modified for Page Preview.

	//#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
	//virtual void pagePreviewKeyEvent(int keyCode,  bool isKeyUp);
	virtual void pagePreviewCancel(BOOL bFlag);
	virtual HRESULT WebControl::pagePreviewscrollPane(bool scroll_event, int dx, int dy );
	//#endif

	virtual void updateFocusedAndActiveState();
	virtual void updateActiveState();

	virtual HRESULT getWmlDoCount( int *out_count ) ;
	virtual HRESULT getWmlDoName( int in_index, char *inout_buffer, int in_bufsize );
	virtual HRESULT executeWmlDo( int in_index );
    
    virtual void AddURLToVisitedLinks(char* szURL);

	/**
	 * @desc set the selected file path
	 * @param      in_szFilePath  [in] The file path
	 * @param      in_iPathLen  [in] The path length
	 */	
	virtual HRESULT setSelectedFilePath(const char* in_szFilePath, int in_iPathLen);

	virtual HRESULT getContentsSize( int *width, int *heigth);

	virtual void setSmartTouchEnabled(bool enabled);

	virtual  void setSmartTouchFocusRange(int range);

	virtual	URILoader::ILoadGroup* getLoadGroup(void);

	virtual	void setLoadGroup(URILoader::ILoadGroup* clinetID);

	virtual void setLcdDimmingMode(bool bIsDimming) ;

	/*
	* @desc Specifiy maximum zoom factor
	*/
	virtual HRESULT setMaxZoomFactor( float factor );
	/*
	* @desc Specifiy minimum zoom factor
	*/
	virtual HRESULT setMinZoomFactor( float factor );


	/*
	* @desc minimize the size of backingstore
	* backingstore means the memory gc used by WebControl
	* WebControl use bigger memmory gc than actual window size, 
	* So If you want to save memory when the webcontrol is hided or unfocused, 
	* use this funtion to shrink the size of memorygc same as window size .
	* The actual minimum backingstore size is calculated by WebContorl implementation.
	*/
	virtual HRESULT minimizeBackingStore();

	/*
	* @desc maximize the size of backingstore
	* Use this funtion to restore the size of backingstore after calling minimizeBackingStore().
	* The actual maximum backingstore size is calculated by WebControl implementation.
	*/
	virtual HRESULT maximizeBackingStore();

	virtual HRESULT setInitialOverviewMode( bool bEnable );

	virtual void stopLingerTimerAfterAjaxRequest();
	virtual void cancelLingerTimer();//Oct 21 Anshul
	virtual void InstanceExit();// this will be called only when SFB_IMPROVE_END_KEY flag is enable in BrConfig.h
	virtual void setEndKeyPressed(bool status);
public:
	static HRESULT New( char* szName, HWIN hParentWin, int x, int y, int w, int h, WebControl **ppWebControl, GCType in_gcType);
	~WebControl();
	WebCore::Page*	GetPage() { return m_pPage; }
	HWIN GetHWIN() { return m_hWin; }
	HWIN GetHGC() { return m_hGC; }

	StateChangeInfo GetStateChangeInfo() { return m_stateChangeInfo; }
	void SetStateChangeInfo(StateChangeInfo pStateInfo) { m_stateChangeInfo = pStateInfo; }
	void*	GetLoadGroup() { return m_pLoadGroup; }
	void SetIsLoadingDone( bool v ) { m_isLoadingDone = v; }	
	IContentsListener* GetContentListener() { if( m_bExitInstance == true ) return NULL; else return m_pContentListener; }
	WebFrame*	GetMainWebFrame(){ return m_pMainWebFrame; }
	IDecisionListener* GetDecisionListener() { if( m_bExitInstance == true ) return NULL; else return m_pDecisionListener; }
	URILoader::IStreamListener* GetDownloadListener() { return m_pDownloadListener; }
	char*				m_openerWindowUrl; //for BUG 1738 Gnana
	void HandleOutOfMemory();
    void onOOMEvent();
#if defined(_SFB_BACKINGSTORE)
	WebBackingStore* GetBackingStore() { return m_pBackingStore; }
#endif
	int GetWidth() { return m_w; }
	int GetHeight() { return m_h; }
	int GetGCType() { return m_nGCType; }

	void closeWindowSoon();


//#ifdef _SFB_PAGE_PREVIEW - Pramod 23/July/08
	//To launch Page pilot during page loading
	HRESULT getPagePilotFlagFirstRendering() { return m_SetToPagePilotAfterFirstRendering; }
	void setPagePilotFlagFirstRendering( bool bFlag ) { m_SetToPagePilotAfterFirstRendering = bFlag; }

	// To reset the page pilot mode
	HRESULT getResetPagePilotFlag() { return m_ReSetPagePilot; }
	void setResetPagePilotFlag( bool bResetFlag ) { m_ReSetPagePilot = bResetFlag; }
//#endif
	bool isZooming(){ return bEnableZoom; }
	//WebCore::String webkit_shp_validate_filename(WebCore::String orgPath,WebCore::String filesPath);

	void touchReleasedTimerFired();//OS timer for zoom

	virtual void setJSConfirmResult(bool bResult) ;
	virtual bool getJSConfirmResult(void) ;

	virtual void setJSPromptResultString(const char *inStrData) ;
	virtual WebCore::String& getJSPromptResultString(void) ;
	
	//for clients of webControl who cant override the default preferences eg widget. default is false ie can override.
	virtual void setoverrideDefaultPreferences(bool set) ;
	
public:
	// Handle for RM callback
	// These APIs will be changed in future if URILoader is used. [todo]
	void ReceivedAuthenticationChallenge(  SFBRmHClient hClient,
											SFBRmReqId reqID,
											int transID,
											char* szProposedUser,
											char* szProposedPass,
											char* szRealm,
											char* pszrl,
											SFBRmAuthenticationCallbackResponse fpRespCallback );
	
	void ReceivedCookieAcceptMethod ( SFBRmHClient hClient,
											SFBRmReqId reqID,
											char* szHostName,
											BOOL bCookieExist,
											char* szCookieString,
											void* pCookie,
											SFBRmCookieCallbackResponse fpCallbackResponse);

	void ReceivedPostStatus(SFBRmHClient hClient,
											SFBRmReqId reqID,
											//int nLenPostedData,
											int nLenTotalPostedData,
											int nLenTotalPostData
											);
	void DisplayUnsupportedFileError();

	void ReceivedPostInit();

	HRESULT ReceivedRedirectionChallenge(  SFBRmHClient hClient,
											SFBRmReqId reqID,
											int transID,
											SFBRmRequestInfo* pRedirectedReq,
											char* pResHeaders,
											int nLenResHeaders,
											SFBRmRedirectCallbackResponse fpCallbackResponse );


	void GetSSLConfirmation(SFBRmHClient hClient,
							SFBRmReqId reqID,
							int transID,
							char* pCertData,
							SFBalRmSSLCallbackResponse fpRespCallback );

	void ReceivedHandlePDPevent(SFBRmHClient hClient, SFBRmPDPContextEventParams *pParams, SFBRmPDPContextCallbackResponse fpCallbackResponse);

	void closeWindowTimerFired();
	bool isSmartTouchEnabled();
	int	                                m_docSize; 

	void setWebControlCloseWindowFired(bool fireCloseWindow) { m_WebControlCloseFired = fireCloseWindow ;}
	bool isWebControlCloseWindowFired() { return m_WebControlCloseFired ;}
	void setExitInstanceStatus(bool status) { m_bExitInstance = status ;}
	bool isExitInstanceCalled() { return (m_bExitInstance || m_bEndKeyPressed) ;}
	//CQ 3977 : Getting Webcontrol Instance Name
	WebCore::String GetInstanceName(){return WebCore::String(m_strName.data()) ; }
# ifdef  SFB_WIDGET
	bool isExposeEventPresent(){return m_ExposeEventPresent; }
	void setExposeEvent(bool bExposeEvent){ m_ExposeEventPresent = bExposeEvent;}
#endif
private:
	WebControl();	
	HRESULT Construct( char* szName, HWIN hParentWin, int x, int y, int w, int h, GCType in_gcType );
	HRESULT Destruct();
	
	HRESULT SetHWIN( HWIN hWin, int x, int y, int w, int h );
	// This API is called when a HIDE event comes to the Simple Window
	HRESULT BrowsingWindowHide( );
	HRESULT StopLoading();
	HRESULT ChangeSetting( ISetting* pSettings );
	HRESULT HandleKeyEvent( int keyCode,  int isKeyUp);
	HRESULT HandleMouseEvent( int recX, int recY, WebCore::MouseEventType in_MouseEventType);

	//HRESULT __TouchClick( int x, int y );
	HRESULT __TouchReleased( int x, int y );

	//void closeWindowTimerFired(WebCore::Timer<WebControl>*);
	//void closeWindowTimerFired();
	void closeWindow();

    // Init and DeInit Static and Globals
    void InitGlobals();
    void DeInitGlobals();

	String								m_strName;
	WebCore::Page*				m_pPage;
	WebFrame*						m_pMainWebFrame;
	HWIN								m_hWin;
	int									m_x, m_y, m_w, m_h;
	HGC									m_hGC;
	int									min_X,min_Y,max_X,max_Y;
	int									m_nGCType;
	//2008.07.29. modified for touch scrollbar. 
	WebCore::IntPoint 					scrollBarPoint;
	bool 					scrollbarPressed;
	bool 					innerTrack;
	// 2008.08.09 bs0425.lee modified for touch press zoom. 
	BOOL								bEnableZoom;
	float								totalZoomY;
	float								zoomValue;
	int									zoomPointX;
	int									zoomPointY;
	WmBitmap*					pBmpWidget;
	WmBitmap*					pBmpWidget1;
	WmBitmap*					pBmpWidget2;
	BOOL						m_bOneClickMode;
	BOOL							m_bFocused;
	BOOL								m_bExitInstance;//this stores the status of instance exit.
	BOOL						m_bEndKeyPressed; //this is updated when end key is pressed 
#if defined(JS_MULTIPLE_WINDOW_SUPPORT)/* Anjana - UI changes for JS New Window */
	BOOL						m_bSecureMode;
#endif
#ifdef SFB_PROGRESSIVE_RENDERING
	WmBitmap*						m_pbmPlaceHolder;
#endif
// bs0425.lee modified for Page Preview.
//#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
		int m_pagePreview_X, m_pagePreview_Y;
	int m_pagepreview_H, m_pagepreview_W;
		BOOL m_Preview;

		bool m_SetToPagePilotAfterFirstRendering; // To launch Page pilot during page loading - Pramod 22/July/'08
		bool m_ReSetPagePilot; // Added to reset page pilot next page loading
//#endif	

	void*									m_pLoadGroup; // [todo] chage it to ILoadGroup* after URILoader is done
	WebPreferences*				m_pWebPreferences;
	bool									m_isLoadingDone;
	IContentsListener*			m_pContentListener;
	int									m_RenderingMode;
	IWindowListener*				m_pWindowListener;
	StateChangeInfo				m_stateChangeInfo;
	IDecisionListener*				m_pDecisionListener;
	URILoader::IStreamListener* m_pDownloadListener;
	char* m_initialUrl ;
#if defined(_SFB_BACKINGSTORE)
	WebBackingStore*				m_pBackingStore;
#endif
	
	ULONG m_prevReleasedTick;
	WebCore::IntPoint m_touchReleasedTimerPoint;
	HTimer hTouchReleasedTimer;//OS timer for zoom
	//bool m_bTouchReleasedTimerIsClick;

    WebCore::Timer<WebControl> *m_pCloseWindowTimer;

	WebCore::String jsPromptResult ;
	WebCore::Node* m_lastFocusedNode;
	bool jsConfirmResult ;
	bool m_WebControlCloseFired ;
# ifdef  SFB_WIDGET
	bool m_ExposeEventPresent;
# endif
	//for clients of webControl who cant override the default preferences eg widget. default is false ie can override.
	bool m_cantOverrideDefaultPreferences ;
	
};


WebCore::Frame* core( WebFrame* pWebFrame );
WebFrame* kit( WebCore::Frame* pFrame );
WebCore::Page* core( WebControl* pWebControl );
WebControl* kit(WebCore::Page* pPage );


/**
 * @desc creates factory function to create IControl
 * @param in_pszDomainName : pointer to Domain ( it means mocha application ) name
 * @param in_ppIControl 
 * @return HRESULT TRUE / FALSE
*/
HRESULT CreateIControl( char* in_pszDomainName, 
					   HWIN hWin, 
					   int x, int y, int w, int h, 
					   IControl** in_ppIControl, GCType in_gcType);


URILoader::ILoadGroup*	CreateClientLoadGroup(char* appName);

void	DestroyClientLoadGroup(URILoader::ILoadGroup* clinetID);

int GetWebControlCount();

#ifdef MULTIPLE_INSTANCE_SUPPORT
//Added to initialise the WebControlInstance structure - Pramod
void InitWebControlInstance();
#endif

} // namespace  WebControl

#endif // __IControl_Defined