/**
 * Samsung Handset Platform
 * Copyright (c) 2000 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics.
 */

/**
 *  This file is about ECOM service.
 * 
 * @name        WebControlECOM.cpp
 * @author      Zafar Hussain ( zafar.h@samsung.com )
 */
#if defined(_JASMINE_ECOM)

//#include "WebControl.h"
#include "IControl.h"
//vprajan: libxml2 is statically linked with Engine
extern "C" BOOL LibxmlDispatch(UINT32 action, ULONG param1, ULONG param2, void* pParam3);

static UINT32	__WebControlAddRef(void);
static UINT32	__WebControlRelease(void);
BOOL WebControlDispatch(UINT32 , ULONG , ULONG , void* );


TWebControl	gWebControl;
static EWebControl WebControlVtbl;
static TWebControl* pSelf = &gWebControl;
extern void browser_vtable_init();

/*
#if defined(_SHP_BROWSER_SFB)
#if defined	(_SHP_SIMUL)
BOOL		(*pSFBalInitialize)(void);
#endif
#endif
*/

/*
 *  __WebControlConstruct .
 * 
 * @return      if succeed, return TRUE or FALSE.
 * @exception   In C, error code. In C++, exception to throw.
 * @see         related information ( function name, module name, etc )
 */
BOOL
__WebControlConstruct(void)
{
	/*
	 * Hook ECOM interfaces and corresponding functions
	 */			
	 #if !defined(_SHP_SIMUL)
	 browser_vtable_init();
	 #endif
	WebControlVtbl.AddRef					= __WebControlAddRef;
	WebControlVtbl.Release					= __WebControlRelease;
	WebControlVtbl.Dispatch					= WebControlDispatch;
	
	WebControlVtbl.WCCreateIControl			= WebControl::CreateIControl;
	WebControlVtbl.GNULibxmlDispatch	    = LibxmlDispatch;
	WebControlVtbl.WCCreateClientLoadGroup			= WebControl::CreateClientLoadGroup;
	WebControlVtbl.WCDestroyClientLoadGroup			= WebControl::DestroyClientLoadGroup;
	WebControlVtbl.WCGetWebControlCount     = WebControl::GetWebControlCount;

	
	INIT_ECOM_VTBL(&gWebControl, EWebControl, WebControlVtbl);   // Initialize the VTBL
//SysDebug((MID_DOC, "[Picsel] Sr=%d, min=%d, sec=%d, yearDay=%d, weekDay=%d \n",dateTime.year, dateTime.month,));
	SysDebug((MID_EXCEPTION, "__WebControlConstruct called. Value assigned to pDllBaseEcomFactory=%d",pDllBaseEcomFactory));
	return SysRegisterClass(NULL, CID_WEBCONTROL, &gWebControl);
}

/*
 *  __WebControlDestruct is destruct.
 * 
 * @return      if succeed, return TRUE or FALSE.
 * @exception   In C, error code. In C++, exception to throw.
 * @see         related information ( function name, module name, etc )
 */
BOOL
__WebControlDestruct(void)
{
	SysDebug((MID_BRSVC, "__WebControlDestruct Called\n"));

	return SysUnregisterClass(NULL, CID_WEBCONTROL);
}

/*
 *  These functions implement EBase interfaces
 * 
 * @return      if succeed, return count.
 * @exception   In C, error code. In C++, exception to throw.
 * @see         related information ( function name, module name, etc )
 */
static UINT32
__WebControlAddRef(void)
{
	return ++pSelf->refCount;
}

/*
 *  This function decrement EBase interface.
 * 
 * @return      count
 * @exception   In C, error code. In C++, exception to throw.
 * @see         related information ( function name, module name, etc )
 */
static UINT32
__WebControlRelease(void)
{
	if (--pSelf->refCount == 0)
	{
	}

	return pSelf->refCount;
}

/**
 *  This function dispatches WebControl services.
 * 
 * @param      action  [in] Specifies an action type to dispatch
 * @param      param1  [in] Specifies an action type to dispatch
 * @param      param2  [in] Specifies an action type to dispatch
 * @param      pParam3  [in] Specifies an action type to dispatch
 * @return	    if succeed, return TRUE or FALSE.
 * @exception  In C, error code. In C++, exception to throw.
 * @see        related information ( function name, module name, etc )
 */
BOOL
WebControlDispatch(UINT32 action, ULONG param1, ULONG param2, void* pParam3)
{
	BOOL	bReturn = FALSE;
	
#if defined	(_SHP_SIMUL)

	HINSTANCE	hInst;

	hInst = LoadLibrary( "WebControl.dll" );

	if ( !hInst )
		return FALSE;

	//pSFBalInitialize	= (BOOL (*))GetProcAddress(hInst, "SFBalInitialize");	
	
#endif

	switch (action)
	{
		case DLL_CREATE:
			bReturn = __WebControlConstruct();
			return bReturn;


		case DLL_DELETE:
			bReturn = __WebControlDestruct();
			return bReturn;
	}
	return FALSE;
}

#endif //_JASMINE_ECOM
/* @} */
