/*
 * Copyright (C) 2006 Nikolas Zimmermann <zimmermann@kde.org> 
 * Copyright (C) 2006 Zack Rusin <zack@kde.org>
 * Copyright (C) 2006 Apple Computer, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
//#include "config.h"

#include "WebEditorClient.h"
#include "WebControl.h"


#include "PlatformKeyboardEvent.h"
#include "EditCommand.h"
#include "Editor.h"
#include "FocusController.h"
#include "Frame.h"
#include "KeyboardCodes.h"
#include "KeyboardEvent.h"
#include "Page.h"

using namespace WebCore;


namespace WebControl {

//using WebCore::String;
WebEditorClient::WebEditorClient( WebControl *pWebControl )
{
	m_pWebControl = pWebControl;
}

bool WebEditorClient::shouldDeleteRange(Range*)
{
    __NotImplemented();
    return true;
}

bool WebEditorClient::shouldShowDeleteInterface(HTMLElement*)
{
    return false;
}

bool WebEditorClient::isContinuousSpellCheckingEnabled()
{
    __NotImplemented();
    return false;
}

bool WebEditorClient::isGrammarCheckingEnabled()
{
    __NotImplemented();
    return false;
}

int WebEditorClient::spellCheckerDocumentTag()
{
    __NotImplemented();
    return 0;
}

bool WebEditorClient::shouldBeginEditing(WebCore::Range*)
{
    __NotImplemented();
    return true;
}

bool WebEditorClient::shouldEndEditing(WebCore::Range*)
{
    __NotImplemented();
    return true;
}

bool WebEditorClient::shouldInsertText(WebCore::String, Range*, EditorInsertAction)
{
    __NotImplemented();
    return true;
}

bool WebEditorClient::shouldChangeSelectedRange(Range*, Range*, EAffinity, bool)
{
    __NotImplemented();
    return true;
}

bool WebEditorClient::shouldApplyStyle(WebCore::CSSStyleDeclaration*,
                                      WebCore::Range*)
{
    __NotImplemented();
    return true;
}

bool WebEditorClient::shouldMoveRangeAfterDelete(WebCore::Range*, WebCore::Range*)
{
    __NotImplemented();
    return true;
}

void WebEditorClient::didBeginEditing()
{
    __NotImplemented();
}

void WebEditorClient::respondToChangedContents()
{
    __NotImplemented();
}

void WebEditorClient::respondToChangedSelection()
{
    __NotImplemented();
}

void WebEditorClient::didEndEditing()
{
    __NotImplemented();
}

void WebEditorClient::didWriteSelectionToPasteboard()
{
    __NotImplemented();
}

void WebEditorClient::didSetSelectionTypesForPasteboard()
{
    __NotImplemented();
}

bool WebEditorClient::isEditable()
{
    //__NotImplemented();
    return false;
}

void WebEditorClient::registerCommandForUndo(WTF::PassRefPtr<WebCore::EditCommand>)
{
    __NotImplemented();
}

void WebEditorClient::registerCommandForRedo(WTF::PassRefPtr<WebCore::EditCommand>)
{
    __NotImplemented();
}

void WebEditorClient::clearUndoRedoOperations()
{
    __NotImplemented();
}

bool WebEditorClient::canUndo() const
{
    __NotImplemented();
    return false;
}

bool WebEditorClient::canRedo() const
{
    __NotImplemented();
    return false;
}

void WebEditorClient::undo()
{
    __NotImplemented();
}

void WebEditorClient::redo()
{
    __NotImplemented();
}

bool WebEditorClient::shouldInsertNode(WebCore::Node*, Range*, EditorInsertAction)
{
    __NotImplemented();
    return true;
}

void WebEditorClient::pageDestroyed()
{
    delete this;
}

bool WebEditorClient::smartInsertDeleteEnabled()
{
    __NotImplemented();
    return false;
}

void WebEditorClient::toggleContinuousSpellChecking()
{
    __NotImplemented();
}

void WebEditorClient::toggleGrammarChecking()
{
}

void WebEditorClient::handleKeypress(KeyboardEvent* event)
{
    Frame* frame = core(m_pWebControl)->focusController()->focusedOrMainFrame();
    if (!frame)
        return;

    const PlatformKeyboardEvent* kevent = event->keyEvent();
    if (!kevent->isKeyUp()) {
        WebCore::Node* start = frame->selectionController()->start().node();
        // VDJ - FIXME not sure why isReadOnly() function is getting called instead of isContentEditable()
        if (start /*&& start->isContentEditable()*/) {
            switch (kevent->windowsVirtualKeyCode()) {
            case VK_BACK:
                frame->editor()->deleteWithDirection(SelectionController::BACKWARD,
                                                     CharacterGranularity, false, true);
                break;
            case VK_DELETE:
                frame->editor()->deleteWithDirection(SelectionController::FORWARD,
                                                     CharacterGranularity, false, true);
                break;
            case VK_LEFT:
                frame->editor()->command("MoveLeft").execute();
                break;
            case VK_RIGHT:
                frame->editor()->command("MoveRight").execute();
                break;
            case VK_UP:
                frame->editor()->command("MoveUp").execute();
                break;
            case VK_DOWN:
                frame->editor()->command("MoveDown").execute();
                break;
            default:
                frame->editor()->insertText(kevent->text(), event);
            }
            event->setDefaultHandled();
        }
    }

}


void WebEditorClient::handleInputMethodKeypress(KeyboardEvent*)
{
    __NotImplemented();
}

void WebEditorClient::textFieldDidBeginEditing(Element*)
{
    __NotImplemented();
}

void WebEditorClient::textFieldDidEndEditing(Element*)
{
    __NotImplemented();
}

void WebEditorClient::textDidChangeInTextField(Element*)
{
    __NotImplemented();
}

bool WebEditorClient::doTextFieldCommandFromEvent(Element*, KeyboardEvent*)
{
    __NotImplemented();
    return false;
}

void WebEditorClient::textWillBeDeletedInTextField(Element*)
{
    __NotImplemented();
}

void WebEditorClient::textDidChangeInTextArea(Element*)
{
    __NotImplemented();
}

void WebEditorClient::ignoreWordInSpellDocument(const WebCore::String&)
{
    __NotImplemented();
}

void WebEditorClient::learnWord(const WebCore::String&)
{
    __NotImplemented();
}

void WebEditorClient::checkSpellingOfString(const UChar*, int, int*, int*)
{
    __NotImplemented();
}

void WebEditorClient::checkGrammarOfString(const UChar*, int, Vector<GrammarDetail>&, int*, int*)
{
    __NotImplemented();
}

void WebEditorClient::updateSpellingUIWithGrammarString(const WebCore::String&, const GrammarDetail&)
{
    __NotImplemented();
}

void WebEditorClient::updateSpellingUIWithMisspelledWord(const WebCore::String&)
{
    __NotImplemented();
}

void WebEditorClient::showSpellingUI(bool)
{
    __NotImplemented();
}

bool WebEditorClient::spellingUIIsShowing()
{
    __NotImplemented();
    return false;
}

void WebEditorClient::getGuessesForWord(const WebCore::String&, Vector<WebCore::String>&)
{
    __NotImplemented();
}

void WebEditorClient::setInputMethodState(bool)
{
}


void WebEditorClient::handleKeyboardEvent(KeyboardEvent* event)
{
    Frame* frame = core(m_pWebControl)->focusController()->focusedOrMainFrame();
    if (!frame || !frame->document()->focusedNode())
        return;

    const PlatformKeyboardEvent* kevent = event->keyEvent();
    if (!kevent || kevent->type() == PlatformKeyboardEvent::KeyUp)
        return;

    WebCore::Node* start = frame->selectionController()->start().node();
    if (!start)
        return;

    // FIXME: Use GtkBindingSet instead of this hard-coded switch
    // http://bugs.webkit.org/show_bug.cgi?id=15911

    if (start->isContentEditable()) {
        switch (kevent->windowsVirtualKeyCode()) {
            case VK_BACK:
                frame->editor()->deleteWithDirection(SelectionController::BACKWARD,
                        kevent->ctrlKey() ? WordGranularity : CharacterGranularity, false, true);
                break;
            case VK_DELETE:
                frame->editor()->deleteWithDirection(SelectionController::FORWARD,
                        kevent->ctrlKey() ? WordGranularity : CharacterGranularity, false, true);
                break;
            case VK_LEFT:
                frame->selectionController()->modify(kevent->shiftKey() ? SelectionController::EXTEND : SelectionController::MOVE,
                        SelectionController::LEFT,
                        kevent->ctrlKey() ? WordGranularity : CharacterGranularity,
                        true);
                break;
            case VK_RIGHT:
                frame->selectionController()->modify(kevent->shiftKey() ? SelectionController::EXTEND : SelectionController::MOVE,
                        SelectionController::RIGHT,
                        kevent->ctrlKey() ? WordGranularity : CharacterGranularity,
                        true);
                break;
            case VK_UP:
                frame->selectionController()->modify(kevent->shiftKey() ? SelectionController::EXTEND : SelectionController::MOVE,
                        SelectionController::BACKWARD,
                        kevent->ctrlKey() ? ParagraphGranularity : LineGranularity,
                        true);
                break;
            case VK_DOWN:
                frame->selectionController()->modify(kevent->shiftKey() ? SelectionController::EXTEND : SelectionController::MOVE,
                        SelectionController::FORWARD,
                        kevent->ctrlKey() ? ParagraphGranularity : LineGranularity,
                        true);
                break;
            case VK_PRIOR:  // PageUp
                frame->editor()->command("MovePageUp").execute();
                break;
            case VK_NEXT:  // PageDown
                frame->editor()->command("MovePageDown").execute();
                break;
            case VK_HOME:
                if (kevent->ctrlKey() && kevent->shiftKey())
                    frame->editor()->command("MoveToBeginningOfDocumentAndModifySelection").execute();
                else if (kevent->ctrlKey())
                    frame->editor()->command("MoveToBeginningOfDocument").execute();
                else if (kevent->shiftKey())
                    frame->editor()->command("MoveToBeginningOfLineAndModifySelection").execute();
                else
                    frame->editor()->command("MoveToBeginningOfLine").execute();
                break;
            case VK_END:
                if (kevent->ctrlKey() && kevent->shiftKey())
                    frame->editor()->command("MoveToEndOfDocumentAndModifySelection").execute();
                else if (kevent->ctrlKey())
                    frame->editor()->command("MoveToEndOfDocument").execute();
                else if (kevent->shiftKey())
                    frame->editor()->command("MoveToEndOfLineAndModifySelection").execute();
                else
                    frame->editor()->command("MoveToEndOfLine").execute();
                break;
            case VK_RETURN:
                frame->editor()->command("InsertLineBreak").execute();
                break;
            case VK_TAB:
                return;
            default:
                if (!kevent->ctrlKey() && !kevent->altKey() && !kevent->text().isEmpty()) {
                    if (kevent->text().length() == 1) {
                        UChar ch = kevent->text()[0];
                        // Don't insert null or control characters as they can result in unexpected behaviour
                        if (ch < ' ')
                            break;
                    }
                    frame->editor()->insertText(kevent->text(), event);
                }else return;
        }
    } else {
        switch (kevent->windowsVirtualKeyCode()) {
            case VK_UP:
                frame->editor()->command("MoveUp").execute();
                break;
            case VK_DOWN:
                frame->editor()->command("MoveDown").execute();
                break;
            case VK_PRIOR:  // PageUp
                frame->editor()->command("MovePageUp").execute();
                break;
            case VK_NEXT:  // PageDown
                frame->editor()->command("MovePageDown").execute();
                break;
            case VK_HOME:
                if (kevent->ctrlKey())
                    frame->editor()->command("MoveToBeginningOfDocument").execute();
                break;
            case VK_END:
                if (kevent->ctrlKey())
                    frame->editor()->command("MoveToEndOfDocument").execute();
                break;
            default:
                return;
        }
    }
    event->setDefaultHandled();
}


void WebEditorClient::handleInputMethodKeydown(KeyboardEvent*)
{
    __NotImplemented();
}


}


