/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		WebDownlaod.h
 * @brief		this class redirect the data stream of WebControl to download engine
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.04.10	v1.0		Created
 */
#ifndef __WebDownlaod_Defined
#define __WebDownlaod_Defined

#include "WebDefs.h"
#include "WebControlConfig.h"


#include "IHttpChannel.h"
#include "WebKitLoader.h"


#include "ResourceResponse.h"
#include "ResourceRequest.h"

class IWebClientBridge;
namespace WebCore 
{
    class ResourceLoader;
}

namespace	WebControl
{

class WebControl;


class WebDownload : public URILoader::IHttpChannel 
{

private:
	WebCore::String m_headers;
	int m_statusCode;
	IWebClientBridge *m_bridge;
    WebCore::ResourceLoader* m_resLoader;
	URILoader::IURI *m_pURI; // id of sfbrm
	int m_transportHandle;
	int m_totalReceived;

public:
	WebDownload( WebControl *, const WebCore::ResourceRequest& request, const WebCore::ResourceResponse& response);
	~WebDownload();

public:
	// Redirect WebKitLoader's data stream to this class
	HRESULT RedirectStream( IWebClientBridge* bridge, WebCore::ResourceLoader* resLoader );

private:
	WebControl * m_pWebControl;
	

public:
	// IHttpChannel's
	virtual URILoader::Result GetName( URILoader::String *pValue ) ;
	virtual URILoader::Result GetStatus( URILoader::Result *pValue  );
	virtual URILoader::Result GetLoadFlags( URILoader::LoadFlags *pValue ) ;
	virtual URILoader::Result SetLoadFlags( URILoader::LoadFlags aValue );
	virtual URILoader::Result IsPending( BOOL *pIsPending );
	virtual URILoader::Result Cancel( URILoader::Result aStatus );
	virtual URILoader::Result Suspend();
	virtual URILoader::Result Pause();
	virtual URILoader::Result Resume();

	virtual URILoader::Result GetOriginalURI( URILoader::IURI **ppValue );
	virtual URILoader::Result SetOriginalURI( URILoader::IURI *pURI ) ;
	virtual URILoader::Result GetURI(URILoader::IURI **ppURI );
	virtual URILoader::Result GetChannelEventListener( URILoader::IChannelEventListener ** pValue );
	virtual URILoader::Result SetChannelEventListener( URILoader::IChannelEventSender* aValue );
	virtual URILoader::Result GetCacheStorage( URILoader::ICacheStorage **pValue );
	virtual URILoader::Result SetCacheStorage( URILoader::ICacheStorage *aValue );
	virtual URILoader::Result GetCookieStorage( URILoader::ICookieStorage **pValue );
	virtual URILoader::Result SetCookieStorage( URILoader::ICookieStorage *aValue );
	virtual URILoader::Result GetContentType( URILoader::String *pValue );
	virtual URILoader::Result SetContentType( URILoader::String *aValue ) ;
	virtual URILoader::Result GetContentCharset( URILoader::String *pValue ) ;
	virtual URILoader::Result SetContentCharset(  URILoader::String *aValue );
	virtual URILoader::Result GetContentLength( long *pValue ) ;
	virtual URILoader::Result SetContentLength( long aValue );
	virtual URILoader::Result AsyncOpen( URILoader::IStreamListener *aListener, void* aContext) ;

	virtual URILoader::Result GetRequestMethod( URILoader::String *pValue ) ;
	virtual URILoader::Result SetRequestMethod( URILoader::String aValue ) ;
	virtual URILoader::Result GetReferrer( URILoader::IURI **ppValue ) ;
	virtual URILoader::Result SetReferrer( URILoader::IURI *aValue ) ;
	virtual URILoader::Result GetResponseStatus( unsigned long *pValue ) ;
	virtual URILoader::Result GetRequestSucceeded( BOOL *pValue ) ; 
	virtual URILoader::Result GetRequestHeader( URILoader::String *aHeader, URILoader::String *pValue );
	virtual URILoader::Result SetRequestHeader( URILoader::String *aHeader,
	                      URILoader::String *aValue,
	                      BOOL aMerge);
	virtual URILoader::Result GetResponseHeader(  URILoader::String *aHeader, URILoader::String *pValue );
	virtual URILoader::Result SetResponseHeader( URILoader::String *aHeader,
	                       URILoader::String aValue,
	                       BOOL aMerge) ;
	virtual URILoader::Result GetResponseHeaderMap( URILoader::HeaderMap* aVisitor) ;
	virtual URILoader::Result GetResponseHeaderString( char **pszHeaders ) ;
	virtual URILoader::Result IsNoStoreResponse( BOOL *pValue );
	virtual URILoader::Result IsNoCacheResponse( BOOL *pValue ) ;
	virtual URILoader::Result SetUploadStream( const char* pData, int len, URILoader::String contentType ) ;
	virtual URILoader::Result SetCallerClientID(int clientID );
	virtual URILoader::Result GetCallerClientID(int* clientID);

public:
	void OnData( char* pData, int len );
	void OnError( int _error );
	void OnComplete();

};

} // namespace  WebControl

#endif // __WebDownlaod_Defined