/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		WebPageInfo.cpp
 * @brief		implement IPageInfo of WebControl
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.18	v1.0		Created
 */

#include "WebPageInfo.h"

#include "CString.h"
#include "DocumentType.h"

namespace WebControl {

WebPageInfo::WebPageInfo( WebControl *pWebControl )
{
	m_pWebControl = pWebControl;
}

/*
 * @desc gets the title of page
 * @param out_pszBuffer : pointer to buffer
 * @param in_nBufferLen : length of buffer
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebPageInfo::getTitle( char* out_pszBuffer, int in_nBufferLen )
{
	WebCore::String title = m_pWebControl->GetMainWebFrame()->GetFrame()->document()->title();
    if( in_nBufferLen <= title.length())
    {
        AcStrncpy( out_pszBuffer, title.latin1().data(), in_nBufferLen - 1);
		//Changed the in_nBufferLen to in_nBufferLen-1 to stop szTitle getting corrupted if we have length of Title greater than 1024. Hema
		out_pszBuffer[in_nBufferLen-1] = 0;
    }
    else
    {
        AcStrcpy(out_pszBuffer, title.latin1().data());
    }

	return HRESULT_OK;
}

/*
 * @desc gets the url of page
 * @param out_pszBuffer : pointer to buffer
 * @param in_nBufferLen : length of buffer
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebPageInfo::getURL( char* out_pszBuffer, int in_nBufferLen )
{
	WebCore::String url = m_pWebControl->GetMainWebFrame()->GetFrame()->document()->documentURI();
    if(in_nBufferLen <= url.length())
    {
        AcStrncpy(out_pszBuffer, url.latin1().data(),  in_nBufferLen- 1);
		//Changed the in_nBufferLen to in_nBufferLen-1 to stop szURL getting corrupted if we have length of URL greater than 1024. Hema
		out_pszBuffer[in_nBufferLen-1]  = '\0';
    }
    else
    {
        AcStrcpy(out_pszBuffer, url.latin1().data());
    }
	return HRESULT_OK;
}
/*
 * @desc gets the size of page
 * @param out_pszBuffer : pointer to buffer
 * @param in_nBufferLen : length of buffer
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebPageInfo::getSize( int *out_size )
{
    *out_size = m_pWebControl->m_docSize;
	return HRESULT_OK;
}
/*
 * @desc gets last modified date of page
 * @param out_pszBuffer : pointer to buffer
 * @param in_nBufferLen : length of buffer
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebPageInfo::getLastModified( char* out_pszBuffer, int in_nBufferLen )
{
	WebCore::String time = m_pWebControl->GetMainWebFrame()->GetFrame()->lastModifiedTime();

    if(in_nBufferLen <= time.length())
    {
        AcStrncpy(out_pszBuffer, time.latin1().data(), in_nBufferLen - 1);
		out_pszBuffer[in_nBufferLen-1] = 0;
        
    }
    else
    {
        AcStrcpy(out_pszBuffer, time.latin1().data());
    }

	return HRESULT_OK;
}
/*
 * @desc gets the mime type of page
 * @param out_pszBuffer : pointer to buffer
 * @param in_nBufferLen : length of buffer
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebPageInfo::getMimetype( char* out_pszBuffer, int in_nBufferLen )
{
	WebCore::DocumentType* docType = m_pWebControl->GetMainWebFrame()->GetFrame()->document()->doctype();
	WebCore::String type;

    if(docType)
        type = docType->name();
    else
        type = "text/html";

    if(in_nBufferLen <= type.length())
    {
        AcStrncpy(out_pszBuffer, type.latin1().data(), in_nBufferLen - 1);
        out_pszBuffer[in_nBufferLen] = 0;
    }
    else
    {
        AcStrcpy(out_pszBuffer, type.latin1().data());
    }
	return HRESULT_OK;
}



} // namespace WebFrame