/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics.
 */
/**
 * @file		WebPerformanceLog.cpp
 * @brief		
 * @author	Sangheon Kim (sangheon77.kim@samsung.com) @n
					
 * @version	1.0
 * @date		2008.09.10	v1.0		Created							shkim @n
 */

#include "ShpJasmine.h"
#include "WebPerformanceLog.h"
#include <map>
#include <string>

typedef struct
{
	unsigned long totalTime;
	unsigned long totalCount;
	unsigned long startTick;
}WebPerfLog;

std::map< std::string, std::map<std::string, WebPerfLog*>*>* gpWebPerfLogMap = NULL;

void WebPerfLogStart( char* id, char* key)
{
	unsigned long tick = SFBSysGetTick();

	if( gpWebPerfLogMap == NULL )
	{
		gpWebPerfLogMap = new std::map< std::string, std::map<std::string, WebPerfLog*>*>();
	}

	std::map< std::string, std::map<std::string, WebPerfLog*>*>::iterator pos;
	pos = gpWebPerfLogMap->find( std::string(id) );

	std::map<std::string, WebPerfLog*>* pPerfLog = NULL;

	if( pos == gpWebPerfLogMap->end() )
	{
		pPerfLog = new std::map<std::string, WebPerfLog*>();
		gpWebPerfLogMap->insert(std::make_pair( std::string(id), pPerfLog ));
	}
	else
	{
		pPerfLog = pos->second;
	}

	std::map<std::string, WebPerfLog*>::iterator ppos;
	ppos = pPerfLog->find( std::string(key) );

	if( ppos == pPerfLog->end() )
	{
		WebPerfLog* newLog = new WebPerfLog();
		newLog->startTick = tick;
		newLog->totalTime = 0;
		newLog->totalCount = 0;

		pPerfLog->insert( std::make_pair( std::string(key), newLog ) );
	}
	else
	{
		WebPerfLog* log = ppos->second;
		log->startTick = tick;
	}
}

void WebPerfLogEnd( char* id, char* key)
{
	unsigned long tick = SFBSysGetTick();

	if( gpWebPerfLogMap == NULL )
	{
		return;
	}

	std::map< std::string, std::map<std::string, WebPerfLog*>*>::iterator pos;
	pos = gpWebPerfLogMap->find( std::string(id) );

	std::map<std::string, WebPerfLog*>* pPerfLog = NULL;

	if( pos == gpWebPerfLogMap->end() )
	{
		return;
	}
	else
	{
		pPerfLog = pos->second;
	}

	std::map<std::string, WebPerfLog*>::iterator ppos;
	ppos = pPerfLog->find( std::string(key) );

	if( ppos == pPerfLog->end() )
	{
		return;
	}
	else
	{
		WebPerfLog* log = ppos->second;
		log->totalCount = log->totalCount + 1;
		log->totalTime = log->totalTime + ( tick-log->startTick);
	}
}

void WebPerfLogPrint( char* id )
{
	if( gpWebPerfLogMap == NULL )
	{
		return;
	}

	std::map< std::string, std::map<std::string, WebPerfLog*>*>::iterator pos;
	pos = gpWebPerfLogMap->find( std::string(id) );

	std::map<std::string, WebPerfLog*>* pPerfLog = NULL;

	if( pos == gpWebPerfLogMap->end() )
	{
		return;
	}
	else
	{
		pPerfLog = pos->second;
	}


	if( pPerfLog->size() <= 0 )
		return;

	std::map<std::string, WebPerfLog*>::iterator ppos;

	
	for (ppos = pPerfLog->begin(); ppos != pPerfLog->end(); ++ppos) 
	{
		WebPerfLog* log;
		log = ppos->second;
		const char *s = ppos->first.c_str();
		SysDebug(( MID_EXCEPTION, "WebPerfLogPrint : id(%s), key(%s), totalcount(%d), totaltime(%d)\n"
			, id, s, log->totalCount, log->totalTime ));

	}
}


void WebPerlLogClean( char* id )
{
	if( gpWebPerfLogMap == NULL )
	{
		return;
	}

	std::map< std::string, std::map<std::string, WebPerfLog*>*>::iterator pos;
	pos = gpWebPerfLogMap->find( std::string(id) );

	std::map<std::string, WebPerfLog*>* pPerfLog = NULL;

	if( pos == gpWebPerfLogMap->end() )
	{
		return;
	}
	else
	{
		pPerfLog = pos->second;
	}


	std::map<std::string, WebPerfLog*>::iterator ppos;

	
	for (ppos = pPerfLog->begin(); ppos != pPerfLog->end(); ++ppos) 
	{
		WebPerfLog* log;
		log = ppos->second;
		if( log )
			delete log;
	}




	delete pPerfLog;

	gpWebPerfLogMap->erase(pos) ;


}


